from roggia import Circuit, ElementType, TwoPortElement

import sys
sys.path.append("../sources")
sys.path.append("../models")
import sources
import models
import numpy as np
import math

# ********************************************************
# ********************************************************
# *** BSIM-CMG 110.0.0 released by Sourabh Khandelwal on 01/01/2016 ****/
# *  BSIM Common Multi-Gate Model Equations (Verilog-A)
# ********************************************************
#
# ********************************************************
# * Copyright 2016 Regents of the University of California.
# * All rights reserved.
# *
# * Project Director: Prof. Chenming Hu.
# * Authors: Sriramkumar V., Navid Paydavosi, Juan Duarte, Sourabh Khandelwal, Darsen Lu,
# *          Chung-Hsun Lin, Mohan Dunga, Shijing Yao,
# *          Ali Niknejad, Chenming Hu
# ********************************************************
# ********************************************************
# *   NONDISCLOSURE STATEMENT
# Software is distributed as is, completely without warranty or service
# support. The University of California and its employees are not liable
# for the condition or performance of the software.
# The University of California owns the copyright and grants users a perpetual,
# irrevocable, worldwide, non-exclusive, royalty-free license with
# respect to the software as set forth below.
# The University of California hereby diself.sclaims all implied warranties.
# The University of California grants the users the right to modify, copy,
# and redistribute the software and documentation, both within the user's
# organization and externally, subject to the following restrictions
# 1. The users agree not to charge for the University of California code
# itself but may charge for additions, extensions, or support.
# 2. In any product based on the software, the users agree to acknowledge
# the University of California that developed the software. This
# acknowledgment shall appear in the product documentation.
# 3. The users agree to obey all U.S. Government restrictions governing
# redistribution or export of the software.
# 4. The users agree to reproduce any copyright notice which appears on
# the software on any copy or modification of such made available
# to others
# Agreed to on __Jan 01, 2016__________________
# By: ___University of California, Berkeley____
#     ___Chenming Hu_____________________
#     ___Professor in Graduate School _______
# ********************************************************
class bsim_cmg():

    # Clamped Exponential Function
    def lexp(self, x):

        if (x > self.EXPL_THRESHOLD):
            lexp  =  self.MAX_EXPL * (1.0 + x - self.EXPL_THRESHOLD);
        elif (x < - self.EXPL_THRESHOLD):
            lexp  =  self.MIN_EXPL
        else:
            lexp  =  math.exp(x);

        return lexp

    # Clamped log Function
    def lln(self, x):
        lln  =  math.log(max(x, self.N_MINLOG)) # natural log

        return lln

    # Hyperbolic Smoothing max Function
    def hypmax(self, x, xmin, c):
        hypmax  =  xmin + 0.5 * (x - xmin - c + math.sqrt((x - xmin - c) * (x - xmin - c) - 4.0 * xmin * c))

        return hypmax

    # Hyperbolic Smoothing Function
    def hypsmooth(self, x, c):
        hypsmooth  =  0.5 * (x + math.sqrt(x * x + 4.0 * c * c))

        return hypsmooth

    # Temperature Dependence Type
    def Tempdep(self, PARAML, PARAMT, DELTEMP, TEMPMOD):
        if (self.TEMPMOD != 0):
            Tempdep  =  PARAML + self.hypmax(PARAMT * DELTEMP, -PARAML, 1.0e-6)
        else:
            Tempdep  =  PARAML * self.hypsmooth(1.0 + PARAMT * DELTEMP - 1.0e-6, 1.0e-3)

        return Tempdep

    # Junction capacitance
    # ex:(ves_jct, self.Czbs, self.PBS_t, SBS, MJS, MJS2, Qes1)
    def BSIM6JunctnCap(self, vex, Cz, PB, SJ, MJ, MJ2):
        if (Cz > 0.0):
            T1 = vex / PB
            if (T1 < 0.9):
                if (SJ > 0.0):   # second-step junction
                    vec =  PB * (1.0 - self.lexp((1.0 / MJ) * self.lln(1.0/SJ)))  # Switch over voltage
                    pb2 =  PB * SJ * MJ2 / MJ / self.lexp(- (1.0 + MJ) * self.lln(1.0 - vec / PB))  # PB for second doping region
                    if (vex > vec):
                        arg =  1.0 - T1
                        if (MJ == 0.5):
                            sarg  =  1.0 / math.sqrt(arg)
                        else:
                            sarg  =  self.lexp(-MJ * self.lln(arg))
                        Qej =  PB * Cz * (1.0 - arg * sarg) / (1.0 - MJ)
                    else:   # vex < vec
                        arg =  1.0 - vec / PB
                        if (MJ == 0.5):
                            sarg  =  1.0 / math.sqrt(arg)
                        else:
                            sarg  =  self.lexp(-MJ * self.lln(arg))
                        Qec =  PB * Cz * (1.0 - arg * sarg) / (1.0 - MJ)
                        arg =  1.0 - (vex - vec) / pb2
                        if (MJ2 == 0.5):
                            sarg  =  1.0 / math.sqrt(arg)
                        else:
                            sarg  =  self.lexp(-MJ2 * self.lln(arg))
                        Qej =  Qec + SJ * pb2 * Cz * (1.0 - arg * sarg) / (1.0 - MJ2)
                else:   # single junction
                    arg =  1.0 - T1
                    if (MJ == 0.5):
                        sarg  =  1.0 / math.sqrt(arg)
                    else:
                        sarg  =  self.lexp(-MJ * self.lln(arg))
                    Qej =  PB * Cz * (1.0 - arg * sarg) / (1.0 - MJ)

            else:   # vex/PB>=0.9
                T2  =  self.lexp(-MJ * self.lln(0.1))
                T3  =  1.0 / (1.0-MJ)
                T4  =  T2 * (T1 - 1.0) * (5.0 * MJ * (T1-1.0) + (1.0 + MJ) )
                T5  =  T3 * (1.0 - 0.05 * MJ * (1.0 + MJ) * T2 )
                Qej =  PB * Cz * (T4 + T5) # Quadratic equation for Qej when vex/PB>=0.9
        else:
            Qej  =  0.0

        return Qej

    def Cfringe_2d(self, block_name, Hg, Hc, Lext, Wfin, Lc, Lg, Tox, Cf1, epssp):
        Hr =  2.3 + 0.2 * ((Hg) + (Tox)) / (Hc)
        Lr =  1.05
        Hgdelta    =  abs((Hg) + (Tox) - (Hc))
        Lmax =  (Lext) * Lr

        y    =  min((Hc), (Hg) + (Tox))
        x    =  (Lext) / (Hr + 1.0)
        Cnon =  1.7e12
        CcgSat =  epssp * (y - x) / (Lext)
        TT1    =  Cnon * CcgSat

        if(TT1 > self.EXPL_THRESHOLD):
            Ccg1 =  CcgSat
        else:
            Ccg1 =  1.0 / Cnon * math.log(1.0 + self.lexp(TT1))

        r1cf =  0.5 * min((Hc) / ((Hg) + (Tox)), ((Hg) + (Tox)) / (Hc))
        Rcf  =  Hgdelta * r1cf
        Ccg2 =  epssp * 2 / math.pi * math.log(((Lext) + 0.5 * math.pi * Rcf) / (Lext))

        Ccg  =  (Wfin) * (Ccg1 + Ccg2)

        x    =  Lmax / (Hg)
        C1   =  4.0 / (math.sqrt(2.0 * (x + 1)) * math.pi)
        C2   =  math.sqrt((Tox) * (Tox) + 2.0 * (Hg) * (Tox) + (Hg) * (Hg) * (x + 1)) * math.sqrt(x + 1) + (Tox) + (Hg) * x + (Hg)
        C3   =  (Tox) * math.sqrt((x + 1) * (x + 4)) + Tox * (x + 2)
        Cfglog =  epssp * (C1 * math.log(C2 / C3) + 12.27)

        dcf    =  Hr * Lr
        TT0    =  math.sqrt(dcf * dcf + 1.0)
        TT1    =  math.sqrt((dcf * dcf + 1) * ((dcf * (Tox)) * (dcf * (Tox)) + 2 * dcf * Lmax * (Tox) + (dcf * dcf + 1) * Lmax * Lmax)) + dcf * (Tox) + dcf * dcf * Lmax + Lmax
        TT2    =  (TT0 + 1.0) * (dcf * (Tox))
        Cfgsat =  2.0 * epssp * math.sqrt(2) / math.pi * (Cf1) * dcf / TT0 * math.log(TT1 / TT2)

        delta  =  1.2e-12
        TT1    =  Cfgsat - Cfglog - delta
        Cfg    =  (Wfin) * (Cfgsat - 0.5 * (TT1 + math.sqrt(TT1 * TT1 + 4 * delta * Cfgsat)))
        Cgg    =  Ccg + Cfg

        return Cgg

    def param_given(self, param):
        if param == "TFIN_TOP":
            return self.TFIN_TOP_given

        if param == "TFIN_BASE":
            return self.TFIN_BASE_given

        if param == "NFINNOM":
            return self.NFINNOM_given

        if param == "RHORSD":
            return self.RHORSD_given

        if param == "CGSO":
            return self.CGSO_given

        if param == "CGDO":
            return self.CGDO_given

        if param == "DLC":
            return self.DLC_given

        if param == "THETASCE":
            return self.THETASCE_given

        if param == "THETASW":
            return self.THETASW_given

        if param == "THETADIBL":
            return self.THETADIBL_given

        if param == "VFBSD":
            return self.VFBSD_given

        if param == "VFBSDCV":
            return self.VFBSDCV_given

        if param == "NVTM":
            return self.NVTM_given


    # Mathematical functions
    def cot(self, x):
        if x >= math.pi / 2:
            val = 0.0
        elif x <= -math.pi / 2:
            val = 0.0
        else:
            val = 1.0 / math.tan(x)

        return val

    def cosh(self, x):
        val  = (0.5 * (self.lexp(x) + self.lexp(-(x))))

        return val

    def define_TNOIMOD1(self):
        self.__TNOIMOD1__ = 1

    def define_RGATEMOD(self):
        self.__RGATEMOD__ = 1

    def define_P_CELSIUS0(self, val):
        self.P_CELSIUS0 = val

    def MPRnb(self,val,uni,        des):
        return val

    def MPRex(self,val,uni,exc,    des):
        if val == exc:
            sys.exit(0)
            print ("MPRex : value excluded")

        return val

    def MPRcc(self,val,uni,lwr,upr,des):
        if lwr <= val <= upr:
            return val

        print ("MPRcc : outwith limits")
        sys.exit(0)

    def MPRoo(self,val,uni,lwr,upr,des):
        if lwr <= val <= upr:
            return val

        print ("MPRoo : outwith limits")
        sys.exit(0)

    def MPRco(self,val,uni,lwr,upr,des):
        if lwr <= val <= upr:
            return val

        print ("MPRco : outwith limits")
        sys.exit(0)

    def MPRoc(self,val,uni,lwr,upr,des):
        if lwr <= val <= upr:
            return val

        print ("MPRoc : outwith limits")
        sys.exit(0)

    def MPRcz(self,val,uni,des):
        if 0 <= val <= math.inf:
            return val

        print ("MPRcz : outwith limits")
        sys.exit(0)

    def MPRoz(self,val,uni,des):
        if 0 <= val <= math.inf:
            return val

        print ("MPRoz : outwith limits")
        sys.exit(0)

    def MPInb(self,val,uni,        des):
        return val

    def MPIex(self,val,uni,exc,    des):
        if val == exc:
            sys.exit(0)
            print ("MPRex : value excluded")

        return val

    def MPIcc(self,val,uni,lwr,upr,des):
        if lwr <= val <= upr:
            return val

        print (des)
        print (val)
        print ("MPIcc : outwith limits")
        sys.exit(0)

    def MPIoo(self,val,uni,lwr,upr,des):
        if lwr <= val <= upr:
            return val

        print ("MPIoo : outwith limits")
        sys.exit(0)

    def MPIco(self,val,uni,lwr,upr,des):
        if lwr <= val <= upr:
            return val

        print ("MPIco : outwith limits")
        sys.exit(0)

    def MPIoc(self,val,uni,lwr,upr,des):
        if lwr <= val <= upr:
            return val

        print ("MPIoc : outwith limits")
        sys.exit(0)

    def MPIcz(self,val,uni,des):
        if 0 <= val <= math.inf:
            return val

        print ("MPIcz : outwith limits")
        sys.exit(0)

    def MPIoz(self,val,uni,des):
        if 0 <= val <= math.inf:
            return val

        print ("MPIoz : outwith limits")
        sys.exit(0)

    def MPIsw(self,val,uni,des):
        if 0 <= val <= 1:
            return val

        print ("MPIsw : outwith limits")
        sys.exit(0)

    def MPIty(self,val,uni,des):
        if -1 <= val <= 1:
            if val != 0:
                return val

        print ("MPIty : outwith limits")
        sys.exit(0)

    def IPRnb(self,val,uni,        des):
        return val

    def IPRex(self,val,uni,exc,    des):
        if val == exc:
            sys.exit(0)
            print ("IPRex : value excluded")

        return val

    def IPRcc(self,val,uni,lwr,upr,des):
        if lwr <= val <= upr:
            return val

        print ("IPRcc : outwith limits")
        sys.exit(0)

    def IPRoo(self,val,uni,lwr,upr,des):
        if lwr <= val <= upr:
            return val

        print ("IPRoo : outwith limits")
        sys.exit(0)

    def IPRco(self,val,uni,lwr,upr,des):
        if lwr <= val <= upr:
            return val

        print ("IPRco : outwith limits")
        sys.exit(0)

    def IPRoc(self,val,uni,lwr,upr,des):
        if lwr <= val <= upr:
            return val

        print ("IPRoc : outwith limits")
        sys.exit(0)

    def IPRcz(self,val,uni,des):
        if 0 <= val <= math.inf:
            return val

        print ("IPRcz : outwith limits")
        sys.exit(0)

    def IPRoz(self,val,uni,des):
        if 0 <= val <= math.inf:
            return val

        print ("IPRoz : outwith limits")
        sys.exit(0)

    def IPInb(self,val,uni,        des):
        return val

    def IPIex(self,val,uni,exc,    des):
        if val == exc:
            sys.exit(0)
            print ("IPRex : value excluded")

        return val

    def IPIcc(self,val,uni,lwr,upr,des):
        if lwr <= val <= upr:
            return val

        print ("IPIcc : outwith limits")
        sys.exit(0)

    def IPIoo(self,val,uni,lwr,upr,des):
        if lwr <= val <= upr:
            return val

        print ("IPIoo : outwith limits")
        sys.exit(0)

    def IPIco(self,val,uni,lwr,upr,des):
        if lwr <= val <= upr:
            return val

        print ("IPIco : outwith limits")
        sys.exit(0)

    def IPIoc(self,val,uni,lwr,upr,des):
        if lwr <= val <= upr:
            return val

        print ("IPIoc : outwith limits")
        sys.exit(0)

    def IPIcz(self,val,uni,des):
        if 0 <= val <= math.inf:
            if val != 0:
                return val

        print ("IPIcz : outwith limits")
        sys.exit(0)

    def IPIoz(self,val,uni,des):
        if 0 <= val <= math.inf:
            if val != 0:
                return val

        print ("IPIoz : outwith limits")
        sys.exit(0)

    def set_ddt(self, method):
        self.bdf_ddt = method

    def ddt(self, q, q_step, delta_t):

        if self.bdf_ddt:
            val = ((147 / 60) * q) + \
                  (-6 * q_step[0]) + \
                  ((15/2) * q_step[1]) + \
                  (-(20/3) * q_step[2]) + \
                  ((15/4) * q_step[3]) + \
                  (-(6/5) * q_step[4]) + \
                  (-(1/6) * q_step[5])

        else:
            val = (q - q_step[0])

        #val = (q - q_step) / delta_t
        return (1 / delta_t) * val

    def update_charge(self):
        for i in range(1, 5):
            self.qd_step[i] = self.qd_step[i-1]
            self.qg_step[i] = self.qg_step[i-1]
            self.Qes_step[i] = self.Qes_step[i-1]
            self.Qed_step[i] = self.Qed_step[i-1]
            self.Qeg_step[i] = self.Qeg_step[i-1]
            self.qgs_parasitic_step[i] = self.qgs_parasitic_step[i-1]
            self.qgd_parasitic_step[i] = self.qgd_parasitic_step[i-1]
            self.qds_fr_step[i] = self.qds_fr_step[i-1]
            self.qg_acc_step[i] = self.qg_acc_step[i-1]
            self.qb_acc_step[i] = self.qb_acc_step[i-1]

        self.qd_step[0]            = self.qd_current
        self.qg_step[0]            = self.qg_current
        self.Qes_step[0]           = self.Qes_current
        self.Qed_step[0]           = self.Qed_current
        self.Qeg_step[0]           = self.Qeg_current
        self.qgs_parasitic_step[0] = self.qgs_parasitic_current
        self.qgd_parasitic_step[0] = self.qgd_parasitic_current
        self.qds_fr_step[0]        = self.qds_fr_current
        self.qg_acc_step[0]        = self.qg_acc_current
        self.qb_acc_step[0]        = self.qb_acc_current


    def update_vccs_mask(self, vccs_mask):
        vccs_mask[:,self.V_IntrinsicGate_si_ref] = 0
        vccs_mask[:,self.V_di_si_ref]            = 0
        vccs_mask[:,self.V_si_di_ref]            = 0
        vccs_mask[:,self.V_d_s_ref]              = 0
        vccs_mask[:,self.V_d_di_ref]             = 0
        vccs_mask[:,self.V_s_si_ref]             = 0
        vccs_mask[:,self.V_IntrinsicGate_di_ref] = 0
        vccs_mask[:,self.V_e_si_ref]             = 0
        vccs_mask[:,self.V_e_di_ref]             = 0
        vccs_mask[:,self.V_IntrinsicGate_e_ref]  = 0
        vccs_mask[:,self.V_GateEdgeNode_si_ref]  = 0
        vccs_mask[:,self.V_GateEdgeNode_di_ref]  = 0

        if (self.CGEOMOD == 1):
            vccs_mask[:,self.V_GateEdgeNode_s_ref] = 0
            vccs_mask[:,self.V_GateEdgeNode_d_ref] = 0

        vccs_mask[:,self.V_e_GateEdgeNode_ref] = 0

        if self.__RGATEMOD__:
            vccs_mask[:,self.V_g_ge_ref] = 0

        return vccs_mask

    def get_vccs_values(self, vccs_values):
        vccs_values[self.V_IntrinsicGate_si_ref] = self.I_IntrinsicGate_si
        vccs_values[self.V_di_si_ref]            = self.I_di_si
        vccs_values[self.V_si_di_ref]            = self.I_si_di
        vccs_values[self.V_d_s_ref]              = self.I_d_s
        vccs_values[self.V_d_di_ref]             = self.I_d_di
        vccs_values[self.V_s_si_ref]             = self.I_s_si
        vccs_values[self.V_IntrinsicGate_di_ref] = self.I_IntrinsicGate_di
        vccs_values[self.V_e_si_ref]             = self.I_e_si
        vccs_values[self.V_e_di_ref]             = self.I_e_di
        vccs_values[self.V_IntrinsicGate_e_ref]  = self.I_IntrinsicGate_e
        vccs_values[self.V_GateEdgeNode_si_ref]  = self.I_GateEdgeNode_si
        vccs_values[self.V_GateEdgeNode_di_ref]  = self.I_GateEdgeNode_di

        if (self.CGEOMOD == 1):
            vccs_values[self.V_GateEdgeNode_s_ref] = self.I_GateEdgeNode_s
            vccs_values[self.V_GateEdgeNode_d_ref] = self.I_GateEdgeNode_d

        vccs_values[self.V_e_GateEdgeNode_ref] = self.I_e_GateEdgeNode

        if self.__RGATEMOD__:
            vccs_values[self.V_g_ge_ref] = self.I_g_ge

        return vccs_values

    def add_edges(self, ckt, g, s, d, e):

        # Get the start edge reference from the target Circuit
        ser = ckt.get_num_edges()

        if self.__RGATEMOD__:
            ge = ckt.get_internal_node()
            GateEdgeNode = ge

            I_g_ge = TwoPortElement()
            I_g_ge.set_type(ElementType.current_src)
            I_g_ge.set_value(0)
            I_g_ge.set_instance("I_g_ge")
            ckt.add_edge(g, ge, I_g_ge)
            self.V_g_ge_ref = ser
            ser             += 1
        else:
            GateEdgeNode = g

        if self.__NQSMOD1__:
            gi = ckt.get_internal_node()
            IntrinsicGate = gi

            I_GateEdgeNode_gi = TwoPortElement()
            I_GateEdgeNode_gi.set_type(ElementType.current_src)
            I_GateEdgeNode_gi.set_value(0)
            I_GateEdgeNode_gi.set_instance("I_GateEdgeNode_gi")
            ckt.add_edge(GateEdgeNode, gi, I_GateEdgeNode_gi)
            self.V_GateEdgeNode_gi_ref = ser
            ser                       += 1
        else:
            IntrinsicGate = GateEdgeNode

        si = ckt.get_internal_node()
        di = ckt.get_internal_node()

        # di_si
        I_di_si = TwoPortElement()
        I_di_si.set_type(ElementType.current_src)
        I_di_si.set_value(0)
        I_di_si.set_instance("I_di_si")
        ckt.add_edge(di, si, I_di_si)
        self.V_di_si_ref = ser
        ser             += 1

        # d_s
        I_d_s = TwoPortElement()
        I_d_s.set_type(ElementType.current_src)
        I_d_s.set_value(0)
        I_d_s.set_instance("I_d_s")
        ckt.add_edge(d, s, I_d_s)
        self.V_d_s_ref   = ser
        ser             += 1

        # d_di
        I_d_di = TwoPortElement()
        I_d_di.set_type(ElementType.current_src)
        I_d_di.set_value(0)
        I_d_di.set_instance("I_d_di")
        ckt.add_edge(d, di, I_d_di)
        self.V_d_di_ref   = ser
        ser              += 1

        # si_di
        I_si_di = TwoPortElement()
        I_si_di.set_type(ElementType.current_src)
        I_si_di.set_value(0)
        I_si_di.set_instance("I_si_di")
        ckt.add_edge(si, di, I_si_di)
        self.V_si_di_ref   = ser
        ser               += 1

        # s_si
        I_s_si = TwoPortElement()
        I_s_si.set_type(ElementType.current_src)
        I_s_si.set_value(0)
        I_s_si.set_instance("I_s_si")
        ckt.add_edge(s, si, I_s_si)
        self.V_s_si_ref    = ser
        ser               += 1

        # e_si
        I_e_si = TwoPortElement()
        I_e_si.set_type(ElementType.current_src)
        I_e_si.set_value(0)
        I_e_si.set_instance("I_e_si")
        ckt.add_edge(e, si, I_e_si)
        self.V_e_si_ref    = ser
        ser               += 1

        # e_di
        I_e_di = TwoPortElement()
        I_e_di.set_type(ElementType.current_src)
        I_e_di.set_value(0)
        I_e_di.set_instance("I_e_di")
        ckt.add_edge(e, di, I_e_di)
        self.V_e_di_ref    = ser
        ser               += 1

        # e_GateEdgeNode
        I_e_GateEdgeNode = TwoPortElement()
        I_e_GateEdgeNode.set_type(ElementType.current_src)
        I_e_GateEdgeNode.set_value(0)
        I_e_GateEdgeNode.set_instance("I_e_GateEdgeNode")
        ckt.add_edge(e, GateEdgeNode, I_e_GateEdgeNode)
        self.V_e_GateEdgeNode_ref   = ser
        ser                        += 1

        # GateEdgeNode_si
        I_GateEdgeNode_si = TwoPortElement()
        I_GateEdgeNode_si.set_type(ElementType.current_src)
        I_GateEdgeNode_si.set_value(0)
        I_GateEdgeNode_si.set_instance("I_GateEdgeNode_si")
        ckt.add_edge(GateEdgeNode, si, I_GateEdgeNode_si)
        self.V_GateEdgeNode_si_ref   = ser
        ser                         += 1

        # GateEdgeNode_di
        I_GateEdgeNode_di = TwoPortElement()
        I_GateEdgeNode_di.set_type(ElementType.current_src)
        I_GateEdgeNode_di.set_value(0)
        I_GateEdgeNode_di.set_instance("I_GateEdgeNode_di")
        ckt.add_edge(GateEdgeNode, di, I_GateEdgeNode_di)
        self.V_GateEdgeNode_di_ref   = ser
        ser                         += 1

        # IntrinsicGate_si
        I_IntrinsicGate_si = TwoPortElement()
        I_IntrinsicGate_si.set_type(ElementType.current_src)
        I_IntrinsicGate_si.set_value(0)
        I_IntrinsicGate_si.set_instance("I_IntrinsicGate_si")
        ckt.add_edge(IntrinsicGate, si, I_IntrinsicGate_si)
        self.V_IntrinsicGate_si_ref   = ser
        ser                          += 1

        # IntrinsicGate_di
        I_IntrinsicGate_di = TwoPortElement()
        I_IntrinsicGate_di.set_type(ElementType.current_src)
        I_IntrinsicGate_di.set_value(0)
        I_IntrinsicGate_di.set_instance("I_IntrinsicGate_di")
        ckt.add_edge(IntrinsicGate, di, I_IntrinsicGate_di)
        self.V_IntrinsicGate_di_ref   = ser
        ser                          += 1

        # IntrinsicGate_e
        I_IntrinsicGate_e = TwoPortElement()
        I_IntrinsicGate_e.set_type(ElementType.current_src)
        I_IntrinsicGate_e.set_value(0)
        I_IntrinsicGate_e.set_instance("I_IntrinsicGate_e")
        ckt.add_edge(IntrinsicGate, e, I_IntrinsicGate_e)
        self.V_IntrinsicGate_e_ref   = ser
        ser                         += 1

        if (self.CGEOMOD == 1):
            # GateEdgeNode_s
            I_GateEdgeNode_s = TwoPortElement()
            I_GateEdgeNode_s.set_type(ElementType.current_src)
            I_GateEdgeNode_s.set_value(0)
            I_GateEdgeNode_s.set_instance("I_GateEdgeNode_s")
            ckt.add_edge(GateEdgeNode, s, I_GateEdgeNode_s)
            self.V_GateEdgeNode_s_ref   = ser
            ser                        += 1

            # GateEdgeNode_d
            I_GateEdgeNode_d = TwoPortElement()
            I_GateEdgeNode_d.set_type(ElementType.current_src)
            I_GateEdgeNode_d.set_value(0)
            I_GateEdgeNode_d.set_instance("I_GateEdgeNode_d")
            ckt.add_edge(GateEdgeNode, d, I_GateEdgeNode_d)
            self.V_GateEdgeNode_d_ref   = ser
            ser

    def __init__(self, params_dict):

        # Numerical Constants
        self.EXPL_THRESHOLD     =  80.0
        self.MAX_EXPL           =  5.540622384e34
        self.MIN_EXPL           =  1.804851387e-35
        self.N_MINLOG           =  1.0e-38
        self.MEXPQM             =  4
        self.DELTA_1            =  0.02
        self.DELTA_ASYMM        =  0.04
        self.CONSTCtoK          =  273.15
        self.REFTEMP            =  300.15    # 27 degrees C

        self.P_CELSIUS0  = 27
        self.temperature = self.P_CELSIUS0

        self.__OPINFO__   = 1
        self.__DEBUG__    = 1
        self.__SHMOD__    = 0 # REVISIT this needs the Temp() function
        self.__RDSMOD__   = 1
        self.__NQSMOD1__  = 0
        self.__NQSMOD2__  = 0
        self.__RGATEMOD__ = 0
        self.__TNOIMOD1__ = 1

        # Model type definitions
        self.ntype = 1
        self.ptype = 0

        # Physical Constants
        self.q             = 1.60219e-19         # Coul
        self.EPS0          = 8.8542e-12          # F/m
        self.HBAR          = 1.05457e-34         # Joule-sec
        self.MEL           = 9.11e-31            # kg
        self.KboQ          = 8.617087e-5         # Joule / degree

        # Param given indicators
        self.TFIN_TOP_given  = 0
        self.TFIN_BASE_given = 0
        self.NFINNOM_given   = 0
        self.RHORSD_given    = 0
        self.CGSO_given      = 0
        self.CGDO_given      = 0
        self.DLC_given       = 0
        self.THETASCE_given  = 0
        self.THETASW_given   = 0
        self.THETADIBL_given = 0
        self.VFBSD_given     = 0
        self.VFBSDCV_given   = 0
        self.NVTM_given      = 0

        # Charge state
        #self.qd_step            = 0
        #self.qg_step            = 0
        #self.Qes_step           = 0
        #self.Qed_step           = 0
        #self.Qeg_step           = 0
        #self.qgs_parasitic_step = 0
        #self.qgd_parasitic_step = 0
        #self.qds_fr_step        = 0
        #self.qg_acc_step        = 0
        #self.qb_acc_step        = 0

        self.qd_step            = np.zeros(6)
        self.qg_step            = np.zeros(6)
        self.Qes_step           = np.zeros(6)
        self.Qed_step           = np.zeros(6)
        self.Qeg_step           = np.zeros(6)
        self.qgs_parasitic_step = np.zeros(6)
        self.qgd_parasitic_step = np.zeros(6)
        self.qds_fr_step        = np.zeros(6)
        self.qg_acc_step        = np.zeros(6)
        self.qb_acc_step        = np.zeros(6)

        self.bdf_ddt            = 0

        # ***************************
        # *   Instance Parameters   *
        # ***************************

        # Note: Some instance parameters are also model parameters. Please refer to the technical note for details.
        if params_dict.get('L') != None:
            val =  params_dict['L']
            self.L       = self.IPRco(val,"m"           ,1.0e-9      ,math.inf    ,"Designed gate length" )
            self.L_given = 1
        else:
            self.L       = self.IPRco(3.0e-8         ,"m"           ,1.0e-9      ,math.inf    ,"Designed gate length" )
            self.L_given = 0

        if params_dict.get('D') != None:
            val =  params_dict['D']
            self.D       = self.IPRco(val,"m"           ,1.0e-9      ,math.inf    ,"Diameter of the cylinder (GEOMOD=3)" )
            self.D_given = 1
        else:
            self.D       = self.IPRco(4.0e-8         ,"m"           ,1.0e-9      ,math.inf    ,"Diameter of the cylinder (GEOMOD=3)" )
            self.D_given = 0

        if params_dict.get('TFIN') != None:
            val =  params_dict['TFIN']
            self.TFIN    = self.IPRco(val,"m"           ,1.0e-9      ,math.inf    ,"Body (fin) thickness" )
            self.TFIN_given = 1
        else:
            self.TFIN    = self.IPRco(1.5e-8         ,"m"           ,1.0e-9      ,math.inf    ,"Body (fin) thickness" )
            self.TFIN_given = 0

        if params_dict.get('FPITCH') != None:
            val =  params_dict['FPITCH']
            self.FPITCH  = self.IPRco(val,"m"           ,self.TFIN   ,math.inf    ,"Fin pitch" )
            self.FPITCH_given = 1
        else:
            self.FPITCH  = self.IPRco(8.0e-8         ,"m"           ,self.TFIN   ,math.inf    ,"Fin pitch" )
            self.FPITCH_given = 0

        if params_dict.get('NF') != None:
            val =  params_dict['NF']
            self.NF      = self.IPIco(val,""            ,1           ,math.inf    ,"Number of fingers" )
            self.NF_given = 1
        else:
            self.NF      = self.IPIco(1              ,""            ,1           ,math.inf    ,"Number of fingers" )
            self.NF_given = 0

        if params_dict.get('NFIN') != None:
            val =  params_dict['NFIN']
            self.NFIN    = self.IPRoz(val,""            ,"Number of fins per finger (real number enables optimization)" )
            self.NFIN_given = 1
        else:
            self.NFIN    = self.IPRoz(1.0            ,""            ,"Number of fins per finger (real number enables optimization)" )
            self.NFIN_given = 0

        if params_dict.get('NGCON') != None:
            val =  params_dict['NGCON']
            self.NGCON   = self.IPIcc(val,""            ,1           ,2           ,"Number of gate contact (1 or 2 sided)" )
            self.NGCON_given = 1
        else:
            self.NGCON   = self.IPIcc(1              ,""            ,1           ,2           ,"Number of gate contact (1 or 2 sided)" )
            self.NGCON_given = 0

        if params_dict.get('ASEO') != None:
            val =  params_dict['ASEO']
            self.ASEO    = self.IPRcz(val,"m^2"         ,"Source-to-substrate overlap area through oxide" )
            self.ASEO_given = 1
        else:
            self.ASEO    = self.IPRcz(0.0            ,"m^2"         ,"Source-to-substrate overlap area through oxide" )
            self.ASEO_given = 0

        if params_dict.get('ADEO') != None:
            val =  params_dict['ADEO']
            self.ADEO    = self.IPRcz(val,"m^2"         ,"Drain-to-substrate overlap area through oxide" )
            self.ADEO_given = 1
        else:
            self.ADEO    = self.IPRcz(0.0            ,"m^2"         ,"Drain-to-substrate overlap area through oxide" )
            self.ADEO_given = 0

        if params_dict.get('PSEO') != None:
            val =  params_dict['PSEO']
            self.PSEO    = self.IPRcz(val,"m"           ,"Perimeter of source-to-substrate overlap region through oxide" )
            self.PSEO_given = 1
        else:
            self.PSEO    = self.IPRcz(0.0            ,"m"           ,"Perimeter of source-to-substrate overlap region through oxide" )
            self.PSEO_given = 0

        if params_dict.get('PDEO') != None:
            val =  params_dict['PDEO']
            self.PDEO    = self.IPRcz(val,"m"           ,"Perimeter of drain-to-substrate overlap region through oxide" )
            self.PDEO_given = 1
        else:
            self.PDEO    = self.IPRcz(0.0            ,"m"           ,"Perimeter of drain-to-substrate overlap region through oxide" )
            self.PDEO_given = 0

        if params_dict.get('ASEJ') != None:
            val =  params_dict['ASEJ']
            self.ASEJ    = self.IPRcz(val,"m^2"         ,"Source junction area (BULKMOD=1 or 2)" )
            self.ASEJ_given = 1
        else:
            self.ASEJ    = self.IPRcz(0.0            ,"m^2"         ,"Source junction area (BULKMOD=1 or 2)" )
            self.ASEJ_given = 0

        if params_dict.get('ADEJ') != None:
            val =  params_dict['ADEJ']
            self.ADEJ    = self.IPRcz(val,"m^2"         ,"Drain junction area (BULKMOD=1 or 2)" )
            self.ADEJ_given = 1
        else:
            self.ADEJ    = self.IPRcz(0.0            ,"m^2"         ,"Drain junction area (BULKMOD=1 or 2)" )
            self.ADEJ_given = 0

        if params_dict.get('PSEJ') != None:
            val =  params_dict['PSEJ']
            self.PSEJ    = self.IPRcz(val,"m"           ,"Source-to-substrate PN junction perimeter (BULKMOD=1 or 2)" )
            self.PSEJ_given = 1
        else:
            self.PSEJ    = self.IPRcz(0.0            ,"m"           ,"Source-to-substrate PN junction perimeter (BULKMOD=1 or 2)" )
            self.PSEJ_given = 0

        if params_dict.get('PDEJ') != None:
            val =  params_dict['PDEJ']
            self.PDEJ    = self.IPRcz(val,"m"           ,"Drain-to-substrate PN junction perimeter (BULKMOD=1 or 2)" )
            self.PDEJ_given = 1
        else:
            self.PDEJ    = self.IPRcz(0.0            ,"m"           ,"Drain-to-substrate PN junction perimeter (BULKMOD=1 or 2)" )
            self.PDEJ_given = 0

        if params_dict.get('COVS') != None:
            val =  params_dict['COVS']
            self.COVS    = self.IPRcz(val,"F/m"         ,"Constant gate-to-source overlap capacitance (CGEOMOD=1)" )
            self.COVS_given = 1
        else:
            self.COVS    = self.IPRcz(0.0            ,"F/m"         ,"Constant gate-to-source overlap capacitance (CGEOMOD=1)" )
            self.COVS_given = 0

        if params_dict.get('COVD') != None:
            val =  params_dict['COVD']
            self.COVD    = self.IPRcz(val,"F/m"         ,"Constant gate-to-drain overlap capacitance (CGEOMOD=1)" )
            self.COVD_given = 1
        else:
            self.COVD    = self.IPRcz(self.COVS      ,"F/m"         ,"Constant gate-to-drain overlap capacitance (CGEOMOD=1)" )
            self.COVD_given = 0

        if params_dict.get('CGSP') != None:
            val =  params_dict['CGSP']
            self.CGSP    = self.IPRcz(val,"F/m"         ,"Constant gate-to-source fringe capacitance (CGEOMOD=1)" )
            self.CGSP_given = 1
        else:
            self.CGSP    = self.IPRcz(0.0            ,"F/m"         ,"Constant gate-to-source fringe capacitance (CGEOMOD=1)" )
            self.CGSP_given = 0

        if params_dict.get('CGDP') != None:
            val =  params_dict['CGDP']
            self.CGDP    = self.IPRcz(val,"F/m"         ,"Constant gate-to-drain fringe capacitance (CGEOMOD=1)" )
            self.CGDP_given = 1
        else:
            self.CGDP    = self.IPRcz(0.0            ,"F/m"         ,"Constant gate-to-drain fringe capacitance (CGEOMOD=1)" )
            self.CGDP_given = 0

        if params_dict.get('CDSP') != None:
            val =  params_dict['CDSP']
            self.CDSP    = self.IPRcz(val,"F"           ,"Constant drain-to-source fringe capacitance (all self.CGEOMOD)" )
            self.CDSP_given = 1
        else:
            self.CDSP    = self.IPRcz(0.0            ,"F"           ,"Constant drain-to-source fringe capacitance (all self.CGEOMOD)" )
            self.CDSP_given = 0

        if params_dict.get('NRS') != None:
            val =  params_dict['NRS']
            self.NRS     = self.IPRcz(val,""            ,"Number of source diffusion squares" )
            self.NRS_given = 1
        else:
            self.NRS     = self.IPRcz(0.0            ,""            ,"Number of source diffusion squares" )
            self.NRS_given = 0

        if params_dict.get('NRD') != None:
            val =  params_dict['NRD']
            self.NRD     = self.IPRcz(val,""            ,"Number of source diffusion squares" )
            self.NRD_given = 1
        else:
            self.NRD     = self.IPRcz(0.0            ,""            ,"Number of source diffusion squares" )
            self.NRD_given = 0

        if params_dict.get('LRSD') != None:
            val =  params_dict['LRSD']
            self.LRSD    = self.IPRoz(val,"m"           ,"Length of the source/drain" )
            self.LRSD_given = 1
        else:
            self.LRSD    = self.IPRoz(self.L         ,"m"           ,"Length of the source/drain" )
            self.LRSD_given = 0

        if params_dict.get('NFINNOM') != None:
            val =  params_dict['NFINNOM']
            self.NFINNOM = self.IPRoz(val,""            ,"Nominal number of fins per finger" )
            self.NFINNOM_given = 1
        else:
            self.NFINNOM = self.IPRoz(1.0            ,""            ,"Nominal number of fins per finger" )
            self.NFINNOM_given = 0


        # Variability Handles
        if params_dict.get('XL') != None:
            val =  params_dict['XL']
            self.XL         = self.MPRnb(val,"m"           ,"L offset for channel length due to mask/etch effect" )
            self.XL_given = 1
        else:
            self.XL         = self.MPRnb(0.0            ,"m"           ,"L offset for channel length due to mask/etch effect" )
            self.XL_given = 0

        if params_dict.get('DTEMP') != None:
            val =  params_dict['DTEMP']
            self.DTEMP      = self.MPRnb(val,"Celsius"     ,"Variability in device temperature" )
            self.DTEMP_given = 1
        else:
            self.DTEMP      = self.MPRnb(0.0            ,"Celsius"     ,"Variability in device temperature" )
            self.DTEMP_given = 0

        if params_dict.get('DELVTRAND') != None:
            val =  params_dict['DELVTRAND']
            self.DELVTRAND  = self.MPRnb(val,"V"           ,"Variability in Vth" )
            self.DELVTRAND_given = 1
        else:
            self.DELVTRAND  = self.MPRnb(0.0            ,"V"           ,"Variability in Vth" )
            self.DELVTRAND_given = 0

        if params_dict.get('U0MULT') != None:
            val =  params_dict['U0MULT']
            self.U0MULT     = self.MPRcz(val,""            ,"Variability in carrier mobility" )
            self.U0MULT_given = 1
        else:
            self.U0MULT     = self.MPRcz(1.0            ,""            ,"Variability in carrier mobility" )
            self.U0MULT_given = 0

        if params_dict.get('IDS0MULT') != None:
            val =  params_dict['IDS0MULT']
            self.IDS0MULT   = self.MPRcz(val,""            ,"Variability in drain current for miscellaneous reasons" )
            self.IDS0MULT_given = 1
        else:
            self.IDS0MULT   = self.MPRcz(1.0            ,""            ,"Variability in drain current for miscellaneous reasons" )
            self.IDS0MULT_given = 0


        # ************************
        # *  Model Parameters    *
        # ************************
        if params_dict.get('DEVTYPE') != None:
            val =  params_dict['DEVTYPE']
            self.DEVTYPE    = self.MPIcc(val,""            ,0           ,1           ,"0: PMOS; 1: NMOS" )
            self.DEVTYPE_given = 1
        else:
            self.DEVTYPE    = self.MPIcc(self.ntype     ,""            ,0           ,1           ,"0: PMOS; 1: NMOS" )
            self.DEVTYPE_given = 0

        if params_dict.get('TYPE') != None:
            val =  params_dict['TYPE']
            self.TYPE       = self.MPIcc(val,""            ,0           ,1           ,"0: PMOS; 1: NMOS" )
            self.TYPE_given = 1
        else:
            self.TYPE       = self.MPIcc(0        ,""            ,0           ,1           ,"0: PMOS; 1: NMOS" )
            self.TYPE_given = 0

        if params_dict.get('BULKMOD') != None:
            val =  params_dict['BULKMOD']
            self.BULKMOD    = self.MPIcc(val,""            ,0           ,2           ,"0: SOI multi-gate; 1: Bulk multi-gate; 2: for decoupled bulk multi-gate" )
            self.BULKMOD_given = 1
        else:
            self.BULKMOD    = self.MPIcc(0              ,""            ,0           ,2           ,"0: SOI multi-gate; 1: Bulk multi-gate; 2: for decoupled bulk multi-gate" )
            self.BULKMOD_given = 0

        if params_dict.get('GEOMOD') != None:
            val =  params_dict['GEOMOD']
            self.GEOMOD     = self.MPIcc(val,""            ,0           ,4           ,"0: Double gate; 1: Triple gate; 2: Quadruple gate; 3: Cylindrical gate; 4: Unified fin Shape" )
            self.GEOMOD_given = 1
        else:
            self.GEOMOD     = self.MPIcc(0              ,""            ,0           ,4           ,"0: Double gate; 1: Triple gate; 2: Quadruple gate; 3: Cylindrical gate; 4: Unified fin Shape" )
            self.GEOMOD_given = 0

        if params_dict.get('CGEO1SW') != None:
            val =  params_dict['CGEO1SW']
            self.CGEO1SW    = self.MPIcc(val,""            ,0           ,1           ,"For CGEOMOD=1 only, this switch enables the parameters self.COVS, self.COVD, self.CGSP, and self.CGDP to be in F per fin, per gate-finger, per unit channel width" )
            self.CGEO1SW_given = 1
        else:
            self.CGEO1SW    = self.MPIcc(0              ,""            ,0           ,1           ,"For CGEOMOD=1 only, this switch enables the parameters self.COVS, self.COVD, self.CGSP, and self.CGDP to be in F per fin, per gate-finger, per unit channel width" )
            self.CGEO1SW_given = 0

        if params_dict.get('RDSMOD') != None:
            val =  params_dict['RDSMOD']
            self.RDSMOD     = self.MPIcc(val,""            ,0           ,2           ,"0: Internal S/D resistance model; 1: External S/D resistance model; 2: Both bias dependent and independent part of S/D resistance internal" )
            self.RDSMOD_given = 1
        else:
            self.RDSMOD     = self.MPIcc(0              ,""            ,0           ,2           ,"0: Internal S/D resistance model; 1: External S/D resistance model; 2: Both bias dependent and independent part of S/D resistance internal" )
            self.RDSMOD_given = 0

        if params_dict.get('ASYMMOD') != None:
            val =  params_dict['ASYMMOD']
            self.ASYMMOD    = self.MPIcc(val,""            ,0           ,1           ,"0: Turn off asymmetry model - forward mode parameters used; 1: Turn on asymmetry model" )
            self.ASYMMOD_given = 1
        else:
            self.ASYMMOD    = self.MPIcc(0              ,""            ,0           ,1           ,"0: Turn off asymmetry model - forward mode parameters used; 1: Turn on asymmetry model" )
            self.ASYMMOD_given = 0

        if params_dict.get('IGCMOD') != None:
            val =  params_dict['IGCMOD']
            self.IGCMOD     = self.MPIcc(val,""            ,0           ,1           ,"0: Turn off Igc, Igs and Igd; 1: Turn on Igc, Igs and Igd" )
            self.IGCMOD_given = 1
        else:
            self.IGCMOD     = self.MPIcc(1              ,""            ,0           ,1           ,"0: Turn off Igc, Igs and Igd; 1: Turn on Igc, Igs and Igd" )
            self.IGCMOD_given = 0

        if params_dict.get('IGBMOD') != None:
            val =  params_dict['IGBMOD']
            self.IGBMOD     = self.MPIcc(val,""            ,0           ,1           ,"0: Turn off Igb; 1: Turn on Igb" )
            self.IGBMOD_given = 1
        else:
            self.IGBMOD     = self.MPIcc(0              ,""            ,0           ,1           ,"0: Turn off Igb; 1: Turn on Igb" )
            self.IGBMOD_given = 0

        if params_dict.get('GIDLMOD') != None:
            val =  params_dict['GIDLMOD']
            self.GIDLMOD    = self.MPIcc(val,""            ,0           ,1           ,"0: Turn off GIDL/GISL current; 1: Turn on GIDL/GISL current" )
            self.GIDLMOD_given = 1
        else:
            self.GIDLMOD    = self.MPIcc(0              ,""            ,0           ,1           ,"0: Turn off GIDL/GISL current; 1: Turn on GIDL/GISL current" )
            self.GIDLMOD_given = 0

        if params_dict.get('IIMOD') != None:
            val =  params_dict['IIMOD']
            self.IIMOD      = self.MPIcc(val,""            ,0           ,2           ,"0: Turn off impact ionization current; 1: BSIM4-based model; 2: BSIMSOI-based model" )
            self.IIMOD_given = 1
        else:
            self.IIMOD      = self.MPIcc(0              ,""            ,0           ,2           ,"0: Turn off impact ionization current; 1: BSIM4-based model; 2: BSIMSOI-based model" )
            self.IIMOD_given = 0

        if params_dict.get('TNOIMOD') != None:
            val =  params_dict['TNOIMOD']
            self.TNOIMOD    = self.MPIcc(val,""            ,0           ,1           ,"0: Charge-based, 1: Correlated thermal noise model" )
            self.TNOIMOD_given = 1
        else:
            self.TNOIMOD    = self.MPIcc(0              ,""            ,0           ,1           ,"0: Charge-based, 1: Correlated thermal noise model" )
            self.TNOIMOD_given = 0

        if params_dict.get('NQSMOD') != None:
            val =  params_dict['NQSMOD']
            self.NQSMOD     = self.MPIcc(val,""            ,0           ,2           ,"0: Turn off NQS model; 1: NQS gate resistance (with gi node); 2: NQS charge deficit model from BSIM4 (with q node)" )
            self.NQSMOD_given = 1
        else:
            self.NQSMOD     = self.MPIcc(0              ,""            ,0           ,2           ,"0: Turn off NQS model; 1: NQS gate resistance (with gi node); 2: NQS charge deficit model from BSIM4 (with q node)" )
            self.NQSMOD_given = 0

        if params_dict.get('SHMOD') != None:
            val =  params_dict['SHMOD']
            self.SHMOD      = self.MPIcc(val,""            ,0           ,1           ,"0: Turn off self-heating; 1: Turn on self-heating" )
            self.SHMOD_given = 1
        else:
            self.SHMOD      = self.MPIcc(0              ,""            ,0           ,1           ,"0: Turn off self-heating; 1: Turn on self-heating" )
            self.SHMOD_given = 0

        if params_dict.get('TEMPMOD') != None:
            val =  params_dict['TEMPMOD']
            self.TEMPMOD    = self.MPIcc(val,""            ,0           ,1           ,"1: Change temperature dependence of specific parameters" )
            self.TEMPMOD_given = 1
        else:
            self.TEMPMOD    = self.MPIcc(0              ,""            ,0           ,1           ,"1: Change temperature dependence of specific parameters" )
            self.TEMPMOD_given = 0

        if params_dict.get('RGATEMOD') != None:
            val =  params_dict['RGATEMOD']
            self.RGATEMOD   = self.MPIcc(val,""            ,0           ,1           ,"0: Turn off gate electrode resistance (without ge node); 1: Turn on gate electrode resistance (with ge node)" )
            self.RGATEMOD_given = 1
        else:
            self.RGATEMOD   = self.MPIcc(0              ,""            ,0           ,1           ,"0: Turn off gate electrode resistance (without ge node); 1: Turn on gate electrode resistance (with ge node)" )
            self.RGATEMOD_given = 0

        if params_dict.get('RGEOMOD') != None:
            val =  params_dict['RGEOMOD']
            self.RGEOMOD    = self.MPIcc(val,""            ,0           ,1           ,"Geometry-dependent source/drain resistance; 0: RSH-based; 1: Holistic" )
            self.RGEOMOD_given = 1
        else:
            self.RGEOMOD    = self.MPIcc(0              ,""            ,0           ,1           ,"Geometry-dependent source/drain resistance; 0: RSH-based; 1: Holistic" )
            self.RGEOMOD_given = 0

        if params_dict.get('CGEOMOD') != None:
            val =  params_dict['CGEOMOD']
            self.CGEOMOD    = self.MPIcc(val,""            ,0           ,2           ,"Geometry-dependent parasitic capacitance model selector" )
            self.CGEOMOD_given = 1
        else:
            self.CGEOMOD    = self.MPIcc(0              ,""            ,0           ,2           ,"Geometry-dependent parasitic capacitance model selector" )
            self.CGEOMOD_given = 0

        if params_dict.get('SH_WARN') != None:
            val =  params_dict['SH_WARN']
            self.SH_WARN    = self.MPIcc(val,""            ,0           ,1           ,"0: Disable self-heating warnings; 1: Enable self-heating warnings" )
            self.SH_WARN_given = 1
        else:
            self.SH_WARN    = self.MPIcc(0              ,""            ,0           ,1           ,"0: Disable self-heating warnings; 1: Enable self-heating warnings" )
            self.SH_WARN_given = 0

        if params_dict.get('IGCLAMP') != None:
            val =  params_dict['IGCLAMP']
            self.IGCLAMP    = self.MPIcc(val,""            ,0           ,1           ,"0: Disable gate current clamps; 1: Enable gate current clamps" )
            self.IGCLAMP_given = 1
        else:
            self.IGCLAMP    = self.MPIcc(1              ,""            ,0           ,1           ,"0: Disable gate current clamps; 1: Enable gate current clamps" )
            self.IGCLAMP_given = 0

        if params_dict.get('LINT') != None:
            val =  params_dict['LINT']
            self.LINT       = self.MPRnb(val,"m"           ,"Length reduction parameter (dopant diffusion effect)" )
            self.LINT_given = 1
        else:
            self.LINT       = self.MPRnb(0.0            ,"m"           ,"Length reduction parameter (dopant diffusion effect)" )
            self.LINT_given = 0

        if params_dict.get('LL') != None:
            val =  params_dict['LL']
            self.LL         = self.MPRnb(val,"m^(LLN+1)"   ,"Length reduction parameter (dopant diffusion effect)" )
            self.LL_given = 1
        else:
            self.LL         = self.MPRnb(0.0            ,"m^(LLN+1)"   ,"Length reduction parameter (dopant diffusion effect)" )
            self.LL_given = 0

        if params_dict.get('LLN') != None:
            val =  params_dict['LLN']
            self.LLN        = self.MPRnb(val,""            ,"Length reduction parameter (dopant diffusion effect)" )
            self.LLN_given = 1
        else:
            self.LLN        = self.MPRnb(1.0            ,""            ,"Length reduction parameter (dopant diffusion effect)" )
            self.LLN_given = 0

        if params_dict.get('DLC') != None:
            val =  params_dict['DLC']
            self.DLC        = self.MPRnb(val,"m"           ,"Delta self.L for C-V model" )
            self.DLC_given = 1
        else:
            self.DLC        = self.MPRnb(0.0            ,"m"           ,"Delta self.L for C-V model" )
            self.DLC_given = 0

        if params_dict.get('DLCACC') != None:
            val =  params_dict['DLCACC']
            self.DLCACC     = self.MPRnb(val,"m"           ,"Delta self.L for C-V model in accumulation region (BULKMOD=1 or 2)" )
            self.DLCACC_given = 1
        else:
            self.DLCACC     = self.MPRnb(0.0            ,"m"           ,"Delta self.L for C-V model in accumulation region (BULKMOD=1 or 2)" )
            self.DLCACC_given = 0

        if params_dict.get('DLBIN') != None:
            val =  params_dict['DLBIN']
            self.DLBIN      = self.MPRnb(val,"m"           ,"Delta self.L for binning" )
            self.DLBIN_given = 1
        else:
            self.DLBIN      = self.MPRnb(0.0            ,"m"           ,"Delta self.L for binning" )
            self.DLBIN_given = 0

        if params_dict.get('LLC') != None:
            val =  params_dict['LLC']
            self.LLC        = self.MPRnb(val,"m^(LLN+1)"   ,"Length reduction parameter (dopant diffusion effect)" )
            self.LLC_given = 1
        else:
            self.LLC        = self.MPRnb(0.0            ,"m^(LLN+1)"   ,"Length reduction parameter (dopant diffusion effect)" )
            self.LLC_given = 0

        if params_dict.get('EOT') != None:
            val =  params_dict['EOT']
            self.EOT        = self.MPRco(val,"m"           ,1.0e-10     ,math.inf         ,"Equivalent oxide thickness" )
            self.EOT_given = 1
        else:
            self.EOT        = self.MPRco(1.0e-9         ,"m"           ,1.0e-10     ,math.inf         ,"Equivalent oxide thickness" )
            self.EOT_given = 0

        if params_dict.get('TOXP') != None:
            val =  params_dict['TOXP']
            self.TOXP       = self.MPRco(val,"m"           ,1.0e-10     ,math.inf         ,"Physical oxide thickness" )
            self.TOXP_given = 1
        else:
            self.TOXP       = self.MPRco(1.2e-9         ,"m"           ,1.0e-10     ,math.inf         ,"Physical oxide thickness" )
            self.TOXP_given = 0

        if params_dict.get('EOTBOX') != None:
            val =  params_dict['EOTBOX']
            self.EOTBOX     = self.MPRco(val,"m"           ,1.0e-9      ,math.inf         ,"Equivalent oxide thickness of the buried oxide (SOI Fmath.infET)" )
            self.EOTBOX_given = 1
        else:
            self.EOTBOX     = self.MPRco(1.4e-7         ,"m"           ,1.0e-9      ,math.inf         ,"Equivalent oxide thickness of the buried oxide (SOI Fmath.infET)" )
            self.EOTBOX_given = 0

        if params_dict.get('HFIN') != None:
            val =  params_dict['HFIN']
            self.HFIN       = self.MPRco(val,"m"           ,1.0e-9      ,math.inf         ,"Fin height" )
            self.HFIN_given = 1
        else:
            self.HFIN       = self.MPRco(3.0e-8         ,"m"           ,1.0e-9      ,math.inf         ,"Fin height" )
            self.HFIN_given = 0

        if params_dict.get('FECH') != None:
            val =  params_dict['FECH']
            self.FECH       = self.MPRcz(val,""            ,"End-channel factor for different orientation/shape" )
            self.FECH_given = 1
        else:
            self.FECH       = self.MPRcz(1.0            ,""            ,"End-channel factor for different orientation/shape" )
            self.FECH_given = 0

        if params_dict.get('DELTAW') != None:
            val =  params_dict['DELTAW']
            self.DELTAW     = self.MPRnb(val,"m"           ,"Change of effective width due to shape of fin/cylinder" )
            self.DELTAW_given = 1
        else:
            self.DELTAW     = self.MPRnb(0.0            ,"m"           ,"Change of effective width due to shape of fin/cylinder" )
            self.DELTAW_given = 0

        if params_dict.get('FECHCV') != None:
            val =  params_dict['FECHCV']
            self.FECHCV     = self.MPRcz(val,""            ,"CV end-channel factor for different orientation/shape" )
            self.FECHCV_given = 1
        else:
            self.FECHCV     = self.MPRcz(1.0            ,""            ,"CV end-channel factor for different orientation/shape" )
            self.FECHCV_given = 0

        if params_dict.get('DELTAWCV') != None:
            val =  params_dict['DELTAWCV']
            self.DELTAWCV   = self.MPRnb(val,"m"           ,"CV change of effective width due to shape of fin/cylinder" )
            self.DELTAWCV_given = 1
        else:
            self.DELTAWCV   = self.MPRnb(0.0            ,"m"           ,"CV change of effective width due to shape of fin/cylinder" )
            self.DELTAWCV_given = 0

        if params_dict.get('NBODY') != None:
            val =  params_dict['NBODY']
            self.NBODY      = self.MPRnb(val,"/m^3"        ,"Channel (body) doping" )
            self.NBODY_given = 1
        else:
            self.NBODY      = self.MPRnb(1.0e22         ,"/m^3"        ,"Channel (body) doping" )
            self.NBODY_given = 0

        if params_dict.get('NBODYN1') != None:
            val =  params_dict['NBODYN1']
            self.NBODYN1    = self.MPRnb(val,""            ,"NFIN dependence of channel (body) doping" )
            self.NBODYN1_given = 1
        else:
            self.NBODYN1    = self.MPRnb(0.0            ,""            ,"NFIN dependence of channel (body) doping" )
            self.NBODYN1_given = 0

        if params_dict.get('NBODYN2') != None:
            val =  params_dict['NBODYN2']
            self.NBODYN2    = self.MPRex(val,""            ,0.0         ,"NFIN dependence of channel (body) doping" )
            self.NBODYN2_given = 1
        else:
            self.NBODYN2    = self.MPRex(1.0e5          ,""            ,0.0         ,"NFIN dependence of channel (body) doping" )
            self.NBODYN2_given = 0

        if params_dict.get('NSD') != None:
            val =  params_dict['NSD']
            self.NSD        = self.MPRcc(val,"/m^3"        ,2.0e25      ,1.0e27      ,"Source/drain active doping concentration" )
            self.NSD_given = 1
        else:
            self.NSD        = self.MPRcc(2.0e26         ,"/m^3"        ,2.0e25      ,1.0e27      ,"Source/drain active doping concentration" )
            self.NSD_given = 0

        if params_dict.get('PHIG') != None:
            val =  params_dict['PHIG']
            self.PHIG       = self.MPRcz(val,"eV"          ,"Gate workfunction" )
            self.PHIG_given = 1
        else:
            self.PHIG       = self.MPRcz(4.61           ,"eV"          ,"Gate workfunction" )
            self.PHIG_given = 0

        if params_dict.get('PHIGL') != None:
            val =  params_dict['PHIGL']
            self.PHIGL      = self.MPRnb(val,"eV/m"        ,"Length dependence of gate workfunction" )
            self.PHIGL_given = 1
        else:
            self.PHIGL      = self.MPRnb(0.0            ,"eV/m"        ,"Length dependence of gate workfunction" )
            self.PHIGL_given = 0

        if params_dict.get('PHIGLT') != None:
            val =  params_dict['PHIGLT']
            self.PHIGLT     = self.MPRnb(val,"/m"          ,"Coupled self.NFIN and length dependence of gate workfunction" )
            self.PHIGLT_given = 1
        else:
            self.PHIGLT     = self.MPRnb(0.0            ,"/m"          ,"Coupled self.NFIN and length dependence of gate workfunction" )
            self.PHIGLT_given = 0

        if params_dict.get('PHIGN1') != None:
            val =  params_dict['PHIGN1']
            self.PHIGN1     = self.MPRnb(val,""            ,"NFIN dependence of gate workfunction" )
            self.PHIGN1_given = 1
        else:
            self.PHIGN1     = self.MPRnb(0.0            ,""            ,"NFIN dependence of gate workfunction" )
            self.PHIGN1_given = 0

        if params_dict.get('PHIGN2') != None:
            val =  params_dict['PHIGN2']
            self.PHIGN2     = self.MPRex(val,""            ,0.0         ,"NFIN dependence of gate workfunction" )
            self.PHIGN2_given = 1
        else:
            self.PHIGN2     = self.MPRex(1.0e5          ,""            ,0.0         ,"NFIN dependence of gate workfunction" )
            self.PHIGN2_given = 0

        if params_dict.get('EPSROX') != None:
            val =  params_dict['EPSROX']
            self.EPSROX     = self.MPRco(val,""            ,1.0         ,math.inf         ,"Relative dielectric constant of the gate dielectric" )
            self.EPSROX_given = 1
        else:
            self.EPSROX     = self.MPRco(3.9            ,""            ,1.0         ,math.inf         ,"Relative dielectric constant of the gate dielectric" )
            self.EPSROX_given = 0

        if params_dict.get('EPSRSUB') != None:
            val =  params_dict['EPSRSUB']
            self.EPSRSUB    = self.MPRco(val,""            ,1.0         ,math.inf         ,"Relative dielectric constant of the channel material" )
            self.EPSRSUB_given = 1
        else:
            self.EPSRSUB    = self.MPRco(11.9           ,""            ,1.0         ,math.inf         ,"Relative dielectric constant of the channel material" )
            self.EPSRSUB_given = 0

        if params_dict.get('EASUB') != None:
            val =  params_dict['EASUB']
            self.EASUB      = self.MPRcz(val,"eV"          ,"Electron affinity of substrate" )
            self.EASUB_given = 1
        else:
            self.EASUB      = self.MPRcz(4.05           ,"eV"          ,"Electron affinity of substrate" )
            self.EASUB_given = 0

        if params_dict.get('NI0SUB') != None:
            val =  params_dict['NI0SUB']
            self.NI0SUB     = self.MPRnb(val,"/m^3"        ,"Intrinsic carrier constant at 300.15K" )
            self.NI0SUB_given = 1
        else:
            self.NI0SUB     = self.MPRnb(1.1e16         ,"/m^3"        ,"Intrinsic carrier constant at 300.15K" )
            self.NI0SUB_given = 0

        if params_dict.get('BG0SUB') != None:
            val =  params_dict['BG0SUB']
            self.BG0SUB     = self.MPRnb(val,"eV"          ,"Bandgap of substrate at 300.15K" )
            self.BG0SUB_given = 1
        else:
            self.BG0SUB     = self.MPRnb(1.12           ,"eV"          ,"Bandgap of substrate at 300.15K" )
            self.BG0SUB_given = 0

        if params_dict.get('NC0SUB') != None:
            val =  params_dict['NC0SUB']
            self.NC0SUB     = self.MPRnb(val,"/m^3"        ,"Conduction band density of states" )
            self.NC0SUB_given = 1
        else:
            self.NC0SUB     = self.MPRnb(2.86e25        ,"/m^3"        ,"Conduction band density of states" )
            self.NC0SUB_given = 0

        if params_dict.get('NGATE') != None:
            val =  params_dict['NGATE']
            self.NGATE      = self.MPRnb(val,"/m^3"        ,"Parameter for poly gate doping. For metal gate please set self.NGATE = 0" )
            self.NGATE_given = 1
        else:
            self.NGATE      = self.MPRnb(0.0            ,"/m^3"        ,"Parameter for poly gate doping. For metal gate please set self.NGATE = 0" )
            self.NGATE_given = 0

        if params_dict.get('Imin') != None:
            val =  params_dict['Imin']
            self.Imin       = self.MPRnb(val,"A/m^2"       ,"Parameter for Vgs clamping for inversion region calculation in accumulation" )
            self.Imin_given = 1
        else:
            self.Imin       = self.MPRnb(1.0e-15        ,"A/m^2"       ,"Parameter for Vgs clamping for inversion region calculation in accumulation" )
            self.Imin_given = 0


        # Short Channel Effects
        if params_dict.get('CIT') != None:
            val =  params_dict['CIT']
            self.CIT       = self.MPRnb(val,"F/m^2"       ,"Parameter for interface trap" )
            self.CIT_given = 1
        else:
            self.CIT       = self.MPRnb(0.0            ,"F/m^2"       ,"Parameter for interface trap" )
            self.CIT_given = 0

        if params_dict.get('CITR') != None:
            val =  params_dict['CITR']
            self.CITR      = self.MPRnb(val,""            ,"Parameter for interface trap in reverse mode for asymmetric model" )
            self.CITR_given = 1
        else:
            self.CITR      = self.MPRnb(self.CIT       ,""            ,"Parameter for interface trap in reverse mode for asymmetric model" )
            self.CITR_given = 0

        if params_dict.get('CDSC') != None:
            val =  params_dict['CDSC']
            self.CDSC      = self.MPRnb(val,"F/m^2"       ,"Coupling capacitance between S/D and channel" )
            self.CDSC_given = 1
        else:
            self.CDSC      = self.MPRnb(7.0e-3         ,"F/m^2"       ,"Coupling capacitance between S/D and channel" )
            self.CDSC_given = 0

        if params_dict.get('CDSCN1') != None:
            val =  params_dict['CDSCN1']
            self.CDSCN1    = self.MPRnb(val,""            ,"NFIN dependence of CDSC" )
            self.CDSCN1_given = 1
        else:
            self.CDSCN1    = self.MPRnb(0.0            ,""            ,"NFIN dependence of CDSC" )
            self.CDSCN1_given = 0

        if params_dict.get('CDSCN2') != None:
            val =  params_dict['CDSCN2']
            self.CDSCN2    = self.MPRnb(val,""            ,"NFIN dependence of CDSC" )
            self.CDSCN2_given = 1
        else:
            self.CDSCN2    = self.MPRnb(1.0e5          ,""            ,"NFIN dependence of CDSC" )
            self.CDSCN2_given = 0

        if params_dict.get('CDSCD') != None:
            val =  params_dict['CDSCD']
            self.CDSCD     = self.MPRnb(val,"F/m^2"       ,"Drain-bias sensitivity of CDSC" )
            self.CDSCD_given = 1
        else:
            self.CDSCD     = self.MPRnb(7.0e-3         ,"F/m^2"       ,"Drain-bias sensitivity of CDSC" )
            self.CDSCD_given = 0

        if params_dict.get('CDSCDN1') != None:
            val =  params_dict['CDSCDN1']
            self.CDSCDN1   = self.MPRnb(val,""            ,"NFIN dependence of CDSCD" )
            self.CDSCDN1_given = 1
        else:
            self.CDSCDN1   = self.MPRnb(0.0            ,""            ,"NFIN dependence of CDSCD" )
            self.CDSCDN1_given = 0

        if params_dict.get('CDSCDN2') != None:
            val =  params_dict['CDSCDN2']
            self.CDSCDN2   = self.MPRex(val,""            ,0.0         ,"NFIN dependence of CDSCD" )
            self.CDSCDN2_given = 1
        else:
            self.CDSCDN2   = self.MPRex(1.0e5          ,""            ,0.0         ,"NFIN dependence of CDSCD" )
            self.CDSCDN2_given = 0

        if params_dict.get('CDSCDR') != None:
            val =  params_dict['CDSCDR']
            self.CDSCDR    = self.MPRnb(val,"F/m^2"       ,"Reverse-mode drain-bias sensitivity of CDSC" )
            self.CDSCDR_given = 1
        else:
            self.CDSCDR    = self.MPRnb(self.CDSCD     ,"F/m^2"       ,"Reverse-mode drain-bias sensitivity of CDSC" )
            self.CDSCDR_given = 0

        if params_dict.get('CDSCDRN1') != None:
            val =  params_dict['CDSCDRN1']
            self.CDSCDRN1  = self.MPRnb(val,""            ,"NFIN dependence of CDSCD" )
            self.CDSCDRN1_given = 1
        else:
            self.CDSCDRN1  = self.MPRnb(self.CDSCDN1   ,""            ,"NFIN dependence of CDSCD" )
            self.CDSCDRN1_given = 0

        if params_dict.get('CDSCDRN2') != None:
            val =  params_dict['CDSCDRN2']
            self.CDSCDRN2  = self.MPRex(val,""            ,0.0         ,"NFIN dependence of CDSCD" )
            self.CDSCDRN2_given = 1
        else:
            self.CDSCDRN2  = self.MPRex(self.CDSCDN2   ,""            ,0.0         ,"NFIN dependence of CDSCD" )
            self.CDSCDRN2_given = 0

        if params_dict.get('DVT0') != None:
            val =  params_dict['DVT0']
            self.DVT0      = self.MPRnb(val,""            ,"SCE coefficient" )
            self.DVT0_given = 1
        else:
            self.DVT0      = self.MPRnb(0.0            ,""            ,"SCE coefficient" )
            self.DVT0_given = 0

        if params_dict.get('DVT1') != None:
            val =  params_dict['DVT1']
            self.DVT1      = self.MPRnb(val,""            ,"SCE exponent coefficient. After binning it should be within (0:math.inf)" )
            self.DVT1_given = 1
        else:
            self.DVT1      = self.MPRnb(0.6            ,""            ,"SCE exponent coefficient. After binning it should be within (0:math.inf)" )
            self.DVT1_given = 0

        if params_dict.get('DVT1SS') != None:
            val =  params_dict['DVT1SS']
            self.DVT1SS    = self.MPRnb(val,""            ,"Subthreshold swing exponent coefficient. After binning it should be within (0:math.inf)" )
            self.DVT1SS_given = 1
        else:
            self.DVT1SS    = self.MPRnb(self.DVT1      ,""            ,"Subthreshold swing exponent coefficient. After binning it should be within (0:math.inf)" )
            self.DVT1SS_given = 0

        if params_dict.get('PHIN') != None:
            val =  params_dict['PHIN']
            self.PHIN      = self.MPRnb(val,"V"           ,"Nonuniform vertical doping effect on surface potential" )
            self.PHIN_given = 1
        else:
            self.PHIN      = self.MPRnb(0.05           ,"V"           ,"Nonuniform vertical doping effect on surface potential" )
            self.PHIN_given = 0

        if params_dict.get('ETA0') != None:
            val =  params_dict['ETA0']
            self.ETA0      = self.MPRnb(val,""            ,"DIBL coefficient" )
            self.ETA0_given = 1
        else:
            self.ETA0      = self.MPRnb(0.6            ,""            ,"DIBL coefficient" )
            self.ETA0_given = 0

        if params_dict.get('ETA0N1') != None:
            val =  params_dict['ETA0N1']
            self.ETA0N1    = self.MPRnb(val,""            ,"NFIN dependence of ETA0" )
            self.ETA0N1_given = 1
        else:
            self.ETA0N1    = self.MPRnb(0.0            ,""            ,"NFIN dependence of ETA0" )
            self.ETA0N1_given = 0

        if params_dict.get('ETA0N2') != None:
            val =  params_dict['ETA0N2']
            self.ETA0N2    = self.MPRco(val,""            ,1.0e-5      ,math.inf         ,"NFIN dependence of ETA0" )
            self.ETA0N2_given = 1
        else:
            self.ETA0N2    = self.MPRco(1.0e5          ,""            ,1.0e-5      ,math.inf         ,"NFIN dependence of ETA0" )
            self.ETA0N2_given = 0

        if params_dict.get('ETA0LT') != None:
            val =  params_dict['ETA0LT']
            self.ETA0LT    = self.MPRnb(val,"/m"          ,"Coupled self.NFIN and length dependence of ETA0" )
            self.ETA0LT_given = 1
        else:
            self.ETA0LT    = self.MPRnb(0.0            ,"/m"          ,"Coupled self.NFIN and length dependence of ETA0" )
            self.ETA0LT_given = 0

        if params_dict.get('TETA0') != None:
            val =  params_dict['TETA0']
            self.TETA0     = self.MPRnb(val,"/K"          ,"Temperature dependence of DIBL coefficient" )
            self.TETA0_given = 1
        else:
            self.TETA0     = self.MPRnb(0.0            ,"/K"          ,"Temperature dependence of DIBL coefficient" )
            self.TETA0_given = 0

        if params_dict.get('ETA0R') != None:
            val =  params_dict['ETA0R']
            self.ETA0R     = self.MPRnb(val,""            ,"Reverse-mode DIBL coefficient" )
            self.ETA0R_given = 1
        else:
            self.ETA0R     = self.MPRnb(self.ETA0      ,""            ,"Reverse-mode DIBL coefficient" )
            self.ETA0R_given = 0

        if params_dict.get('TETA0R') != None:
            val =  params_dict['TETA0R']
            self.TETA0R    = self.MPRnb(val,"/K"          ,"Temperature dependence of reverse-mode DIBL coefficient" )
            self.TETA0R_given = 1
        else:
            self.TETA0R    = self.MPRnb(self.TETA0     ,"/K"          ,"Temperature dependence of reverse-mode DIBL coefficient" )
            self.TETA0R_given = 0

        if params_dict.get('DSUB') != None:
            val =  params_dict['DSUB']
            self.DSUB      = self.MPRnb(val,""            ,"DIBL exponent coefficient" )
            self.DSUB_given = 1
        else:
            self.DSUB      = self.MPRnb(1.06           ,""            ,"DIBL exponent coefficient" )
            self.DSUB_given = 0

        if params_dict.get('DVTP0') != None:
            val =  params_dict['DVTP0']
            self.DVTP0     = self.MPRnb(val,""            ,"Coefficient for drain-induced Vth shift (DITS)" )
            self.DVTP0_given = 1
        else:
            self.DVTP0     = self.MPRnb(0.0            ,""            ,"Coefficient for drain-induced Vth shift (DITS)" )
            self.DVTP0_given = 0

        if params_dict.get('DVTP1') != None:
            val =  params_dict['DVTP1']
            self.DVTP1     = self.MPRnb(val,""            ,"DITS exponent coefficient" )
            self.DVTP1_given = 1
        else:
            self.DVTP1     = self.MPRnb(0.0            ,""            ,"DITS exponent coefficient" )
            self.DVTP1_given = 0

        if params_dict.get('ADVTP0') != None:
            val =  params_dict['ADVTP0']
            self.ADVTP0    = self.MPRnb(val,""            ,"Pre-exponential coefficient for DITS" )
            self.ADVTP0_given = 1
        else:
            self.ADVTP0    = self.MPRnb(0.0            ,""            ,"Pre-exponential coefficient for DITS" )
            self.ADVTP0_given = 0

        if params_dict.get('BDVTP0') != None:
            val =  params_dict['BDVTP0']
            self.BDVTP0    = self.MPRex(val,""            ,0.0         ,"Exponential coefficient for DITS" )
            self.BDVTP0_given = 1
        else:
            self.BDVTP0    = self.MPRex(1.0e-7         ,""            ,0.0         ,"Exponential coefficient for DITS" )
            self.BDVTP0_given = 0

        if params_dict.get('ADVTP1') != None:
            val =  params_dict['ADVTP1']
            self.ADVTP1    = self.MPRnb(val,""            ,"Pre-exponential coefficient for DVTP1" )
            self.ADVTP1_given = 1
        else:
            self.ADVTP1    = self.MPRnb(0.0            ,""            ,"Pre-exponential coefficient for DVTP1" )
            self.ADVTP1_given = 0

        if params_dict.get('BDVTP1') != None:
            val =  params_dict['BDVTP1']
            self.BDVTP1    = self.MPRex(val,""            ,0.0         ,"Exponential coefficient for DVTP1" )
            self.BDVTP1_given = 1
        else:
            self.BDVTP1    = self.MPRex(1.0e-7         ,""            ,0.0         ,"Exponential coefficient for DVTP1" )
            self.BDVTP1_given = 0

        if params_dict.get('DVTP2') != None:
            val =  params_dict['DVTP2']
            self.DVTP2     = self.MPRnb(val,""            ,"DITS model parameter" )
            self.DVTP2_given = 1
        else:
            self.DVTP2     = self.MPRnb(0.0            ,""            ,"DITS model parameter" )
            self.DVTP2_given = 0

        if params_dict.get('K1RSCE') != None:
            val =  params_dict['K1RSCE']
            self.K1RSCE    = self.MPRnb(val,"/V^(0.5)"    ,"K1 for reverse short channel effect calculation" )
            self.K1RSCE_given = 1
        else:
            self.K1RSCE    = self.MPRnb(0.0            ,"/V^(0.5)"    ,"K1 for reverse short channel effect calculation" )
            self.K1RSCE_given = 0

        if params_dict.get('LPE0') != None:
            val =  params_dict['LPE0']
            self.LPE0      = self.MPRnb(val,"m"           ,"Equivalent length of pocket region at zero bias" )
            self.LPE0_given = 1
        else:
            self.LPE0      = self.MPRnb(5.0e-9         ,"m"           ,"Equivalent length of pocket region at zero bias" )
            self.LPE0_given = 0

        if params_dict.get('DVTSHIFT') != None:
            val =  params_dict['DVTSHIFT']
            self.DVTSHIFT  = self.MPRnb(val,"V"           ,"Vth shift handle" )
            self.DVTSHIFT_given = 1
        else:
            self.DVTSHIFT  = self.MPRnb(0.0            ,"V"           ,"Vth shift handle" )
            self.DVTSHIFT_given = 0

        if params_dict.get('DVTSHIFTR') != None:
            val =  params_dict['DVTSHIFTR']
            self.DVTSHIFTR = self.MPRnb(val,""            ,"Vth shift handle for asymmetric mode" )
            self.DVTSHIFTR_given = 1
        else:
            self.DVTSHIFTR = self.MPRnb(self.DVTSHIFT  ,""            ,"Vth shift handle for asymmetric mode" )
            self.DVTSHIFTR_given = 0

        if params_dict.get('THETASCE') != None:
            val =  params_dict['THETASCE']
            self.THETASCE  = self.MPRnb(val,""            ,"Vth roll-off length dependence. If defined by user, it will overwrite self.Theta_SCE in the code")
            self.THETASCE_given = 1
        else:
            self.THETASCE  = self.MPRnb(0.0            ,""            ,"Vth roll-off length dependence. If defined by user, it will overwrite self.Theta_SCE in the code")
            self.THETASCE_given = 0

        if params_dict.get('THETADIBL') != None:
            val =  params_dict['THETADIBL']
            self.THETADIBL = self.MPRnb(val,""            ,"DIBL length dependence. If defined by user, will overwrite self.Theta_DIBL in the code" )
            self.THETADIBL_given = 1
        else:
            self.THETADIBL = self.MPRnb(0.0            ,""            ,"DIBL length dependence. If defined by user, will overwrite self.Theta_DIBL in the code" )
            self.THETADIBL_given = 0

        if params_dict.get('THETASW') != None:
            val =  params_dict['THETASW']
            self.THETASW   = self.MPRnb(val,""            ,"Subthreshold swing length dependence. If defined by user, it will overwrite self.Theta_SW in the code" )
            self.THETASW_given = 1
        else:
            self.THETASW   = self.MPRnb(0.0            ,""            ,"Subthreshold swing length dependence. If defined by user, it will overwrite self.Theta_SW in the code" )
            self.THETASW_given = 0

        if params_dict.get('NVTM') != None:
            val =  params_dict['NVTM']
            self.NVTM      = self.MPRnb(val,"V"           ,"Subthreshold swing factor multiplied by Vtm. If defined by user, it will overwrite nVtm in the code" )
            self.NVTM_given = 1
        else:
            self.NVTM      = self.MPRnb(0.0            ,"V"           ,"Subthreshold swing factor multiplied by Vtm. If defined by user, it will overwrite nVtm in the code" )
            self.NVTM_given = 0


        # Lateral non-uniform doping effect (IV-CV Vth shift)
        if params_dict.get('K0') != None:
            val =  params_dict['K0']
            self.K0        = self.MPRnb(val,"V"           ,"Lateral self.NUD voltage parameter" )
            self.K0_given = 1
        else:
            self.K0        = self.MPRnb(0.0            ,"V"           ,"Lateral self.NUD voltage parameter" )
            self.K0_given = 0

        if params_dict.get('K01') != None:
            val =  params_dict['K01']
            self.K01       = self.MPRnb(val,"V/K"         ,"Temperature dependence of lateral self.NUD voltage parameter" )
            self.K01_given = 1
        else:
            self.K01       = self.MPRnb(0.0            ,"V/K"         ,"Temperature dependence of lateral self.NUD voltage parameter" )
            self.K01_given = 0

        if params_dict.get('K0SI') != None:
            val =  params_dict['K0SI']
            self.K0SI      = self.MPRnb(val,""            ,"Correction factor for strong inversion used in Mnud. After binning it should be within (0:math.inf)" )
            self.K0SI_given = 1
        else:
            self.K0SI      = self.MPRnb(1.0            ,""            ,"Correction factor for strong inversion used in Mnud. After binning it should be within (0:math.inf)" )
            self.K0SI_given = 0

        if params_dict.get('K0SI1') != None:
            val =  params_dict['K0SI1']
            self.K0SI1     = self.MPRnb(val,"/K"          ,"Temperature dependence of K0SI" )
            self.K0SI1_given = 1
        else:
            self.K0SI1     = self.MPRnb(0.0            ,"/K"          ,"Temperature dependence of K0SI" )
            self.K0SI1_given = 0

        if params_dict.get('K2SI') != None:
            val =  params_dict['K2SI']
            self.K2SI      = self.MPRnb(val,""            ,"Correction factor for strong inversion used in Mob" )
            self.K2SI_given = 1
        else:
            self.K2SI      = self.MPRnb(self.K0SI      ,""            ,"Correction factor for strong inversion used in Mob" )
            self.K2SI_given = 0

        if params_dict.get('K2SI1') != None:
            val =  params_dict['K2SI1']
            self.K2SI1     = self.MPRnb(val,""            ,"Temperature dependence of K2SI" )
            self.K2SI1_given = 1
        else:
            self.K2SI1     = self.MPRnb(self.K0SI1     ,""            ,"Temperature dependence of K2SI" )
            self.K2SI1_given = 0

        if params_dict.get('K0SISAT') != None:
            val =  params_dict['K0SISAT']
            self.K0SISAT   = self.MPRnb(val,""            ,"Correction factor for strong inversion used in Mnud" )
            self.K0SISAT_given = 1
        else:
            self.K0SISAT   = self.MPRnb(0.0            ,""            ,"Correction factor for strong inversion used in Mnud" )
            self.K0SISAT_given = 0

        if params_dict.get('K0SISAT1') != None:
            val =  params_dict['K0SISAT1']
            self.K0SISAT1  = self.MPRnb(val,""            ,"Temperature dependence of K0SISAT" )
            self.K0SISAT1_given = 1
        else:
            self.K0SISAT1  = self.MPRnb(0.0            ,""            ,"Temperature dependence of K0SISAT" )
            self.K0SISAT1_given = 0

        if params_dict.get('K2SISAT') != None:
            val =  params_dict['K2SISAT']
            self.K2SISAT   = self.MPRnb(val,""            ,"Correction factor for strong inversion used in Mob" )
            self.K2SISAT_given = 1
        else:
            self.K2SISAT   = self.MPRnb(self.K0SISAT   ,""            ,"Correction factor for strong inversion used in Mob" )
            self.K2SISAT_given = 0

        if params_dict.get('K2SISAT1') != None:
            val =  params_dict['K2SISAT1']
            self.K2SISAT1  = self.MPRnb(val,""            ,"Temperature dependence of K2SISAT" )
            self.K2SISAT1_given = 1
        else:
            self.K2SISAT1  = self.MPRnb(self.K0SISAT1  ,""            ,"Temperature dependence of K2SISAT" )
            self.K2SISAT1_given = 0


        # Body Effect for MG Devices on Bulk Substrate (ex: FinFETs on BULK)
        if params_dict.get('PHIBE') != None:
            val =  params_dict['PHIBE']
            self.PHIBE   = self.MPRnb(val,"V"           ,"Body effect voltage parameter. After binning it should be within [0.2:1.2]" )
            self.PHIBE_given = 1
        else:
            self.PHIBE   = self.MPRnb(0.7            ,"V"           ,"Body effect voltage parameter. After binning it should be within [0.2:1.2]" )
            self.PHIBE_given = 0

        if params_dict.get('K1') != None:
            val =  params_dict['K1']
            self.K1      = self.MPRco(val,"V^(0.5)"     ,1.0e-6      ,math.inf         ,"Body effect coefficient for subthreshold region" )
            self.K1_given = 1
        else:
            self.K1      = self.MPRco(1.0e-6         ,"V^(0.5)"     ,1.0e-6      ,math.inf         ,"Body effect coefficient for subthreshold region" )
            self.K1_given = 0

        if params_dict.get('K11') != None:
            val =  params_dict['K11']
            self.K11     = self.MPRnb(val,"V^(0.5)/K"   ,"Temperature dependence of K1" )
            self.K11_given = 1
        else:
            self.K11     = self.MPRnb(0.0            ,"V^(0.5)/K"   ,"Temperature dependence of K1" )
            self.K11_given = 0

        if params_dict.get('K2SAT') != None:
            val =  params_dict['K2SAT']
            self.K2SAT   = self.MPRnb(val,""            ,"Correction factor for self.K2 in saturation (high Vds)" )
            self.K2SAT_given = 1
        else:
            self.K2SAT   = self.MPRnb(0.0            ,""            ,"Correction factor for self.K2 in saturation (high Vds)" )
            self.K2SAT_given = 0

        if params_dict.get('K2SAT1') != None:
            val =  params_dict['K2SAT1']
            self.K2SAT1  = self.MPRnb(val,""            ,"Temperature dependence of K2SAT" )
            self.K2SAT1_given = 1
        else:
            self.K2SAT1  = self.MPRnb(0.0            ,""            ,"Temperature dependence of K2SAT" )
            self.K2SAT1_given = 0

        if params_dict.get('K2') != None:
            val =  params_dict['K2']
            self.K2      = self.MPRnb(val,""            ,"Body effect coefficient for BULKMOD==2" )
            self.K2_given = 1
        else:
            self.K2      = self.MPRnb(0.0            ,""            ,"Body effect coefficient for BULKMOD==2" )
            self.K2_given = 0

        if params_dict.get('K21') != None:
            val =  params_dict['K21']
            self.K21     = self.MPRnb(val,""            ,"Temperature dependence of K2" )
            self.K21_given = 1
        else:
            self.K21     = self.MPRnb(0.0            ,""            ,"Temperature dependence of K2" )
            self.K21_given = 0


        # Quantum Mechanical Effect
        if params_dict.get('QMFACTOR') != None:
            val =  params_dict['QMFACTOR']
            self.QMFACTOR    = self.MPRnb(val,""            ,"Prefactor + switch for QM Vth correction" )
            self.QMFACTOR_given = 1
        else:
            self.QMFACTOR    = self.MPRnb(0.0            ,""            ,"Prefactor + switch for QM Vth correction" )
            self.QMFACTOR_given = 0

        if params_dict.get('QMTCENCV') != None:
            val =  params_dict['QMTCENCV']
            self.QMTCENCV    = self.MPRnb(val,""            ,"Prefactor + switch for QM Width and Toxeff correction for CV" )
            self.QMTCENCV_given = 1
        else:
            self.QMTCENCV    = self.MPRnb(0.0            ,""            ,"Prefactor + switch for QM Width and Toxeff correction for CV" )
            self.QMTCENCV_given = 0

        if params_dict.get('QMTCENCVA') != None:
            val =  params_dict['QMTCENCVA']
            self.QMTCENCVA   = self.MPRnb(val,""            ,"Prefactor + switch for QM Width and Toxeff correction for CV (accumulation region)" )
            self.QMTCENCVA_given = 1
        else:
            self.QMTCENCVA   = self.MPRnb(0.0            ,""            ,"Prefactor + switch for QM Width and Toxeff correction for CV (accumulation region)" )
            self.QMTCENCVA_given = 0

        if params_dict.get('AQMTCEN') != None:
            val =  params_dict['AQMTCEN']
            self.AQMTCEN     = self.MPRnb(val,""            ,"Parameter for geometric dependence of Tcen on R/TFIN/HFIN" )
            self.AQMTCEN_given = 1
        else:
            self.AQMTCEN     = self.MPRnb(0.0            ,""            ,"Parameter for geometric dependence of Tcen on R/TFIN/HFIN" )
            self.AQMTCEN_given = 0

        if params_dict.get('BQMTCEN') != None:
            val =  params_dict['BQMTCEN']
            self.BQMTCEN     = self.MPRex(val,""            ,0.0         ,"Parameter for geometric dependence of Tcen on R/TFIN/HFIN" )
            self.BQMTCEN_given = 1
        else:
            self.BQMTCEN     = self.MPRex(1.2e-8         ,""            ,0.0         ,"Parameter for geometric dependence of Tcen on R/TFIN/HFIN" )
            self.BQMTCEN_given = 0

        if params_dict.get('ETAQM') != None:
            val =  params_dict['ETAQM']
            self.ETAQM       = self.MPRnb(val,""            ,"Bulk charge coefficient for Tcen" )
            self.ETAQM_given = 1
        else:
            self.ETAQM       = self.MPRnb(0.54           ,""            ,"Bulk charge coefficient for Tcen" )
            self.ETAQM_given = 0

        if params_dict.get('QM0') != None:
            val =  params_dict['QM0']
            self.QM0         = self.MPRnb(val,"V"           ,"Knee-point for Tcen in inversion (Charge normalized to Cox)" )
            self.QM0_given = 1
        else:
            self.QM0         = self.MPRnb(1.0e-3         ,"V"           ,"Knee-point for Tcen in inversion (Charge normalized to Cox)" )
            self.QM0_given = 0

        if params_dict.get('PQM') != None:
            val =  params_dict['PQM']
            self.PQM         = self.MPRnb(val,""            ,"Slope of normalized Tcen in inversion" )
            self.PQM_given = 1
        else:
            self.PQM         = self.MPRnb(0.66           ,""            ,"Slope of normalized Tcen in inversion" )
            self.PQM_given = 0

        if params_dict.get('QM0ACC') != None:
            val =  params_dict['QM0ACC']
            self.QM0ACC      = self.MPRnb(val,"V"           ,"Knee-point for Tcen in accumulation (Charge normalized to Cox)" )
            self.QM0ACC_given = 1
        else:
            self.QM0ACC      = self.MPRnb(1.0e-3         ,"V"           ,"Knee-point for Tcen in accumulation (Charge normalized to Cox)" )
            self.QM0ACC_given = 0

        if params_dict.get('PQMACC') != None:
            val =  params_dict['PQMACC']
            self.PQMACC      = self.MPRnb(val,""            ,"Slope of normalized Tcen in accumulation" )
            self.PQMACC_given = 1
        else:
            self.PQMACC      = self.MPRnb(0.66           ,""            ,"Slope of normalized Tcen in accumulation" )
            self.PQMACC_given = 0


        # Velocity Saturation Model
        if params_dict.get('VSAT') != None:
            val =  params_dict['VSAT']
            self.VSAT          = self.MPRnb(val,"m/s"         ,"Saturation velocity for the saturation region" )
            self.VSAT_given = 1
        else:
            self.VSAT          = self.MPRnb(8.5e4          ,"m/s"         ,"Saturation velocity for the saturation region" )
            self.VSAT_given = 0

        if params_dict.get('VSATR') != None:
            val =  params_dict['VSATR']
            self.VSATR         = self.MPRnb(val,"m/s"         ,"Saturation velocity for the saturation region in the reverse mode" )
            self.VSATR_given = 1
        else:
            self.VSATR         = self.MPRnb(self.VSAT      ,"m/s"         ,"Saturation velocity for the saturation region in the reverse mode" )
            self.VSATR_given = 0

        if params_dict.get('VSATN1') != None:
            val =  params_dict['VSATN1']
            self.VSATN1        = self.MPRnb(val,""            ,"NFIN dependence of VSAT" )
            self.VSATN1_given = 1
        else:
            self.VSATN1        = self.MPRnb(0.0            ,""            ,"NFIN dependence of VSAT" )
            self.VSATN1_given = 0

        if params_dict.get('VSATN2') != None:
            val =  params_dict['VSATN2']
            self.VSATN2        = self.MPRex(val,""            ,0.0         ,"NFIN dependence of VSAT" )
            self.VSATN2_given = 1
        else:
            self.VSATN2        = self.MPRex(1.0e5          ,""            ,0.0         ,"NFIN dependence of VSAT" )
            self.VSATN2_given = 0

        if params_dict.get('VSATRN1') != None:
            val =  params_dict['VSATRN1']
            self.VSATRN1       = self.MPRnb(val,""            ,"NFIN dependence of VSATR" )
            self.VSATRN1_given = 1
        else:
            self.VSATRN1       = self.MPRnb(self.VSATN1    ,""            ,"NFIN dependence of VSATR" )
            self.VSATRN1_given = 0

        if params_dict.get('VSATRN2') != None:
            val =  params_dict['VSATRN2']
            self.VSATRN2       = self.MPRex(val,""            ,0.0         ,"NFIN dependence of VSATR" )
            self.VSATRN2_given = 1
        else:
            self.VSATRN2       = self.MPRex(self.VSATN2    ,""            ,0.0         ,"NFIN dependence of VSATR" )
            self.VSATRN2_given = 0

        if params_dict.get('AVSAT') != None:
            val =  params_dict['AVSAT']
            self.AVSAT         = self.MPRnb(val,""            ,"Pre-exponential coefficient for VSAT" )
            self.AVSAT_given = 1
        else:
            self.AVSAT         = self.MPRnb(0.0            ,""            ,"Pre-exponential coefficient for VSAT" )
            self.AVSAT_given = 0

        if params_dict.get('BVSAT') != None:
            val =  params_dict['BVSAT']
            self.BVSAT         = self.MPRex(val,""            ,0.0         ,"Exponential coefficient for VSAT" )
            self.BVSAT_given = 1
        else:
            self.BVSAT         = self.MPRex(1.0e-7         ,""            ,0.0         ,"Exponential coefficient for VSAT" )
            self.BVSAT_given = 0

        if params_dict.get('VSAT1') != None:
            val =  params_dict['VSAT1']
            self.VSAT1         = self.MPRnb(val,"m/s"         ,"Velocity saturation parameter for Ion degradation - forward mode" )
            self.VSAT1_given = 1
        else:
            self.VSAT1         = self.MPRnb(self.VSAT      ,"m/s"         ,"Velocity saturation parameter for Ion degradation - forward mode" )
            self.VSAT1_given = 0

        if params_dict.get('VSAT1N1') != None:
            val =  params_dict['VSAT1N1']
            self.VSAT1N1       = self.MPRnb(val,""            ,"NFIN dependence of VSAT1" )
            self.VSAT1N1_given = 1
        else:
            self.VSAT1N1       = self.MPRnb(self.VSATN1    ,""            ,"NFIN dependence of VSAT1" )
            self.VSAT1N1_given = 0

        if params_dict.get('VSAT1N2') != None:
            val =  params_dict['VSAT1N2']
            self.VSAT1N2       = self.MPRex(val,""            ,0.0         ,"NFIN dependence of VSAT1" )
            self.VSAT1N2_given = 1
        else:
            self.VSAT1N2       = self.MPRex(self.VSATN2    ,""            ,0.0         ,"NFIN dependence of VSAT1" )
            self.VSAT1N2_given = 0

        if params_dict.get('VSAT1R') != None:
            val =  params_dict['VSAT1R']
            self.VSAT1R        = self.MPRnb(val,"m/s"         ,"Velocity saturation parameter for Ion degradation - reverse mode" )
            self.VSAT1R_given = 1
        else:
            self.VSAT1R        = self.MPRnb(self.VSAT1     ,"m/s"         ,"Velocity saturation parameter for Ion degradation - reverse mode" )
            self.VSAT1R_given = 0

        if params_dict.get('VSAT1RN1') != None:
            val =  params_dict['VSAT1RN1']
            self.VSAT1RN1      = self.MPRnb(val,""            ,"NFIN dependence of VSAT1R" )
            self.VSAT1RN1_given = 1
        else:
            self.VSAT1RN1      = self.MPRnb(self.VSAT1N1   ,""            ,"NFIN dependence of VSAT1R" )
            self.VSAT1RN1_given = 0

        if params_dict.get('VSAT1RN2') != None:
            val =  params_dict['VSAT1RN2']
            self.VSAT1RN2      = self.MPRex(val,""            ,0.0         ,"NFIN dependence of VSAT1R" )
            self.VSAT1RN2_given = 1
        else:
            self.VSAT1RN2      = self.MPRex(self.VSAT1N2   ,""            ,0.0         ,"NFIN dependence of VSAT1R" )
            self.VSAT1RN2_given = 0

        if params_dict.get('AVSAT1') != None:
            val =  params_dict['AVSAT1']
            self.AVSAT1        = self.MPRnb(val,""            ,"Pre-exponential coefficient for VSAT1" )
            self.AVSAT1_given = 1
        else:
            self.AVSAT1        = self.MPRnb(self.AVSAT     ,""            ,"Pre-exponential coefficient for VSAT1" )
            self.AVSAT1_given = 0

        if params_dict.get('BVSAT1') != None:
            val =  params_dict['BVSAT1']
            self.BVSAT1        = self.MPRex(val,""            ,0.0         ,"Exponential coefficient for VSAT1" )
            self.BVSAT1_given = 1
        else:
            self.BVSAT1        = self.MPRex(self.BVSAT     ,""            ,0.0         ,"Exponential coefficient for VSAT1" )
            self.BVSAT1_given = 0

        if params_dict.get('DELTAVSAT') != None:
            val =  params_dict['DELTAVSAT']
            self.DELTAVSAT     = self.MPRnb(val,""            ,"velocity saturation parameter in the linear region" )
            self.DELTAVSAT_given = 1
        else:
            self.DELTAVSAT     = self.MPRnb(1.0            ,""            ,"velocity saturation parameter in the linear region" )
            self.DELTAVSAT_given = 0

        if params_dict.get('PSAT') != None:
            val =  params_dict['PSAT']
            self.PSAT          = self.MPRnb(val,""            ,"Velocity saturation exponent, after binnig should be from [2.0:math.inf)" )
            self.PSAT_given = 1
        else:
            self.PSAT          = self.MPRnb(2.0            ,""            ,"Velocity saturation exponent, after binnig should be from [2.0:math.inf)" )
            self.PSAT_given = 0

        if params_dict.get('APSAT') != None:
            val =  params_dict['APSAT']
            self.APSAT         = self.MPRnb(val,""            ,"Pre-exponential coefficient for PSAT" )
            self.APSAT_given = 1
        else:
            self.APSAT         = self.MPRnb(0.0            ,""            ,"Pre-exponential coefficient for PSAT" )
            self.APSAT_given = 0

        if params_dict.get('BPSAT') != None:
            val =  params_dict['BPSAT']
            self.BPSAT         = self.MPRex(val,""            ,0.0         ,"Exponential coefficient for PSAT" )
            self.BPSAT_given = 1
        else:
            self.BPSAT         = self.MPRex(1.0            ,""            ,0.0         ,"Exponential coefficient for PSAT" )
            self.BPSAT_given = 0

        if params_dict.get('KSATIV') != None:
            val =  params_dict['KSATIV']
            self.KSATIV        = self.MPRnb(val,""            ,"Parameter for long channel Vdsat" )
            self.KSATIV_given = 1
        else:
            self.KSATIV        = self.MPRnb(1.0            ,""            ,"Parameter for long channel Vdsat" )
            self.KSATIV_given = 0

        if params_dict.get('KSATIVR') != None:
            val =  params_dict['KSATIVR']
            self.KSATIVR       = self.MPRnb(val,""            ,"KSATIV in asymmetric mode" )
            self.KSATIVR_given = 1
        else:
            self.KSATIVR       = self.MPRnb(self.KSATIV    ,""            ,"KSATIV in asymmetric mode" )
            self.KSATIVR_given = 0

        if params_dict.get('VSATCV') != None:
            val =  params_dict['VSATCV']
            self.VSATCV        = self.MPRnb(val,"m/s"         ,"Velocity saturation parameter for CV" )
            self.VSATCV_given = 1
        else:
            self.VSATCV        = self.MPRnb(self.VSAT      ,"m/s"         ,"Velocity saturation parameter for CV" )
            self.VSATCV_given = 0

        if params_dict.get('AVSATCV') != None:
            val =  params_dict['AVSATCV']
            self.AVSATCV       = self.MPRnb(val,""            ,"Pre-exponential coefficient for VSATCV" )
            self.AVSATCV_given = 1
        else:
            self.AVSATCV       = self.MPRnb(self.AVSAT     ,""            ,"Pre-exponential coefficient for VSATCV" )
            self.AVSATCV_given = 0

        if params_dict.get('BVSATCV') != None:
            val =  params_dict['BVSATCV']
            self.BVSATCV       = self.MPRex(val,""            ,0.0         ,"Exponential coefficient for VSATCV" )
            self.BVSATCV_given = 1
        else:
            self.BVSATCV       = self.MPRex(self.BVSAT     ,""            ,0.0         ,"Exponential coefficient for VSATCV" )
            self.BVSATCV_given = 0

        if params_dict.get('DELTAVSATCV') != None:
            val =  params_dict['DELTAVSATCV']
            self.DELTAVSATCV   = self.MPRnb(val,""            ,"Velocity saturation parameter in the linear region for the capacitance model" )
            self.DELTAVSATCV_given = 1
        else:
            self.DELTAVSATCV   = self.MPRnb(self.DELTAVSAT ,""            ,"Velocity saturation parameter in the linear region for the capacitance model" )
            self.DELTAVSATCV_given = 0

        if params_dict.get('PSATCV') != None:
            val =  params_dict['PSATCV']
            self.PSATCV        = self.MPRnb(val,""            ,"Velocity saturation exponent for C-V" )
            self.PSATCV_given = 1
        else:
            self.PSATCV        = self.MPRnb(self.PSAT      ,""            ,"Velocity saturation exponent for C-V" )
            self.PSATCV_given = 0

        if params_dict.get('APSATCV') != None:
            val =  params_dict['APSATCV']
            self.APSATCV       = self.MPRnb(val,""            ,"Pre-exponential coefficient for PSATCV" )
            self.APSATCV_given = 1
        else:
            self.APSATCV       = self.MPRnb(self.APSAT     ,""            ,"Pre-exponential coefficient for PSATCV" )
            self.APSATCV_given = 0

        if params_dict.get('BPSATCV') != None:
            val =  params_dict['BPSATCV']
            self.BPSATCV       = self.MPRex(val,""            ,0.0         ,"Exponential coefficient for PSATCV" )
            self.BPSATCV_given = 1
        else:
            self.BPSATCV       = self.MPRex(self.BPSAT     ,""            ,0.0         ,"Exponential coefficient for PSATCV" )
            self.BPSATCV_given = 0

        if params_dict.get('MEXP') != None:
            val =  params_dict['MEXP']
            self.MEXP          = self.MPRnb(val,""            ,"Smoothing function factor for Vdsat" )
            self.MEXP_given = 1
        else:
            self.MEXP          = self.MPRnb(4.0            ,""            ,"Smoothing function factor for Vdsat" )
            self.MEXP_given = 0

        if params_dict.get('AMEXP') != None:
            val =  params_dict['AMEXP']
            self.AMEXP         = self.MPRnb(val,""            ,"Pre-exponential coefficient for MEXP" )
            self.AMEXP_given = 1
        else:
            self.AMEXP         = self.MPRnb(0.0            ,""            ,"Pre-exponential coefficient for MEXP" )
            self.AMEXP_given = 0

        if params_dict.get('BMEXP') != None:
            val =  params_dict['BMEXP']
            self.BMEXP         = self.MPRnb(val,""            ,"Exponential coefficient for MEXP" )
            self.BMEXP_given = 1
        else:
            self.BMEXP         = self.MPRnb(1.0            ,""            ,"Exponential coefficient for MEXP" )
            self.BMEXP_given = 0

        if params_dict.get('MEXPR') != None:
            val =  params_dict['MEXPR']
            self.MEXPR         = self.MPRnb(val,""            ,"Reverse-mode smoothing function factor for Vdsat" )
            self.MEXPR_given = 1
        else:
            self.MEXPR         = self.MPRnb(self.MEXP      ,""            ,"Reverse-mode smoothing function factor for Vdsat" )
            self.MEXPR_given = 0

        if params_dict.get('AMEXPR') != None:
            val =  params_dict['AMEXPR']
            self.AMEXPR        = self.MPRnb(val,""            ,"Pre-exponential coefficient for MEXPR" )
            self.AMEXPR_given = 1
        else:
            self.AMEXPR        = self.MPRnb(self.AMEXP     ,""            ,"Pre-exponential coefficient for MEXPR" )
            self.AMEXPR_given = 0

        if params_dict.get('BMEXPR') != None:
            val =  params_dict['BMEXPR']
            self.BMEXPR        = self.MPRnb(val,""            ,"Exponential coefficient for MEXPR" )
            self.BMEXPR_given = 1
        else:
            self.BMEXPR        = self.MPRnb(self.BMEXP     ,""            ,"Exponential coefficient for MEXPR" )
            self.BMEXPR_given = 0

        if params_dict.get('PTWG') != None:
            val =  params_dict['PTWG']
            self.PTWG          = self.MPRnb(val,"/V^2"        ,"Gmsat degradation parameter - forward mode" )
            self.PTWG_given = 1
        else:
            self.PTWG          = self.MPRnb(0.0            ,"/V^2"        ,"Gmsat degradation parameter - forward mode" )
            self.PTWG_given = 0

        if params_dict.get('PTWGR') != None:
            val =  params_dict['PTWGR']
            self.PTWGR         = self.MPRnb(val,"/V^2"        ,"Gmsat degradation parameter - reverse mode" )
            self.PTWGR_given = 1
        else:
            self.PTWGR         = self.MPRnb(self.PTWG      ,"/V^2"        ,"Gmsat degradation parameter - reverse mode" )
            self.PTWGR_given = 0

        if params_dict.get('APTWG') != None:
            val =  params_dict['APTWG']
            self.APTWG         = self.MPRnb(val,""            ,"Pre-exponential coefficient for PTWG" )
            self.APTWG_given = 1
        else:
            self.APTWG         = self.MPRnb(0.0            ,""            ,"Pre-exponential coefficient for PTWG" )
            self.APTWG_given = 0

        if params_dict.get('BPTWG') != None:
            val =  params_dict['BPTWG']
            self.BPTWG         = self.MPRex(val,""            ,0.0         ,"Exponential coefficient for PTWG" )
            self.BPTWG_given = 1
        else:
            self.BPTWG         = self.MPRex(1.0e-7         ,""            ,0.0         ,"Exponential coefficient for PTWG" )
            self.BPTWG_given = 0

        if params_dict.get('AT') != None:
            val =  params_dict['AT']
            self.AT            = self.MPRnb(val,"/K"          ,"Saturation velocity temperature coefficient" )
            self.AT_given = 1
        else:
            self.AT            = self.MPRnb(-1.56e-3       ,"/K"          ,"Saturation velocity temperature coefficient" )
            self.AT_given = 0

        if params_dict.get('ATR') != None:
            val =  params_dict['ATR']
            self.ATR           = self.MPRnb(val,""            ,"Reverse-mode saturation velocity temperature coefficient" )
            self.ATR_given = 1
        else:
            self.ATR           = self.MPRnb(self.AT        ,""            ,"Reverse-mode saturation velocity temperature coefficient" )
            self.ATR_given = 0

        if params_dict.get('ATCV') != None:
            val =  params_dict['ATCV']
            self.ATCV          = self.MPRnb(val,"/K"          ,"Saturation velocity temperature coefficient for CV" )
            self.ATCV_given = 1
        else:
            self.ATCV          = self.MPRnb(self.AT        ,"/K"          ,"Saturation velocity temperature coefficient for CV" )
            self.ATCV_given = 0

        if params_dict.get('TMEXP') != None:
            val =  params_dict['TMEXP']
            self.TMEXP         = self.MPRnb(val,"/K"          ,"Temperature coefficient for Vdseff smoothing" )
            self.TMEXP_given = 1
        else:
            self.TMEXP         = self.MPRnb(0.0            ,"/K"          ,"Temperature coefficient for Vdseff smoothing" )
            self.TMEXP_given = 0

        if params_dict.get('TMEXPR') != None:
            val =  params_dict['TMEXPR']
            self.TMEXPR        = self.MPRnb(val,"/K"          ,"Reverse-mode temperature coefficient for Vdseff smoothing" )
            self.TMEXPR_given = 1
        else:
            self.TMEXPR        = self.MPRnb(self.TMEXP     ,"/K"          ,"Reverse-mode temperature coefficient for Vdseff smoothing" )
            self.TMEXPR_given = 0

        if params_dict.get('PTWGT') != None:
            val =  params_dict['PTWGT']
            self.PTWGT         = self.MPRnb(val,"/K"          ,"PTWG temperature coefficient" )
            self.PTWGT_given = 1
        else:
            self.PTWGT         = self.MPRnb(4.0e-3         ,"/K"          ,"PTWG temperature coefficient" )
            self.PTWGT_given = 0


        # Mobility Model
        if params_dict.get('U0') != None:
            val =  params_dict['U0']
            self.U0         = self.MPRnb(val,"m^2/(V*s)"   ,"Low-field mobility" )
            self.U0_given = 1
        else:
            self.U0         = self.MPRnb(3.0e-2         ,"m^2/(V*s)"   ,"Low-field mobility" )
            self.U0_given = 0

        if params_dict.get('U0R') != None:
            val =  params_dict['U0R']
            self.U0R        = self.MPRnb(val,"m^2/(V*s)"   ,"Reverse-mode low-field mobility" )
            self.U0R_given = 1
        else:
            self.U0R        = self.MPRnb(self.U0        ,"m^2/(V*s)"   ,"Reverse-mode low-field mobility" )
            self.U0R_given = 0

        if params_dict.get('U0N1') != None:
            val =  params_dict['U0N1']
            self.U0N1       = self.MPRnb(val,""            ,"NFIN dependence of U0" )
            self.U0N1_given = 1
        else:
            self.U0N1       = self.MPRnb(0.0            ,""            ,"NFIN dependence of U0" )
            self.U0N1_given = 0

        if params_dict.get('U0N1R') != None:
            val =  params_dict['U0N1R']
            self.U0N1R      = self.MPRnb(val,""            ,"Reverse-mode self.NFIN dependence of U0" )
            self.U0N1R_given = 1
        else:
            self.U0N1R      = self.MPRnb(self.U0N1      ,""            ,"Reverse-mode self.NFIN dependence of U0" )
            self.U0N1R_given = 0

        if params_dict.get('U0N2') != None:
            val =  params_dict['U0N2']
            self.U0N2       = self.MPRex(val,""            ,0.0         ,"NFIN dependence of U0" )
            self.U0N2_given = 1
        else:
            self.U0N2       = self.MPRex(1.0e5          ,""            ,0.0         ,"NFIN dependence of U0" )
            self.U0N2_given = 0

        if params_dict.get('U0N2R') != None:
            val =  params_dict['U0N2R']
            self.U0N2R      = self.MPRex(val,""            ,0.0         ,"Reverse-mode self.NFIN dependence of U0" )
            self.U0N2R_given = 1
        else:
            self.U0N2R      = self.MPRex(self.U0N2      ,""            ,0.0         ,"Reverse-mode self.NFIN dependence of U0" )
            self.U0N2R_given = 0

        if params_dict.get('U0LT') != None:
            val =  params_dict['U0LT']
            self.U0LT       = self.MPRnb(val,"/m"          ,"Coupled self.NFIN and length dependence of U0" )
            self.U0LT_given = 1
        else:
            self.U0LT       = self.MPRnb(0.0            ,"/m"          ,"Coupled self.NFIN and length dependence of U0" )
            self.U0LT_given = 0

        if params_dict.get('ETAMOB') != None:
            val =  params_dict['ETAMOB']
            self.ETAMOB     = self.MPRnb(val,""            ,"Effective field parameter" )
            self.ETAMOB_given = 1
        else:
            self.ETAMOB     = self.MPRnb(2.0            ,""            ,"Effective field parameter" )
            self.ETAMOB_given = 0

        if params_dict.get('UP') != None:
            val =  params_dict['UP']
            self.UP         = self.MPRnb(val,"um^LPA"      ,"Mobility self.L coefficient" )
            self.UP_given = 1
        else:
            self.UP         = self.MPRnb(0.0            ,"um^LPA"      ,"Mobility self.L coefficient" )
            self.UP_given = 0

        if params_dict.get('LPA') != None:
            val =  params_dict['LPA']
            self.LPA        = self.MPRnb(val,""            ,"Mobility self.L power coefficient" )
            self.LPA_given = 1
        else:
            self.LPA        = self.MPRnb(1.0            ,""            ,"Mobility self.L power coefficient" )
            self.LPA_given = 0

        if params_dict.get('UPR') != None:
            val =  params_dict['UPR']
            self.UPR        = self.MPRnb(val,"um^LPA"      ,"Reverse-mode mobility self.L coefficient" )
            self.UPR_given = 1
        else:
            self.UPR        = self.MPRnb(self.UP        ,"um^LPA"      ,"Reverse-mode mobility self.L coefficient" )
            self.UPR_given = 0

        if params_dict.get('LPAR') != None:
            val =  params_dict['LPAR']
            self.LPAR       = self.MPRnb(val,""            ,"Reverse-mode mobility self.L power coefficient" )
            self.LPAR_given = 1
        else:
            self.LPAR       = self.MPRnb(self.LPA       ,""            ,"Reverse-mode mobility self.L power coefficient" )
            self.LPAR_given = 0

        if params_dict.get('UA') != None:
            val =  params_dict['UA']
            self.UA         = self.MPRnb(val,"(cm/MV)^EU"  ,"Phonon/surface roughness scattering parameter" )
            self.UA_given = 1
        else:
            self.UA         = self.MPRnb(0.3            ,"(cm/MV)^EU"  ,"Phonon/surface roughness scattering parameter" )
            self.UA_given = 0

        if params_dict.get('UAR') != None:
            val =  params_dict['UAR']
            self.UAR        = self.MPRnb(val,"(cm/MV)^EU"  ,"Reverse-mode phonon/surface roughness scattering parameter" )
            self.UAR_given = 1
        else:
            self.UAR        = self.MPRnb(self.UA        ,"(cm/MV)^EU"  ,"Reverse-mode phonon/surface roughness scattering parameter" )
            self.UAR_given = 0

        if params_dict.get('AUA') != None:
            val =  params_dict['AUA']
            self.AUA        = self.MPRnb(val,""            ,"Pre-exponential coefficient for UA" )
            self.AUA_given = 1
        else:
            self.AUA        = self.MPRnb(0.0            ,""            ,"Pre-exponential coefficient for UA" )
            self.AUA_given = 0

        if params_dict.get('AUAR') != None:
            val =  params_dict['AUAR']
            self.AUAR       = self.MPRnb(val,""            ,"Reverse-mode pre-exponential coefficient for UA" )
            self.AUAR_given = 1
        else:
            self.AUAR       = self.MPRnb(self.AUA       ,""            ,"Reverse-mode pre-exponential coefficient for UA" )
            self.AUAR_given = 0

        if params_dict.get('BUA') != None:
            val =  params_dict['BUA']
            self.BUA        = self.MPRex(val,""            ,0.0         ,"Exponential coefficient for UA" )
            self.BUA_given = 1
        else:
            self.BUA        = self.MPRex(1.0e-7         ,""            ,0.0         ,"Exponential coefficient for UA" )
            self.BUA_given = 0

        if params_dict.get('BUAR') != None:
            val =  params_dict['BUAR']
            self.BUAR       = self.MPRex(val,""            ,0.0         ,"Reverse-mode exponential coefficient for UAR" )
            self.BUAR_given = 1
        else:
            self.BUAR       = self.MPRex(self.BUA       ,""            ,0.0         ,"Reverse-mode exponential coefficient for UAR" )
            self.BUAR_given = 0

        if params_dict.get('UC') != None:
            val =  params_dict['UC']
            self.UC         = self.MPRnb(val,"(1e-6*cm/MV^2)^EU"        ,"Body effect for mobility degradation parameter - BULKMOD=1 or 2" )
            self.UC_given = 1
        else:
            self.UC         = self.MPRnb(0.0            ,"(1e-6*cm/MV^2)^EU"        ,"Body effect for mobility degradation parameter - BULKMOD=1 or 2" )
            self.UC_given = 0

        if params_dict.get('UCR') != None:
            val =  params_dict['UCR']
            self.UCR        = self.MPRnb(val,""            ,"Reverse-mode body effect for mobility degradation parameter - BULKMOD=1 or 2" )
            self.UCR_given = 1
        else:
            self.UCR        = self.MPRnb(self.UC        ,""            ,"Reverse-mode body effect for mobility degradation parameter - BULKMOD=1 or 2" )
            self.UCR_given = 0

        if params_dict.get('EU') != None:
            val =  params_dict['EU']
            self.EU         = self.MPRnb(val,"cm/MV"       ,"Phonon/surface roughness scattering parameter" )
            self.EU_given = 1
        else:
            self.EU         = self.MPRnb(2.5            ,"cm/MV"       ,"Phonon/surface roughness scattering parameter" )
            self.EU_given = 0

        if params_dict.get('EUR') != None:
            val =  params_dict['EUR']
            self.EUR        = self.MPRnb(val,"cm/MV"       ,"Reverse-mode phonon/surface roughness scattering parameter" )
            self.EUR_given = 1
        else:
            self.EUR        = self.MPRnb(self.EU        ,"cm/MV"       ,"Reverse-mode phonon/surface roughness scattering parameter" )
            self.EUR_given = 0

        if params_dict.get('AEU') != None:
            val =  params_dict['AEU']
            self.AEU        = self.MPRnb(val,""            ,"Pre-exponential coefficient for EU" )
            self.AEU_given = 1
        else:
            self.AEU        = self.MPRnb(0.0            ,""            ,"Pre-exponential coefficient for EU" )
            self.AEU_given = 0

        if params_dict.get('AEUR') != None:
            val =  params_dict['AEUR']
            self.AEUR       = self.MPRnb(val,""            ,"Reverse-mode pre-exponential coefficient for EU" )
            self.AEUR_given = 1
        else:
            self.AEUR       = self.MPRnb(self.AEU       ,""            ,"Reverse-mode pre-exponential coefficient for EU" )
            self.AEUR_given = 0

        if params_dict.get('BEU') != None:
            val =  params_dict['BEU']
            self.BEU        = self.MPRex(val,""            ,0.0         ,"Exponential coefficient for EU" )
            self.BEU_given = 1
        else:
            self.BEU        = self.MPRex(1.0e-7         ,""            ,0.0         ,"Exponential coefficient for EU" )
            self.BEU_given = 0

        if params_dict.get('BEUR') != None:
            val =  params_dict['BEUR']
            self.BEUR       = self.MPRex(val,""            ,0.0         ,"Reverse-mode exponential coefficient for EU" )
            self.BEUR_given = 1
        else:
            self.BEUR       = self.MPRex(self.BEU       ,""            ,0.0         ,"Reverse-mode exponential coefficient for EU" )
            self.BEUR_given = 0

        if params_dict.get('UD') != None:
            val =  params_dict['UD']
            self.UD         = self.MPRnb(val,"cm/MV"       ,"Columbic scattering parameter" )
            self.UD_given = 1
        else:
            self.UD         = self.MPRnb(0.0            ,"cm/MV"       ,"Columbic scattering parameter" )
            self.UD_given = 0

        if params_dict.get('UDR') != None:
            val =  params_dict['UDR']
            self.UDR        = self.MPRnb(val,"cm/MV"       ,"Reverse-mode columbic scattering parameter" )
            self.UDR_given = 1
        else:
            self.UDR        = self.MPRnb(self.UD        ,"cm/MV"       ,"Reverse-mode columbic scattering parameter" )
            self.UDR_given = 0

        if params_dict.get('AUD') != None:
            val =  params_dict['AUD']
            self.AUD        = self.MPRnb(val,""            ,"Pre-exponential coefficient for UD" )
            self.AUD_given = 1
        else:
            self.AUD        = self.MPRnb(0.0            ,""            ,"Pre-exponential coefficient for UD" )
            self.AUD_given = 0

        if params_dict.get('AUDR') != None:
            val =  params_dict['AUDR']
            self.AUDR       = self.MPRnb(val,""            ,"Reverse-mode pre-exponential coefficient for UD" )
            self.AUDR_given = 1
        else:
            self.AUDR       = self.MPRnb(self.AUD       ,""            ,"Reverse-mode pre-exponential coefficient for UD" )
            self.AUDR_given = 0

        if params_dict.get('BUD') != None:
            val =  params_dict['BUD']
            self.BUD        = self.MPRex(val,""            ,0.0         ,"Exponential coefficient for UD" )
            self.BUD_given = 1
        else:
            self.BUD        = self.MPRex(5.0e-8         ,""            ,0.0         ,"Exponential coefficient for UD" )
            self.BUD_given = 0

        if params_dict.get('BUDR') != None:
            val =  params_dict['BUDR']
            self.BUDR       = self.MPRex(val,""            ,0.0         ,"Reverse-mode exponential coefficient for UD" )
            self.BUDR_given = 1
        else:
            self.BUDR       = self.MPRex(self.BUD       ,""            ,0.0         ,"Reverse-mode exponential coefficient for UD" )
            self.BUDR_given = 0

        if params_dict.get('UCS') != None:
            val =  params_dict['UCS']
            self.UCS        = self.MPRnb(val,""            ,"Columbic scattering parameter" )
            self.UCS_given = 1
        else:
            self.UCS        = self.MPRnb(1.0            ,""            ,"Columbic scattering parameter" )
            self.UCS_given = 0

        if params_dict.get('UTE') != None:
            val =  params_dict['UTE']
            self.UTE        = self.MPRnb(val,""            ,"Mobility temperature coefficient" )
            self.UTE_given = 1
        else:
            self.UTE        = self.MPRnb(0.0            ,""            ,"Mobility temperature coefficient" )
            self.UTE_given = 0

        if params_dict.get('UTER') != None:
            val =  params_dict['UTER']
            self.UTER       = self.MPRnb(val,""            ,"Reverse-mode for mobility temperature coefficient" )
            self.UTER_given = 1
        else:
            self.UTER       = self.MPRnb(self.UTE       ,""            ,"Reverse-mode for mobility temperature coefficient" )
            self.UTER_given = 0

        if params_dict.get('UTL') != None:
            val =  params_dict['UTL']
            self.UTL        = self.MPRnb(val,""            ,"Mobility temperature coefficient" )
            self.UTL_given = 1
        else:
            self.UTL        = self.MPRnb(-1.5e-3        ,""            ,"Mobility temperature coefficient" )
            self.UTL_given = 0

        if params_dict.get('UTLR') != None:
            val =  params_dict['UTLR']
            self.UTLR       = self.MPRnb(val,""            ,"Reverse-mode for mobility temperature coefficient" )
            self.UTLR_given = 1
        else:
            self.UTLR       = self.MPRnb(self.UTL       ,""            ,"Reverse-mode for mobility temperature coefficient" )
            self.UTLR_given = 0

        if params_dict.get('EMOBT') != None:
            val =  params_dict['EMOBT']
            self.EMOBT      = self.MPRnb(val,""            ,"Temperature coefficient of ETAMOB" )
            self.EMOBT_given = 1
        else:
            self.EMOBT      = self.MPRnb(0.0            ,""            ,"Temperature coefficient of ETAMOB" )
            self.EMOBT_given = 0

        if params_dict.get('UA1') != None:
            val =  params_dict['UA1']
            self.UA1        = self.MPRnb(val,""            ,"Mobility temperature coefficient for UA" )
            self.UA1_given = 1
        else:
            self.UA1        = self.MPRnb(1.032e-3       ,""            ,"Mobility temperature coefficient for UA" )
            self.UA1_given = 0

        if params_dict.get('UA1R') != None:
            val =  params_dict['UA1R']
            self.UA1R       = self.MPRnb(val,""            ,"Reverse-mode mobility temperature coefficient for UA" )
            self.UA1R_given = 1
        else:
            self.UA1R       = self.MPRnb(self.UA1       ,""            ,"Reverse-mode mobility temperature coefficient for UA" )
            self.UA1R_given = 0

        if params_dict.get('UC1') != None:
            val =  params_dict['UC1']
            self.UC1        = self.MPRnb(val,""            ,"Mobility temperature coefficient for UC" )
            self.UC1_given = 1
        else:
            self.UC1        = self.MPRnb(5.6e-11        ,""            ,"Mobility temperature coefficient for UC" )
            self.UC1_given = 0

        if params_dict.get('UC1R') != None:
            val =  params_dict['UC1R']
            self.UC1R       = self.MPRnb(val,""            ,"Reverse-mode mobility temperature coefficient for UC" )
            self.UC1R_given = 1
        else:
            self.UC1R       = self.MPRnb(self.UC1       ,""            ,"Reverse-mode mobility temperature coefficient for UC" )
            self.UC1R_given = 0

        if params_dict.get('UD1') != None:
            val =  params_dict['UD1']
            self.UD1        = self.MPRnb(val,""            ,"Mobility temperature coefficient for UC" )
            self.UD1_given = 1
        else:
            self.UD1        = self.MPRnb(0.0            ,""            ,"Mobility temperature coefficient for UC" )
            self.UD1_given = 0

        if params_dict.get('UD1R') != None:
            val =  params_dict['UD1R']
            self.UD1R       = self.MPRnb(val,""            ,"Reverse-mode mobility temperature coefficient for UD" )
            self.UD1R_given = 1
        else:
            self.UD1R       = self.MPRnb(self.UD1       ,""            ,"Reverse-mode mobility temperature coefficient for UD" )
            self.UD1R_given = 0

        if params_dict.get('UCSTE') != None:
            val =  params_dict['UCSTE']
            self.UCSTE      = self.MPRnb(val,""            ,"Mobility temperature coefficient" )
            self.UCSTE_given = 1
        else:
            self.UCSTE      = self.MPRnb(-4.775e-3      ,""            ,"Mobility temperature coefficient" )
            self.UCSTE_given = 0

        if params_dict.get('CHARGEWF') != None:
            val =  params_dict['CHARGEWF']
            self.CHARGEWF   = self.MPRcc(val,""            ,-1.0        ,1.0         ,"Average channel charge weighting factor, +1: source-side, 0: middle, -1: drain-side" )
            self.CHARGEWF_given = 1
        else:
            self.CHARGEWF   = self.MPRcc(0.0            ,""            ,-1.0        ,1.0         ,"Average channel charge weighting factor, +1: source-side, 0: middle, -1: drain-side" )
            self.CHARGEWF_given = 0


        # Access Resistance Model
        if params_dict.get('RDSWMIN') != None:
            val =  params_dict['RDSWMIN']
            self.RDSWMIN  = self.MPRnb(val,"ohm*(um^(WR))"            ,"RDSMOD = 0 S/D extension resistance per unit width at high Vgs" )
            self.RDSWMIN_given = 1
        else:
            self.RDSWMIN  = self.MPRnb(0.0            ,"ohm*(um^(WR))"            ,"RDSMOD = 0 S/D extension resistance per unit width at high Vgs" )
            self.RDSWMIN_given = 0

        if params_dict.get('RDSW') != None:
            val =  params_dict['RDSW']
            self.RDSW     = self.MPRnb(val,"ohm*(um^(WR))"            ,"RDSMOD = 0 zero bias S/D extension resistance per unit width" )
            self.RDSW_given = 1
        else:
            self.RDSW     = self.MPRnb(1.0e2          ,"ohm*(um^(WR))"            ,"RDSMOD = 0 zero bias S/D extension resistance per unit width" )
            self.RDSW_given = 0

        if params_dict.get('ARDSW') != None:
            val =  params_dict['ARDSW']
            self.ARDSW    = self.MPRnb(val,""            ,"Pre-exponential coefficient for RDSW" )
            self.ARDSW_given = 1
        else:
            self.ARDSW    = self.MPRnb(0.0            ,""            ,"Pre-exponential coefficient for RDSW" )
            self.ARDSW_given = 0

        if params_dict.get('BRDSW') != None:
            val =  params_dict['BRDSW']
            self.BRDSW    = self.MPRex(val,""            ,0.0         ,"exponential coefficient for RDSW" )
            self.BRDSW_given = 1
        else:
            self.BRDSW    = self.MPRex(1.0e-7         ,""            ,0.0         ,"exponential coefficient for RDSW" )
            self.BRDSW_given = 0

        if params_dict.get('RSWMIN') != None:
            val =  params_dict['RSWMIN']
            self.RSWMIN   = self.MPRnb(val,"ohm*(um^(WR))"            ,"RDSMOD = 1 source extension resistance per unit width at high Vgs" )
            self.RSWMIN_given = 1
        else:
            self.RSWMIN   = self.MPRnb(0.0            ,"ohm*(um^(WR))"            ,"RDSMOD = 1 source extension resistance per unit width at high Vgs" )
            self.RSWMIN_given = 0

        if params_dict.get('RSW') != None:
            val =  params_dict['RSW']
            self.RSW      = self.MPRnb(val,"ohm*(um^(WR))"            ,"RDSMOD = 1 zero bias source extension resistance per unit width" )
            self.RSW_given = 1
        else:
            self.RSW      = self.MPRnb(5.0e1          ,"ohm*(um^(WR))"            ,"RDSMOD = 1 zero bias source extension resistance per unit width" )
            self.RSW_given = 0

        if params_dict.get('ARSW') != None:
            val =  params_dict['ARSW']
            self.ARSW     = self.MPRnb(val,""            ,"Pre-exponential coefficient for RSW" )
            self.ARSW_given = 1
        else:
            self.ARSW     = self.MPRnb(0.0            ,""            ,"Pre-exponential coefficient for RSW" )
            self.ARSW_given = 0

        if params_dict.get('BRSW') != None:
            val =  params_dict['BRSW']
            self.BRSW     = self.MPRex(val,""            ,0.0         ,"Exponential coefficient for RSW" )
            self.BRSW_given = 1
        else:
            self.BRSW     = self.MPRex(1.0e-7         ,""            ,0.0         ,"Exponential coefficient for RSW" )
            self.BRSW_given = 0

        if params_dict.get('RDWMIN') != None:
            val =  params_dict['RDWMIN']
            self.RDWMIN   = self.MPRnb(val,"ohm*(um^(WR))"            ,"RDSMOD = 1 drain extension resistance per unit width at high Vgs" )
            self.RDWMIN_given = 1
        else:
            self.RDWMIN   = self.MPRnb(0.0            ,"ohm*(um^(WR))"            ,"RDSMOD = 1 drain extension resistance per unit width at high Vgs" )
            self.RDWMIN_given = 0

        if params_dict.get('RDW') != None:
            val =  params_dict['RDW']
            self.RDW      = self.MPRnb(val,""            ,"RDSMOD = 1 zero bias drain extension resistance per unit width" )
            self.RDW_given = 1
        else:
            self.RDW      = self.MPRnb(5.0e1          ,""            ,"RDSMOD = 1 zero bias drain extension resistance per unit width" )
            self.RDW_given = 0

        if params_dict.get('ARDW') != None:
            val =  params_dict['ARDW']
            self.ARDW     = self.MPRnb(val,""            ,"Pre-exponential coefficient for RDW" )
            self.ARDW_given = 1
        else:
            self.ARDW     = self.MPRnb(0.0            ,""            ,"Pre-exponential coefficient for RDW" )
            self.ARDW_given = 0

        if params_dict.get('BRDW') != None:
            val =  params_dict['BRDW']
            self.BRDW     = self.MPRex(val,""            ,0.0         ,"Exponential coefficient for RDW" )
            self.BRDW_given = 1
        else:
            self.BRDW     = self.MPRex(1.0e-7         ,""            ,0.0         ,"Exponential coefficient for RDW" )
            self.BRDW_given = 0

        if params_dict.get('RSDR') != None:
            val =  params_dict['RSDR']
            self.RSDR     = self.MPRcz(val,"V^(-PRSDR)"  ,"Source-side drift resistance parameter - forward mode" )
            self.RSDR_given = 1
        else:
            self.RSDR     = self.MPRcz(0.0            ,"V^(-PRSDR)"  ,"Source-side drift resistance parameter - forward mode" )
            self.RSDR_given = 0

        if params_dict.get('RSDRR') != None:
            val =  params_dict['RSDRR']
            self.RSDRR    = self.MPRcz(val,"V^(-PRSDR)"  ,"Source-side drift resistance parameter - reverse mode" )
            self.RSDRR_given = 1
        else:
            self.RSDRR    = self.MPRcz(self.RSDR      ,"V^(-PRSDR)"  ,"Source-side drift resistance parameter - reverse mode" )
            self.RSDRR_given = 0

        if params_dict.get('RDDR') != None:
            val =  params_dict['RDDR']
            self.RDDR     = self.MPRcz(val,"V^(-PRDDR)"  ,"Drain-side drift resistance parameter - forward mode" )
            self.RDDR_given = 1
        else:
            self.RDDR     = self.MPRcz(self.RSDR      ,"V^(-PRDDR)"  ,"Drain-side drift resistance parameter - forward mode" )
            self.RDDR_given = 0

        if params_dict.get('RDDRR') != None:
            val =  params_dict['RDDRR']
            self.RDDRR    = self.MPRcz(val,"V^(-PRDDR)"  ,"Drain-side drift resistance parameter - reverse mode" )
            self.RDDRR_given = 1
        else:
            self.RDDRR    = self.MPRcz(self.RDDR      ,"V^(-PRDDR)"  ,"Drain-side drift resistance parameter - reverse mode" )
            self.RDDRR_given = 0

        if params_dict.get('PRSDR') != None:
            val =  params_dict['PRSDR']
            self.PRSDR    = self.MPRnb(val,""            ,"Source-side quasi-saturation parameter" )
            self.PRSDR_given = 1
        else:
            self.PRSDR    = self.MPRnb(1.0            ,""            ,"Source-side quasi-saturation parameter" )
            self.PRSDR_given = 0

        if params_dict.get('PRDDR') != None:
            val =  params_dict['PRDDR']
            self.PRDDR    = self.MPRnb(val,""            ,"Drain-side quasi-saturation parameter" )
            self.PRDDR_given = 1
        else:
            self.PRDDR    = self.MPRnb(self.PRSDR     ,""            ,"Drain-side quasi-saturation parameter" )
            self.PRDDR_given = 0

        if params_dict.get('PRWGS') != None:
            val =  params_dict['PRWGS']
            self.PRWGS    = self.MPRnb(val,"/V"          ,"Gate bias dependence of source extension resistance" )
            self.PRWGS_given = 1
        else:
            self.PRWGS    = self.MPRnb(0.0            ,"/V"          ,"Gate bias dependence of source extension resistance" )
            self.PRWGS_given = 0

        if params_dict.get('PRWGD') != None:
            val =  params_dict['PRWGD']
            self.PRWGD    = self.MPRnb(val,"/V"          ,"Gate bias dependence of drain extension resistance" )
            self.PRWGD_given = 1
        else:
            self.PRWGD    = self.MPRnb(self.PRWGS     ,"/V"          ,"Gate bias dependence of drain extension resistance" )
            self.PRWGD_given = 0

        if params_dict.get('WR') != None:
            val =  params_dict['WR']
            self.WR       = self.MPRnb(val,""            ,"W dependence parameter of S/D extension resistance" )
            self.WR_given = 1
        else:
            self.WR       = self.MPRnb(1.0            ,""            ,"W dependence parameter of S/D extension resistance" )
            self.WR_given = 0

        if params_dict.get('PRT') != None:
            val =  params_dict['PRT']
            self.PRT      = self.MPRnb(val,"/K"          ,"Series resistance temperature coefficient" )
            self.PRT_given = 1
        else:
            self.PRT      = self.MPRnb(1.0e-3         ,"/K"          ,"Series resistance temperature coefficient" )
            self.PRT_given = 0

        if params_dict.get('TRSDR') != None:
            val =  params_dict['TRSDR']
            self.TRSDR    = self.MPRnb(val,"/K"          ,"Source-side drift resistance temperature coefficient" )
            self.TRSDR_given = 1
        else:
            self.TRSDR    = self.MPRnb(0.0            ,"/K"          ,"Source-side drift resistance temperature coefficient" )
            self.TRSDR_given = 0

        if params_dict.get('TRDDR') != None:
            val =  params_dict['TRDDR']
            self.TRDDR    = self.MPRnb(val,"/K"          ,"Drain-side drift resistance temperature coefficient" )
            self.TRDDR_given = 1
        else:
            self.TRDDR    = self.MPRnb(self.TRSDR     ,"/K"          ,"Drain-side drift resistance temperature coefficient" )
            self.TRDDR_given = 0


        # DIBL Model
        if params_dict.get('PDIBL1') != None:
            val =  params_dict['PDIBL1']
            self.PDIBL1   = self.MPRnb(val,""            ,"DIBL output conductance parameter - forward mode" )
            self.PDIBL1_given = 1
        else:
            self.PDIBL1   = self.MPRnb(1.3            ,""            ,"DIBL output conductance parameter - forward mode" )
            self.PDIBL1_given = 0

        if params_dict.get('PDIBL1R') != None:
            val =  params_dict['PDIBL1R']
            self.PDIBL1R  = self.MPRnb(val,""            ,"DIBL output conductance parameter - reverse mode" )
            self.PDIBL1R_given = 1
        else:
            self.PDIBL1R  = self.MPRnb(self.PDIBL1    ,""            ,"DIBL output conductance parameter - reverse mode" )
            self.PDIBL1R_given = 0

        if params_dict.get('PDIBL2') != None:
            val =  params_dict['PDIBL2']
            self.PDIBL2   = self.MPRnb(val,""            ,"DIBL output conductance parameter" )
            self.PDIBL2_given = 1
        else:
            self.PDIBL2   = self.MPRnb(2.0e-4         ,""            ,"DIBL output conductance parameter" )
            self.PDIBL2_given = 0

        if params_dict.get('PDIBL2R') != None:
            val =  params_dict['PDIBL2R']
            self.PDIBL2R  = self.MPRnb(val,""            ,"DIBL output conductance parameter - reverse mode" )
            self.PDIBL2R_given = 1
        else:
            self.PDIBL2R  = self.MPRnb(self.PDIBL2    ,""            ,"DIBL output conductance parameter - reverse mode" )
            self.PDIBL2R_given = 0

        if params_dict.get('DROUT') != None:
            val =  params_dict['DROUT']
            self.DROUT    = self.MPRnb(val,""            ,"L dependence of DIBL effect on Rout" )
            self.DROUT_given = 1
        else:
            self.DROUT    = self.MPRnb(1.06           ,""            ,"L dependence of DIBL effect on Rout" )
            self.DROUT_given = 0

        if params_dict.get('PVAG') != None:
            val =  params_dict['PVAG']
            self.PVAG     = self.MPRnb(val,""            ,"Vgs dependence on early voltage" )
            self.PVAG_given = 1
        else:
            self.PVAG     = self.MPRnb(1.0            ,""            ,"Vgs dependence on early voltage" )
            self.PVAG_given = 0


        # Channel Length Modulation Effect
        if params_dict.get('PCLM') != None:
            val =  params_dict['PCLM']
            self.PCLM     = self.MPRnb(val,""            ,"Channel length modulation (CLM) parameter" )
            self.PCLM_given = 1
        else:
            self.PCLM     = self.MPRnb(1.3e-2         ,""            ,"Channel length modulation (CLM) parameter" )
            self.PCLM_given = 0

        if params_dict.get('PCLMR') != None:
            val =  params_dict['PCLMR']
            self.PCLMR    = self.MPRnb(val,""            ,"Reverse model self.PCLM parameter" )
            self.PCLMR_given = 1
        else:
            self.PCLMR    = self.MPRnb(self.PCLM      ,""            ,"Reverse model self.PCLM parameter" )
            self.PCLMR_given = 0

        if params_dict.get('APCLM') != None:
            val =  params_dict['APCLM']
            self.APCLM    = self.MPRnb(val,""            ,"Pre-exponential coefficient for PCLM" )
            self.APCLM_given = 1
        else:
            self.APCLM    = self.MPRnb(0.0            ,""            ,"Pre-exponential coefficient for PCLM" )
            self.APCLM_given = 0

        if params_dict.get('APCLMR') != None:
            val =  params_dict['APCLMR']
            self.APCLMR   = self.MPRnb(val,""            ,"Reverse-mode pre-exponential coefficient for PCLM" )
            self.APCLMR_given = 1
        else:
            self.APCLMR   = self.MPRnb(self.APCLM     ,""            ,"Reverse-mode pre-exponential coefficient for PCLM" )
            self.APCLMR_given = 0

        if params_dict.get('BPCLM') != None:
            val =  params_dict['BPCLM']
            self.BPCLM    = self.MPRex(val,""            ,0.0         ,"Exponential coefficient for PCLM" )
            self.BPCLM_given = 1
        else:
            self.BPCLM    = self.MPRex(1.0e-7         ,""            ,0.0         ,"Exponential coefficient for PCLM" )
            self.BPCLM_given = 0

        if params_dict.get('BPCLMR') != None:
            val =  params_dict['BPCLMR']
            self.BPCLMR   = self.MPRex(val,""            ,0.0         ,"Reverse-mode exponential coefficient for PCLM" )
            self.BPCLMR_given = 1
        else:
            self.BPCLMR   = self.MPRex(self.BPCLM     ,""            ,0.0         ,"Reverse-mode exponential coefficient for PCLM" )
            self.BPCLMR_given = 0

        if params_dict.get('PCLMG') != None:
            val =  params_dict['PCLMG']
            self.PCLMG    = self.MPRnb(val,""            ,"Gate bias dependence parameter for CLM" )
            self.PCLMG_given = 1
        else:
            self.PCLMG    = self.MPRnb(0.0            ,""            ,"Gate bias dependence parameter for CLM" )
            self.PCLMG_given = 0

        if params_dict.get('PCLMCV') != None:
            val =  params_dict['PCLMCV']
            self.PCLMCV   = self.MPRnb(val,""            ,"CLM parameter for short-channel CV" )
            self.PCLMCV_given = 1
        else:
            self.PCLMCV   = self.MPRnb(self.PCLM      ,""            ,"CLM parameter for short-channel CV" )
            self.PCLMCV_given = 0


        # Non-Saturation Effect
        if params_dict.get('A1') != None:
            val =  params_dict['A1']
            self.A1    = self.MPRnb(val,""            ,"Non-saturation effect parameter for strong inversion Region" )
            self.A1_given = 1
        else:
            self.A1    = self.MPRnb(0.0            ,""            ,"Non-saturation effect parameter for strong inversion Region" )
            self.A1_given = 0

        if params_dict.get('A11') != None:
            val =  params_dict['A11']
            self.A11   = self.MPRnb(val,"(V^-2)/K"    ,"Temperature dependence of A1" )
            self.A11_given = 1
        else:
            self.A11   = self.MPRnb(0.0            ,"(V^-2)/K"    ,"Temperature dependence of A1" )
            self.A11_given = 0

        if params_dict.get('A2') != None:
            val =  params_dict['A2']
            self.A2    = self.MPRnb(val,""            ,"Non-saturation effect parameter for moderate Inversion Region" )
            self.A2_given = 1
        else:
            self.A2    = self.MPRnb(0.0            ,""            ,"Non-saturation effect parameter for moderate Inversion Region" )
            self.A2_given = 0

        if params_dict.get('A21') != None:
            val =  params_dict['A21']
            self.A21   = self.MPRnb(val,"(V^-1)/K"    ,"Temperature dependence of A2" )
            self.A21_given = 1
        else:
            self.A21   = self.MPRnb(0.0            ,"(V^-1)/K"    ,"Temperature dependence of A2" )
            self.A21_given = 0


        # Gate Electrode Resistance
        if params_dict.get('RGEXT') != None:
            val =  params_dict['RGEXT']
            self.RGEXT  = self.MPRcz(val,"ohm"         ,"Effective gate electrode external resistance" )
            self.RGEXT_given = 1
        else:
            self.RGEXT  = self.MPRcz(0.0            ,"ohm"         ,"Effective gate electrode external resistance" )
            self.RGEXT_given = 0

        if params_dict.get('RGFIN') != None:
            val =  params_dict['RGFIN']
            self.RGFIN  = self.MPRco(val,"ohm"         ,1.0e-3      ,math.inf         ,"Effective gate electrode per finger per fin resistance" )
            self.RGFIN_given = 1
        else:
            self.RGFIN  = self.MPRco(1.0e-3         ,"ohm"         ,1.0e-3      ,math.inf         ,"Effective gate electrode per finger per fin resistance" )
            self.RGFIN_given = 0


        # Geometry Dependent Source/Drain Resistance of RGEOMOD = 0
        if params_dict.get('RSHS') != None:
            val =  params_dict['RSHS']
            self.RSHS   = self.MPRnb(val,"ohm"         ,"Source-side sheet resistance" )
            self.RSHS_given = 1
        else:
            self.RSHS   = self.MPRnb(0.0            ,"ohm"         ,"Source-side sheet resistance" )
            self.RSHS_given = 0

        if params_dict.get('RSHD') != None:
            val =  params_dict['RSHD']
            self.RSHD   = self.MPRnb(val,"ohm"         ,"Drain-side sheet resistance" )
            self.RSHD_given = 1
        else:
            self.RSHD   = self.MPRnb(self.RSHS      ,"ohm"         ,"Drain-side sheet resistance" )
            self.RSHD_given = 0


        # Geometry Dependent Source/Drain Resistance of RGEOMOD = 1 for variability modeling
        # These parameters are shared with CGEOMOD = 2
        if params_dict.get('HEPI') != None:
            val =  params_dict['HEPI']
            self.HEPI       = self.MPRnb(val,"m"           ,"Height of the raised source/drain on top of the fin" )
            self.HEPI_given = 1
        else:
            self.HEPI       = self.MPRnb(1.0e-8         ,"m"           ,"Height of the raised source/drain on top of the fin" )
            self.HEPI_given = 0

        if params_dict.get('TSILI') != None:
            val =  params_dict['TSILI']
            self.TSILI      = self.MPRnb(val,"m"           ,"Thickness of the silicide on top of the raised source/drain" )
            self.TSILI_given = 1
        else:
            self.TSILI      = self.MPRnb(1.0e-8         ,"m"           ,"Thickness of the silicide on top of the raised source/drain" )
            self.TSILI_given = 0

        if params_dict.get('RHOC') != None:
            val =  params_dict['RHOC']
            self.RHOC       = self.MPRcc(val,"ohm*(m^2)"   ,1.0e-18     ,1.0e-9      ,"Contact resistivity at the silicon/silicide interface" )
            self.RHOC_given = 1
        else:
            self.RHOC       = self.MPRcc(1.0e-12        ,"ohm*(m^2)"   ,1.0e-18     ,1.0e-9      ,"Contact resistivity at the silicon/silicide interface" )
            self.RHOC_given = 0

        if params_dict.get('RHORSD') != None:
            val =  params_dict['RHORSD']
            self.RHORSD     = self.MPRoz(val,"ohm*(m)"     ,"Average resistivity of silicon in the raised source/drain region" )
            self.RHORSD_given = 1
        else:
            self.RHORSD     = self.MPRoz(1.0            ,"ohm*(m)"     ,"Average resistivity of silicon in the raised source/drain region" )
            self.RHORSD_given = 0

        if params_dict.get('CRATIO') != None:
            val =  params_dict['CRATIO']
            self.CRATIO     = self.MPRcc(val,""            ,0.0         ,1.0         ,"Ratio of the corner area filled with silicon to the total corner area" )
            self.CRATIO_given = 1
        else:
            self.CRATIO     = self.MPRcc(0.5            ,""            ,0.0         ,1.0         ,"Ratio of the corner area filled with silicon to the total corner area" )
            self.CRATIO_given = 0

        if params_dict.get('DELTAPRSD') != None:
            val =  params_dict['DELTAPRSD']
            self.DELTAPRSD  = self.MPRoo(val,"m"           ,-self.FPITCH     ,math.inf         ,"Change in silicon/silicide interface length due to non-rectangular epi" )
            self.DELTAPRSD_given = 1
        else:
            self.DELTAPRSD  = self.MPRoo(0.0            ,"m"           ,-self.FPITCH     ,math.inf         ,"Change in silicon/silicide interface length due to non-rectangular epi" )
            self.DELTAPRSD_given = 0

        if params_dict.get('SDTERM') != None:
            val =  params_dict['SDTERM']
            self.SDTERM     = self.MPIcc(val,""            ,0           ,1           ,"Indicator of whether the source/drain are terminated with silicide" )
            self.SDTERM_given = 1
        else:
            self.SDTERM     = self.MPIcc(0              ,""            ,0           ,1           ,"Indicator of whether the source/drain are terminated with silicide" )
            self.SDTERM_given = 0

        if params_dict.get('LSP') != None:
            val =  params_dict['LSP']
            self.LSP        = self.MPRnb(val                      ,"m" ,"Thickness of the gate sidewall spacer" )
            self.LSP_given = 1
        else:
            self.LSP        = self.MPRnb((0.2*(self.L + self.XL)) ,"m" ,"Thickness of the gate sidewall spacer" )
            self.LSP_given = 0

        if params_dict.get('EPSRSP') != None:
            val =  params_dict['EPSRSP']
            self.EPSRSP     = self.MPRco(val,""            ,1.0         ,math.inf         ,"Relative dielectric constant of the spacer" )
            self.EPSRSP_given = 1
        else:
            self.EPSRSP     = self.MPRco(3.9            ,""            ,1.0         ,math.inf         ,"Relative dielectric constant of the spacer" )
            self.EPSRSP_given = 0

        if params_dict.get('TGATE') != None:
            val =  params_dict['TGATE']
            self.TGATE      = self.MPRoz(val,"m"           ,"Gate height on top of the hard mask" )
            self.TGATE_given = 1
        else:
            self.TGATE      = self.MPRoz(3.0e-8         ,"m"           ,"Gate height on top of the hard mask" )
            self.TGATE_given = 0

        if params_dict.get('TMASK') != None:
            val =  params_dict['TMASK']
            self.TMASK      = self.MPRcz(val,"m"           ,"Height of hard mask on top of the fin" )
            self.TMASK_given = 1
        else:
            self.TMASK      = self.MPRcz(3.0e-8         ,"m"           ,"Height of hard mask on top of the fin" )
            self.TMASK_given = 0

        if params_dict.get('ASILIEND') != None:
            val =  params_dict['ASILIEND']
            self.ASILIEND   = self.MPRcz(val,"m^2"         ,"Extra silicide cross sectional area at the two ends of the FinFET" )
            self.ASILIEND_given = 1
        else:
            self.ASILIEND   = self.MPRcz(0.0            ,"m^2"         ,"Extra silicide cross sectional area at the two ends of the FinFET" )
            self.ASILIEND_given = 0

        if params_dict.get('ARSDEND') != None:
            val =  params_dict['ARSDEND']
            self.ARSDEND    = self.MPRcz(val,"m^2"         ,"Extra raised source/drain cross sectional areaat the two ends of the FinFET" )
            self.ARSDEND_given = 1
        else:
            self.ARSDEND    = self.MPRcz(0.0            ,"m^2"         ,"Extra raised source/drain cross sectional areaat the two ends of the FinFET" )
            self.ARSDEND_given = 0

        if params_dict.get('PRSDEND') != None:
            val =  params_dict['PRSDEND']
            self.PRSDEND    = self.MPRcz(val,"m"           ,"Extra silicon/silicide interface perimeter at the two ends of the FinFET" )
            self.PRSDEND_given = 1
        else:
            self.PRSDEND    = self.MPRcz(0.0            ,"m"           ,"Extra silicon/silicide interface perimeter at the two ends of the FinFET" )
            self.PRSDEND_given = 0

        if params_dict.get('NSDE') != None:
            val =  params_dict['NSDE']
            self.NSDE       = self.MPRcc(val,"/(m^3)"      ,1.0e25      ,1.0e26      ,"Source/drain active doping concentration at self.Leff edge" )
            self.NSDE_given = 1
        else:
            self.NSDE       = self.MPRcc(2.0e25         ,"/(m^3)"      ,1.0e25      ,1.0e26      ,"Source/drain active doping concentration at self.Leff edge" )
            self.NSDE_given = 0

        if params_dict.get('RGEOA') != None:
            val =  params_dict['RGEOA']
            self.RGEOA      = self.MPRnb(val,""            ,"Fitting parameter for RGEOMOD=1" )
            self.RGEOA_given = 1
        else:
            self.RGEOA      = self.MPRnb(1.0            ,""            ,"Fitting parameter for RGEOMOD=1" )
            self.RGEOA_given = 0

        if params_dict.get('RGEOB') != None:
            val =  params_dict['RGEOB']
            self.RGEOB      = self.MPRnb(val,"/m"          ,"Fitting parameter for RGEOMOD=1" )
            self.RGEOB_given = 1
        else:
            self.RGEOB      = self.MPRnb(0.0            ,"/m"          ,"Fitting parameter for RGEOMOD=1" )
            self.RGEOB_given = 0

        if params_dict.get('RGEOC') != None:
            val =  params_dict['RGEOC']
            self.RGEOC      = self.MPRnb(val,"/m"          ,"Fitting parameter for RGEOMOD=1" )
            self.RGEOC_given = 1
        else:
            self.RGEOC      = self.MPRnb(0.0            ,"/m"          ,"Fitting parameter for RGEOMOD=1" )
            self.RGEOC_given = 0

        if params_dict.get('RGEOD') != None:
            val =  params_dict['RGEOD']
            self.RGEOD      = self.MPRnb(val,"/m"          ,"Fitting parameter for RGEOMOD=1" )
            self.RGEOD_given = 1
        else:
            self.RGEOD      = self.MPRnb(0.0            ,"/m"          ,"Fitting parameter for RGEOMOD=1" )
            self.RGEOD_given = 0

        if params_dict.get('RGEOE') != None:
            val =  params_dict['RGEOE']
            self.RGEOE      = self.MPRnb(val,"/m"          ,"Fitting parameter for RGEOMOD=1" )
            self.RGEOE_given = 1
        else:
            self.RGEOE      = self.MPRnb(0.0            ,"/m"          ,"Fitting parameter for RGEOMOD=1" )
            self.RGEOE_given = 0

        if params_dict.get('CGEOA') != None:
            val =  params_dict['CGEOA']
            self.CGEOA      = self.MPRnb(val,""            ,"Fitting parameter for CGEOMOD=2" )
            self.CGEOA_given = 1
        else:
            self.CGEOA      = self.MPRnb(1.0            ,""            ,"Fitting parameter for CGEOMOD=2" )
            self.CGEOA_given = 0

        if params_dict.get('CGEOB') != None:
            val =  params_dict['CGEOB']
            self.CGEOB      = self.MPRnb(val,"/m"          ,"Fitting parameter for CGEOMOD=2" )
            self.CGEOB_given = 1
        else:
            self.CGEOB      = self.MPRnb(0.0            ,"/m"          ,"Fitting parameter for CGEOMOD=2" )
            self.CGEOB_given = 0

        if params_dict.get('CGEOC') != None:
            val =  params_dict['CGEOC']
            self.CGEOC      = self.MPRnb(val,"/m"          ,"Fitting parameter for CGEOMOD=2" )
            self.CGEOC_given = 1
        else:
            self.CGEOC      = self.MPRnb(0.0            ,"/m"          ,"Fitting parameter for CGEOMOD=2" )
            self.CGEOC_given = 0

        if params_dict.get('CGEOD') != None:
            val =  params_dict['CGEOD']
            self.CGEOD      = self.MPRnb(val,"/m"          ,"Fitting parameter for CGEOMOD=2" )
            self.CGEOD_given = 1
        else:
            self.CGEOD      = self.MPRnb(0.0            ,"/m"          ,"Fitting parameter for CGEOMOD=2" )
            self.CGEOD_given = 0

        if params_dict.get('CGEOE') != None:
            val =  params_dict['CGEOE']
            self.CGEOE      = self.MPRcz(val,""            ,"Fitting parameter for CGEOMOD=2" )
            self.CGEOE_given = 1
        else:
            self.CGEOE      = self.MPRcz(1.0            ,""            ,"Fitting parameter for CGEOMOD=2" )
            self.CGEOE_given = 0


        # Gate Current
        if params_dict.get('AIGBINV') != None:
            val =  params_dict['AIGBINV']
            self.AIGBINV    = self.MPRnb(val,"((F*s^2/g)^0.5)*m^-1"     ,"Parameter for Igb in inversion" )
            self.AIGBINV_given = 1
        else:
            self.AIGBINV    = self.MPRnb(1.11e-2        ,"((F*s^2/g)^0.5)*m^-1"     ,"Parameter for Igb in inversion" )
            self.AIGBINV_given = 0

        if params_dict.get('AIGBINV1') != None:
            val =  params_dict['AIGBINV1']
            self.AIGBINV1   = self.MPRnb(val,"((F*s^2/g)^0.5)*m^-1/K"   ,"Parameter for Igb in inversion" )
            self.AIGBINV1_given = 1
        else:
            self.AIGBINV1   = self.MPRnb(0.0            ,"((F*s^2/g)^0.5)*m^-1/K"   ,"Parameter for Igb in inversion" )
            self.AIGBINV1_given = 0

        if params_dict.get('BIGBINV') != None:
            val =  params_dict['BIGBINV']
            self.BIGBINV    = self.MPRnb(val,"((F*s^2/g)^0.5)*(m*V)^-1" ,"Parameter for Igb in inversion" )
            self.BIGBINV_given = 1
        else:
            self.BIGBINV    = self.MPRnb(9.49e-4        ,"((F*s^2/g)^0.5)*(m*V)^-1" ,"Parameter for Igb in inversion" )
            self.BIGBINV_given = 0

        if params_dict.get('CIGBINV') != None:
            val =  params_dict['CIGBINV']
            self.CIGBINV    = self.MPRnb(val,"/V"          ,"Parameter for Igb in inversion" )
            self.CIGBINV_given = 1
        else:
            self.CIGBINV    = self.MPRnb(6.0e-3         ,"/V"          ,"Parameter for Igb in inversion" )
            self.CIGBINV_given = 0

        if params_dict.get('EIGBINV') != None:
            val =  params_dict['EIGBINV']
            self.EIGBINV    = self.MPRnb(val,"V"           ,"Parameter for Igb in inversion" )
            self.EIGBINV_given = 1
        else:
            self.EIGBINV    = self.MPRnb(1.1            ,"V"           ,"Parameter for Igb in inversion" )
            self.EIGBINV_given = 0

        if params_dict.get('NIGBINV') != None:
            val =  params_dict['NIGBINV']
            self.NIGBINV    = self.MPRnb(val,""            ,"Parameter for Igb in inversion" )
            self.NIGBINV_given = 1
        else:
            self.NIGBINV    = self.MPRnb(3.0            ,""            ,"Parameter for Igb in inversion" )
            self.NIGBINV_given = 0

        if params_dict.get('AIGBACC') != None:
            val =  params_dict['AIGBACC']
            self.AIGBACC    = self.MPRnb(val,"((F*s^2/g)^0.5)*m^-1"     ,"Parameter for Igb in accumulation" )
            self.AIGBACC_given = 1
        else:
            self.AIGBACC    = self.MPRnb(1.36e-2        ,"((F*s^2/g)^0.5)*m^-1"     ,"Parameter for Igb in accumulation" )
            self.AIGBACC_given = 0

        if params_dict.get('AIGBACC1') != None:
            val =  params_dict['AIGBACC1']
            self.AIGBACC1   = self.MPRnb(val,"((F*s^2/g)^0.5)*m^-1/K"   ,"Parameter for Igb in accumulation" )
            self.AIGBACC1_given = 1
        else:
            self.AIGBACC1   = self.MPRnb(0.0            ,"((F*s^2/g)^0.5)*m^-1/K"   ,"Parameter for Igb in accumulation" )
            self.AIGBACC1_given = 0

        if params_dict.get('BIGBACC') != None:
            val =  params_dict['BIGBACC']
            self.BIGBACC    = self.MPRnb(val,"((F*s^2/g)^0.5)*(m*V)^-1" ,"Parameter for Igb in accumulation" )
            self.BIGBACC_given = 1
        else:
            self.BIGBACC    = self.MPRnb(1.71e-3        ,"((F*s^2/g)^0.5)*(m*V)^-1" ,"Parameter for Igb in accumulation" )
            self.BIGBACC_given = 0

        if params_dict.get('CIGBACC') != None:
            val =  params_dict['CIGBACC']
            self.CIGBACC    = self.MPRnb(val,"/V"          ,"Parameter for Igb in accumulation" )
            self.CIGBACC_given = 1
        else:
            self.CIGBACC    = self.MPRnb(7.5e-2         ,"/V"          ,"Parameter for Igb in accumulation" )
            self.CIGBACC_given = 0

        if params_dict.get('NIGBACC') != None:
            val =  params_dict['NIGBACC']
            self.NIGBACC    = self.MPRnb(val,""            ,"Parameter for Igb in accumulation" )
            self.NIGBACC_given = 1
        else:
            self.NIGBACC    = self.MPRnb(1.0            ,""            ,"Parameter for Igb in accumulation" )
            self.NIGBACC_given = 0

        if params_dict.get('AIGC') != None:
            val =  params_dict['AIGC']
            self.AIGC       = self.MPRnb(val,"((F*s^2/g)^0.5)*m^-1"     ,"Parameter for Igc in inversion" )
            self.AIGC_given = 1
        else:
            self.AIGC       = self.MPRnb(1.36e-2        ,"((F*s^2/g)^0.5)*m^-1"     ,"Parameter for Igc in inversion" )
            self.AIGC_given = 0

        if params_dict.get('AIGC1') != None:
            val =  params_dict['AIGC1']
            self.AIGC1      = self.MPRnb(val,"((F*s^2/g)^0.5)*m^-1/K"   ,"Parameter for Igc in inversion" )
            self.AIGC1_given = 1
        else:
            self.AIGC1      = self.MPRnb(0.0            ,"((F*s^2/g)^0.5)*m^-1/K"   ,"Parameter for Igc in inversion" )
            self.AIGC1_given = 0

        if params_dict.get('BIGC') != None:
            val =  params_dict['BIGC']
            self.BIGC       = self.MPRnb(val,"((F*s^2/g)^0.5)*(m*V)^-1" ,"Parameter for Igc in inversion" )
            self.BIGC_given = 1
        else:
            self.BIGC       = self.MPRnb(1.71e-3        ,"((F*s^2/g)^0.5)*(m*V)^-1" ,"Parameter for Igc in inversion" )
            self.BIGC_given = 0

        if params_dict.get('CIGC') != None:
            val =  params_dict['CIGC']
            self.CIGC       = self.MPRnb(val,"/V"          ,"Parameter for Igc in inversion" )
            self.CIGC_given = 1
        else:
            self.CIGC       = self.MPRnb(7.5e-2         ,"/V"          ,"Parameter for Igc in inversion" )
            self.CIGC_given = 0

        if params_dict.get('PIGCD') != None:
            val =  params_dict['PIGCD']
            self.PIGCD      = self.MPRnb(val,""            ,"Parameter for Igc partition" )
            self.PIGCD_given = 1
        else:
            self.PIGCD      = self.MPRnb(1.0            ,""            ,"Parameter for Igc partition" )
            self.PIGCD_given = 0

        if params_dict.get('DLCIGS') != None:
            val =  params_dict['DLCIGS']
            self.DLCIGS     = self.MPRnb(val,"m"           ,"Delta self.L for Igs model" )
            self.DLCIGS_given = 1
        else:
            self.DLCIGS     = self.MPRnb(0.0            ,"m"           ,"Delta self.L for Igs model" )
            self.DLCIGS_given = 0

        if params_dict.get('AIGS') != None:
            val =  params_dict['AIGS']
            self.AIGS       = self.MPRnb(val,"((F*s^2/g)^0.5)*m^-1"     ,"Parameter for Igs in inversion" )
            self.AIGS_given = 1
        else:
            self.AIGS       = self.MPRnb(1.36e-2        ,"((F*s^2/g)^0.5)*m^-1"     ,"Parameter for Igs in inversion" )
            self.AIGS_given = 0

        if params_dict.get('AIGS1') != None:
            val =  params_dict['AIGS1']
            self.AIGS1      = self.MPRnb(val,"((F*s^2/g)^0.5)*m^-1/K"   ,"Parameter for Igs in inversion" )
            self.AIGS1_given = 1
        else:
            self.AIGS1      = self.MPRnb(0.0            ,"((F*s^2/g)^0.5)*m^-1/K"   ,"Parameter for Igs in inversion" )
            self.AIGS1_given = 0

        if params_dict.get('BIGS') != None:
            val =  params_dict['BIGS']
            self.BIGS       = self.MPRnb(val,"((F*s^2/g)^0.5)*(m*V)^-1" ,"Parameter for Igs in inversion" )
            self.BIGS_given = 1
        else:
            self.BIGS       = self.MPRnb(1.71e-3        ,"((F*s^2/g)^0.5)*(m*V)^-1" ,"Parameter for Igs in inversion" )
            self.BIGS_given = 0

        if params_dict.get('CIGS') != None:
            val =  params_dict['CIGS']
            self.CIGS       = self.MPRnb(val,"/V"          ,"Parameter for Igs in inversion" )
            self.CIGS_given = 1
        else:
            self.CIGS       = self.MPRnb(7.5e-2         ,"/V"          ,"Parameter for Igs in inversion" )
            self.CIGS_given = 0

        if params_dict.get('DLCIGD') != None:
            val =  params_dict['DLCIGD']
            self.DLCIGD     = self.MPRnb(val,"m"           ,"Delta self.L for Igd model" )
            self.DLCIGD_given = 1
        else:
            self.DLCIGD     = self.MPRnb(self.DLCIGS    ,"m"           ,"Delta self.L for Igd model" )
            self.DLCIGD_given = 0

        if params_dict.get('AIGD') != None:
            val =  params_dict['AIGD']
            self.AIGD       = self.MPRnb(val,"((F*s^2/g)^0.5)*m^-1"     ,"Parameter for Igd in inversion" )
            self.AIGD_given = 1
        else:
            self.AIGD       = self.MPRnb(self.AIGS      ,"((F*s^2/g)^0.5)*m^-1"     ,"Parameter for Igd in inversion" )
            self.AIGD_given = 0

        if params_dict.get('AIGD1') != None:
            val =  params_dict['AIGD1']
            self.AIGD1      = self.MPRnb(val,"((F*s^2/g)^0.5)*m^-1/K"   ,"Parameter for Igd in inversion" )
            self.AIGD1_given = 1
        else:
            self.AIGD1      = self.MPRnb(self.AIGS1     ,"((F*s^2/g)^0.5)*m^-1/K"   ,"Parameter for Igd in inversion" )
            self.AIGD1_given = 0

        if params_dict.get('BIGD') != None:
            val =  params_dict['BIGD']
            self.BIGD       = self.MPRnb(val,"((F*s^2/g)^0.5)*(m*V)^-1" ,"Parameter for Igd in inversion" )
            self.BIGD_given = 1
        else:
            self.BIGD       = self.MPRnb(self.BIGS      ,"((F*s^2/g)^0.5)*(m*V)^-1" ,"Parameter for Igd in inversion" )
            self.BIGD_given = 0

        if params_dict.get('CIGD') != None:
            val =  params_dict['CIGD']
            self.CIGD       = self.MPRnb(val,"/V"          ,"Parameter for Igd in inversion" )
            self.CIGD_given = 1
        else:
            self.CIGD       = self.MPRnb(self.CIGS      ,"/V"          ,"Parameter for Igd in inversion" )
            self.CIGD_given = 0

        if params_dict.get('VFBSD') != None:
            val =  params_dict['VFBSD']
            self.VFBSD      = self.MPRnb(val,"V"           ,"Flatband voltage for S/D region" )
            self.VFBSD_given = 1
        else:
            self.VFBSD      = self.MPRnb(0.0            ,"V"           ,"Flatband voltage for S/D region" )
            self.VFBSD_given = 0

        if params_dict.get('VFBSDCV') != None:
            val =  params_dict['VFBSDCV']
            self.VFBSDCV    = self.MPRnb(val,"V"           ,"Flatband voltage for S/D region for C-V calculations" )
            self.VFBSDCV_given = 1
        else:
            self.VFBSDCV    = self.MPRnb(self.VFBSD     ,"V"           ,"Flatband voltage for S/D region for C-V calculations" )
            self.VFBSDCV_given = 0

        if params_dict.get('TOXREF') != None:
            val =  params_dict['TOXREF']
            self.TOXREF     = self.MPRoz(val,"m"           ,"Target tox value" )
            self.TOXREF_given = 1
        else:
            self.TOXREF     = self.MPRoz(1.2e-9         ,"m"           ,"Target tox value" )
            self.TOXREF_given = 0

        if params_dict.get('TOXG') != None:
            val =  params_dict['TOXG']
            self.TOXG       = self.MPRnb(val,"m"           ,"Oxide thickness for gate current model" )
            self.TOXG_given = 1
        else:
            self.TOXG       = self.MPRnb(self.TOXP      ,"m"           ,"Oxide thickness for gate current model" )
            self.TOXG_given = 0

        if params_dict.get('NTOX') != None:
            val =  params_dict['NTOX']
            self.NTOX       = self.MPRnb(val,""            ,"Exponent for Tox ratio" )
            self.NTOX_given = 1
        else:
            self.NTOX       = self.MPRnb(1.0            ,""            ,"Exponent for Tox ratio" )
            self.NTOX_given = 0

        if params_dict.get('POXEDGE') != None:
            val =  params_dict['POXEDGE']
            self.POXEDGE    = self.MPRnb(val,""            ,"Factor for the gate edge Tox" )
            self.POXEDGE_given = 1
        else:
            self.POXEDGE    = self.MPRnb(1.0            ,""            ,"Factor for the gate edge Tox" )
            self.POXEDGE_given = 0


        # GIDL/GISL Current
        if params_dict.get('AGISL') != None:
            val =  params_dict['AGISL']
            self.AGISL  = self.MPRnb(val,"mho"         ,"Pre-exponential coefficient for GISL" )
            self.AGISL_given = 1
        else:
            self.AGISL  = self.MPRnb(6.055e-12      ,"mho"         ,"Pre-exponential coefficient for GISL" )
            self.AGISL_given = 0

        if params_dict.get('BGISL') != None:
            val =  params_dict['BGISL']
            self.BGISL  = self.MPRnb(val,"V/m"         ,"Exponential coefficient for GISL" )
            self.BGISL_given = 1
        else:
            self.BGISL  = self.MPRnb(3.0e8          ,"V/m"         ,"Exponential coefficient for GISL" )
            self.BGISL_given = 0

        if params_dict.get('CGISL') != None:
            val =  params_dict['CGISL']
            self.CGISL  = self.MPRnb(val,"V^3"         ,"Parameter for body-effect of GISL" )
            self.CGISL_given = 1
        else:
            self.CGISL  = self.MPRnb(0.5            ,"V^3"         ,"Parameter for body-effect of GISL" )
            self.CGISL_given = 0

        if params_dict.get('EGISL') != None:
            val =  params_dict['EGISL']
            self.EGISL  = self.MPRnb(val,"V"           ,"Band bending parameter for GISL" )
            self.EGISL_given = 1
        else:
            self.EGISL  = self.MPRnb(0.2            ,"V"           ,"Band bending parameter for GISL" )
            self.EGISL_given = 0

        if params_dict.get('PGISL') != None:
            val =  params_dict['PGISL']
            self.PGISL  = self.MPRnb(val,""            ,"Parameter for body-bias effect on GISL" )
            self.PGISL_given = 1
        else:
            self.PGISL  = self.MPRnb(1.0            ,""            ,"Parameter for body-bias effect on GISL" )
            self.PGISL_given = 0

        if params_dict.get('AGIDL') != None:
            val =  params_dict['AGIDL']
            self.AGIDL  = self.MPRnb(val,"mho"         ,"Pre-exponential coefficient for GIDL" )
            self.AGIDL_given = 1
        else:
            self.AGIDL  = self.MPRnb(self.AGISL     ,"mho"         ,"Pre-exponential coefficient for GIDL" )
            self.AGIDL_given = 0

        if params_dict.get('BGIDL') != None:
            val =  params_dict['BGIDL']
            self.BGIDL  = self.MPRnb(val,"V/m"         ,"Exponential coefficient for GIDL" )
            self.BGIDL_given = 1
        else:
            self.BGIDL  = self.MPRnb(self.BGISL     ,"V/m"         ,"Exponential coefficient for GIDL" )
            self.BGIDL_given = 0

        if params_dict.get('CGIDL') != None:
            val =  params_dict['CGIDL']
            self.CGIDL  = self.MPRnb(val,"V^3"         ,"Parameter for body-effect of GIDL" )
            self.CGIDL_given = 1
        else:
            self.CGIDL  = self.MPRnb(self.CGISL     ,"V^3"         ,"Parameter for body-effect of GIDL" )
            self.CGIDL_given = 0

        if params_dict.get('EGIDL') != None:
            val =  params_dict['EGIDL']
            self.EGIDL  = self.MPRnb(val,"V"           ,"Band bending parameter for GIDL" )
            self.EGIDL_given = 1
        else:
            self.EGIDL  = self.MPRnb(self.EGISL     ,"V"           ,"Band bending parameter for GIDL" )
            self.EGIDL_given = 0

        if params_dict.get('PGIDL') != None:
            val =  params_dict['PGIDL']
            self.PGIDL  = self.MPRnb(val,""            ,"Parameter for body-bias effect on GIDL" )
            self.PGIDL_given = 1
        else:
            self.PGIDL  = self.MPRnb(self.PGISL     ,""            ,"Parameter for body-bias effect on GIDL" )
            self.PGIDL_given = 0


        # Impact Ionization Current
        # IIMOD = 1
        if params_dict.get('ALPHA0') != None:
            val =  params_dict['ALPHA0']
            self.ALPHA0    = self.MPRnb(val,"m/V"         ,"First parameter of Iii" )
            self.ALPHA0_given = 1
        else:
            self.ALPHA0    = self.MPRnb(0.0            ,"m/V"         ,"First parameter of Iii" )
            self.ALPHA0_given = 0

        if params_dict.get('ALPHA01') != None:
            val =  params_dict['ALPHA01']
            self.ALPHA01   = self.MPRnb(val,"m/V/K"       ,"Temperature dependence of ALPHA0" )
            self.ALPHA01_given = 1
        else:
            self.ALPHA01   = self.MPRnb(0.0            ,"m/V/K"       ,"Temperature dependence of ALPHA0" )
            self.ALPHA01_given = 0

        if params_dict.get('ALPHA1') != None:
            val =  params_dict['ALPHA1']
            self.ALPHA1    = self.MPRnb(val,"/V"          ,"L scaling parameter of Iii" )
            self.ALPHA1_given = 1
        else:
            self.ALPHA1    = self.MPRnb(0.0            ,"/V"          ,"L scaling parameter of Iii" )
            self.ALPHA1_given = 0

        if params_dict.get('ALPHA11') != None:
            val =  params_dict['ALPHA11']
            self.ALPHA11   = self.MPRnb(val,"/V/K"        ,"Temperature dependence ALPHA1" )
            self.ALPHA11_given = 1
        else:
            self.ALPHA11   = self.MPRnb(0.0            ,"/V/K"        ,"Temperature dependence ALPHA1" )
            self.ALPHA11_given = 0

        if params_dict.get('BETA0') != None:
            val =  params_dict['BETA0']
            self.BETA0     = self.MPRnb(val,"/V"          ,"Vds dependence parameter of Iii" )
            self.BETA0_given = 1
        else:
            self.BETA0     = self.MPRnb(0.0            ,"/V"          ,"Vds dependence parameter of Iii" )
            self.BETA0_given = 0


        # IIMOD = 2
        if params_dict.get('ALPHAII0') != None:
            val =  params_dict['ALPHAII0']
            self.ALPHAII0      = self.MPRnb(val,"m/V"         ,"First parameter of Iii for IIMOD=2" )
            self.ALPHAII0_given = 1
        else:
            self.ALPHAII0      = self.MPRnb(0.0            ,"m/V"         ,"First parameter of Iii for IIMOD=2" )
            self.ALPHAII0_given = 0

        if params_dict.get('ALPHAII01') != None:
            val =  params_dict['ALPHAII01']
            self.ALPHAII01     = self.MPRnb(val,"m/V/K"       ,"Temperature dependence of ALPHAII0" )
            self.ALPHAII01_given = 1
        else:
            self.ALPHAII01     = self.MPRnb(0.0            ,"m/V/K"       ,"Temperature dependence of ALPHAII0" )
            self.ALPHAII01_given = 0

        if params_dict.get('ALPHAII1') != None:
            val =  params_dict['ALPHAII1']
            self.ALPHAII1      = self.MPRnb(val,"/V"          ,"L scaling parameter of Iii for IIMOD=2" )
            self.ALPHAII1_given = 1
        else:
            self.ALPHAII1      = self.MPRnb(0.0            ,"/V"          ,"L scaling parameter of Iii for IIMOD=2" )
            self.ALPHAII1_given = 0

        if params_dict.get('ALPHAII11') != None:
            val =  params_dict['ALPHAII11']
            self.ALPHAII11     = self.MPRnb(val,"m/V/K"       ,"Temperature dependence of ALPHAII1" )
            self.ALPHAII11_given = 1
        else:
            self.ALPHAII11     = self.MPRnb(0.0            ,"m/V/K"       ,"Temperature dependence of ALPHAII1" )
            self.ALPHAII11_given = 0

        if params_dict.get('BETAII0') != None:
            val =  params_dict['BETAII0']
            self.BETAII0       = self.MPRnb(val,"/V"          ,"Vds dependence parameter of Iii" )
            self.BETAII0_given = 1
        else:
            self.BETAII0       = self.MPRnb(0.0            ,"/V"          ,"Vds dependence parameter of Iii" )
            self.BETAII0_given = 0

        if params_dict.get('BETAII1') != None:
            val =  params_dict['BETAII1']
            self.BETAII1       = self.MPRnb(val,""            ,"Vds dependence parameter of Iii" )
            self.BETAII1_given = 1
        else:
            self.BETAII1       = self.MPRnb(0.0            ,""            ,"Vds dependence parameter of Iii" )
            self.BETAII1_given = 0

        if params_dict.get('BETAII2') != None:
            val =  params_dict['BETAII2']
            self.BETAII2       = self.MPRnb(val,"V"           ,"Vds dependence parameter of Iii" )
            self.BETAII2_given = 1
        else:
            self.BETAII2       = self.MPRnb(0.1            ,"V"           ,"Vds dependence parameter of Iii" )
            self.BETAII2_given = 0

        if params_dict.get('ESATII') != None:
            val =  params_dict['ESATII']
            self.ESATII        = self.MPRnb(val,"V/m"         ,"Saturation channel E-field for Iii" )
            self.ESATII_given = 1
        else:
            self.ESATII        = self.MPRnb(1.0e7          ,"V/m"         ,"Saturation channel E-field for Iii" )
            self.ESATII_given = 0

        if params_dict.get('LII') != None:
            val =  params_dict['LII']
            self.LII           = self.MPRnb(val,"V*m"         ,"Channel length dependence parameter of Iii" )
            self.LII_given = 1
        else:
            self.LII           = self.MPRnb(0.5e-9         ,"V*m"         ,"Channel length dependence parameter of Iii" )
            self.LII_given = 0

        if params_dict.get('SII0') != None:
            val =  params_dict['SII0']
            self.SII0          = self.MPRnb(val,"/V"          ,"Vgs dependence parameter of Iii" )
            self.SII0_given = 1
        else:
            self.SII0          = self.MPRnb(0.5            ,"/V"          ,"Vgs dependence parameter of Iii" )
            self.SII0_given = 0

        if params_dict.get('SII1') != None:
            val =  params_dict['SII1']
            self.SII1          = self.MPRnb(val,""            ,"1st Vgs dependence parameter of Iii" )
            self.SII1_given = 1
        else:
            self.SII1          = self.MPRnb(0.1            ,""            ,"1st Vgs dependence parameter of Iii" )
            self.SII1_given = 0

        if params_dict.get('SII2') != None:
            val =  params_dict['SII2']
            self.SII2          = self.MPRnb(val,"V"           ,"2nd Vgs dependence parameter of Iii" )
            self.SII2_given = 1
        else:
            self.SII2          = self.MPRnb(0.0            ,"V"           ,"2nd Vgs dependence parameter of Iii" )
            self.SII2_given = 0

        if params_dict.get('SIID') != None:
            val =  params_dict['SIID']
            self.SIID          = self.MPRnb(val,"V"           ,"3rd Vds dependence parameter of Iii" )
            self.SIID_given = 1
        else:
            self.SIID          = self.MPRnb(0.0            ,"V"           ,"3rd Vds dependence parameter of Iii" )
            self.SIID_given = 0

        if params_dict.get('IIMOD2CLAMP1') != None:
            val =  params_dict['IIMOD2CLAMP1']
            self.IIMOD2CLAMP1  = self.MPRoo(val,"V"           ,0.0         ,math.inf         ,"Clamp1 of SII1*Vg term in IIMOD=2 model" )
            self.IIMOD2CLAMP1_given = 1
        else:
            self.IIMOD2CLAMP1  = self.MPRoo(0.1            ,"V"           ,0.0         ,math.inf         ,"Clamp1 of SII1*Vg term in IIMOD=2 model" )
            self.IIMOD2CLAMP1_given = 0

        if params_dict.get('IIMOD2CLAMP2') != None:
            val =  params_dict['IIMOD2CLAMP2']
            self.IIMOD2CLAMP2  = self.MPRoo(val,"V"           ,0.0         ,math.inf         ,"Clamp2 of SII0*Vg term in IIMOD=2 model" )
            self.IIMOD2CLAMP2_given = 1
        else:
            self.IIMOD2CLAMP2  = self.MPRoo(0.1            ,"V"           ,0.0         ,math.inf         ,"Clamp2 of SII0*Vg term in IIMOD=2 model" )
            self.IIMOD2CLAMP2_given = 0

        if params_dict.get('IIMOD2CLAMP3') != None:
            val =  params_dict['IIMOD2CLAMP3']
            self.IIMOD2CLAMP3  = self.MPRoo(val,"V"           ,0.0         ,math.inf         ,"Clamp3 of Ratio term in IIMOD=2 model" )
            self.IIMOD2CLAMP3_given = 1
        else:
            self.IIMOD2CLAMP3  = self.MPRoo(0.1            ,"V"           ,0.0         ,math.inf         ,"Clamp3 of Ratio term in IIMOD=2 model" )
            self.IIMOD2CLAMP3_given = 0


        # Accumulation Capacitance
        if params_dict.get('EOTACC') != None:
            val =  params_dict['EOTACC']
            self.EOTACC     = self.MPRco(val,"m"           ,1.0e-10     ,math.inf         ,"Equivalent oxide thickness for accumulation region" )
            self.EOTACC_given = 1
        else:
            self.EOTACC     = self.MPRco(self.EOT       ,"m"           ,1.0e-10     ,math.inf         ,"Equivalent oxide thickness for accumulation region" )
            self.EOTACC_given = 0

        if params_dict.get('DELVFBACC') != None:
            val =  params_dict['DELVFBACC']
            self.DELVFBACC  = self.MPRnb(val,"V"           ,"Change in flatband voltage: Vfb_accumulation - Vfb_inversion" )
            self.DELVFBACC_given = 1
        else:
            self.DELVFBACC  = self.MPRnb(0.0            ,"V"           ,"Change in flatband voltage: Vfb_accumulation - Vfb_inversion" )
            self.DELVFBACC_given = 0


        # Fringe Capacitance
        # CGEOMOD=0
        if params_dict.get('CFS') != None:
            val =  params_dict['CFS']
            self.CFS   = self.MPRcz(val,"F/m"         ,"Outer fringe capacitance at source side" )
            self.CFS_given = 1
        else:
            self.CFS   = self.MPRcz(2.5e-11        ,"F/m"         ,"Outer fringe capacitance at source side" )
            self.CFS_given = 0

        if params_dict.get('CFD') != None:
            val =  params_dict['CFD']
            self.CFD   = self.MPRcz(val,"F/m"         ,"Outer fringe capacitance at drain side" )
            self.CFD_given = 1
        else:
            self.CFD   = self.MPRcz(self.CFS            ,"F/m"         ,"Outer fringe capacitance at drain side" )
            self.CFD_given = 0


        # Overlap Capacitance for CGEOMOD = 0 and 2
        if params_dict.get('CGSO') != None:
            val =  params_dict['CGSO']
            self.CGSO       = self.MPRcz(val,"F/m"         ,"Non LDD region source-gate overlap capacitance per unit channel width" )
            self.CGSO_given = 1
        else:
            self.CGSO       = self.MPRcz(0.0            ,"F/m"         ,"Non LDD region source-gate overlap capacitance per unit channel width" )
            self.CGSO_given = 0

        if params_dict.get('CGDO') != None:
            val =  params_dict['CGDO']
            self.CGDO       = self.MPRcz(val,"F/m"         ,"Non LDD region drain-gate overlap capacitance per unit channel width" )
            self.CGDO_given = 1
        else:
            self.CGDO       = self.MPRcz(self.CGSO      ,"F/m"         ,"Non LDD region drain-gate overlap capacitance per unit channel width" )
            self.CGDO_given = 0

        if params_dict.get('CGSL') != None:
            val =  params_dict['CGSL']
            self.CGSL       = self.MPRcz(val,"F/m"         ,"Overlap capacitance between gate and lightly-doped source region (for self.CGEOMOD = 0, 2)" )
            self.CGSL_given = 1
        else:
            self.CGSL       = self.MPRcz(0.0            ,"F/m"         ,"Overlap capacitance between gate and lightly-doped source region (for self.CGEOMOD = 0, 2)" )
            self.CGSL_given = 0

        if params_dict.get('CGDL') != None:
            val =  params_dict['CGDL']
            self.CGDL       = self.MPRcz(val,"F/m"         ,"Overlap capacitance between gate and lightly-doped drain region (for self.CGEOMOD = 0, 2)" )
            self.CGDL_given = 1
        else:
            self.CGDL       = self.MPRcz(self.CGSL      ,"F/m"         ,"Overlap capacitance between gate and lightly-doped drain region (for self.CGEOMOD = 0, 2)" )
            self.CGDL_given = 0

        if params_dict.get('CKAPPAS') != None:
            val =  params_dict['CKAPPAS']
            self.CKAPPAS    = self.MPRco(val,"V"           ,2.0e-2      ,math.inf         ,"Coefficient of bias-dependent overlap capacitance for the source side (for self.CGEOMOD = 0, 2)" )
            self.CKAPPAS_given = 1
        else:
            self.CKAPPAS    = self.MPRco(0.6            ,"V"           ,2.0e-2      ,math.inf         ,"Coefficient of bias-dependent overlap capacitance for the source side (for self.CGEOMOD = 0, 2)" )
            self.CKAPPAS_given = 0

        if params_dict.get('CKAPPAD') != None:
            val =  params_dict['CKAPPAD']
            self.CKAPPAD    = self.MPRco(val,"V"           ,2.0e-2      ,math.inf         ,"Coefficient of bias-dependent overlap capacitance for the drain side (for self.CGEOMOD = 0, 2)" )
            self.CKAPPAD_given = 1
        else:
            self.CKAPPAD    = self.MPRco(self.CKAPPAS   ,"V"           ,2.0e-2      ,math.inf         ,"Coefficient of bias-dependent overlap capacitance for the drain side (for self.CGEOMOD = 0, 2)" )
            self.CKAPPAD_given = 0

        if params_dict.get('CGBO') != None:
            val =  params_dict['CGBO']
            self.CGBO       = self.MPRcz(val,"F/m"         ,"Gate-to-substrate overlap capacitance per unit channel length per finger per NGCON" )
            self.CGBO_given = 1
        else:
            self.CGBO       = self.MPRcz(0.0            ,"F/m"         ,"Gate-to-substrate overlap capacitance per unit channel length per finger per NGCON" )
            self.CGBO_given = 0

        if params_dict.get('CGBN') != None:
            val =  params_dict['CGBN']
            self.CGBN       = self.MPRcz(val,"F/m"         ,"Gate-to-substrate overlap capacitance per unit channel length per fin per finger" )
            self.CGBN_given = 1
        else:
            self.CGBN       = self.MPRcz(0.0            ,"F/m"         ,"Gate-to-substrate overlap capacitance per unit channel length per fin per finger" )
            self.CGBN_given = 0

        if params_dict.get('CGBL') != None:
            val =  params_dict['CGBL']
            self.CGBL       = self.MPRcz(val,"F/m"         ,"Bias dependent component of gate-to-substrate overlap capacitance per unit channel length per fin per finger" )
            self.CGBL_given = 1
        else:
            self.CGBL       = self.MPRcz(0.0            ,"F/m"         ,"Bias dependent component of gate-to-substrate overlap capacitance per unit channel length per fin per finger" )
            self.CGBL_given = 0

        if params_dict.get('CKAPPAB') != None:
            val =  params_dict['CKAPPAB']
            self.CKAPPAB    = self.MPRco(val,""            ,2.0e-2      ,math.inf         ,"Bias dependent gate-to-substrate parasitic capacitance" )
            self.CKAPPAB_given = 1
        else:
            self.CKAPPAB    = self.MPRco(0.6            ,""            ,2.0e-2      ,math.inf         ,"Bias dependent gate-to-substrate parasitic capacitance" )
            self.CKAPPAB_given = 0


        # Source/Drain-to-Substrate Sidewall Capacitance
        if params_dict.get('CSDESW') != None:
            val =  params_dict['CSDESW']
            self.CSDESW     = self.MPRcz(val,"F/m"         ,"Coefficient for source/drain-to-substrate sidewall capacitance" )
            self.CSDESW_given = 1
        else:
            self.CSDESW     = self.MPRcz(0.0            ,"F/m"         ,"Coefficient for source/drain-to-substrate sidewall capacitance" )
            self.CSDESW_given = 0


        # Junction Current and Capacitance
        # Junction Capacitance
        if params_dict.get('CJS') != None:
            val =  params_dict['CJS']
            self.CJS        = self.MPRnb(val,"F/m^2"       ,"Unit area source-side junction capacitance at zero bias" )
            self.CJS_given = 1
        else:
            self.CJS        = self.MPRnb(5.0e-4         ,"F/m^2"       ,"Unit area source-side junction capacitance at zero bias" )
            self.CJS_given = 0

        if params_dict.get('CJD') != None:
            val =  params_dict['CJD']
            self.CJD        = self.MPRnb(val,"F/m^2"       ,"Unit area drain-side junction capacitance at zero bias" )
            self.CJD_given = 1
        else:
            self.CJD        = self.MPRnb(self.CJS       ,"F/m^2"       ,"Unit area drain-side junction capacitance at zero bias" )
            self.CJD_given = 0

        if params_dict.get('CJSWS') != None:
            val =  params_dict['CJSWS']
            self.CJSWS      = self.MPRnb(val,"F/m"         ,"Unit length source-side sidewall junction capacitance at zero bias" )
            self.CJSWS_given = 1
        else:
            self.CJSWS      = self.MPRnb(5.0e-10        ,"F/m"         ,"Unit length source-side sidewall junction capacitance at zero bias" )
            self.CJSWS_given = 0

        if params_dict.get('CJSWD') != None:
            val =  params_dict['CJSWD']
            self.CJSWD      = self.MPRnb(val,"F/m"         ,"Unit length drain-side sidewall junction capacitance at zero bias" )
            self.CJSWD_given = 1
        else:
            self.CJSWD      = self.MPRnb(self.CJSWS     ,"F/m"         ,"Unit length drain-side sidewall junction capacitance at zero bias" )
            self.CJSWD_given = 0

        if params_dict.get('CJSWGS') != None:
            val =  params_dict['CJSWGS']
            self.CJSWGS     = self.MPRnb(val,"F/m"         ,"Unit length source-side gate sidewall junction capacitance at zero bias" )
            self.CJSWGS_given = 1
        else:
            self.CJSWGS     = self.MPRnb(0.0            ,"F/m"         ,"Unit length source-side gate sidewall junction capacitance at zero bias" )
            self.CJSWGS_given = 0

        if params_dict.get('CJSWGD') != None:
            val =  params_dict['CJSWGD']
            self.CJSWGD     = self.MPRnb(val,"F/m"         ,"Unit length drain-side gate sidewall junction capacitance at zero bias" )
            self.CJSWGD_given = 1
        else:
            self.CJSWGD     = self.MPRnb(self.CJSWGS    ,"F/m"         ,"Unit length drain-side gate sidewall junction capacitance at zero bias" )
            self.CJSWGD_given = 0

        if params_dict.get('PBS') != None:
            val =  params_dict['PBS']
            self.PBS        = self.MPRnb(val,"V"           ,"Source-side bulk junction built-in potential" )
            self.PBS_given = 1
        else:
            self.PBS        = self.MPRnb(1.0            ,"V"           ,"Source-side bulk junction built-in potential" )
            self.PBS_given = 0

        if params_dict.get('PBD') != None:
            val =  params_dict['PBD']
            self.PBD        = self.MPRnb(val,"V"           ,"Drain-side bulk junction built-in potential" )
            self.PBD_given = 1
        else:
            self.PBD        = self.MPRnb(self.PBS       ,"V"           ,"Drain-side bulk junction built-in potential" )
            self.PBD_given = 0

        if params_dict.get('PBSWS') != None:
            val =  params_dict['PBSWS']
            self.PBSWS      = self.MPRnb(val,"V"           ,"Built-in potential for Source-side sidewall junction capacitance" )
            self.PBSWS_given = 1
        else:
            self.PBSWS      = self.MPRnb(1.0            ,"V"           ,"Built-in potential for Source-side sidewall junction capacitance" )
            self.PBSWS_given = 0

        if params_dict.get('PBSWD') != None:
            val =  params_dict['PBSWD']
            self.PBSWD      = self.MPRnb(val,"V"           ,"Built-in potential for Drain-side sidewall junction capacitance" )
            self.PBSWD_given = 1
        else:
            self.PBSWD      = self.MPRnb(self.PBSWS     ,"V"           ,"Built-in potential for Drain-side sidewall junction capacitance" )
            self.PBSWD_given = 0

        if params_dict.get('PBSWGS') != None:
            val =  params_dict['PBSWGS']
            self.PBSWGS     = self.MPRnb(val,"V"           ,"Built-in potential for Source-side gate sidewall junction capacitance" )
            self.PBSWGS_given = 1
        else:
            self.PBSWGS     = self.MPRnb(self.PBSWS     ,"V"           ,"Built-in potential for Source-side gate sidewall junction capacitance" )
            self.PBSWGS_given = 0

        if params_dict.get('PBSWGD') != None:
            val =  params_dict['PBSWGD']
            self.PBSWGD     = self.MPRnb(val,"V"           ,"Built-in potential for Drain-side gate sidewall junction capacitance" )
            self.PBSWGD_given = 1
        else:
            self.PBSWGD     = self.MPRnb(self.PBSWGS    ,"V"           ,"Built-in potential for Drain-side gate sidewall junction capacitance" )
            self.PBSWGD_given = 0

        if params_dict.get('MJS') != None:
            val =  params_dict['MJS']
            self.MJS        = self.MPRnb(val,""            ,"Source bottom junction capacitance grading coefficient" )
            self.MJS_given = 1
        else:
            self.MJS        = self.MPRnb(0.5            ,""            ,"Source bottom junction capacitance grading coefficient" )
            self.MJS_given = 0

        if params_dict.get('MJD') != None:
            val =  params_dict['MJD']
            self.MJD        = self.MPRnb(val,""            ,"Drain bottom junction capacitance grading coefficient" )
            self.MJD_given = 1
        else:
            self.MJD        = self.MPRnb(self.MJS       ,""            ,"Drain bottom junction capacitance grading coefficient" )
            self.MJD_given = 0

        if params_dict.get('MJSWS') != None:
            val =  params_dict['MJSWS']
            self.MJSWS      = self.MPRnb(val,""            ,"Source sidewall junction capacitance grading coefficient" )
            self.MJSWS_given = 1
        else:
            self.MJSWS      = self.MPRnb(0.33           ,""            ,"Source sidewall junction capacitance grading coefficient" )
            self.MJSWS_given = 0

        if params_dict.get('MJSWD') != None:
            val =  params_dict['MJSWD']
            self.MJSWD      = self.MPRnb(val,""            ,"Drain sidewall junction capacitance grading coefficient" )
            self.MJSWD_given = 1
        else:
            self.MJSWD      = self.MPRnb(self.MJSWS     ,""            ,"Drain sidewall junction capacitance grading coefficient" )
            self.MJSWD_given = 0

        if params_dict.get('MJSWGS') != None:
            val =  params_dict['MJSWGS']
            self.MJSWGS     = self.MPRnb(val,""            ,"Source-side gate sidewall junction capacitance grading coefficient" )
            self.MJSWGS_given = 1
        else:
            self.MJSWGS     = self.MPRnb(self.MJSWS     ,""            ,"Source-side gate sidewall junction capacitance grading coefficient" )
            self.MJSWGS_given = 0

        if params_dict.get('MJSWGD') != None:
            val =  params_dict['MJSWGD']
            self.MJSWGD     = self.MPRnb(val,""            ,"Drain-side gate sidewall junction capacitance grading coefficient" )
            self.MJSWGD_given = 1
        else:
            self.MJSWGD     = self.MPRnb(self.MJSWGS    ,""            ,"Drain-side gate sidewall junction capacitance grading coefficient" )
            self.MJSWGD_given = 0


        # Second Junction for Two-Step Junction Capacitance
        if params_dict.get('SJS') != None:
            val =  params_dict['SJS']
            self.SJS        = self.MPRcz(val,""            ,"Constant for source-side two-step second junction" )
            self.SJS_given = 1
        else:
            self.SJS        = self.MPRcz(0.0            ,""            ,"Constant for source-side two-step second junction" )
            self.SJS_given = 0

        if params_dict.get('SJD') != None:
            val =  params_dict['SJD']
            self.SJD        = self.MPRcz(val,""            ,"Constant for drain-side two-step second junction" )
            self.SJD_given = 1
        else:
            self.SJD        = self.MPRcz(self.SJS       ,""            ,"Constant for drain-side two-step second junction" )
            self.SJD_given = 0

        if params_dict.get('SJSWS') != None:
            val =  params_dict['SJSWS']
            self.SJSWS      = self.MPRcz(val,""            ,"Constant for source-side sidewall two-step second junction" )
            self.SJSWS_given = 1
        else:
            self.SJSWS      = self.MPRcz(0.0            ,""            ,"Constant for source-side sidewall two-step second junction" )
            self.SJSWS_given = 0

        if params_dict.get('SJSWD') != None:
            val =  params_dict['SJSWD']
            self.SJSWD      = self.MPRcz(val,""            ,"Constant for drain-side sidewall two-step second junction" )
            self.SJSWD_given = 1
        else:
            self.SJSWD      = self.MPRcz(self.SJSWS     ,""            ,"Constant for drain-side sidewall two-step second junction" )
            self.SJSWD_given = 0

        if params_dict.get('SJSWGS') != None:
            val =  params_dict['SJSWGS']
            self.SJSWGS     = self.MPRcz(val,""            ,"Constant for source-side gate sidewall two-step second junction" )
            self.SJSWGS_given = 1
        else:
            self.SJSWGS     = self.MPRcz(0.0            ,""            ,"Constant for source-side gate sidewall two-step second junction" )
            self.SJSWGS_given = 0

        if params_dict.get('SJSWGD') != None:
            val =  params_dict['SJSWGD']
            self.SJSWGD     = self.MPRcz(val,""            ,"Constant for source-side gate sidewall two-step second junction" )
            self.SJSWGD_given = 1
        else:
            self.SJSWGD     = self.MPRcz(self.SJSWGS    ,""            ,"Constant for source-side gate sidewall two-step second junction" )
            self.SJSWGD_given = 0

        if params_dict.get('MJS2') != None:
            val =  params_dict['MJS2']
            self.MJS2       = self.MPRnb(val,""            ,"Source bottom two-step second junction capacitance grading coefficient" )
            self.MJS2_given = 1
        else:
            self.MJS2       = self.MPRnb(1.25e-1        ,""            ,"Source bottom two-step second junction capacitance grading coefficient" )
            self.MJS2_given = 0

        if params_dict.get('MJD2') != None:
            val =  params_dict['MJD2']
            self.MJD2       = self.MPRnb(val,""            ,"Drain bottom two-step second junction capacitance grading coefficient" )
            self.MJD2_given = 1
        else:
            self.MJD2       = self.MPRnb(self.MJS2      ,""            ,"Drain bottom two-step second junction capacitance grading coefficient" )
            self.MJD2_given = 0

        if params_dict.get('MJSWS2') != None:
            val =  params_dict['MJSWS2']
            self.MJSWS2     = self.MPRnb(val,""            ,"Source sidewall two-step second junction capacitance grading coefficient" )
            self.MJSWS2_given = 1
        else:
            self.MJSWS2     = self.MPRnb(8.3e-2         ,""            ,"Source sidewall two-step second junction capacitance grading coefficient" )
            self.MJSWS2_given = 0

        if params_dict.get('MJSWD2') != None:
            val =  params_dict['MJSWD2']
            self.MJSWD2     = self.MPRnb(val,""            ,"Drain sidewall two-step second junction capacitance grading coefficient" )
            self.MJSWD2_given = 1
        else:
            self.MJSWD2     = self.MPRnb(self.MJSWS2    ,""            ,"Drain sidewall two-step second junction capacitance grading coefficient" )
            self.MJSWD2_given = 0

        if params_dict.get('MJSWGS2') != None:
            val =  params_dict['MJSWGS2']
            self.MJSWGS2    = self.MPRnb(val,""            ,"Source-side gate sidewall two-step second junction capacitance grading coefficient" )
            self.MJSWGS2_given = 1
        else:
            self.MJSWGS2    = self.MPRnb(self.MJSWS2    ,""            ,"Source-side gate sidewall two-step second junction capacitance grading coefficient" )
            self.MJSWGS2_given = 0

        if params_dict.get('MJSWGD2') != None:
            val =  params_dict['MJSWGD2']
            self.MJSWGD2    = self.MPRnb(val,""            ,"Drain-side gate sidewall two-step second junction capacitance grading coefficient" )
            self.MJSWGD2_given = 1
        else:
            self.MJSWGD2    = self.MPRnb(self.MJSWGS2   ,""            ,"Drain-side gate sidewall two-step second junction capacitance grading coefficient" )
            self.MJSWGD2_given = 0


        # Junction Current
        if params_dict.get('JSS') != None:
            val =  params_dict['JSS']
            self.JSS        = self.MPRnb(val,"A/m^2"       ,"Bottom source junction reverse saturation current density" )
            self.JSS_given = 1
        else:
            self.JSS        = self.MPRnb(1.0e-4         ,"A/m^2"       ,"Bottom source junction reverse saturation current density" )
            self.JSS_given = 0

        if params_dict.get('JSD') != None:
            val =  params_dict['JSD']
            self.JSD        = self.MPRnb(val,"A/m^2"       ,"Bottom drain junction reverse saturation current density" )
            self.JSD_given = 1
        else:
            self.JSD        = self.MPRnb(self.JSS       ,"A/m^2"       ,"Bottom drain junction reverse saturation current density" )
            self.JSD_given = 0

        if params_dict.get('JSWS') != None:
            val =  params_dict['JSWS']
            self.JSWS       = self.MPRnb(val,"A/m"         ,"Unit length reverse saturation current for sidewall source junction" )
            self.JSWS_given = 1
        else:
            self.JSWS       = self.MPRnb(0.0            ,"A/m"         ,"Unit length reverse saturation current for sidewall source junction" )
            self.JSWS_given = 0

        if params_dict.get('JSWD') != None:
            val =  params_dict['JSWD']
            self.JSWD       = self.MPRnb(val,"A/m"         ,"Unit length reverse saturation current for sidewall drain junction" )
            self.JSWD_given = 1
        else:
            self.JSWD       = self.MPRnb(self.JSWS      ,"A/m"         ,"Unit length reverse saturation current for sidewall drain junction" )
            self.JSWD_given = 0

        if params_dict.get('JSWGS') != None:
            val =  params_dict['JSWGS']
            self.JSWGS      = self.MPRnb(val,"A/m"         ,"Unit length reverse saturation current for gate-edge sidewall source junction" )
            self.JSWGS_given = 1
        else:
            self.JSWGS      = self.MPRnb(0.0            ,"A/m"         ,"Unit length reverse saturation current for gate-edge sidewall source junction" )
            self.JSWGS_given = 0

        if params_dict.get('JSWGD') != None:
            val =  params_dict['JSWGD']
            self.JSWGD      = self.MPRnb(val,"A/m"         ,"Unit length reverse saturation current for gate-edge sidewall drain junction" )
            self.JSWGD_given = 1
        else:
            self.JSWGD      = self.MPRnb(self.JSWGS     ,"A/m"         ,"Unit length reverse saturation current for gate-edge sidewall drain junction" )
            self.JSWGD_given = 0

        if params_dict.get('NJS') != None:
            val =  params_dict['NJS']
            self.NJS        = self.MPRex(val,""            ,0.0         ,"Source junction emission coefficient" )
            self.NJS_given = 1
        else:
            self.NJS        = self.MPRex(1.0            ,""            ,0.0         ,"Source junction emission coefficient" )
            self.NJS_given = 0

        if params_dict.get('NJD') != None:
            val =  params_dict['NJD']
            self.NJD        = self.MPRex(val,""            ,0.0         ,"Drain junction emission coefficient" )
            self.NJD_given = 1
        else:
            self.NJD        = self.MPRex(self.NJS       ,""            ,0.0         ,"Drain junction emission coefficient" )
            self.NJD_given = 0

        if params_dict.get('IJTHSFWD') != None:
            val =  params_dict['IJTHSFWD']
            self.IJTHSFWD   = self.MPRnb(val,"A"           ,"Forward source diode breakdown limiting current" )
            self.IJTHSFWD_given = 1
        else:
            self.IJTHSFWD   = self.MPRnb(0.1            ,"A"           ,"Forward source diode breakdown limiting current" )
            self.IJTHSFWD_given = 0

        if params_dict.get('IJTHDFWD') != None:
            val =  params_dict['IJTHDFWD']
            self.IJTHDFWD   = self.MPRnb(val,"A"           ,"Forward drain diode breakdown limiting current" )
            self.IJTHDFWD_given = 1
        else:
            self.IJTHDFWD   = self.MPRnb(self.IJTHSFWD  ,"A"           ,"Forward drain diode breakdown limiting current" )
            self.IJTHDFWD_given = 0

        if params_dict.get('IJTHSREV') != None:
            val =  params_dict['IJTHSREV']
            self.IJTHSREV   = self.MPRnb(val,"A"           ,"Reverse source diode breakdown limiting current" )
            self.IJTHSREV_given = 1
        else:
            self.IJTHSREV   = self.MPRnb(0.1            ,"A"           ,"Reverse source diode breakdown limiting current" )
            self.IJTHSREV_given = 0

        if params_dict.get('IJTHDREV') != None:
            val =  params_dict['IJTHDREV']
            self.IJTHDREV   = self.MPRnb(val,"A"           ,"Reverse drain diode breakdown limiting current" )
            self.IJTHDREV_given = 1
        else:
            self.IJTHDREV   = self.MPRnb(self.IJTHSREV  ,"A"           ,"Reverse drain diode breakdown limiting current" )
            self.IJTHDREV_given = 0

        if params_dict.get('BVS') != None:
            val =  params_dict['BVS']
            self.BVS        = self.MPRnb(val,"V"           ,"Source diode breakdown voltage" )
            self.BVS_given = 1
        else:
            self.BVS        = self.MPRnb(1.0e1          ,"V"           ,"Source diode breakdown voltage" )
            self.BVS_given = 0

        if params_dict.get('BVD') != None:
            val =  params_dict['BVD']
            self.BVD        = self.MPRnb(val,"V"           ,"Drain diode breakdown voltage" )
            self.BVD_given = 1
        else:
            self.BVD        = self.MPRnb(self.BVS       ,"V"           ,"Drain diode breakdown voltage" )
            self.BVD_given = 0

        if params_dict.get('XJBVS') != None:
            val =  params_dict['XJBVS']
            self.XJBVS      = self.MPRnb(val,""            ,"Fitting parameter for source diode breakdown current" )
            self.XJBVS_given = 1
        else:
            self.XJBVS      = self.MPRnb(1.0            ,""            ,"Fitting parameter for source diode breakdown current" )
            self.XJBVS_given = 0

        if params_dict.get('XJBVD') != None:
            val =  params_dict['XJBVD']
            self.XJBVD      = self.MPRnb(val,""            ,"Fitting parameter for drain diode breakdown current" )
            self.XJBVD_given = 1
        else:
            self.XJBVD      = self.MPRnb(self.XJBVS     ,""            ,"Fitting parameter for drain diode breakdown current" )
            self.XJBVD_given = 0


        # Tunneling Component of Junction Current
        if params_dict.get('JTSS') != None:
            val =  params_dict['JTSS']
            self.JTSS        = self.MPRnb(val,"A/m^2"       ,"Bottom source junction trap-assisted saturation current density" )
            self.JTSS_given = 1
        else:
            self.JTSS        = self.MPRnb(0.0            ,"A/m^2"       ,"Bottom source junction trap-assisted saturation current density" )
            self.JTSS_given = 0

        if params_dict.get('JTSD') != None:
            val =  params_dict['JTSD']
            self.JTSD        = self.MPRnb(val,"A/m^2"       ,"Bottom drain junction trap-assisted saturation current density" )
            self.JTSD_given = 1
        else:
            self.JTSD        = self.MPRnb(self.JTSS      ,"A/m^2"       ,"Bottom drain junction trap-assisted saturation current density" )
            self.JTSD_given = 0

        if params_dict.get('JTSSWS') != None:
            val =  params_dict['JTSSWS']
            self.JTSSWS      = self.MPRnb(val,"A/m"         ,"Unit length trap-assisted saturation current for sidewall source junction" )
            self.JTSSWS_given = 1
        else:
            self.JTSSWS      = self.MPRnb(0.0            ,"A/m"         ,"Unit length trap-assisted saturation current for sidewall source junction" )
            self.JTSSWS_given = 0

        if params_dict.get('JTSSWD') != None:
            val =  params_dict['JTSSWD']
            self.JTSSWD      = self.MPRnb(val,"A/m"         ,"Unit length trap-assisted saturation current for sidewall drain junction" )
            self.JTSSWD_given = 1
        else:
            self.JTSSWD      = self.MPRnb(self.JTSSWS    ,"A/m"         ,"Unit length trap-assisted saturation current for sidewall drain junction" )
            self.JTSSWD_given = 0

        if params_dict.get('JTSSWGS') != None:
            val =  params_dict['JTSSWGS']
            self.JTSSWGS     = self.MPRnb(val,"A/m"         ,"Unit length trap-assisted saturation current for gate-edge sidewall source junction" )
            self.JTSSWGS_given = 1
        else:
            self.JTSSWGS     = self.MPRnb(0.0            ,"A/m"         ,"Unit length trap-assisted saturation current for gate-edge sidewall source junction" )
            self.JTSSWGS_given = 0

        if params_dict.get('JTSSWGD') != None:
            val =  params_dict['JTSSWGD']
            self.JTSSWGD     = self.MPRnb(val,"A/m"         ,"Unit length trap-assisted saturation current for gate-edge sidewall drain junction" )
            self.JTSSWGD_given = 1
        else:
            self.JTSSWGD     = self.MPRnb(self.JTSSWGS   ,"A/m"         ,"Unit length trap-assisted saturation current for gate-edge sidewall drain junction" )
            self.JTSSWGD_given = 0

        if params_dict.get('JTWEFF') != None:
            val =  params_dict['JTWEFF']
            self.JTWEFF      = self.MPRnb(val,"m"           ,"Trap-assisted tunneling current width dependence" )
            self.JTWEFF_given = 1
        else:
            self.JTWEFF      = self.MPRnb(0.0            ,"m"           ,"Trap-assisted tunneling current width dependence" )
            self.JTWEFF_given = 0

        if params_dict.get('NJTS') != None:
            val =  params_dict['NJTS']
            self.NJTS        = self.MPRnb(val,""            ,"Non-ideality factor for JTSS" )
            self.NJTS_given = 1
        else:
            self.NJTS        = self.MPRnb(2.0e1          ,""            ,"Non-ideality factor for JTSS" )
            self.NJTS_given = 0

        if params_dict.get('NJTSD') != None:
            val =  params_dict['NJTSD']
            self.NJTSD       = self.MPRnb(val,""            ,"Non-ideality factor for JTSD" )
            self.NJTSD_given = 1
        else:
            self.NJTSD       = self.MPRnb(self.NJTS      ,""            ,"Non-ideality factor for JTSD" )
            self.NJTSD_given = 0

        if params_dict.get('NJTSSW') != None:
            val =  params_dict['NJTSSW']
            self.NJTSSW      = self.MPRnb(val,""            ,"Non-ideality factor for JTSSWS" )
            self.NJTSSW_given = 1
        else:
            self.NJTSSW      = self.MPRnb(2.0e1          ,""            ,"Non-ideality factor for JTSSWS" )
            self.NJTSSW_given = 0

        if params_dict.get('NJTSSWD') != None:
            val =  params_dict['NJTSSWD']
            self.NJTSSWD     = self.MPRnb(val,""            ,"Non-ideality factor for JTSSWD" )
            self.NJTSSWD_given = 1
        else:
            self.NJTSSWD     = self.MPRnb(self.NJTSSW    ,""            ,"Non-ideality factor for JTSSWD" )
            self.NJTSSWD_given = 0

        if params_dict.get('NJTSSWG') != None:
            val =  params_dict['NJTSSWG']
            self.NJTSSWG     = self.MPRnb(val,""            ,"Non-ideality factor for JTSSWGS" )
            self.NJTSSWG_given = 1
        else:
            self.NJTSSWG     = self.MPRnb(2.0e1          ,""            ,"Non-ideality factor for JTSSWGS" )
            self.NJTSSWG_given = 0

        if params_dict.get('NJTSSWGD') != None:
            val =  params_dict['NJTSSWGD']
            self.NJTSSWGD    = self.MPRnb(val,""            ,"Non-ideality factor for JTSSWGD" )
            self.NJTSSWGD_given = 1
        else:
            self.NJTSSWGD    = self.MPRnb(self.NJTSSWG   ,""            ,"Non-ideality factor for JTSSWGD" )
            self.NJTSSWGD_given = 0

        if params_dict.get('VTSS') != None:
            val =  params_dict['VTSS']
            self.VTSS        = self.MPRnb(val,"V"           ,"Bottom source junction trap-assisted current voltage dependent parameter" )
            self.VTSS_given = 1
        else:
            self.VTSS        = self.MPRnb(1.0e1          ,"V"           ,"Bottom source junction trap-assisted current voltage dependent parameter" )
            self.VTSS_given = 0

        if params_dict.get('VTSD') != None:
            val =  params_dict['VTSD']
            self.VTSD        = self.MPRnb(val,"V"           ,"Bottom drain junction trap-assisted current voltage dependent parameter" )
            self.VTSD_given = 1
        else:
            self.VTSD        = self.MPRnb(self.VTSS      ,"V"           ,"Bottom drain junction trap-assisted current voltage dependent parameter" )
            self.VTSD_given = 0

        if params_dict.get('VTSSWS') != None:
            val =  params_dict['VTSSWS']
            self.VTSSWS      = self.MPRnb(val,"V"           ,"Unit length trap-assisted current voltage dependent parameter for sidewall source junction" )
            self.VTSSWS_given = 1
        else:
            self.VTSSWS      = self.MPRnb(1.0e1          ,"V"           ,"Unit length trap-assisted current voltage dependent parameter for sidewall source junction" )
            self.VTSSWS_given = 0

        if params_dict.get('VTSSWD') != None:
            val =  params_dict['VTSSWD']
            self.VTSSWD      = self.MPRnb(val,"V"           ,"Unit length trap-assisted current voltage dependent parameter for sidewall drain junction" )
            self.VTSSWD_given = 1
        else:
            self.VTSSWD      = self.MPRnb(self.VTSSWS    ,"V"           ,"Unit length trap-assisted current voltage dependent parameter for sidewall drain junction" )
            self.VTSSWD_given = 0

        if params_dict.get('VTSSWGS') != None:
            val =  params_dict['VTSSWGS']
            self.VTSSWGS     = self.MPRnb(val,"V"           ,"Unit length trap-assisted current voltage dependent parameter for gate-edge sidewall source junction" )
            self.VTSSWGS_given = 1
        else:
            self.VTSSWGS     = self.MPRnb(1.0e1          ,"V"           ,"Unit length trap-assisted current voltage dependent parameter for gate-edge sidewall source junction" )
            self.VTSSWGS_given = 0

        if params_dict.get('VTSSWGD') != None:
            val =  params_dict['VTSSWGD']
            self.VTSSWGD     = self.MPRnb(val,"V"           ,"Unit length trap-assisted current voltage dependent parameter for gate-edge sidewall drain junction" )
            self.VTSSWGD_given = 1
        else:
            self.VTSSWGD     = self.MPRnb(self.VTSSWGS   ,"V"           ,"Unit length trap-assisted current voltage dependent parameter for gate-edge sidewall drain junction" )
            self.VTSSWGD_given = 0


        # Recombination-Generation Current
        if params_dict.get('LINTIGEN') != None:
            val =  params_dict['LINTIGEN']
            self.LINTIGEN  = self.MPRnb(val,"m"           ,"Lint for thermal generation current" )
            self.LINTIGEN_given = 1
        else:
            self.LINTIGEN  = self.MPRnb(0.0            ,"m"           ,"Lint for thermal generation current" )
            self.LINTIGEN_given = 0

        if params_dict.get('NTGEN') != None:
            val =  params_dict['NTGEN']
            self.NTGEN     = self.MPRnb(val,""            ,"Thermal generation current parameter" )
            self.NTGEN_given = 1
        else:
            self.NTGEN     = self.MPRnb(1.0            ,""            ,"Thermal generation current parameter" )
            self.NTGEN_given = 0

        if params_dict.get('AIGEN') != None:
            val =  params_dict['AIGEN']
            self.AIGEN     = self.MPRnb(val,"(m^-3)*(V^-1)"            ,"Thermal generation current parameter" )
            self.AIGEN_given = 1
        else:
            self.AIGEN     = self.MPRnb(0.0            ,"(m^-3)*(V^-1)"            ,"Thermal generation current parameter" )
            self.AIGEN_given = 0

        if params_dict.get('BIGEN') != None:
            val =  params_dict['BIGEN']
            self.BIGEN     = self.MPRnb(val,"(m^-3)*(V^-3)"            ,"Thermal generation current parameter" )
            self.BIGEN_given = 1
        else:
            self.BIGEN     = self.MPRnb(0.0            ,"(m^-3)*(V^-3)"            ,"Thermal generation current parameter" )
            self.BIGEN_given = 0


        # NQS Gate Resistance Model & NQS Charge Deficit Model
        # For NQSMOD=1, Set XRCRG1=0 to turn off NQS gate resistance
        if params_dict.get('XRCRG1') != None:
            val =  params_dict['XRCRG1']
            self.XRCRG1  = self.MPRnb(val,""            ,"Parameter for non-quasistatic gate resistance (self.NQSMOD = 1) and self.NQSMOD = 2" )
            self.XRCRG1_given = 1
        else:
            self.XRCRG1  = self.MPRnb(1.2e1          ,""            ,"Parameter for non-quasistatic gate resistance (self.NQSMOD = 1) and self.NQSMOD = 2" )
            self.XRCRG1_given = 0

        if params_dict.get('XRCRG2') != None:
            val =  params_dict['XRCRG2']
            self.XRCRG2  = self.MPRnb(val,""            ,"Parameter for non-quasistatic gate resistance (self.NQSMOD = 1) and self.NQSMOD = 2" )
            self.XRCRG2_given = 1
        else:
            self.XRCRG2  = self.MPRnb(1.0            ,""            ,"Parameter for non-quasistatic gate resistance (self.NQSMOD = 1) and self.NQSMOD = 2" )
            self.XRCRG2_given = 0


        # NQS Charge Segmentation Model
        if params_dict.get('NSEG') != None:
            val =  params_dict['NSEG']
            self.NSEG = self.MPIcc(val,""            ,4           ,10          ,"Number of segments for NQSMOD=3 (3, 5 & 10 supported)" )
            self.NSEG_given = 1
        else:
            self.NSEG = self.MPIcc(4              ,""            ,4           ,10          ,"Number of segments for NQSMOD=3 (3, 5 & 10 supported)" )
            self.NSEG_given = 0


        # Flicker Noise
        if params_dict.get('EF') != None:
            val =  params_dict['EF']
            self.EF       = self.MPRnb(val,""            ,"Flicker noise frequency exponent" )
            self.EF_given = 1
        else:
            self.EF       = self.MPRnb(1.0            ,""            ,"Flicker noise frequency exponent" )
            self.EF_given = 0

        if params_dict.get('EM') != None:
            val =  params_dict['EM']
            self.EM       = self.MPRnb(val,"V/m"         ,"Flicker noise parameter" )
            self.EM_given = 1
        else:
            self.EM       = self.MPRnb(4.1e7          ,"V/m"         ,"Flicker noise parameter" )
            self.EM_given = 0

        if params_dict.get('NOIA') != None:
            val =  params_dict['NOIA']
            self.NOIA     = self.MPRnb(val,"(eV^-1)*(s^(1-EF))*(m^-3)"             ,"Flicker noise parameter" )
            self.NOIA_given = 1
        else:
            self.NOIA     = self.MPRnb(6.25e39        ,"(eV^-1)*(s^(1-EF))*(m^-3)"             ,"Flicker noise parameter" )
            self.NOIA_given = 0

        if params_dict.get('NOIB') != None:
            val =  params_dict['NOIB']
            self.NOIB     = self.MPRnb(val,"(eV^-1)*(s^(1-EF))*(m^-1)"             ,"Flicker noise parameter" )
            self.NOIB_given = 1
        else:
            self.NOIB     = self.MPRnb(3.125e24       ,"(eV^-1)*(s^(1-EF))*(m^-1)"             ,"Flicker noise parameter" )
            self.NOIB_given = 0

        if params_dict.get('NOIC') != None:
            val =  params_dict['NOIC']
            self.NOIC     = self.MPRnb(val,"(eV^-1)*(s^(1-EF))*(m)"                ,"Flicker noise parameter" )
            self.NOIC_given = 1
        else:
            self.NOIC     = self.MPRnb(8.75e7         ,"(eV^-1)*(s^(1-EF))*(m)"                ,"Flicker noise parameter" )
            self.NOIC_given = 0

        if params_dict.get('LINTNOI') != None:
            val =  params_dict['LINTNOI']
            self.LINTNOI  = self.MPRnb(val,"m^2"         ,"L offset for flicker noise calculation" )
            self.LINTNOI_given = 1
        else:
            self.LINTNOI  = self.MPRnb(0.0            ,"m^2"         ,"L offset for flicker noise calculation" )
            self.LINTNOI_given = 0


        # Thermal Noise
        if params_dict.get('NTNOI') != None:
            val =  params_dict['NTNOI']
            self.NTNOI  = self.MPRcz(val,""            ,"Thermal noise parameter" )
            self.NTNOI_given = 1
        else:
            self.NTNOI  = self.MPRcz(1.0            ,""            ,"Thermal noise parameter" )
            self.NTNOI_given = 0

        if params_dict.get('TNOIA') != None:
            val =  params_dict['TNOIA']
            self.TNOIA  = self.MPRnb(val,"/m"          ,"Thermal noise parameter" )
            self.TNOIA_given = 1
        else:
            self.TNOIA  = self.MPRnb(1.5            ,"/m"          ,"Thermal noise parameter" )
            self.TNOIA_given = 0

        if params_dict.get('TNOIB') != None:
            val =  params_dict['TNOIB']
            self.TNOIB  = self.MPRnb(val,"/m"          ,"Thermal noise parameter" )
            self.TNOIB_given = 1
        else:
            self.TNOIB  = self.MPRnb(3.5            ,"/m"          ,"Thermal noise parameter" )
            self.TNOIB_given = 0

        if params_dict.get('RNOIA') != None:
            val =  params_dict['RNOIA']
            self.RNOIA  = self.MPRnb(val,""            ,"Thermal noise coefficient" )
            self.RNOIA_given = 1
        else:
            self.RNOIA  = self.MPRnb(5.77e-1        ,""            ,"Thermal noise coefficient" )
            self.RNOIA_given = 0

        if params_dict.get('RNOIB') != None:
            val =  params_dict['RNOIB']
            self.RNOIB  = self.MPRnb(val,""            ,"Thermal noise coefficient" )
            self.RNOIB_given = 1
        else:
            self.RNOIB  = self.MPRnb(3.7e-1         ,""            ,"Thermal noise coefficient" )
            self.RNOIB_given = 0


        # Parameters Controlled by Correlated Thermal Noise Switch
        if self.__TNOIMOD1__:
            if params_dict.get('TNOIC') != None:
                val =  params_dict['TNOIC']
                self.TNOIC   = self.MPRnb(val,""            ,"Thermal noise parameter for TNOIMOD=1" )
                self.TNOIC_given = 1
            else:
                self.TNOIC   = self.MPRnb(3.5            ,""            ,"Thermal noise parameter for TNOIMOD=1" )
                self.TNOIC_given = 0

            if params_dict.get('RNOIC') != None:
                val =  params_dict['RNOIC']
                self.RNOIC   = self.MPRnb(val,""            ,"Thermal noise coefficient for TNOIMOD=1" )
                self.RNOIC_given = 1
            else:
                self.RNOIC   = self.MPRnb(3.95e-1        ,""            ,"Thermal noise coefficient for TNOIMOD=1" )
                self.RNOIC_given = 0

            if params_dict.get('SCALEN') != None:
                val =  params_dict['SCALEN']
                self.SCALEN  = self.MPRex(val,""            ,0.0         ,"Noise scaling parameter for TNOIMOD=1" )
                self.SCALEN_given = 1
            else:
                self.SCALEN  = self.MPRex(1.0e5          ,""            ,0.0         ,"Noise scaling parameter for TNOIMOD=1" )
                self.SCALEN_given = 0


        # Temperature Effects
        if params_dict.get('TNOM') != None:
            val =  params_dict['TNOM']
            self.TNOM        = self.MPRco(val,"Celsius"     ,-self.P_CELSIUS0,math.inf         ,"Temperature at which the model is extracted" )
            self.TNOM_given = 1
        else:
            self.TNOM        = self.MPRco(27.0           ,"Celsius"     ,-self.P_CELSIUS0,math.inf         ,"Temperature at which the model is extracted" )
            self.TNOM_given = 0

        if params_dict.get('TBGASUB') != None:
            val =  params_dict['TBGASUB']
            self.TBGASUB     = self.MPRnb(val,"eV/K"        ,"Bandgap temperature coefficient" )
            self.TBGASUB_given = 1
        else:
            self.TBGASUB     = self.MPRnb(7.02e-4        ,"eV/K"        ,"Bandgap temperature coefficient" )
            self.TBGASUB_given = 0

        if params_dict.get('TBGBSUB') != None:
            val =  params_dict['TBGBSUB']
            self.TBGBSUB     = self.MPRnb(val,"K"           ,"Bandgap temperature coefficient" )
            self.TBGBSUB_given = 1
        else:
            self.TBGBSUB     = self.MPRnb(1.108e3        ,"K"           ,"Bandgap temperature coefficient" )
            self.TBGBSUB_given = 0

        if params_dict.get('KT1') != None:
            val =  params_dict['KT1']
            self.KT1         = self.MPRnb(val,"V"           ,"Vth temperature coefficient" )
            self.KT1_given = 1
        else:
            self.KT1         = self.MPRnb(0.0            ,"V"           ,"Vth temperature coefficient" )
            self.KT1_given = 0

        if params_dict.get('KT1L') != None:
            val =  params_dict['KT1L']
            self.KT1L        = self.MPRnb(val,"V*m"         ,"Vth temperature self.L coefficient" )
            self.KT1L_given = 1
        else:
            self.KT1L        = self.MPRnb(0.0            ,"V*m"         ,"Vth temperature self.L coefficient" )
            self.KT1L_given = 0

        if params_dict.get('TSS') != None:
            val =  params_dict['TSS']
            self.TSS         = self.MPRnb(val,"/K"          ,"Swing temperature coefficient" )
            self.TSS_given = 1
        else:
            self.TSS         = self.MPRnb(0.0            ,"/K"          ,"Swing temperature coefficient" )
            self.TSS_given = 0

        if params_dict.get('IIT') != None:
            val =  params_dict['IIT']
            self.IIT         = self.MPRnb(val,""            ,"Impact ionization temperature dependence for self.IIMOD = 1" )
            self.IIT_given = 1
        else:
            self.IIT         = self.MPRnb(-0.5           ,""            ,"Impact ionization temperature dependence for self.IIMOD = 1" )
            self.IIT_given = 0

        if params_dict.get('TII') != None:
            val =  params_dict['TII']
            self.TII         = self.MPRnb(val,""            ,"Impact ionization temperature dependence for self.IIMOD = 2" )
            self.TII_given = 1
        else:
            self.TII         = self.MPRnb(0.0            ,""            ,"Impact ionization temperature dependence for self.IIMOD = 2" )
            self.TII_given = 0

        if params_dict.get('TGIDL') != None:
            val =  params_dict['TGIDL']
            self.TGIDL       = self.MPRnb(val,"/K"          ,"GIDL/GISL temperature dependence" )
            self.TGIDL_given = 1
        else:
            self.TGIDL       = self.MPRnb(-3.0e-3        ,"/K"          ,"GIDL/GISL temperature dependence" )
            self.TGIDL_given = 0

        if params_dict.get('IGT') != None:
            val =  params_dict['IGT']
            self.IGT         = self.MPRnb(val,""            ,"Gate current temperature dependence" )
            self.IGT_given = 1
        else:
            self.IGT         = self.MPRnb(2.5            ,""            ,"Gate current temperature dependence" )
            self.IGT_given = 0

        if params_dict.get('TCJ') != None:
            val =  params_dict['TCJ']
            self.TCJ         = self.MPRnb(val,"/K"          ,"Temperature coefficient for CJS/CJD" )
            self.TCJ_given = 1
        else:
            self.TCJ         = self.MPRnb(0.0            ,"/K"          ,"Temperature coefficient for CJS/CJD" )
            self.TCJ_given = 0

        if params_dict.get('TCJSW') != None:
            val =  params_dict['TCJSW']
            self.TCJSW       = self.MPRnb(val,"/K"          ,"Temperature coefficient for CJSWS/CJSWD" )
            self.TCJSW_given = 1
        else:
            self.TCJSW       = self.MPRnb(0.0            ,"/K"          ,"Temperature coefficient for CJSWS/CJSWD" )
            self.TCJSW_given = 0

        if params_dict.get('TCJSWG') != None:
            val =  params_dict['TCJSWG']
            self.TCJSWG      = self.MPRnb(val,"/K"          ,"Temperature coefficient for CJSWGS/CJSWGD" )
            self.TCJSWG_given = 1
        else:
            self.TCJSWG      = self.MPRnb(0.0            ,"/K"          ,"Temperature coefficient for CJSWGS/CJSWGD" )
            self.TCJSWG_given = 0

        if params_dict.get('TPB') != None:
            val =  params_dict['TPB']
            self.TPB         = self.MPRnb(val,"/K"          ,"Temperature coefficient for PBS/PBD" )
            self.TPB_given = 1
        else:
            self.TPB         = self.MPRnb(0.0            ,"/K"          ,"Temperature coefficient for PBS/PBD" )
            self.TPB_given = 0

        if params_dict.get('TPBSW') != None:
            val =  params_dict['TPBSW']
            self.TPBSW       = self.MPRnb(val,"/K"          ,"Temperature coefficient for PBSWS/PBSWD" )
            self.TPBSW_given = 1
        else:
            self.TPBSW       = self.MPRnb(0.0            ,"/K"          ,"Temperature coefficient for PBSWS/PBSWD" )
            self.TPBSW_given = 0

        if params_dict.get('TPBSWG') != None:
            val =  params_dict['TPBSWG']
            self.TPBSWG      = self.MPRnb(val,"/K"          ,"Temperature coefficient for PBSWGS/PBSWGD" )
            self.TPBSWG_given = 1
        else:
            self.TPBSWG      = self.MPRnb(0.0            ,"/K"          ,"Temperature coefficient for PBSWGS/PBSWGD" )
            self.TPBSWG_given = 0

        if params_dict.get('XTIS') != None:
            val =  params_dict['XTIS']
            self.XTIS        = self.MPRnb(val,""            ,"Source junction current temperature exponent" )
            self.XTIS_given = 1
        else:
            self.XTIS        = self.MPRnb(3.0            ,""            ,"Source junction current temperature exponent" )
            self.XTIS_given = 0

        if params_dict.get('XTID') != None:
            val =  params_dict['XTID']
            self.XTID        = self.MPRnb(val,""            ,"Drain junction current temperature exponent" )
            self.XTID_given = 1
        else:
            self.XTID        = self.MPRnb(self.XTIS      ,""            ,"Drain junction current temperature exponent" )
            self.XTID_given = 0

        if params_dict.get('XTSS') != None:
            val =  params_dict['XTSS']
            self.XTSS        = self.MPRnb(val,""            ,"Power dependence of self.JTSS on temperature" )
            self.XTSS_given = 1
        else:
            self.XTSS        = self.MPRnb(2.0e-2         ,""            ,"Power dependence of self.JTSS on temperature" )
            self.XTSS_given = 0

        if params_dict.get('XTSD') != None:
            val =  params_dict['XTSD']
            self.XTSD        = self.MPRnb(val,""            ,"Power dependence of self.JTSD on temperature" )
            self.XTSD_given = 1
        else:
            self.XTSD        = self.MPRnb(self.XTSS      ,""            ,"Power dependence of self.JTSD on temperature" )
            self.XTSD_given = 0

        if params_dict.get('XTSSWS') != None:
            val =  params_dict['XTSSWS']
            self.XTSSWS      = self.MPRnb(val,""            ,"Power dependence of self.JTSSWS on temperature" )
            self.XTSSWS_given = 1
        else:
            self.XTSSWS      = self.MPRnb(2.0e-2         ,""            ,"Power dependence of self.JTSSWS on temperature" )
            self.XTSSWS_given = 0

        if params_dict.get('XTSSWD') != None:
            val =  params_dict['XTSSWD']
            self.XTSSWD      = self.MPRnb(val,""            ,"Power dependence of self.JTSSWD on temperature" )
            self.XTSSWD_given = 1
        else:
            self.XTSSWD      = self.MPRnb(self.XTSSWS    ,""            ,"Power dependence of self.JTSSWD on temperature" )
            self.XTSSWD_given = 0

        if params_dict.get('XTSSWGS') != None:
            val =  params_dict['XTSSWGS']
            self.XTSSWGS     = self.MPRnb(val,""            ,"Power dependence of self.JTSSWGS on temperature" )
            self.XTSSWGS_given = 1
        else:
            self.XTSSWGS     = self.MPRnb(2.0e-2         ,""            ,"Power dependence of self.JTSSWGS on temperature" )
            self.XTSSWGS_given = 0

        if params_dict.get('XTSSWGD') != None:
            val =  params_dict['XTSSWGD']
            self.XTSSWGD     = self.MPRnb(val,""            ,"Power dependence of self.JTSSWGD on temperature" )
            self.XTSSWGD_given = 1
        else:
            self.XTSSWGD     = self.MPRnb(self.XTSSWGS   ,""            ,"Power dependence of self.JTSSWGD on temperature" )
            self.XTSSWGD_given = 0

        if params_dict.get('TNJTS') != None:
            val =  params_dict['TNJTS']
            self.TNJTS       = self.MPRnb(val,""            ,"Temperature coefficient for NJTS" )
            self.TNJTS_given = 1
        else:
            self.TNJTS       = self.MPRnb(0.0            ,""            ,"Temperature coefficient for NJTS" )
            self.TNJTS_given = 0

        if params_dict.get('TNJTSD') != None:
            val =  params_dict['TNJTSD']
            self.TNJTSD      = self.MPRnb(val,""            ,"Temperature coefficient for NJTSD" )
            self.TNJTSD_given = 1
        else:
            self.TNJTSD      = self.MPRnb(self.TNJTS     ,""            ,"Temperature coefficient for NJTSD" )
            self.TNJTSD_given = 0

        if params_dict.get('TNJTSSW') != None:
            val =  params_dict['TNJTSSW']
            self.TNJTSSW     = self.MPRnb(val,""            ,"Temperature coefficient for NJTSSW" )
            self.TNJTSSW_given = 1
        else:
            self.TNJTSSW     = self.MPRnb(0.0            ,""            ,"Temperature coefficient for NJTSSW" )
            self.TNJTSSW_given = 0

        if params_dict.get('TNJTSSWD') != None:
            val =  params_dict['TNJTSSWD']
            self.TNJTSSWD    = self.MPRnb(val,""            ,"Temperature coefficient for NJTSSWD" )
            self.TNJTSSWD_given = 1
        else:
            self.TNJTSSWD    = self.MPRnb(self.TNJTSSW   ,""            ,"Temperature coefficient for NJTSSWD" )
            self.TNJTSSWD_given = 0

        if params_dict.get('TNJTSSWG') != None:
            val =  params_dict['TNJTSSWG']
            self.TNJTSSWG    = self.MPRnb(val,""            ,"Temperature coefficient for NJTSSWG" )
            self.TNJTSSWG_given = 1
        else:
            self.TNJTSSWG    = self.MPRnb(0.0            ,""            ,"Temperature coefficient for NJTSSWG" )
            self.TNJTSSWG_given = 0

        if params_dict.get('TNJTSSWGD') != None:
            val =  params_dict['TNJTSSWGD']
            self.TNJTSSWGD   = self.MPRnb(val,""            ,"Temperature coefficient for NJTSSWGD" )
            self.TNJTSSWGD_given = 1
        else:
            self.TNJTSSWGD   = self.MPRnb(self.TNJTSSWG  ,""            ,"Temperature coefficient for NJTSSWGD" )
            self.TNJTSSWGD_given = 0


        # Self Heating
        if params_dict.get('RTH0') != None:
            val =  params_dict['RTH0']
            self.RTH0    = self.MPRcz(val,"ohm*m*K/W"   ,"Thermal resistance" )
            self.RTH0_given = 1
        else:
            self.RTH0    = self.MPRcz(1.0e-2         ,"ohm*m*K/W"   ,"Thermal resistance" )
            self.RTH0_given = 0

        if params_dict.get('CTH0') != None:
            val =  params_dict['CTH0']
            self.CTH0    = self.MPRcz(val,"W*s/m/K"     ,"Thermal capacitance" )
            self.CTH0_given = 1
        else:
            self.CTH0    = self.MPRcz(1.0e-5         ,"W*s/m/K"     ,"Thermal capacitance" )
            self.CTH0_given = 0

        if params_dict.get('WTH0') != None:
            val =  params_dict['WTH0']
            self.WTH0    = self.MPRcz(val,"m"           ,"Width dependence coefficient for Rth and Cth" )
            self.WTH0_given = 1
        else:
            self.WTH0    = self.MPRcz(0.0            ,"m"           ,"Width dependence coefficient for Rth and Cth" )
            self.WTH0_given = 0

        if params_dict.get('ASHEXP') != None:
            val =  params_dict['ASHEXP']
            self.ASHEXP  = self.MPRcz(val,""            ,"Exponent to tune RTH dependence of NFINTOTAL" )
            self.ASHEXP_given = 1
        else:
            self.ASHEXP  = self.MPRcz(1.0            ,""            ,"Exponent to tune RTH dependence of NFINTOTAL" )
            self.ASHEXP_given = 0

        if params_dict.get('BSHEXP') != None:
            val =  params_dict['BSHEXP']
            self.BSHEXP  = self.MPRcz(val,""            ,"Exponent to tune RTH dependence of NF" )
            self.BSHEXP_given = 1
        else:
            self.BSHEXP  = self.MPRcz(1.0            ,""            ,"Exponent to tune RTH dependence of NF" )
            self.BSHEXP_given = 0


        # Unified Model
        if params_dict.get('ACH_UFCM') != None:
            val =  params_dict['ACH_UFCM']
            self.ACH_UFCM    = self.MPRoz(val,"m^2"         ,"Area of the channel for the unified Model" )
            self.ACH_UFCM_given = 1
        else:
            self.ACH_UFCM    = self.MPRoz(1.0            ,"m^2"         ,"Area of the channel for the unified Model" )
            self.ACH_UFCM_given = 0

        if params_dict.get('CINS_UFCM') != None:
            val =  params_dict['CINS_UFCM']
            self.CINS_UFCM   = self.MPRoz(val,"F/m"         ,"Insulator capacitance for the unified Model" )
            self.CINS_UFCM_given = 1
        else:
            self.CINS_UFCM   = self.MPRoz(1.0            ,"F/m"         ,"Insulator capacitance for the unified Model" )
            self.CINS_UFCM_given = 0

        if params_dict.get('W_UFCM') != None:
            val =  params_dict['W_UFCM']
            self.W_UFCM      = self.MPRoz(val,"m"           ,"Effective channel width for the unified Model" )
            self.W_UFCM_given = 1
        else:
            self.W_UFCM      = self.MPRoz(1.0            ,"m"           ,"Effective channel width for the unified Model" )
            self.W_UFCM_given = 0

        if params_dict.get('TFIN_TOP') != None:
            val =  params_dict['TFIN_TOP']
            self.TFIN_TOP    = self.MPRcz(val,"m"           ,"Top body (fin) thickness for trapezoidal triple gate" )
            self.TFIN_TOP_given = 1
        else:
            self.TFIN_TOP    = self.MPRcz(1.5e-8         ,"m"           ,"Top body (fin) thickness for trapezoidal triple gate" )
            self.TFIN_TOP_given = 0

        if params_dict.get('TFIN_BASE') != None:
            val =  params_dict['TFIN_BASE']
            self.TFIN_BASE   = self.MPRco(val,"m"           ,1.0e-9      ,math.inf         ,"Base body (fin) thickness for trapezoidal triple gate" )
            self.TFIN_BASE_given = 1
        else:
            self.TFIN_BASE   = self.MPRco(1.5e-8         ,"m"           ,1.0e-9      ,math.inf         ,"Base body (fin) thickness for trapezoidal triple gate" )
            self.TFIN_BASE_given = 0

        if params_dict.get('QMFACTORCV') != None:
            val =  params_dict['QMFACTORCV']
            self.QMFACTORCV  = self.MPRcz(val,""            ,"Charge dependence taking QM effects into account" )
            self.QMFACTORCV_given = 1
        else:
            self.QMFACTORCV  = self.MPRcz(0.0            ,""            ,"Charge dependence taking QM effects into account" )
            self.QMFACTORCV_given = 0

        if params_dict.get('ALPHA_UFCM') != None:
            val =  params_dict['ALPHA_UFCM']
            self.ALPHA_UFCM  = self.MPRcz(val,""            ,"Mobile charge scaling term taking QM effects into account" )
            self.ALPHA_UFCM_given = 1
        else:
            self.ALPHA_UFCM  = self.MPRcz(0.5556         ,""            ,"Mobile charge scaling term taking QM effects into account" )
            self.ALPHA_UFCM_given = 0

        # Binning Parameters
        if params_dict.get('LNBODY') != None:
            val =  params_dict['LNBODY']
            self.LNBODY       = self.MPRnb(val,"m^-2"                  ,"" )
            self.LNBODY_given = 1
        else:
            self.LNBODY       = self.MPRnb(0.0            ,"m^-2"                  ,"" )
            self.LNBODY_given = 0

        if params_dict.get('NNBODY') != None:
            val =  params_dict['NNBODY']
            self.NNBODY       = self.MPRnb(val,"m^-2"                  ,"" )
            self.NNBODY_given = 1
        else:
            self.NNBODY       = self.MPRnb(0.0            ,"m^-2"                  ,"" )
            self.NNBODY_given = 0

        if params_dict.get('PNBODY') != None:
            val =  params_dict['PNBODY']
            self.PNBODY       = self.MPRnb(val,"m^-1"                  ,"" )
            self.PNBODY_given = 1
        else:
            self.PNBODY       = self.MPRnb(0.0            ,"m^-1"                  ,"" )
            self.PNBODY_given = 0

        if params_dict.get('LPHIG') != None:
            val =  params_dict['LPHIG']
            self.LPHIG        = self.MPRnb(val,"m*eV"                  ,"" )
            self.LPHIG_given = 1
        else:
            self.LPHIG        = self.MPRnb(0.0            ,"m*eV"                  ,"" )
            self.LPHIG_given = 0

        if params_dict.get('NPHIG') != None:
            val =  params_dict['NPHIG']
            self.NPHIG        = self.MPRnb(val,"m*eV"                  ,"" )
            self.NPHIG_given = 1
        else:
            self.NPHIG        = self.MPRnb(0.0            ,"m*eV"                  ,"" )
            self.NPHIG_given = 0

        if params_dict.get('PPHIG') != None:
            val =  params_dict['PPHIG']
            self.PPHIG        = self.MPRnb(val,"(m^2)*eV"              ,"" )
            self.PPHIG_given = 1
        else:
            self.PPHIG        = self.MPRnb(0.0            ,"(m^2)*eV"              ,"" )
            self.PPHIG_given = 0

        if params_dict.get('LNGATE') != None:
            val =  params_dict['LNGATE']
            self.LNGATE       = self.MPRnb(val,"m^-2"                  ,"" )
            self.LNGATE_given = 1
        else:
            self.LNGATE       = self.MPRnb(0.0            ,"m^-2"                  ,"" )
            self.LNGATE_given = 0

        if params_dict.get('NNGATE') != None:
            val =  params_dict['NNGATE']
            self.NNGATE       = self.MPRnb(val,"m^-2"                  ,"" )
            self.NNGATE_given = 1
        else:
            self.NNGATE       = self.MPRnb(0.0            ,"m^-2"                  ,"" )
            self.NNGATE_given = 0

        if params_dict.get('PNGATE') != None:
            val =  params_dict['PNGATE']
            self.PNGATE       = self.MPRnb(val,"m^-1"                  ,"" )
            self.PNGATE_given = 1
        else:
            self.PNGATE       = self.MPRnb(0.0            ,"m^-1"                  ,"" )
            self.PNGATE_given = 0

        if params_dict.get('LCIT') != None:
            val =  params_dict['LCIT']
            self.LCIT         = self.MPRnb(val,"F/m"                   ,"" )
            self.LCIT_given = 1
        else:
            self.LCIT         = self.MPRnb(0.0            ,"F/m"                   ,"" )
            self.LCIT_given = 0

        if params_dict.get('NCIT') != None:
            val =  params_dict['NCIT']
            self.NCIT         = self.MPRnb(val,"F/m"                   ,"" )
            self.NCIT_given = 1
        else:
            self.NCIT         = self.MPRnb(0.0            ,"F/m"                   ,"" )
            self.NCIT_given = 0

        if params_dict.get('PCIT') != None:
            val =  params_dict['PCIT']
            self.PCIT         = self.MPRnb(val,"F"                     ,"" )
            self.PCIT_given = 1
        else:
            self.PCIT         = self.MPRnb(0.0            ,"F"                     ,"" )
            self.PCIT_given = 0

        if params_dict.get('LCITR') != None:
            val =  params_dict['LCITR']
            self.LCITR        = self.MPRnb(val,""                      ,"" )
            self.LCITR_given = 1
        else:
            self.LCITR        = self.MPRnb(self.LCIT           ,""                      ,"" )
            self.LCITR_given = 0

        if params_dict.get('NCITR') != None:
            val =  params_dict['NCITR']
            self.NCITR        = self.MPRnb(val,""                      ,"" )
            self.NCITR_given = 1
        else:
            self.NCITR        = self.MPRnb(self.NCIT           ,""                      ,"" )
            self.NCITR_given = 0

        if params_dict.get('PCITR') != None:
            val =  params_dict['PCITR']
            self.PCITR        = self.MPRnb(val,""                      ,"" )
            self.PCITR_given = 1
        else:
            self.PCITR        = self.MPRnb(self.PCIT           ,""                      ,"" )
            self.PCITR_given = 0

        if params_dict.get('LCDSC') != None:
            val =  params_dict['LCDSC']
            self.LCDSC        = self.MPRnb(val,"F/m"                   ,"" )
            self.LCDSC_given = 1
        else:
            self.LCDSC        = self.MPRnb(0.0            ,"F/m"                   ,"" )
            self.LCDSC_given = 0

        if params_dict.get('NCDSC') != None:
            val =  params_dict['NCDSC']
            self.NCDSC        = self.MPRnb(val,"F/m"                   ,"" )
            self.NCDSC_given = 1
        else:
            self.NCDSC        = self.MPRnb(0.0            ,"F/m"                   ,"" )
            self.NCDSC_given = 0

        if params_dict.get('PCDSC') != None:
            val =  params_dict['PCDSC']
            self.PCDSC        = self.MPRnb(val,"F"                     ,"" )
            self.PCDSC_given = 1
        else:
            self.PCDSC        = self.MPRnb(0.0            ,"F"                     ,"" )
            self.PCDSC_given = 0

        if params_dict.get('LCDSCD') != None:
            val =  params_dict['LCDSCD']
            self.LCDSCD       = self.MPRnb(val,"F/m"                   ,"" )
            self.LCDSCD_given = 1
        else:
            self.LCDSCD       = self.MPRnb(0.0            ,"F/m"                   ,"" )
            self.LCDSCD_given = 0

        if params_dict.get('NCDSCD') != None:
            val =  params_dict['NCDSCD']
            self.NCDSCD       = self.MPRnb(val,"F/m"                   ,"" )
            self.NCDSCD_given = 1
        else:
            self.NCDSCD       = self.MPRnb(0.0            ,"F/m"                   ,"" )
            self.NCDSCD_given = 0

        if params_dict.get('PCDSCD') != None:
            val =  params_dict['PCDSCD']
            self.PCDSCD       = self.MPRnb(val,"F"                     ,"" )
            self.PCDSCD_given = 1
        else:
            self.PCDSCD       = self.MPRnb(0.0            ,"F"                     ,"" )
            self.PCDSCD_given = 0

        if params_dict.get('LCDSCDR') != None:
            val =  params_dict['LCDSCDR']
            self.LCDSCDR      = self.MPRnb(val,"F/m"                   ,"" )
            self.LCDSCDR_given = 1
        else:
            self.LCDSCDR      = self.MPRnb(self.LCDSCD         ,"F/m"                   ,"" )
            self.LCDSCDR_given = 0

        if params_dict.get('NCDSCDR') != None:
            val =  params_dict['NCDSCDR']
            self.NCDSCDR      = self.MPRnb(val,"F/m"                   ,"" )
            self.NCDSCDR_given = 1
        else:
            self.NCDSCDR      = self.MPRnb(self.NCDSCD         ,"F/m"                   ,"" )
            self.NCDSCDR_given = 0

        if params_dict.get('PCDSCDR') != None:
            val =  params_dict['PCDSCDR']
            self.PCDSCDR      = self.MPRnb(val,"F"                     ,"" )
            self.PCDSCDR_given = 1
        else:
            self.PCDSCDR      = self.MPRnb(self.PCDSCD         ,"F"                     ,"" )
            self.PCDSCDR_given = 0

        if params_dict.get('LDVT0') != None:
            val =  params_dict['LDVT0']
            self.LDVT0        = self.MPRnb(val,""                      ,"" )
            self.LDVT0_given = 1
        else:
            self.LDVT0        = self.MPRnb(0.0            ,""                      ,"" )
            self.LDVT0_given = 0

        if params_dict.get('NDVT0') != None:
            val =  params_dict['NDVT0']
            self.NDVT0        = self.MPRnb(val,""                      ,"" )
            self.NDVT0_given = 1
        else:
            self.NDVT0        = self.MPRnb(0.0            ,""                      ,"" )
            self.NDVT0_given = 0

        if params_dict.get('PDVT0') != None:
            val =  params_dict['PDVT0']
            self.PDVT0        = self.MPRnb(val,""                      ,"" )
            self.PDVT0_given = 1
        else:
            self.PDVT0        = self.MPRnb(0.0            ,""                      ,"" )
            self.PDVT0_given = 0

        if params_dict.get('LDVT1') != None:
            val =  params_dict['LDVT1']
            self.LDVT1        = self.MPRnb(val,""                      ,"" )
            self.LDVT1_given = 1
        else:
            self.LDVT1        = self.MPRnb(0.0            ,""                      ,"" )
            self.LDVT1_given = 0

        if params_dict.get('NDVT1') != None:
            val =  params_dict['NDVT1']
            self.NDVT1        = self.MPRnb(val,""                      ,"" )
            self.NDVT1_given = 1
        else:
            self.NDVT1        = self.MPRnb(0.0            ,""                      ,"" )
            self.NDVT1_given = 0

        if params_dict.get('PDVT1') != None:
            val =  params_dict['PDVT1']
            self.PDVT1        = self.MPRnb(val,""                      ,"" )
            self.PDVT1_given = 1
        else:
            self.PDVT1        = self.MPRnb(0.0            ,""                      ,"" )
            self.PDVT1_given = 0

        if params_dict.get('LDVT1SS') != None:
            val =  params_dict['LDVT1SS']
            self.LDVT1SS      = self.MPRnb(val,""                      ,"" )
            self.LDVT1SS_given = 1
        else:
            self.LDVT1SS      = self.MPRnb(self.LDVT1          ,""                      ,"" )
            self.LDVT1SS_given = 0

        if params_dict.get('NDVT1SS') != None:
            val =  params_dict['NDVT1SS']
            self.NDVT1SS      = self.MPRnb(val,""                      ,"" )
            self.NDVT1SS_given = 1
        else:
            self.NDVT1SS      = self.MPRnb(self.NDVT1          ,""                      ,"" )
            self.NDVT1SS_given = 0

        if params_dict.get('PDVT1SS') != None:
            val =  params_dict['PDVT1SS']
            self.PDVT1SS      = self.MPRnb(val,""                      ,"" )
            self.PDVT1SS_given = 1
        else:
            self.PDVT1SS      = self.MPRnb(self.PDVT1          ,""                      ,"" )
            self.PDVT1SS_given = 0

        if params_dict.get('LPHIN') != None:
            val =  params_dict['LPHIN']
            self.LPHIN        = self.MPRnb(val,"m*V"                   ,"" )
            self.LPHIN_given = 1
        else:
            self.LPHIN        = self.MPRnb(0.0            ,"m*V"                   ,"" )
            self.LPHIN_given = 0

        if params_dict.get('NPHIN') != None:
            val =  params_dict['NPHIN']
            self.NPHIN        = self.MPRnb(val,"m*V"                   ,"" )
            self.NPHIN_given = 1
        else:
            self.NPHIN        = self.MPRnb(0.0            ,"m*V"                   ,"" )
            self.NPHIN_given = 0

        if params_dict.get('PPHIN') != None:
            val =  params_dict['PPHIN']
            self.PPHIN        = self.MPRnb(val,"(m^2)*V"               ,"" )
            self.PPHIN_given = 1
        else:
            self.PPHIN        = self.MPRnb(0.0            ,"(m^2)*V"               ,"" )
            self.PPHIN_given = 0

        if params_dict.get('LETA0') != None:
            val =  params_dict['LETA0']
            self.LETA0        = self.MPRnb(val,""                      ,"" )
            self.LETA0_given = 1
        else:
            self.LETA0        = self.MPRnb(0.0            ,""                      ,"" )
            self.LETA0_given = 0

        if params_dict.get('NETA0') != None:
            val =  params_dict['NETA0']
            self.NETA0        = self.MPRnb(val,""                      ,"" )
            self.NETA0_given = 1
        else:
            self.NETA0        = self.MPRnb(0.0            ,""                      ,"" )
            self.NETA0_given = 0

        if params_dict.get('PETA0') != None:
            val =  params_dict['PETA0']
            self.PETA0        = self.MPRnb(val,""                      ,"" )
            self.PETA0_given = 1
        else:
            self.PETA0        = self.MPRnb(0.0            ,""                      ,"" )
            self.PETA0_given = 0

        if params_dict.get('LETA0R') != None:
            val =  params_dict['LETA0R']
            self.LETA0R       = self.MPRnb(val,""                      ,"" )
            self.LETA0R_given = 1
        else:
            self.LETA0R       = self.MPRnb(self.LETA0          ,""                      ,"" )
            self.LETA0R_given = 0

        if params_dict.get('NETA0R') != None:
            val =  params_dict['NETA0R']
            self.NETA0R       = self.MPRnb(val,""                      ,"" )
            self.NETA0R_given = 1
        else:
            self.NETA0R       = self.MPRnb(self.NETA0          ,""                      ,"" )
            self.NETA0R_given = 0

        if params_dict.get('PETA0R') != None:
            val =  params_dict['PETA0R']
            self.PETA0R       = self.MPRnb(val,""                      ,"" )
            self.PETA0R_given = 1
        else:
            self.PETA0R       = self.MPRnb(self.PETA0          ,""                      ,"" )
            self.PETA0R_given = 0

        if params_dict.get('LDSUB') != None:
            val =  params_dict['LDSUB']
            self.LDSUB        = self.MPRnb(val,""                      ,"" )
            self.LDSUB_given = 1
        else:
            self.LDSUB        = self.MPRnb(0.0            ,""                      ,"" )
            self.LDSUB_given = 0

        if params_dict.get('NDSUB') != None:
            val =  params_dict['NDSUB']
            self.NDSUB        = self.MPRnb(val,""                      ,"" )
            self.NDSUB_given = 1
        else:
            self.NDSUB        = self.MPRnb(0.0            ,""                      ,"" )
            self.NDSUB_given = 0

        if params_dict.get('PDSUB') != None:
            val =  params_dict['PDSUB']
            self.PDSUB        = self.MPRnb(val,""                      ,"" )
            self.PDSUB_given = 1
        else:
            self.PDSUB        = self.MPRnb(0.0            ,""                      ,"" )
            self.PDSUB_given = 0

        if params_dict.get('LK1RSCE') != None:
            val =  params_dict['LK1RSCE']
            self.LK1RSCE      = self.MPRnb(val,"m*V^(1/2)"             ,"" )
            self.LK1RSCE_given = 1
        else:
            self.LK1RSCE      = self.MPRnb(0.0            ,"m*V^(1/2)"             ,"" )
            self.LK1RSCE_given = 0

        if params_dict.get('NK1RSCE') != None:
            val =  params_dict['NK1RSCE']
            self.NK1RSCE      = self.MPRnb(val,"m*V^(1/2)"             ,"" )
            self.NK1RSCE_given = 1
        else:
            self.NK1RSCE      = self.MPRnb(0.0            ,"m*V^(1/2)"             ,"" )
            self.NK1RSCE_given = 0

        if params_dict.get('PK1RSCE') != None:
            val =  params_dict['PK1RSCE']
            self.PK1RSCE      = self.MPRnb(val,"(m^2)*V^(1/2)"         ,"" )
            self.PK1RSCE_given = 1
        else:
            self.PK1RSCE      = self.MPRnb(0.0            ,"(m^2)*V^(1/2)"         ,"" )
            self.PK1RSCE_given = 0

        if params_dict.get('LLPE0') != None:
            val =  params_dict['LLPE0']
            self.LLPE0        = self.MPRnb(val,"m^2"                   ,"" )
            self.LLPE0_given = 1
        else:
            self.LLPE0        = self.MPRnb(0.0            ,"m^2"                   ,"" )
            self.LLPE0_given = 0

        if params_dict.get('NLPE0') != None:
            val =  params_dict['NLPE0']
            self.NLPE0        = self.MPRnb(val,"m^2"                   ,"" )
            self.NLPE0_given = 1
        else:
            self.NLPE0        = self.MPRnb(0.0            ,"m^2"                   ,"" )
            self.NLPE0_given = 0

        if params_dict.get('PLPE0') != None:
            val =  params_dict['PLPE0']
            self.PLPE0        = self.MPRnb(val,"m^3"                   ,"" )
            self.PLPE0_given = 1
        else:
            self.PLPE0        = self.MPRnb(0.0            ,"m^3"                   ,"" )
            self.PLPE0_given = 0

        if params_dict.get('LDVTSHIFT') != None:
            val =  params_dict['LDVTSHIFT']
            self.LDVTSHIFT    = self.MPRnb(val,"m*V"                   ,"" )
            self.LDVTSHIFT_given = 1
        else:
            self.LDVTSHIFT    = self.MPRnb(0.0            ,"m*V"                   ,"" )
            self.LDVTSHIFT_given = 0

        if params_dict.get('NDVTSHIFT') != None:
            val =  params_dict['NDVTSHIFT']
            self.NDVTSHIFT    = self.MPRnb(val,"m*V"                   ,"" )
            self.NDVTSHIFT_given = 1
        else:
            self.NDVTSHIFT    = self.MPRnb(0.0            ,"m*V"                   ,"" )
            self.NDVTSHIFT_given = 0

        if params_dict.get('PDVTSHIFT') != None:
            val =  params_dict['PDVTSHIFT']
            self.PDVTSHIFT    = self.MPRnb(val,"(m^2)*V"               ,"" )
            self.PDVTSHIFT_given = 1
        else:
            self.PDVTSHIFT    = self.MPRnb(0.0            ,"(m^2)*V"               ,"" )
            self.PDVTSHIFT_given = 0

        if params_dict.get('LDVTSHIFTR') != None:
            val =  params_dict['LDVTSHIFTR']
            self.LDVTSHIFTR   = self.MPRnb(val,""                      ,"" )
            self.LDVTSHIFTR_given = 1
        else:
            self.LDVTSHIFTR   = self.MPRnb(self.LDVTSHIFT      ,""                      ,"" )
            self.LDVTSHIFTR_given = 0

        if params_dict.get('NDVTSHIFTR') != None:
            val =  params_dict['NDVTSHIFTR']
            self.NDVTSHIFTR   = self.MPRnb(val,""                      ,"" )
            self.NDVTSHIFTR_given = 1
        else:
            self.NDVTSHIFTR   = self.MPRnb(self.NDVTSHIFT      ,""                      ,"" )
            self.NDVTSHIFTR_given = 0

        if params_dict.get('PDVTSHIFTR') != None:
            val =  params_dict['PDVTSHIFTR']
            self.PDVTSHIFTR   = self.MPRnb(val,""                      ,"" )
            self.PDVTSHIFTR_given = 1
        else:
            self.PDVTSHIFTR   = self.MPRnb(self.PDVTSHIFT      ,""                      ,"" )
            self.PDVTSHIFTR_given = 0

        if params_dict.get('LPHIBE') != None:
            val =  params_dict['LPHIBE']
            self.LPHIBE       = self.MPRnb(val,"m*V"                   ,"" )
            self.LPHIBE_given = 1
        else:
            self.LPHIBE       = self.MPRnb(0.0            ,"m*V"                   ,"" )
            self.LPHIBE_given = 0

        if params_dict.get('NPHIBE') != None:
            val =  params_dict['NPHIBE']
            self.NPHIBE       = self.MPRnb(val,"m*V"                   ,"" )
            self.NPHIBE_given = 1
        else:
            self.NPHIBE       = self.MPRnb(0.0            ,"m*V"                   ,"" )
            self.NPHIBE_given = 0

        if params_dict.get('PPHIBE') != None:
            val =  params_dict['PPHIBE']
            self.PPHIBE       = self.MPRnb(val,"(m^2)*V"               ,"" )
            self.PPHIBE_given = 1
        else:
            self.PPHIBE       = self.MPRnb(0.0            ,"(m^2)*V"               ,"" )
            self.PPHIBE_given = 0

        if params_dict.get('LK0') != None:
            val =  params_dict['LK0']
            self.LK0          = self.MPRnb(val,"m*V"                   ,"" )
            self.LK0_given = 1
        else:
            self.LK0          = self.MPRnb(0.0            ,"m*V"                   ,"" )
            self.LK0_given = 0

        if params_dict.get('NK0') != None:
            val =  params_dict['NK0']
            self.NK0          = self.MPRnb(val,"m*V"                   ,"" )
            self.NK0_given = 1
        else:
            self.NK0          = self.MPRnb(0.0            ,"m*V"                   ,"" )
            self.NK0_given = 0

        if params_dict.get('PK0') != None:
            val =  params_dict['PK0']
            self.PK0          = self.MPRnb(val,"(m^2)*V"               ,"" )
            self.PK0_given = 1
        else:
            self.PK0          = self.MPRnb(0.0            ,"(m^2)*V"               ,"" )
            self.PK0_given = 0

        if params_dict.get('LK01') != None:
            val =  params_dict['LK01']
            self.LK01         = self.MPRnb(val,"(m*V)/K"               ,"" )
            self.LK01_given = 1
        else:
            self.LK01         = self.MPRnb(0.0            ,"(m*V)/K"               ,"" )
            self.LK01_given = 0

        if params_dict.get('NK01') != None:
            val =  params_dict['NK01']
            self.NK01         = self.MPRnb(val,"(m*V)/K"               ,"" )
            self.NK01_given = 1
        else:
            self.NK01         = self.MPRnb(0.0            ,"(m*V)/K"               ,"" )
            self.NK01_given = 0

        if params_dict.get('PK01') != None:
            val =  params_dict['PK01']
            self.PK01         = self.MPRnb(val,"(m^2*V)/K"             ,"" )
            self.PK01_given = 1
        else:
            self.PK01         = self.MPRnb(0.0            ,"(m^2*V)/K"             ,"" )
            self.PK01_given = 0

        if params_dict.get('LK0SI') != None:
            val =  params_dict['LK0SI']
            self.LK0SI        = self.MPRnb(val,""                      ,"" )
            self.LK0SI_given = 1
        else:
            self.LK0SI        = self.MPRnb(0.0            ,""                      ,"" )
            self.LK0SI_given = 0

        if params_dict.get('NK0SI') != None:
            val =  params_dict['NK0SI']
            self.NK0SI        = self.MPRnb(val,""                      ,"" )
            self.NK0SI_given = 1
        else:
            self.NK0SI        = self.MPRnb(0.0            ,""                      ,"" )
            self.NK0SI_given = 0

        if params_dict.get('PK0SI') != None:
            val =  params_dict['PK0SI']
            self.PK0SI        = self.MPRnb(val,""                      ,"" )
            self.PK0SI_given = 1
        else:
            self.PK0SI        = self.MPRnb(0.0            ,""                      ,"" )
            self.PK0SI_given = 0

        if params_dict.get('LK0SI1') != None:
            val =  params_dict['LK0SI1']
            self.LK0SI1       = self.MPRnb(val,"m/K"                   ,"" )
            self.LK0SI1_given = 1
        else:
            self.LK0SI1       = self.MPRnb(0.0            ,"m/K"                   ,"" )
            self.LK0SI1_given = 0

        if params_dict.get('NK0SI1') != None:
            val =  params_dict['NK0SI1']
            self.NK0SI1       = self.MPRnb(val,"m/K"                   ,"" )
            self.NK0SI1_given = 1
        else:
            self.NK0SI1       = self.MPRnb(0.0            ,"m/K"                   ,"" )
            self.NK0SI1_given = 0

        if params_dict.get('PK0SI1') != None:
            val =  params_dict['PK0SI1']
            self.PK0SI1       = self.MPRnb(val,"(m^2)/K"               ,"" )
            self.PK0SI1_given = 1
        else:
            self.PK0SI1       = self.MPRnb(0.0            ,"(m^2)/K"               ,"" )
            self.PK0SI1_given = 0

        if params_dict.get('LK1') != None:
            val =  params_dict['LK1']
            self.LK1          = self.MPRnb(val,"m*V^(1/2)"             ,"" )
            self.LK1_given = 1
        else:
            self.LK1          = self.MPRnb(0.0            ,"m*V^(1/2)"             ,"" )
            self.LK1_given = 0

        if params_dict.get('NK1') != None:
            val =  params_dict['NK1']
            self.NK1          = self.MPRnb(val,"m*V^(1/2)"             ,"" )
            self.NK1_given = 1
        else:
            self.NK1          = self.MPRnb(0.0            ,"m*V^(1/2)"             ,"" )
            self.NK1_given = 0

        if params_dict.get('PK1') != None:
            val =  params_dict['PK1']
            self.PK1          = self.MPRnb(val,"(m^2)*V^(1/2)"         ,"" )
            self.PK1_given = 1
        else:
            self.PK1          = self.MPRnb(0.0            ,"(m^2)*V^(1/2)"         ,"" )
            self.PK1_given = 0

        if params_dict.get('LK11') != None:
            val =  params_dict['LK11']
            self.LK11         = self.MPRnb(val,"(m*V^(-1/2))/K"        ,"" )
            self.LK11_given = 1
        else:
            self.LK11         = self.MPRnb(0.0            ,"(m*V^(-1/2))/K"        ,"" )
            self.LK11_given = 0

        if params_dict.get('NK11') != None:
            val =  params_dict['NK11']
            self.NK11         = self.MPRnb(val,"(m*V^(-1/2))/K"        ,"" )
            self.NK11_given = 1
        else:
            self.NK11         = self.MPRnb(0.0            ,"(m*V^(-1/2))/K"        ,"" )
            self.NK11_given = 0

        if params_dict.get('PK11') != None:
            val =  params_dict['PK11']
            self.PK11         = self.MPRnb(val,"(m^2*V^(-1/2))/K"      ,"" )
            self.PK11_given = 1
        else:
            self.PK11         = self.MPRnb(0.0            ,"(m^2*V^(-1/2))/K"      ,"" )
            self.PK11_given = 0

        if params_dict.get('LK2SI') != None:
            val =  params_dict['LK2SI']
            self.LK2SI        = self.MPRnb(val,""                      ,"" )
            self.LK2SI_given = 1
        else:
            self.LK2SI        = self.MPRnb(self.LK0SI          ,""                      ,"" )
            self.LK2SI_given = 0

        if params_dict.get('NK2SI') != None:
            val =  params_dict['NK2SI']
            self.NK2SI        = self.MPRnb(val,""                      ,"" )
            self.NK2SI_given = 1
        else:
            self.NK2SI        = self.MPRnb(self.NK0SI          ,""                      ,"" )
            self.NK2SI_given = 0

        if params_dict.get('PK2SI') != None:
            val =  params_dict['PK2SI']
            self.PK2SI        = self.MPRnb(val,""                      ,"" )
            self.PK2SI_given = 1
        else:
            self.PK2SI        = self.MPRnb(self.PK0SI          ,""                      ,"" )
            self.PK2SI_given = 0

        if params_dict.get('LK2SI1') != None:
            val =  params_dict['LK2SI1']
            self.LK2SI1       = self.MPRnb(val,""                      ,"" )
            self.LK2SI1_given = 1
        else:
            self.LK2SI1       = self.MPRnb(self.LK0SI1         ,""                      ,"" )
            self.LK2SI1_given = 0

        if params_dict.get('NK2SI1') != None:
            val =  params_dict['NK2SI1']
            self.NK2SI1       = self.MPRnb(val,""                      ,"" )
            self.NK2SI1_given = 1
        else:
            self.NK2SI1       = self.MPRnb(self.NK0SI1         ,""                      ,"" )
            self.NK2SI1_given = 0

        if params_dict.get('PK2SI1') != None:
            val =  params_dict['PK2SI1']
            self.PK2SI1       = self.MPRnb(val,""                      ,"" )
            self.PK2SI1_given = 1
        else:
            self.PK2SI1       = self.MPRnb(self.PK0SI1         ,""                      ,"" )
            self.PK2SI1_given = 0

        if params_dict.get('LK0SISAT') != None:
            val =  params_dict['LK0SISAT']
            self.LK0SISAT     = self.MPRnb(val,""                      ,"" )
            self.LK0SISAT_given = 1
        else:
            self.LK0SISAT     = self.MPRnb(0.0            ,""                      ,"" )
            self.LK0SISAT_given = 0

        if params_dict.get('NK0SISAT') != None:
            val =  params_dict['NK0SISAT']
            self.NK0SISAT     = self.MPRnb(val,""                      ,"" )
            self.NK0SISAT_given = 1
        else:
            self.NK0SISAT     = self.MPRnb(0.0            ,""                      ,"" )
            self.NK0SISAT_given = 0

        if params_dict.get('PK0SISAT') != None:
            val =  params_dict['PK0SISAT']
            self.PK0SISAT     = self.MPRnb(val,""                      ,"" )
            self.PK0SISAT_given = 1
        else:
            self.PK0SISAT     = self.MPRnb(0.0            ,""                      ,"" )
            self.PK0SISAT_given = 0

        if params_dict.get('LK0SISAT1') != None:
            val =  params_dict['LK0SISAT1']
            self.LK0SISAT1    = self.MPRnb(val,""                      ,"" )
            self.LK0SISAT1_given = 1
        else:
            self.LK0SISAT1    = self.MPRnb(0.0            ,""                      ,"" )
            self.LK0SISAT1_given = 0

        if params_dict.get('NK0SISAT1') != None:
            val =  params_dict['NK0SISAT1']
            self.NK0SISAT1    = self.MPRnb(val,""                      ,"" )
            self.NK0SISAT1_given = 1
        else:
            self.NK0SISAT1    = self.MPRnb(0.0            ,""                      ,"" )
            self.NK0SISAT1_given = 0

        if params_dict.get('PK0SISAT1') != None:
            val =  params_dict['PK0SISAT1']
            self.PK0SISAT1    = self.MPRnb(val,""                      ,"" )
            self.PK0SISAT1_given = 1
        else:
            self.PK0SISAT1    = self.MPRnb(0.0            ,""                      ,"" )
            self.PK0SISAT1_given = 0

        if params_dict.get('LK2SISAT') != None:
            val =  params_dict['LK2SISAT']
            self.LK2SISAT     = self.MPRnb(val,""                      ,"" )
            self.LK2SISAT_given = 1
        else:
            self.LK2SISAT     = self.MPRnb(self.LK0SISAT       ,""                      ,"" )
            self.LK2SISAT_given = 0

        if params_dict.get('NK2SISAT') != None:
            val =  params_dict['NK2SISAT']
            self.NK2SISAT     = self.MPRnb(val,""                      ,"" )
            self.NK2SISAT_given = 1
        else:
            self.NK2SISAT     = self.MPRnb(self.NK0SISAT       ,""                      ,"" )
            self.NK2SISAT_given = 0

        if params_dict.get('PK2SISAT') != None:
            val =  params_dict['PK2SISAT']
            self.PK2SISAT     = self.MPRnb(val,""                      ,"" )
            self.PK2SISAT_given = 1
        else:
            self.PK2SISAT     = self.MPRnb(self.PK0SISAT       ,""                      ,"" )
            self.PK2SISAT_given = 0

        if params_dict.get('LK2SISAT1') != None:
            val =  params_dict['LK2SISAT1']
            self.LK2SISAT1    = self.MPRnb(val,""                      ,"" )
            self.LK2SISAT1_given = 1
        else:
            self.LK2SISAT1    = self.MPRnb(self.LK0SISAT1      ,""                      ,"" )
            self.LK2SISAT1_given = 0

        if params_dict.get('NK2SISAT1') != None:
            val =  params_dict['NK2SISAT1']
            self.NK2SISAT1    = self.MPRnb(val,""                      ,"" )
            self.NK2SISAT1_given = 1
        else:
            self.NK2SISAT1    = self.MPRnb(self.NK0SISAT1      ,""                      ,"" )
            self.NK2SISAT1_given = 0

        if params_dict.get('PK2SISAT1') != None:
            val =  params_dict['PK2SISAT1']
            self.PK2SISAT1    = self.MPRnb(val,""                      ,"" )
            self.PK2SISAT1_given = 1
        else:
            self.PK2SISAT1    = self.MPRnb(self.PK0SISAT1      ,""                      ,"" )
            self.PK2SISAT1_given = 0

        if params_dict.get('LK2SAT') != None:
            val =  params_dict['LK2SAT']
            self.LK2SAT       = self.MPRnb(val,""                      ,"" )
            self.LK2SAT_given = 1
        else:
            self.LK2SAT       = self.MPRnb(0.0            ,""                      ,"" )
            self.LK2SAT_given = 0

        if params_dict.get('NK2SAT') != None:
            val =  params_dict['NK2SAT']
            self.NK2SAT       = self.MPRnb(val,""                      ,"" )
            self.NK2SAT_given = 1
        else:
            self.NK2SAT       = self.MPRnb(0.0            ,""                      ,"" )
            self.NK2SAT_given = 0

        if params_dict.get('PK2SAT') != None:
            val =  params_dict['PK2SAT']
            self.PK2SAT       = self.MPRnb(val,""                      ,"" )
            self.PK2SAT_given = 1
        else:
            self.PK2SAT       = self.MPRnb(0.0            ,""                      ,"" )
            self.PK2SAT_given = 0

        if params_dict.get('LK2SAT1') != None:
            val =  params_dict['LK2SAT1']
            self.LK2SAT1      = self.MPRnb(val,""                      ,"" )
            self.LK2SAT1_given = 1
        else:
            self.LK2SAT1      = self.MPRnb(0.0            ,""                      ,"" )
            self.LK2SAT1_given = 0

        if params_dict.get('NK2SAT1') != None:
            val =  params_dict['NK2SAT1']
            self.NK2SAT1      = self.MPRnb(val,""                      ,"" )
            self.NK2SAT1_given = 1
        else:
            self.NK2SAT1      = self.MPRnb(0.0            ,""                      ,"" )
            self.NK2SAT1_given = 0

        if params_dict.get('PK2SAT1') != None:
            val =  params_dict['PK2SAT1']
            self.PK2SAT1      = self.MPRnb(val,""                      ,"" )
            self.PK2SAT1_given = 1
        else:
            self.PK2SAT1      = self.MPRnb(0.0            ,""                      ,"" )
            self.PK2SAT1_given = 0

        if params_dict.get('LK2') != None:
            val =  params_dict['LK2']
            self.LK2          = self.MPRnb(val,""                      ,"" )
            self.LK2_given = 1
        else:
            self.LK2          = self.MPRnb(0.0            ,""                      ,"" )
            self.LK2_given = 0

        if params_dict.get('NK2') != None:
            val =  params_dict['NK2']
            self.NK2          = self.MPRnb(val,""                      ,"" )
            self.NK2_given = 1
        else:
            self.NK2          = self.MPRnb(0.0            ,""                      ,"" )
            self.NK2_given = 0

        if params_dict.get('PK2') != None:
            val =  params_dict['PK2']
            self.PK2          = self.MPRnb(val,""                      ,"" )
            self.PK2_given = 1
        else:
            self.PK2          = self.MPRnb(0.0            ,""                      ,"" )
            self.PK2_given = 0

        if params_dict.get('LK21') != None:
            val =  params_dict['LK21']
            self.LK21         = self.MPRnb(val,""                      ,"" )
            self.LK21_given = 1
        else:
            self.LK21         = self.MPRnb(0.0            ,""                      ,"" )
            self.LK21_given = 0

        if params_dict.get('NK21') != None:
            val =  params_dict['NK21']
            self.NK21         = self.MPRnb(val,""                      ,"" )
            self.NK21_given = 1
        else:
            self.NK21         = self.MPRnb(0.0            ,""                      ,"" )
            self.NK21_given = 0

        if params_dict.get('PK21') != None:
            val =  params_dict['PK21']
            self.PK21         = self.MPRnb(val,""                      ,"" )
            self.PK21_given = 1
        else:
            self.PK21         = self.MPRnb(0.0            ,""                      ,"" )
            self.PK21_given = 0

        if params_dict.get('LDVTB') != None:
            val =  params_dict['LDVTB']
            self.LDVTB        = self.MPRnb(val,""                      ,"" )
            self.LDVTB_given = 1
        else:
            self.LDVTB        = self.MPRnb(0.0            ,""                      ,"" )
            self.LDVTB_given = 0

        if params_dict.get('NDVTB') != None:
            val =  params_dict['NDVTB']
            self.NDVTB        = self.MPRnb(val,""                      ,"" )
            self.NDVTB_given = 1
        else:
            self.NDVTB        = self.MPRnb(0.0            ,""                      ,"" )
            self.NDVTB_given = 0

        if params_dict.get('PDVTB') != None:
            val =  params_dict['PDVTB']
            self.PDVTB        = self.MPRnb(val,""                      ,"" )
            self.PDVTB_given = 1
        else:
            self.PDVTB        = self.MPRnb(0.0            ,""                      ,"" )
            self.PDVTB_given = 0

        if params_dict.get('LLPEB') != None:
            val =  params_dict['LLPEB']
            self.LLPEB        = self.MPRnb(val,""                      ,"" )
            self.LLPEB_given = 1
        else:
            self.LLPEB        = self.MPRnb(0.0            ,""                      ,"" )
            self.LLPEB_given = 0

        if params_dict.get('NLPEB') != None:
            val =  params_dict['NLPEB']
            self.NLPEB        = self.MPRnb(val,""                      ,"" )
            self.NLPEB_given = 1
        else:
            self.NLPEB        = self.MPRnb(0.0            ,""                      ,"" )
            self.NLPEB_given = 0

        if params_dict.get('PLPEB') != None:
            val =  params_dict['PLPEB']
            self.PLPEB        = self.MPRnb(val,""                      ,"" )
            self.PLPEB_given = 1
        else:
            self.PLPEB        = self.MPRnb(0.0            ,""                      ,"" )
            self.PLPEB_given = 0

        if params_dict.get('LQMFACTOR') != None:
            val =  params_dict['LQMFACTOR']
            self.LQMFACTOR    = self.MPRnb(val,""                      ,"" )
            self.LQMFACTOR_given = 1
        else:
            self.LQMFACTOR    = self.MPRnb(0.0            ,""                      ,"" )
            self.LQMFACTOR_given = 0

        if params_dict.get('NQMFACTOR') != None:
            val =  params_dict['NQMFACTOR']
            self.NQMFACTOR    = self.MPRnb(val,""                      ,"" )
            self.NQMFACTOR_given = 1
        else:
            self.NQMFACTOR    = self.MPRnb(0.0            ,""                      ,"" )
            self.NQMFACTOR_given = 0

        if params_dict.get('PQMFACTOR') != None:
            val =  params_dict['PQMFACTOR']
            self.PQMFACTOR    = self.MPRnb(val,""                      ,"" )
            self.PQMFACTOR_given = 1
        else:
            self.PQMFACTOR    = self.MPRnb(0.0            ,""                      ,"" )
            self.PQMFACTOR_given = 0

        if params_dict.get('LQMTCENCV') != None:
            val =  params_dict['LQMTCENCV']
            self.LQMTCENCV    = self.MPRnb(val,""                      ,"" )
            self.LQMTCENCV_given = 1
        else:
            self.LQMTCENCV    = self.MPRnb(0.0            ,""                      ,"" )
            self.LQMTCENCV_given = 0

        if params_dict.get('NQMTCENCV') != None:
            val =  params_dict['NQMTCENCV']
            self.NQMTCENCV    = self.MPRnb(val,""                      ,"" )
            self.NQMTCENCV_given = 1
        else:
            self.NQMTCENCV    = self.MPRnb(0.0            ,""                      ,"" )
            self.NQMTCENCV_given = 0

        if params_dict.get('PQMTCENCV') != None:
            val =  params_dict['PQMTCENCV']
            self.PQMTCENCV    = self.MPRnb(val,""                      ,"" )
            self.PQMTCENCV_given = 1
        else:
            self.PQMTCENCV    = self.MPRnb(0.0            ,""                      ,"" )
            self.PQMTCENCV_given = 0

        if params_dict.get('LQMTCENCVA') != None:
            val =  params_dict['LQMTCENCVA']
            self.LQMTCENCVA   = self.MPRnb(val,""                      ,"" )
            self.LQMTCENCVA_given = 1
        else:
            self.LQMTCENCVA   = self.MPRnb(0.0            ,""                      ,"" )
            self.LQMTCENCVA_given = 0

        if params_dict.get('NQMTCENCVA') != None:
            val =  params_dict['NQMTCENCVA']
            self.NQMTCENCVA   = self.MPRnb(val,""                      ,"" )
            self.NQMTCENCVA_given = 1
        else:
            self.NQMTCENCVA   = self.MPRnb(0.0            ,""                      ,"" )
            self.NQMTCENCVA_given = 0

        if params_dict.get('PQMTCENCVA') != None:
            val =  params_dict['PQMTCENCVA']
            self.PQMTCENCVA   = self.MPRnb(val,""                      ,"" )
            self.PQMTCENCVA_given = 1
        else:
            self.PQMTCENCVA   = self.MPRnb(0.0            ,""                      ,"" )
            self.PQMTCENCVA_given = 0

        if params_dict.get('LVSAT') != None:
            val =  params_dict['LVSAT']
            self.LVSAT        = self.MPRnb(val,"(m^2)/s"               ,"" )
            self.LVSAT_given = 1
        else:
            self.LVSAT        = self.MPRnb(0.0            ,"(m^2)/s"               ,"" )
            self.LVSAT_given = 0

        if params_dict.get('NVSAT') != None:
            val =  params_dict['NVSAT']
            self.NVSAT        = self.MPRnb(val,"(m^2)/s"               ,"" )
            self.NVSAT_given = 1
        else:
            self.NVSAT        = self.MPRnb(0.0            ,"(m^2)/s"               ,"" )
            self.NVSAT_given = 0

        if params_dict.get('PVSAT') != None:
            val =  params_dict['PVSAT']
            self.PVSAT        = self.MPRnb(val,"(m^3)/s"               ,"" )
            self.PVSAT_given = 1
        else:
            self.PVSAT        = self.MPRnb(0.0            ,"(m^3)/s"               ,"" )
            self.PVSAT_given = 0

        if params_dict.get('LVSATR') != None:
            val =  params_dict['LVSATR']
            self.LVSATR       = self.MPRnb(val,""                      ,"" )
            self.LVSATR_given = 1
        else:
            self.LVSATR       = self.MPRnb(self.LVSAT          ,""                      ,"" )
            self.LVSATR_given = 0

        if params_dict.get('NVSATR') != None:
            val =  params_dict['NVSATR']
            self.NVSATR       = self.MPRnb(val,""                      ,"" )
            self.NVSATR_given = 1
        else:
            self.NVSATR       = self.MPRnb(self.NVSAT          ,""                      ,"" )
            self.NVSATR_given = 0

        if params_dict.get('PVSATR') != None:
            val =  params_dict['PVSATR']
            self.PVSATR       = self.MPRnb(val,""                      ,"" )
            self.PVSATR_given = 1
        else:
            self.PVSATR       = self.MPRnb(self.PVSAT          ,""                      ,"" )
            self.PVSATR_given = 0

        if params_dict.get('LVSAT1') != None:
            val =  params_dict['LVSAT1']
            self.LVSAT1       = self.MPRnb(val,""                      ,"" )
            self.LVSAT1_given = 1
        else:
            self.LVSAT1       = self.MPRnb(self.LVSAT          ,""                      ,"" )
            self.LVSAT1_given = 0

        if params_dict.get('NVSAT1') != None:
            val =  params_dict['NVSAT1']
            self.NVSAT1       = self.MPRnb(val,""                      ,"" )
            self.NVSAT1_given = 1
        else:
            self.NVSAT1       = self.MPRnb(self.NVSAT          ,""                      ,"" )
            self.NVSAT1_given = 0

        if params_dict.get('PVSAT1') != None:
            val =  params_dict['PVSAT1']
            self.PVSAT1       = self.MPRnb(val,""                      ,"" )
            self.PVSAT1_given = 1
        else:
            self.PVSAT1       = self.MPRnb(self.PVSAT          ,""                      ,"" )
            self.PVSAT1_given = 0

        if params_dict.get('LVSAT1R') != None:
            val =  params_dict['LVSAT1R']
            self.LVSAT1R      = self.MPRnb(val,"(m^2)/s"               ,"" )
            self.LVSAT1R_given = 1
        else:
            self.LVSAT1R      = self.MPRnb(self.LVSAT1         ,"(m^2)/s"               ,"" )
            self.LVSAT1R_given = 0

        if params_dict.get('NVSAT1R') != None:
            val =  params_dict['NVSAT1R']
            self.NVSAT1R      = self.MPRnb(val,"(m^2)/s"               ,"" )
            self.NVSAT1R_given = 1
        else:
            self.NVSAT1R      = self.MPRnb(self.NVSAT1         ,"(m^2)/s"               ,"" )
            self.NVSAT1R_given = 0

        if params_dict.get('PVSAT1R') != None:
            val =  params_dict['PVSAT1R']
            self.PVSAT1R      = self.MPRnb(val,"(m^3)/s"              ,"" )
            self.PVSAT1R_given = 1
        else:
            self.PVSAT1R      = self.MPRnb(self.PVSAT1         ,"(m^3)/s"              ,"" )
            self.PVSAT1R_given = 0

        if params_dict.get('LPSAT') != None:
            val =  params_dict['LPSAT']
            self.LPSAT        = self.MPRnb(val,""                      ,"" )
            self.LPSAT_given = 1
        else:
            self.LPSAT        = self.MPRnb(0.0            ,""                      ,"" )
            self.LPSAT_given = 0

        if params_dict.get('NPSAT') != None:
            val =  params_dict['NPSAT']
            self.NPSAT        = self.MPRnb(val,""                      ,"" )
            self.NPSAT_given = 1
        else:
            self.NPSAT        = self.MPRnb(0.0            ,""                      ,"" )
            self.NPSAT_given = 0

        if params_dict.get('PPSAT') != None:
            val =  params_dict['PPSAT']
            self.PPSAT        = self.MPRnb(val,""                      ,"" )
            self.PPSAT_given = 1
        else:
            self.PPSAT        = self.MPRnb(0.0            ,""                      ,"" )
            self.PPSAT_given = 0

        if params_dict.get('LDELTAVSAT') != None:
            val =  params_dict['LDELTAVSAT']
            self.LDELTAVSAT   = self.MPRnb(val,""                      ,"" )
            self.LDELTAVSAT_given = 1
        else:
            self.LDELTAVSAT   = self.MPRnb(0.0            ,""                      ,"" )
            self.LDELTAVSAT_given = 0

        if params_dict.get('NDELTAVSAT') != None:
            val =  params_dict['NDELTAVSAT']
            self.NDELTAVSAT   = self.MPRnb(val,""                      ,"" )
            self.NDELTAVSAT_given = 1
        else:
            self.NDELTAVSAT   = self.MPRnb(0.0            ,""                      ,"" )
            self.NDELTAVSAT_given = 0

        if params_dict.get('PDELTAVSAT') != None:
            val =  params_dict['PDELTAVSAT']
            self.PDELTAVSAT   = self.MPRnb(val,""                      ,"" )
            self.PDELTAVSAT_given = 1
        else:
            self.PDELTAVSAT   = self.MPRnb(0.0            ,""                      ,"" )
            self.PDELTAVSAT_given = 0

        if params_dict.get('LKSATIV') != None:
            val =  params_dict['LKSATIV']
            self.LKSATIV      = self.MPRnb(val,""                      ,"" )
            self.LKSATIV_given = 1
        else:
            self.LKSATIV      = self.MPRnb(0.0            ,""                      ,"" )
            self.LKSATIV_given = 0

        if params_dict.get('NKSATIV') != None:
            val =  params_dict['NKSATIV']
            self.NKSATIV      = self.MPRnb(val,""                      ,"" )
            self.NKSATIV_given = 1
        else:
            self.NKSATIV      = self.MPRnb(0.0            ,""                      ,"" )
            self.NKSATIV_given = 0

        if params_dict.get('PKSATIV') != None:
            val =  params_dict['PKSATIV']
            self.PKSATIV      = self.MPRnb(val,""                      ,"" )
            self.PKSATIV_given = 1
        else:
            self.PKSATIV      = self.MPRnb(0.0            ,""                      ,"" )
            self.PKSATIV_given = 0

        if params_dict.get('LKSATIVR') != None:
            val =  params_dict['LKSATIVR']
            self.LKSATIVR     = self.MPRnb(val,""                      ,"" )
            self.LKSATIVR_given = 1
        else:
            self.LKSATIVR     = self.MPRnb(self.LKSATIV        ,""                      ,"" )
            self.LKSATIVR_given = 0

        if params_dict.get('NKSATIVR') != None:
            val =  params_dict['NKSATIVR']
            self.NKSATIVR     = self.MPRnb(val,""                      ,"" )
            self.NKSATIVR_given = 1
        else:
            self.NKSATIVR     = self.MPRnb(self.NKSATIV        ,""                      ,"" )
            self.NKSATIVR_given = 0

        if params_dict.get('PKSATIVR') != None:
            val =  params_dict['PKSATIVR']
            self.PKSATIVR     = self.MPRnb(val,""                      ,"" )
            self.PKSATIVR_given = 1
        else:
            self.PKSATIVR     = self.MPRnb(self.PKSATIV        ,""                      ,"" )
            self.PKSATIVR_given = 0

        if params_dict.get('LVSATCV') != None:
            val =  params_dict['LVSATCV']
            self.LVSATCV      = self.MPRnb(val,"(m^2)/s"               ,"" )
            self.LVSATCV_given = 1
        else:
            self.LVSATCV      = self.MPRnb(0.0            ,"(m^2)/s"               ,"" )
            self.LVSATCV_given = 0

        if params_dict.get('NVSATCV') != None:
            val =  params_dict['NVSATCV']
            self.NVSATCV      = self.MPRnb(val,"(m^2)/s"               ,"" )
            self.NVSATCV_given = 1
        else:
            self.NVSATCV      = self.MPRnb(0.0            ,"(m^2)/s"               ,"" )
            self.NVSATCV_given = 0

        if params_dict.get('PVSATCV') != None:
            val =  params_dict['PVSATCV']
            self.PVSATCV      = self.MPRnb(val,"(m^3)/s"               ,"" )
            self.PVSATCV_given = 1
        else:
            self.PVSATCV      = self.MPRnb(0.0            ,"(m^3)/s"               ,"" )
            self.PVSATCV_given = 0

        if params_dict.get('LPSATCV') != None:
            val =  params_dict['LPSATCV']
            self.LPSATCV      = self.MPRnb(val,""                      ,"" )
            self.LPSATCV_given = 1
        else:
            self.LPSATCV      = self.MPRnb(0.0            ,""                      ,"" )
            self.LPSATCV_given = 0

        if params_dict.get('NPSATCV') != None:
            val =  params_dict['NPSATCV']
            self.NPSATCV      = self.MPRnb(val,""                      ,"" )
            self.NPSATCV_given = 1
        else:
            self.NPSATCV      = self.MPRnb(0.0            ,""                      ,"" )
            self.NPSATCV_given = 0

        if params_dict.get('PPSATCV') != None:
            val =  params_dict['PPSATCV']
            self.PPSATCV      = self.MPRnb(val,""                      ,"" )
            self.PPSATCV_given = 1
        else:
            self.PPSATCV      = self.MPRnb(0.0            ,""                      ,"" )
            self.PPSATCV_given = 0

        if params_dict.get('LDELTAVSATCV') != None:
            val =  params_dict['LDELTAVSATCV']
            self.LDELTAVSATCV = self.MPRnb(val,""                      ,"" )
            self.LDELTAVSATCV_given = 1
        else:
            self.LDELTAVSATCV = self.MPRnb(0.0            ,""                      ,"" )
            self.LDELTAVSATCV_given = 0

        if params_dict.get('NDELTAVSATCV') != None:
            val =  params_dict['NDELTAVSATCV']
            self.NDELTAVSATCV = self.MPRnb(val,""                      ,"" )
            self.NDELTAVSATCV_given = 1
        else:
            self.NDELTAVSATCV = self.MPRnb(0.0            ,""                      ,"" )
            self.NDELTAVSATCV_given = 0

        if params_dict.get('PDELTAVSATCV') != None:
            val =  params_dict['PDELTAVSATCV']
            self.PDELTAVSATCV = self.MPRnb(val,""                      ,"" )
            self.PDELTAVSATCV_given = 1
        else:
            self.PDELTAVSATCV = self.MPRnb(0.0            ,""                      ,"" )
            self.PDELTAVSATCV_given = 0

        if params_dict.get('LMEXP') != None:
            val =  params_dict['LMEXP']
            self.LMEXP        = self.MPRnb(val,""                      ,"" )
            self.LMEXP_given = 1
        else:
            self.LMEXP        = self.MPRnb(0.0            ,""                      ,"" )
            self.LMEXP_given = 0

        if params_dict.get('NMEXP') != None:
            val =  params_dict['NMEXP']
            self.NMEXP        = self.MPRnb(val,""                      ,"" )
            self.NMEXP_given = 1
        else:
            self.NMEXP        = self.MPRnb(0.0            ,""                      ,"" )
            self.NMEXP_given = 0

        if params_dict.get('PMEXP') != None:
            val =  params_dict['PMEXP']
            self.PMEXP        = self.MPRnb(val,""                      ,"" )
            self.PMEXP_given = 1
        else:
            self.PMEXP        = self.MPRnb(0.0            ,""                      ,"" )
            self.PMEXP_given = 0

        if params_dict.get('LMEXPR') != None:
            val =  params_dict['LMEXPR']
            self.LMEXPR       = self.MPRnb(val,""                      ,"" )
            self.LMEXPR_given = 1
        else:
            self.LMEXPR       = self.MPRnb(self.LMEXP          ,""                      ,"" )
            self.LMEXPR_given = 0

        if params_dict.get('NMEXPR') != None:
            val =  params_dict['NMEXPR']
            self.NMEXPR       = self.MPRnb(val,""                      ,"" )
            self.NMEXPR_given = 1
        else:
            self.NMEXPR       = self.MPRnb(self.NMEXP          ,""                      ,"" )
            self.NMEXPR_given = 0

        if params_dict.get('PMEXPR') != None:
            val =  params_dict['PMEXPR']
            self.PMEXPR       = self.MPRnb(val,""                      ,"" )
            self.PMEXPR_given = 1
        else:
            self.PMEXPR       = self.MPRnb(self.PMEXP          ,""                      ,"" )
            self.PMEXPR_given = 0

        if params_dict.get('LPTWG') != None:
            val =  params_dict['LPTWG']
            self.LPTWG        = self.MPRnb(val,"m*(V^-2)"              ,"" )
            self.LPTWG_given = 1
        else:
            self.LPTWG        = self.MPRnb(0.0            ,"m*(V^-2)"              ,"" )
            self.LPTWG_given = 0

        if params_dict.get('NPTWG') != None:
            val =  params_dict['NPTWG']
            self.NPTWG        = self.MPRnb(val,"m*(V^-2)"              ,"" )
            self.NPTWG_given = 1
        else:
            self.NPTWG        = self.MPRnb(0.0            ,"m*(V^-2)"              ,"" )
            self.NPTWG_given = 0

        if params_dict.get('PPTWG') != None:
            val =  params_dict['PPTWG']
            self.PPTWG        = self.MPRnb(val,"m^2*(V^-2)"            ,"" )
            self.PPTWG_given = 1
        else:
            self.PPTWG        = self.MPRnb(0.0            ,"m^2*(V^-2)"            ,"" )
            self.PPTWG_given = 0

        if params_dict.get('LPTWGR') != None:
            val =  params_dict['LPTWGR']
            self.LPTWGR       = self.MPRnb(val,"m*(V^-2)"              ,"" )
            self.LPTWGR_given = 1
        else:
            self.LPTWGR       = self.MPRnb(self.LPTWG          ,"m*(V^-2)"              ,"" )
            self.LPTWGR_given = 0

        if params_dict.get('NPTWGR') != None:
            val =  params_dict['NPTWGR']
            self.NPTWGR       = self.MPRnb(val,"m*(V^-2)"              ,"" )
            self.NPTWGR_given = 1
        else:
            self.NPTWGR       = self.MPRnb(self.NPTWG          ,"m*(V^-2)"              ,"" )
            self.NPTWGR_given = 0

        if params_dict.get('PPTWGR') != None:
            val =  params_dict['PPTWGR']
            self.PPTWGR       = self.MPRnb(val,"m^2*(V^-2)"            ,"" )
            self.PPTWGR_given = 1
        else:
            self.PPTWGR       = self.MPRnb(self.PPTWG          ,"m^2*(V^-2)"            ,"" )
            self.PPTWGR_given = 0

        if params_dict.get('LU0') != None:
            val =  params_dict['LU0']
            self.LU0          = self.MPRnb(val,"(m^3)/V*s"             ,"" )
            self.LU0_given = 1
        else:
            self.LU0          = self.MPRnb(0.0            ,"(m^3)/V*s"             ,"" )
            self.LU0_given = 0

        if params_dict.get('NU0') != None:
            val =  params_dict['NU0']
            self.NU0          = self.MPRnb(val,"(m^3)/V*s"             ,"" )
            self.NU0_given = 1
        else:
            self.NU0          = self.MPRnb(0.0            ,"(m^3)/V*s"             ,"" )
            self.NU0_given = 0

        if params_dict.get('PU0') != None:
            val =  params_dict['PU0']
            self.PU0          = self.MPRnb(val,"(m^4)/V*s"             ,"" )
            self.PU0_given = 1
        else:
            self.PU0          = self.MPRnb(0.0            ,"(m^4)/V*s"             ,"" )
            self.PU0_given = 0

        if params_dict.get('LU0R') != None:
            val =  params_dict['LU0R']
            self.LU0R         = self.MPRnb(val,""                      ,"" )
            self.LU0R_given = 1
        else:
            self.LU0R         = self.MPRnb(self.LU0            ,""                      ,"" )
            self.LU0R_given = 0

        if params_dict.get('NU0R') != None:
            val =  params_dict['NU0R']
            self.NU0R         = self.MPRnb(val,""                      ,"" )
            self.NU0R_given = 1
        else:
            self.NU0R         = self.MPRnb(self.NU0            ,""                      ,"" )
            self.NU0R_given = 0

        if params_dict.get('PU0R') != None:
            val =  params_dict['PU0R']
            self.PU0R         = self.MPRnb(val,""                      ,"" )
            self.PU0R_given = 1
        else:
            self.PU0R         = self.MPRnb(self.PU0            ,""                      ,"" )
            self.PU0R_given = 0

        if params_dict.get('LETAMOB') != None:
            val =  params_dict['LETAMOB']
            self.LETAMOB      = self.MPRnb(val,""                      ,"" )
            self.LETAMOB_given = 1
        else:
            self.LETAMOB      = self.MPRnb(0.0            ,""                      ,"" )
            self.LETAMOB_given = 0

        if params_dict.get('NETAMOB') != None:
            val =  params_dict['NETAMOB']
            self.NETAMOB      = self.MPRnb(val,""                      ,"" )
            self.NETAMOB_given = 1
        else:
            self.NETAMOB      = self.MPRnb(0.0            ,""                      ,"" )
            self.NETAMOB_given = 0

        if params_dict.get('PETAMOB') != None:
            val =  params_dict['PETAMOB']
            self.PETAMOB      = self.MPRnb(val,""                      ,"" )
            self.PETAMOB_given = 1
        else:
            self.PETAMOB      = self.MPRnb(0.0            ,""                      ,"" )
            self.PETAMOB_given = 0

        if params_dict.get('LUP') != None:
            val =  params_dict['LUP']
            self.LUP          = self.MPRnb(val,"m*(um^LPA)"            ,"" )
            self.LUP_given = 1
        else:
            self.LUP          = self.MPRnb(0.0            ,"m*(um^LPA)"            ,"" )
            self.LUP_given = 0

        if params_dict.get('NUP') != None:
            val =  params_dict['NUP']
            self.NUP          = self.MPRnb(val,"m*(um^LPA)"            ,"" )
            self.NUP_given = 1
        else:
            self.NUP          = self.MPRnb(0.0            ,"m*(um^LPA)"            ,"" )
            self.NUP_given = 0

        if params_dict.get('PUP') != None:
            val =  params_dict['PUP']
            self.PUP          = self.MPRnb(val,"m^2*(um^LPA)"          ,"" )
            self.PUP_given = 1
        else:
            self.PUP          = self.MPRnb(0.0            ,"m^2*(um^LPA)"          ,"" )
            self.PUP_given = 0

        if params_dict.get('LUPR') != None:
            val =  params_dict['LUPR']
            self.LUPR         = self.MPRnb(val,""                      ,"" )
            self.LUPR_given = 1
        else:
            self.LUPR         = self.MPRnb(self.LUP            ,""                      ,"" )
            self.LUPR_given = 0

        if params_dict.get('NUPR') != None:
            val =  params_dict['NUPR']
            self.NUPR         = self.MPRnb(val,""                      ,"" )
            self.NUPR_given = 1
        else:
            self.NUPR         = self.MPRnb(self.NUP            ,""                      ,"" )
            self.NUPR_given = 0

        if params_dict.get('PUPR') != None:
            val =  params_dict['PUPR']
            self.PUPR         = self.MPRnb(val,""                      ,"" )
            self.PUPR_given = 1
        else:
            self.PUPR         = self.MPRnb(self.PUP            ,""                      ,"" )
            self.PUPR_given = 0

        if params_dict.get('LUA') != None:
            val =  params_dict['LUA']
            self.LUA          = self.MPRnb(val,"m*((cm/MV)^EU)"        ,"" )
            self.LUA_given = 1
        else:
            self.LUA          = self.MPRnb(0.0            ,"m*((cm/MV)^EU)"        ,"" )
            self.LUA_given = 0

        if params_dict.get('NUA') != None:
            val =  params_dict['NUA']
            self.NUA          = self.MPRnb(val,"m*((cm/MV)^EU)"        ,"" )
            self.NUA_given = 1
        else:
            self.NUA          = self.MPRnb(0.0            ,"m*((cm/MV)^EU)"        ,"" )
            self.NUA_given = 0

        if params_dict.get('PUA') != None:
            val =  params_dict['PUA']
            self.PUA          = self.MPRnb(val,"m^2*((cm/MV)^EU)"      ,"" )
            self.PUA_given = 1
        else:
            self.PUA          = self.MPRnb(0.0            ,"m^2*((cm/MV)^EU)"      ,"" )
            self.PUA_given = 0

        if params_dict.get('LUAR') != None:
            val =  params_dict['LUAR']
            self.LUAR         = self.MPRnb(val,""                      ,"" )
            self.LUAR_given = 1
        else:
            self.LUAR         = self.MPRnb(self.LUA            ,""                      ,"" )
            self.LUAR_given = 0

        if params_dict.get('NUAR') != None:
            val =  params_dict['NUAR']
            self.NUAR         = self.MPRnb(val,""                      ,"" )
            self.NUAR_given = 1
        else:
            self.NUAR         = self.MPRnb(self.NUA            ,""                      ,"" )
            self.NUAR_given = 0

        if params_dict.get('PUAR') != None:
            val =  params_dict['PUAR']
            self.PUAR         = self.MPRnb(val,""                      ,"" )
            self.PUAR_given = 1
        else:
            self.PUAR         = self.MPRnb(self.PUA            ,""                      ,"" )
            self.PUAR_given = 0

        if params_dict.get('LUC') != None:
            val =  params_dict['LUC']
            self.LUC          = self.MPRnb(val,""                      ,"" )
            self.LUC_given = 1
        else:
            self.LUC          = self.MPRnb(0.0            ,""                      ,"" )
            self.LUC_given = 0

        if params_dict.get('NUC') != None:
            val =  params_dict['NUC']
            self.NUC          = self.MPRnb(val,""                      ,"" )
            self.NUC_given = 1
        else:
            self.NUC          = self.MPRnb(0.0            ,""                      ,"" )
            self.NUC_given = 0

        if params_dict.get('PUC') != None:
            val =  params_dict['PUC']
            self.PUC          = self.MPRnb(val,""                      ,"" )
            self.PUC_given = 1
        else:
            self.PUC          = self.MPRnb(0.0            ,""                      ,"" )
            self.PUC_given = 0

        if params_dict.get('LUCR') != None:
            val =  params_dict['LUCR']
            self.LUCR         = self.MPRnb(val,""                      ,"" )
            self.LUCR_given = 1
        else:
            self.LUCR         = self.MPRnb(self.LUC            ,""                      ,"" )
            self.LUCR_given = 0

        if params_dict.get('NUCR') != None:
            val =  params_dict['NUCR']
            self.NUCR         = self.MPRnb(val,""                      ,"" )
            self.NUCR_given = 1
        else:
            self.NUCR         = self.MPRnb(self.NUC            ,""                      ,"" )
            self.NUCR_given = 0

        if params_dict.get('PUCR') != None:
            val =  params_dict['PUCR']
            self.PUCR         = self.MPRnb(val,""                      ,"" )
            self.PUCR_given = 1
        else:
            self.PUCR         = self.MPRnb(self.PUC            ,""                      ,"" )
            self.PUCR_given = 0

        if params_dict.get('LEU') != None:
            val =  params_dict['LEU']
            self.LEU          = self.MPRnb(val,"m*(cm/MV)"             ,"" )
            self.LEU_given = 1
        else:
            self.LEU          = self.MPRnb(0.0            ,"m*(cm/MV)"             ,"" )
            self.LEU_given = 0

        if params_dict.get('NEU') != None:
            val =  params_dict['NEU']
            self.NEU          = self.MPRnb(val,"m*(cm/MV)"             ,"" )
            self.NEU_given = 1
        else:
            self.NEU          = self.MPRnb(0.0            ,"m*(cm/MV)"             ,"" )
            self.NEU_given = 0

        if params_dict.get('PEU') != None:
            val =  params_dict['PEU']
            self.PEU          = self.MPRnb(val,"m^2*(cm/MV)"           ,"" )
            self.PEU_given = 1
        else:
            self.PEU          = self.MPRnb(0.0            ,"m^2*(cm/MV)"           ,"" )
            self.PEU_given = 0

        if params_dict.get('LEUR') != None:
            val =  params_dict['LEUR']
            self.LEUR         = self.MPRnb(val,""                      ,"" )
            self.LEUR_given = 1
        else:
            self.LEUR         = self.MPRnb(self.LEU            ,""                      ,"" )
            self.LEUR_given = 0

        if params_dict.get('NEUR') != None:
            val =  params_dict['NEUR']
            self.NEUR         = self.MPRnb(val,""                      ,"" )
            self.NEUR_given = 1
        else:
            self.NEUR         = self.MPRnb(self.NEU            ,""                      ,"" )
            self.NEUR_given = 0

        if params_dict.get('PEUR') != None:
            val =  params_dict['PEUR']
            self.PEUR         = self.MPRnb(val,""                      ,"" )
            self.PEUR_given = 1
        else:
            self.PEUR         = self.MPRnb(self.PEU            ,""                      ,"" )
            self.PEUR_given = 0

        if params_dict.get('LUD') != None:
            val =  params_dict['LUD']
            self.LUD          = self.MPRnb(val,"m*(cm/MV)"             ,"" )
            self.LUD_given = 1
        else:
            self.LUD          = self.MPRnb(0.0            ,"m*(cm/MV)"             ,"" )
            self.LUD_given = 0

        if params_dict.get('NUD') != None:
            val =  params_dict['NUD']
            self.NUD          = self.MPRnb(val,"m*(cm/MV)"             ,"" )
            self.NUD_given = 1
        else:
            self.NUD          = self.MPRnb(0.0            ,"m*(cm/MV)"             ,"" )
            self.NUD_given = 0

        if params_dict.get('PUD') != None:
            val =  params_dict['PUD']
            self.PUD          = self.MPRnb(val,"m^2*(cm/MV)"           ,"" )
            self.PUD_given = 1
        else:
            self.PUD          = self.MPRnb(0.0            ,"m^2*(cm/MV)"           ,"" )
            self.PUD_given = 0

        if params_dict.get('LUDR') != None:
            val =  params_dict['LUDR']
            self.LUDR         = self.MPRnb(val,""                      ,"" )
            self.LUDR_given = 1
        else:
            self.LUDR         = self.MPRnb(self.LUD            ,""                      ,"" )
            self.LUDR_given = 0

        if params_dict.get('NUDR') != None:
            val =  params_dict['NUDR']
            self.NUDR         = self.MPRnb(val,""                      ,"" )
            self.NUDR_given = 1
        else:
            self.NUDR         = self.MPRnb(self.NUD            ,""                      ,"" )
            self.NUDR_given = 0

        if params_dict.get('PUDR') != None:
            val =  params_dict['PUDR']
            self.PUDR         = self.MPRnb(val,""                      ,"" )
            self.PUDR_given = 1
        else:
            self.PUDR         = self.MPRnb(self.PUD            ,""                      ,"" )
            self.PUDR_given = 0

        if params_dict.get('LUCS') != None:
            val =  params_dict['LUCS']
            self.LUCS         = self.MPRnb(val,""                      ,"" )
            self.LUCS_given = 1
        else:
            self.LUCS         = self.MPRnb(0.0            ,""                      ,"" )
            self.LUCS_given = 0

        if params_dict.get('NUCS') != None:
            val =  params_dict['NUCS']
            self.NUCS         = self.MPRnb(val,""                      ,"" )
            self.NUCS_given = 1
        else:
            self.NUCS         = self.MPRnb(0.0            ,""                      ,"" )
            self.NUCS_given = 0

        if params_dict.get('PUCS') != None:
            val =  params_dict['PUCS']
            self.PUCS         = self.MPRnb(val,""                      ,"" )
            self.PUCS_given = 1
        else:
            self.PUCS         = self.MPRnb(0.0            ,""                      ,"" )
            self.PUCS_given = 0

        if params_dict.get('LPCLM') != None:
            val =  params_dict['LPCLM']
            self.LPCLM        = self.MPRnb(val,""                      ,"" )
            self.LPCLM_given = 1
        else:
            self.LPCLM        = self.MPRnb(0.0            ,""                      ,"" )
            self.LPCLM_given = 0

        if params_dict.get('NPCLM') != None:
            val =  params_dict['NPCLM']
            self.NPCLM        = self.MPRnb(val,""                      ,"" )
            self.NPCLM_given = 1
        else:
            self.NPCLM        = self.MPRnb(0.0            ,""                      ,"" )
            self.NPCLM_given = 0

        if params_dict.get('PPCLM') != None:
            val =  params_dict['PPCLM']
            self.PPCLM        = self.MPRnb(val,""                      ,"" )
            self.PPCLM_given = 1
        else:
            self.PPCLM        = self.MPRnb(0.0            ,""                      ,"" )
            self.PPCLM_given = 0

        if params_dict.get('LPCLMR') != None:
            val =  params_dict['LPCLMR']
            self.LPCLMR       = self.MPRnb(val,""                      ,"" )
            self.LPCLMR_given = 1
        else:
            self.LPCLMR       = self.MPRnb(self.LPCLM          ,""                      ,"" )
            self.LPCLMR_given = 0

        if params_dict.get('NPCLMR') != None:
            val =  params_dict['NPCLMR']
            self.NPCLMR       = self.MPRnb(val,""                      ,"" )
            self.NPCLMR_given = 1
        else:
            self.NPCLMR       = self.MPRnb(self.NPCLM          ,""                      ,"" )
            self.NPCLMR_given = 0

        if params_dict.get('PPCLMR') != None:
            val =  params_dict['PPCLMR']
            self.PPCLMR       = self.MPRnb(val,""                      ,"" )
            self.PPCLMR_given = 1
        else:
            self.PPCLMR       = self.MPRnb(self.PPCLM          ,""                      ,"" )
            self.PPCLMR_given = 0

        if params_dict.get('LPCLMG') != None:
            val =  params_dict['LPCLMG']
            self.LPCLMG       = self.MPRnb(val,""                      ,"" )
            self.LPCLMG_given = 1
        else:
            self.LPCLMG       = self.MPRnb(0.0            ,""                      ,"" )
            self.LPCLMG_given = 0

        if params_dict.get('NPCLMG') != None:
            val =  params_dict['NPCLMG']
            self.NPCLMG       = self.MPRnb(val,""                      ,"" )
            self.NPCLMG_given = 1
        else:
            self.NPCLMG       = self.MPRnb(0.0            ,""                      ,"" )
            self.NPCLMG_given = 0

        if params_dict.get('PPCLMG') != None:
            val =  params_dict['PPCLMG']
            self.PPCLMG       = self.MPRnb(val,""                      ,"" )
            self.PPCLMG_given = 1
        else:
            self.PPCLMG       = self.MPRnb(0.0            ,""                      ,"" )
            self.PPCLMG_given = 0

        if params_dict.get('LPCLMCV') != None:
            val =  params_dict['LPCLMCV']
            self.LPCLMCV      = self.MPRnb(val,""                      ,"" )
            self.LPCLMCV_given = 1
        else:
            self.LPCLMCV      = self.MPRnb(self.LPCLM          ,""                      ,"" )
            self.LPCLMCV_given = 0

        if params_dict.get('NPCLMCV') != None:
            val =  params_dict['NPCLMCV']
            self.NPCLMCV      = self.MPRnb(val,""                      ,"" )
            self.NPCLMCV_given = 1
        else:
            self.NPCLMCV      = self.MPRnb(self.NPCLM          ,""                      ,"" )
            self.NPCLMCV_given = 0

        if params_dict.get('PPCLMCV') != None:
            val =  params_dict['PPCLMCV']
            self.PPCLMCV      = self.MPRnb(val,""                      ,"" )
            self.PPCLMCV_given = 1
        else:
            self.PPCLMCV      = self.MPRnb(self.PPCLM          ,""                      ,"" )
            self.PPCLMCV_given = 0

        if params_dict.get('LA1') != None:
            val =  params_dict['LA1']
            self.LA1          = self.MPRnb(val,"m*(V^-2)"              ,"" )
            self.LA1_given = 1
        else:
            self.LA1          = self.MPRnb(0.0            ,"m*(V^-2)"              ,"" )
            self.LA1_given = 0

        if params_dict.get('NA1') != None:
            val =  params_dict['NA1']
            self.NA1          = self.MPRnb(val,"m*(V^-2)"              ,"" )
            self.NA1_given = 1
        else:
            self.NA1          = self.MPRnb(0.0            ,"m*(V^-2)"              ,"" )
            self.NA1_given = 0

        if params_dict.get('PA1') != None:
            val =  params_dict['PA1']
            self.PA1          = self.MPRnb(val,"m^2*(V^-2)"            ,"" )
            self.PA1_given = 1
        else:
            self.PA1          = self.MPRnb(0.0            ,"m^2*(V^-2)"            ,"" )
            self.PA1_given = 0

        if params_dict.get('LA11') != None:
            val =  params_dict['LA11']
            self.LA11         = self.MPRnb(val,"m*(V^-2/K)"            ,"" )
            self.LA11_given = 1
        else:
            self.LA11         = self.MPRnb(0.0            ,"m*(V^-2/K)"            ,"" )
            self.LA11_given = 0

        if params_dict.get('NA11') != None:
            val =  params_dict['NA11']
            self.NA11         = self.MPRnb(val,"m*(V^-2/K)"            ,"" )
            self.NA11_given = 1
        else:
            self.NA11         = self.MPRnb(0.0            ,"m*(V^-2/K)"            ,"" )
            self.NA11_given = 0

        if params_dict.get('PA11') != None:
            val =  params_dict['PA11']
            self.PA11         = self.MPRnb(val,"m^2*(V^-2/K)"          ,"" )
            self.PA11_given = 1
        else:
            self.PA11         = self.MPRnb(0.0            ,"m^2*(V^-2/K)"          ,"" )
            self.PA11_given = 0

        if params_dict.get('LA2') != None:
            val =  params_dict['LA2']
            self.LA2          = self.MPRnb(val,"m*(V^-1)"              ,"" )
            self.LA2_given = 1
        else:
            self.LA2          = self.MPRnb(0.0            ,"m*(V^-1)"              ,"" )
            self.LA2_given = 0

        if params_dict.get('NA2') != None:
            val =  params_dict['NA2']
            self.NA2          = self.MPRnb(val,"m*(V^-1)"              ,"" )
            self.NA2_given = 1
        else:
            self.NA2          = self.MPRnb(0.0            ,"m*(V^-1)"              ,"" )
            self.NA2_given = 0

        if params_dict.get('PA2') != None:
            val =  params_dict['PA2']
            self.PA2          = self.MPRnb(val,"m^2*(V^-1)"            ,"" )
            self.PA2_given = 1
        else:
            self.PA2          = self.MPRnb(0.0            ,"m^2*(V^-1)"            ,"" )
            self.PA2_given = 0

        if params_dict.get('LA21') != None:
            val =  params_dict['LA21']
            self.LA21         = self.MPRnb(val,"m*(V^-1/K)"            ,"" )
            self.LA21_given = 1
        else:
            self.LA21         = self.MPRnb(0.0            ,"m*(V^-1/K)"            ,"" )
            self.LA21_given = 0

        if params_dict.get('NA21') != None:
            val =  params_dict['NA21']
            self.NA21         = self.MPRnb(val,"m*(V^-1/K)"            ,"" )
            self.NA21_given = 1
        else:
            self.NA21         = self.MPRnb(0.0            ,"m*(V^-1/K)"            ,"" )
            self.NA21_given = 0

        if params_dict.get('PA21') != None:
            val =  params_dict['PA21']
            self.PA21         = self.MPRnb(val,"m^2*(V^-1/K)"          ,"" )
            self.PA21_given = 1
        else:
            self.PA21         = self.MPRnb(0.0            ,"m^2*(V^-1/K)"          ,"" )
            self.PA21_given = 0

        if params_dict.get('LRDSW') != None:
            val =  params_dict['LRDSW']
            self.LRDSW        = self.MPRnb(val,"m*(ohm-um^WR)"         ,"" )
            self.LRDSW_given = 1
        else:
            self.LRDSW        = self.MPRnb(0.0            ,"m*(ohm-um^WR)"         ,"" )
            self.LRDSW_given = 0

        if params_dict.get('NRDSW') != None:
            val =  params_dict['NRDSW']
            self.NRDSW        = self.MPRnb(val,"m*(ohm-um^WR)"         ,"" )
            self.NRDSW_given = 1
        else:
            self.NRDSW        = self.MPRnb(0.0            ,"m*(ohm-um^WR)"         ,"" )
            self.NRDSW_given = 0

        if params_dict.get('PRDSW') != None:
            val =  params_dict['PRDSW']
            self.PRDSW        = self.MPRnb(val,"(m^2)*(ohm-um^WR)"     ,"" )
            self.PRDSW_given = 1
        else:
            self.PRDSW        = self.MPRnb(0.0            ,"(m^2)*(ohm-um^WR)"     ,"" )
            self.PRDSW_given = 0

        if params_dict.get('LRSW') != None:
            val =  params_dict['LRSW']
            self.LRSW         = self.MPRnb(val,"m*(ohm-um^WR)"         ,"" )
            self.LRSW_given = 1
        else:
            self.LRSW         = self.MPRnb(0.0            ,"m*(ohm-um^WR)"         ,"" )
            self.LRSW_given = 0

        if params_dict.get('NRSW') != None:
            val =  params_dict['NRSW']
            self.NRSW         = self.MPRnb(val,"m*(ohm-um^WR)"         ,"" )
            self.NRSW_given = 1
        else:
            self.NRSW         = self.MPRnb(0.0            ,"m*(ohm-um^WR)"         ,"" )
            self.NRSW_given = 0

        if params_dict.get('PRSW') != None:
            val =  params_dict['PRSW']
            self.PRSW         = self.MPRnb(val,"(m^2)*(ohm-um^WR)"     ,"" )
            self.PRSW_given = 1
        else:
            self.PRSW         = self.MPRnb(0.0            ,"(m^2)*(ohm-um^WR)"     ,"" )
            self.PRSW_given = 0

        if params_dict.get('LRDW') != None:
            val =  params_dict['LRDW']
            self.LRDW         = self.MPRnb(val,"m*(ohm-um^WR)"         ,"" )
            self.LRDW_given = 1
        else:
            self.LRDW         = self.MPRnb(0.0            ,"m*(ohm-um^WR)"         ,"" )
            self.LRDW_given = 0

        if params_dict.get('NRDW') != None:
            val =  params_dict['NRDW']
            self.NRDW         = self.MPRnb(val,"m*(ohm-um^WR)"         ,"" )
            self.NRDW_given = 1
        else:
            self.NRDW         = self.MPRnb(0.0            ,"m*(ohm-um^WR)"         ,"" )
            self.NRDW_given = 0

        if params_dict.get('PRDW') != None:
            val =  params_dict['PRDW']
            self.PRDW         = self.MPRnb(val,"(m^2)*(ohm-um^WR)"     ,"" )
            self.PRDW_given = 1
        else:
            self.PRDW         = self.MPRnb(0.0            ,"(m^2)*(ohm-um^WR)"     ,"" )
            self.PRDW_given = 0

        if params_dict.get('LPRWGS') != None:
            val =  params_dict['LPRWGS']
            self.LPRWGS       = self.MPRnb(val,"m/V"                   ,"" )
            self.LPRWGS_given = 1
        else:
            self.LPRWGS       = self.MPRnb(0.0            ,"m/V"                   ,"" )
            self.LPRWGS_given = 0

        if params_dict.get('NPRWGS') != None:
            val =  params_dict['NPRWGS']
            self.NPRWGS       = self.MPRnb(val,"m/V"                   ,"" )
            self.NPRWGS_given = 1
        else:
            self.NPRWGS       = self.MPRnb(0.0            ,"m/V"                   ,"" )
            self.NPRWGS_given = 0

        if params_dict.get('PPRWGS') != None:
            val =  params_dict['PPRWGS']
            self.PPRWGS       = self.MPRnb(val,"(m^2)/V"               ,"" )
            self.PPRWGS_given = 1
        else:
            self.PPRWGS       = self.MPRnb(0.0            ,"(m^2)/V"               ,"" )
            self.PPRWGS_given = 0

        if params_dict.get('LPRWGD') != None:
            val =  params_dict['LPRWGD']
            self.LPRWGD       = self.MPRnb(val,"m/V"                   ,"" )
            self.LPRWGD_given = 1
        else:
            self.LPRWGD       = self.MPRnb(0.0            ,"m/V"                   ,"" )
            self.LPRWGD_given = 0

        if params_dict.get('NPRWGD') != None:
            val =  params_dict['NPRWGD']
            self.NPRWGD       = self.MPRnb(val,"m/V"                   ,"" )
            self.NPRWGD_given = 1
        else:
            self.NPRWGD       = self.MPRnb(0.0            ,"m/V"                   ,"" )
            self.NPRWGD_given = 0

        if params_dict.get('PPRWGD') != None:
            val =  params_dict['PPRWGD']
            self.PPRWGD       = self.MPRnb(val,"(m^2)/V"               ,"" )
            self.PPRWGD_given = 1
        else:
            self.PPRWGD       = self.MPRnb(0.0            ,"(m^2)/V"               ,"" )
            self.PPRWGD_given = 0

        if params_dict.get('LWR') != None:
            val =  params_dict['LWR']
            self.LWR          = self.MPRnb(val,""                      ,"" )
            self.LWR_given = 1
        else:
            self.LWR          = self.MPRnb(0.0            ,""                      ,"" )
            self.LWR_given = 0

        if params_dict.get('NWR') != None:
            val =  params_dict['NWR']
            self.NWR          = self.MPRnb(val,""                      ,"" )
            self.NWR_given = 1
        else:
            self.NWR          = self.MPRnb(0.0            ,""                      ,"" )
            self.NWR_given = 0

        if params_dict.get('PWR') != None:
            val =  params_dict['PWR']
            self.PWR          = self.MPRnb(val,""                      ,"" )
            self.PWR_given = 1
        else:
            self.PWR          = self.MPRnb(0.0            ,""                      ,"" )
            self.PWR_given = 0

        if params_dict.get('LPDIBL1') != None:
            val =  params_dict['LPDIBL1']
            self.LPDIBL1      = self.MPRnb(val,""                      ,"" )
            self.LPDIBL1_given = 1
        else:
            self.LPDIBL1      = self.MPRnb(0.0            ,""                      ,"" )
            self.LPDIBL1_given = 0

        if params_dict.get('NPDIBL1') != None:
            val =  params_dict['NPDIBL1']
            self.NPDIBL1      = self.MPRnb(val,""                      ,"" )
            self.NPDIBL1_given = 1
        else:
            self.NPDIBL1      = self.MPRnb(0.0            ,""                      ,"" )
            self.NPDIBL1_given = 0

        if params_dict.get('PPDIBL1') != None:
            val =  params_dict['PPDIBL1']
            self.PPDIBL1      = self.MPRnb(val,""                      ,"" )
            self.PPDIBL1_given = 1
        else:
            self.PPDIBL1      = self.MPRnb(0.0            ,""                      ,"" )
            self.PPDIBL1_given = 0

        if params_dict.get('LPDIBL1R') != None:
            val =  params_dict['LPDIBL1R']
            self.LPDIBL1R     = self.MPRnb(val,""                      ,"" )
            self.LPDIBL1R_given = 1
        else:
            self.LPDIBL1R     = self.MPRnb(self.LPDIBL1        ,""                      ,"" )
            self.LPDIBL1R_given = 0

        if params_dict.get('NPDIBL1R') != None:
            val =  params_dict['NPDIBL1R']
            self.NPDIBL1R     = self.MPRnb(val,""                      ,"" )
            self.NPDIBL1R_given = 1
        else:
            self.NPDIBL1R     = self.MPRnb(self.NPDIBL1        ,""                      ,"" )
            self.NPDIBL1R_given = 0

        if params_dict.get('PPDIBL1R') != None:
            val =  params_dict['PPDIBL1R']
            self.PPDIBL1R     = self.MPRnb(val,""                      ,"" )
            self.PPDIBL1R_given = 1
        else:
            self.PPDIBL1R     = self.MPRnb(self.PPDIBL1        ,""                      ,"" )
            self.PPDIBL1R_given = 0

        if params_dict.get('LPDIBL2') != None:
            val =  params_dict['LPDIBL2']
            self.LPDIBL2      = self.MPRnb(val,""                      ,"" )
            self.LPDIBL2_given = 1
        else:
            self.LPDIBL2      = self.MPRnb(0.0            ,""                      ,"" )
            self.LPDIBL2_given = 0

        if params_dict.get('NPDIBL2') != None:
            val =  params_dict['NPDIBL2']
            self.NPDIBL2      = self.MPRnb(val,""                      ,"" )
            self.NPDIBL2_given = 1
        else:
            self.NPDIBL2      = self.MPRnb(0.0            ,""                      ,"" )
            self.NPDIBL2_given = 0

        if params_dict.get('PPDIBL2') != None:
            val =  params_dict['PPDIBL2']
            self.PPDIBL2      = self.MPRnb(val,""                      ,"" )
            self.PPDIBL2_given = 1
        else:
            self.PPDIBL2      = self.MPRnb(0.0            ,""                      ,"" )
            self.PPDIBL2_given = 0

        if params_dict.get('LPDIBL2R') != None:
            val =  params_dict['LPDIBL2R']
            self.LPDIBL2R     = self.MPRnb(val,""                      ,"" )
            self.LPDIBL2R_given = 1
        else:
            self.LPDIBL2R     = self.MPRnb(self.LPDIBL2        ,""                      ,"" )
            self.LPDIBL2R_given = 0

        if params_dict.get('NPDIBL2R') != None:
            val =  params_dict['NPDIBL2R']
            self.NPDIBL2R     = self.MPRnb(val,""                      ,"" )
            self.NPDIBL2R_given = 1
        else:
            self.NPDIBL2R     = self.MPRnb(self.NPDIBL2        ,""                      ,"" )
            self.NPDIBL2R_given = 0

        if params_dict.get('PPDIBL2R') != None:
            val =  params_dict['PPDIBL2R']
            self.PPDIBL2R     = self.MPRnb(val,""                      ,"" )
            self.PPDIBL2R_given = 1
        else:
            self.PPDIBL2R     = self.MPRnb(self.PPDIBL2        ,""                      ,"" )
            self.PPDIBL2R_given = 0

        if params_dict.get('LDROUT') != None:
            val =  params_dict['LDROUT']
            self.LDROUT       = self.MPRnb(val,""                      ,"" )
            self.LDROUT_given = 1
        else:
            self.LDROUT       = self.MPRnb(0.0            ,""                      ,"" )
            self.LDROUT_given = 0

        if params_dict.get('NDROUT') != None:
            val =  params_dict['NDROUT']
            self.NDROUT       = self.MPRnb(val,""                      ,"" )
            self.NDROUT_given = 1
        else:
            self.NDROUT       = self.MPRnb(0.0            ,""                      ,"" )
            self.NDROUT_given = 0

        if params_dict.get('PDROUT') != None:
            val =  params_dict['PDROUT']
            self.PDROUT       = self.MPRnb(val,""                      ,"" )
            self.PDROUT_given = 1
        else:
            self.PDROUT       = self.MPRnb(0.0            ,""                      ,"" )
            self.PDROUT_given = 0

        if params_dict.get('LPVAG') != None:
            val =  params_dict['LPVAG']
            self.LPVAG        = self.MPRnb(val,""                      ,"" )
            self.LPVAG_given = 1
        else:
            self.LPVAG        = self.MPRnb(0.0            ,""                      ,"" )
            self.LPVAG_given = 0

        if params_dict.get('NPVAG') != None:
            val =  params_dict['NPVAG']
            self.NPVAG        = self.MPRnb(val,""                      ,"" )
            self.NPVAG_given = 1
        else:
            self.NPVAG        = self.MPRnb(0.0            ,""                      ,"" )
            self.NPVAG_given = 0

        if params_dict.get('PPVAG') != None:
            val =  params_dict['PPVAG']
            self.PPVAG        = self.MPRnb(val,""                      ,"" )
            self.PPVAG_given = 1
        else:
            self.PPVAG        = self.MPRnb(0.0            ,""                      ,"" )
            self.PPVAG_given = 0

        if params_dict.get('LAIGBINV') != None:
            val =  params_dict['LAIGBINV']
            self.LAIGBINV     = self.MPRnb(val,"(F*s^2/g)^0.5"         ,"" )
            self.LAIGBINV_given = 1
        else:
            self.LAIGBINV     = self.MPRnb(0.0            ,"(F*s^2/g)^0.5"         ,"" )
            self.LAIGBINV_given = 0

        if params_dict.get('NAIGBINV') != None:
            val =  params_dict['NAIGBINV']
            self.NAIGBINV     = self.MPRnb(val,"(F*s^2/g)^0.5"         ,"" )
            self.NAIGBINV_given = 1
        else:
            self.NAIGBINV     = self.MPRnb(0.0            ,"(F*s^2/g)^0.5"         ,"" )
            self.NAIGBINV_given = 0

        if params_dict.get('PAIGBINV') != None:
            val =  params_dict['PAIGBINV']
            self.PAIGBINV     = self.MPRnb(val,"((F*s^2/g)^0.5)*m"     ,"" )
            self.PAIGBINV_given = 1
        else:
            self.PAIGBINV     = self.MPRnb(0.0            ,"((F*s^2/g)^0.5)*m"     ,"" )
            self.PAIGBINV_given = 0

        if params_dict.get('LAIGBINV1') != None:
            val =  params_dict['LAIGBINV1']
            self.LAIGBINV1    = self.MPRnb(val,"((F*s^2/g)^0.5)/K"     ,"" )
            self.LAIGBINV1_given = 1
        else:
            self.LAIGBINV1    = self.MPRnb(0.0            ,"((F*s^2/g)^0.5)/K"     ,"" )
            self.LAIGBINV1_given = 0

        if params_dict.get('NAIGBINV1') != None:
            val =  params_dict['NAIGBINV1']
            self.NAIGBINV1    = self.MPRnb(val,"((F*s^2/g)^0.5)/K"     ,"" )
            self.NAIGBINV1_given = 1
        else:
            self.NAIGBINV1    = self.MPRnb(0.0            ,"((F*s^2/g)^0.5)/K"     ,"" )
            self.NAIGBINV1_given = 0

        if params_dict.get('PAIGBINV1') != None:
            val =  params_dict['PAIGBINV1']
            self.PAIGBINV1    = self.MPRnb(val,"((F*s^2/g)^0.5)*(m/K)" ,"" )
            self.PAIGBINV1_given = 1
        else:
            self.PAIGBINV1    = self.MPRnb(0.0            ,"((F*s^2/g)^0.5)*(m/K)" ,"" )
            self.PAIGBINV1_given = 0

        if params_dict.get('LBIGBINV') != None:
            val =  params_dict['LBIGBINV']
            self.LBIGBINV     = self.MPRnb(val,"((F*s^2/g)^0.5)/V"     ,"" )
            self.LBIGBINV_given = 1
        else:
            self.LBIGBINV     = self.MPRnb(0.0            ,"((F*s^2/g)^0.5)/V"     ,"" )
            self.LBIGBINV_given = 0

        if params_dict.get('NBIGBINV') != None:
            val =  params_dict['NBIGBINV']
            self.NBIGBINV     = self.MPRnb(val,"((F*s^2/g)^0.5)/V"     ,"" )
            self.NBIGBINV_given = 1
        else:
            self.NBIGBINV     = self.MPRnb(0.0            ,"((F*s^2/g)^0.5)/V"     ,"" )
            self.NBIGBINV_given = 0

        if params_dict.get('PBIGBINV') != None:
            val =  params_dict['PBIGBINV']
            self.PBIGBINV     = self.MPRnb(val,"((F*s^2/g)^0.5)*(m/V)" ,"" )
            self.PBIGBINV_given = 1
        else:
            self.PBIGBINV     = self.MPRnb(0.0            ,"((F*s^2/g)^0.5)*(m/V)" ,"" )
            self.PBIGBINV_given = 0

        if params_dict.get('LCIGBINV') != None:
            val =  params_dict['LCIGBINV']
            self.LCIGBINV     = self.MPRnb(val,"m/V"                   ,"" )
            self.LCIGBINV_given = 1
        else:
            self.LCIGBINV     = self.MPRnb(0.0            ,"m/V"                   ,"" )
            self.LCIGBINV_given = 0

        if params_dict.get('NCIGBINV') != None:
            val =  params_dict['NCIGBINV']
            self.NCIGBINV     = self.MPRnb(val,"m/V"                   ,"" )
            self.NCIGBINV_given = 1
        else:
            self.NCIGBINV     = self.MPRnb(0.0            ,"m/V"                   ,"" )
            self.NCIGBINV_given = 0

        if params_dict.get('PCIGBINV') != None:
            val =  params_dict['PCIGBINV']
            self.PCIGBINV     = self.MPRnb(val,"(m^2)/V"               ,"" )
            self.PCIGBINV_given = 1
        else:
            self.PCIGBINV     = self.MPRnb(0.0            ,"(m^2)/V"               ,"" )
            self.PCIGBINV_given = 0

        if params_dict.get('LEIGBINV') != None:
            val =  params_dict['LEIGBINV']
            self.LEIGBINV     = self.MPRnb(val,"m*V"                   ,"" )
            self.LEIGBINV_given = 1
        else:
            self.LEIGBINV     = self.MPRnb(0.0            ,"m*V"                   ,"" )
            self.LEIGBINV_given = 0

        if params_dict.get('NEIGBINV') != None:
            val =  params_dict['NEIGBINV']
            self.NEIGBINV     = self.MPRnb(val,"m*V"                   ,"" )
            self.NEIGBINV_given = 1
        else:
            self.NEIGBINV     = self.MPRnb(0.0            ,"m*V"                   ,"" )
            self.NEIGBINV_given = 0

        if params_dict.get('PEIGBINV') != None:
            val =  params_dict['PEIGBINV']
            self.PEIGBINV     = self.MPRnb(val,"(m^2)*V"               ,"" )
            self.PEIGBINV_given = 1
        else:
            self.PEIGBINV     = self.MPRnb(0.0            ,"(m^2)*V"               ,"" )
            self.PEIGBINV_given = 0

        if params_dict.get('LNIGBINV') != None:
            val =  params_dict['LNIGBINV']
            self.LNIGBINV     = self.MPRnb(val,""                      ,"" )
            self.LNIGBINV_given = 1
        else:
            self.LNIGBINV     = self.MPRnb(0.0            ,""                      ,"" )
            self.LNIGBINV_given = 0

        if params_dict.get('NNIGBINV') != None:
            val =  params_dict['NNIGBINV']
            self.NNIGBINV     = self.MPRnb(val,""                      ,"" )
            self.NNIGBINV_given = 1
        else:
            self.NNIGBINV     = self.MPRnb(0.0            ,""                      ,"" )
            self.NNIGBINV_given = 0

        if params_dict.get('PNIGBINV') != None:
            val =  params_dict['PNIGBINV']
            self.PNIGBINV     = self.MPRnb(val,""                      ,"" )
            self.PNIGBINV_given = 1
        else:
            self.PNIGBINV     = self.MPRnb(0.0            ,""                      ,"" )
            self.PNIGBINV_given = 0

        if params_dict.get('LAIGBACC') != None:
            val =  params_dict['LAIGBACC']
            self.LAIGBACC     = self.MPRnb(val,"(F*s^2/g)^0.5"         ,"" )
            self.LAIGBACC_given = 1
        else:
            self.LAIGBACC     = self.MPRnb(0.0            ,"(F*s^2/g)^0.5"         ,"" )
            self.LAIGBACC_given = 0

        if params_dict.get('NAIGBACC') != None:
            val =  params_dict['NAIGBACC']
            self.NAIGBACC     = self.MPRnb(val,"(F*s^2/g)^0.5"         ,"" )
            self.NAIGBACC_given = 1
        else:
            self.NAIGBACC     = self.MPRnb(0.0            ,"(F*s^2/g)^0.5"         ,"" )
            self.NAIGBACC_given = 0

        if params_dict.get('PAIGBACC') != None:
            val =  params_dict['PAIGBACC']
            self.PAIGBACC     = self.MPRnb(val,"((F*s^2/g)^0.5)*m"     ,"" )
            self.PAIGBACC_given = 1
        else:
            self.PAIGBACC     = self.MPRnb(0.0            ,"((F*s^2/g)^0.5)*m"     ,"" )
            self.PAIGBACC_given = 0

        if params_dict.get('LAIGBACC1') != None:
            val =  params_dict['LAIGBACC1']
            self.LAIGBACC1    = self.MPRnb(val,"((F*s^2/g)^0.5)/K"     ,"" )
            self.LAIGBACC1_given = 1
        else:
            self.LAIGBACC1    = self.MPRnb(0.0            ,"((F*s^2/g)^0.5)/K"     ,"" )
            self.LAIGBACC1_given = 0

        if params_dict.get('NAIGBACC1') != None:
            val =  params_dict['NAIGBACC1']
            self.NAIGBACC1    = self.MPRnb(val,"((F*s^2/g)^0.5)/K"     ,"" )
            self.NAIGBACC1_given = 1
        else:
            self.NAIGBACC1    = self.MPRnb(0.0            ,"((F*s^2/g)^0.5)/K"     ,"" )
            self.NAIGBACC1_given = 0

        if params_dict.get('PAIGBACC1') != None:
            val =  params_dict['PAIGBACC1']
            self.PAIGBACC1    = self.MPRnb(val,"((F*s^2/g)^0.5)*(m/K)" ,"" )
            self.PAIGBACC1_given = 1
        else:
            self.PAIGBACC1    = self.MPRnb(0.0            ,"((F*s^2/g)^0.5)*(m/K)" ,"" )
            self.PAIGBACC1_given = 0

        if params_dict.get('LBIGBACC') != None:
            val =  params_dict['LBIGBACC']
            self.LBIGBACC     = self.MPRnb(val,"((F*s^2/g)^0.5)/V"     ,"" )
            self.LBIGBACC_given = 1
        else:
            self.LBIGBACC     = self.MPRnb(0.0            ,"((F*s^2/g)^0.5)/V"     ,"" )
            self.LBIGBACC_given = 0

        if params_dict.get('NBIGBACC') != None:
            val =  params_dict['NBIGBACC']
            self.NBIGBACC     = self.MPRnb(val,"((F*s^2/g)^0.5)/V"     ,"" )
            self.NBIGBACC_given = 1
        else:
            self.NBIGBACC     = self.MPRnb(0.0            ,"((F*s^2/g)^0.5)/V"     ,"" )
            self.NBIGBACC_given = 0

        if params_dict.get('PBIGBACC') != None:
            val =  params_dict['PBIGBACC']
            self.PBIGBACC     = self.MPRnb(val,"((F*s^2/g)^0.5)*(m/V)" ,"" )
            self.PBIGBACC_given = 1
        else:
            self.PBIGBACC     = self.MPRnb(0.0            ,"((F*s^2/g)^0.5)*(m/V)" ,"" )
            self.PBIGBACC_given = 0

        if params_dict.get('LCIGBACC') != None:
            val =  params_dict['LCIGBACC']
            self.LCIGBACC     = self.MPRnb(val,"m/V"                   ,"" )
            self.LCIGBACC_given = 1
        else:
            self.LCIGBACC     = self.MPRnb(0.0            ,"m/V"                   ,"" )
            self.LCIGBACC_given = 0

        if params_dict.get('NCIGBACC') != None:
            val =  params_dict['NCIGBACC']
            self.NCIGBACC     = self.MPRnb(val,"m/V"                   ,"" )
            self.NCIGBACC_given = 1
        else:
            self.NCIGBACC     = self.MPRnb(0.0            ,"m/V"                   ,"" )
            self.NCIGBACC_given = 0

        if params_dict.get('PCIGBACC') != None:
            val =  params_dict['PCIGBACC']
            self.PCIGBACC     = self.MPRnb(val,"(m^2)/V"               ,"" )
            self.PCIGBACC_given = 1
        else:
            self.PCIGBACC     = self.MPRnb(0.0            ,"(m^2)/V"               ,"" )
            self.PCIGBACC_given = 0

        if params_dict.get('LNIGBACC') != None:
            val =  params_dict['LNIGBACC']
            self.LNIGBACC     = self.MPRnb(val,""                      ,"" )
            self.LNIGBACC_given = 1
        else:
            self.LNIGBACC     = self.MPRnb(0.0            ,""                      ,"" )
            self.LNIGBACC_given = 0

        if params_dict.get('NNIGBACC') != None:
            val =  params_dict['NNIGBACC']
            self.NNIGBACC     = self.MPRnb(val,""                      ,"" )
            self.NNIGBACC_given = 1
        else:
            self.NNIGBACC     = self.MPRnb(0.0            ,""                      ,"" )
            self.NNIGBACC_given = 0

        if params_dict.get('PNIGBACC') != None:
            val =  params_dict['PNIGBACC']
            self.PNIGBACC     = self.MPRnb(val,""                      ,"" )
            self.PNIGBACC_given = 1
        else:
            self.PNIGBACC     = self.MPRnb(0.0            ,""                      ,"" )
            self.PNIGBACC_given = 0

        if params_dict.get('LAIGC') != None:
            val =  params_dict['LAIGC']
            self.LAIGC        = self.MPRnb(val,"(F*s^2/g)^0.5"         ,"" )
            self.LAIGC_given = 1
        else:
            self.LAIGC        = self.MPRnb(0.0            ,"(F*s^2/g)^0.5"         ,"" )
            self.LAIGC_given = 0

        if params_dict.get('NAIGC') != None:
            val =  params_dict['NAIGC']
            self.NAIGC        = self.MPRnb(val,"(F*s^2/g)^0.5"         ,"" )
            self.NAIGC_given = 1
        else:
            self.NAIGC        = self.MPRnb(0.0            ,"(F*s^2/g)^0.5"         ,"" )
            self.NAIGC_given = 0

        if params_dict.get('PAIGC') != None:
            val =  params_dict['PAIGC']
            self.PAIGC        = self.MPRnb(val,"((F*s^2/g)^0.5)*m"     ,"" )
            self.PAIGC_given = 1
        else:
            self.PAIGC        = self.MPRnb(0.0            ,"((F*s^2/g)^0.5)*m"     ,"" )
            self.PAIGC_given = 0

        if params_dict.get('LAIGC1') != None:
            val =  params_dict['LAIGC1']
            self.LAIGC1       = self.MPRnb(val,"((F*s^2/g)^0.5)/K"     ,"" )
            self.LAIGC1_given = 1
        else:
            self.LAIGC1       = self.MPRnb(0.0            ,"((F*s^2/g)^0.5)/K"     ,"" )
            self.LAIGC1_given = 0

        if params_dict.get('NAIGC1') != None:
            val =  params_dict['NAIGC1']
            self.NAIGC1       = self.MPRnb(val,"((F*s^2/g)^0.5)/K"     ,"" )
            self.NAIGC1_given = 1
        else:
            self.NAIGC1       = self.MPRnb(0.0            ,"((F*s^2/g)^0.5)/K"     ,"" )
            self.NAIGC1_given = 0

        if params_dict.get('PAIGC1') != None:
            val =  params_dict['PAIGC1']
            self.PAIGC1       = self.MPRnb(val,"((F*s^2/g)^0.5)*(m/K)" ,"" )
            self.PAIGC1_given = 1
        else:
            self.PAIGC1       = self.MPRnb(0.0            ,"((F*s^2/g)^0.5)*(m/K)" ,"" )
            self.PAIGC1_given = 0

        if params_dict.get('LBIGC') != None:
            val =  params_dict['LBIGC']
            self.LBIGC        = self.MPRnb(val,"((F*s^2/g)^0.5)/V"     ,"" )
            self.LBIGC_given = 1
        else:
            self.LBIGC        = self.MPRnb(0.0            ,"((F*s^2/g)^0.5)/V"     ,"" )
            self.LBIGC_given = 0

        if params_dict.get('NBIGC') != None:
            val =  params_dict['NBIGC']
            self.NBIGC        = self.MPRnb(val,"((F*s^2/g)^0.5)/V"     ,"" )
            self.NBIGC_given = 1
        else:
            self.NBIGC        = self.MPRnb(0.0            ,"((F*s^2/g)^0.5)/V"     ,"" )
            self.NBIGC_given = 0

        if params_dict.get('PBIGC') != None:
            val =  params_dict['PBIGC']
            self.PBIGC        = self.MPRnb(val,"((F*s^2/g)^0.5)*(m/V)" ,"" )
            self.PBIGC_given = 1
        else:
            self.PBIGC        = self.MPRnb(0.0            ,"((F*s^2/g)^0.5)*(m/V)" ,"" )
            self.PBIGC_given = 0

        if params_dict.get('LCIGC') != None:
            val =  params_dict['LCIGC']
            self.LCIGC        = self.MPRnb(val,"m/V"                   ,"" )
            self.LCIGC_given = 1
        else:
            self.LCIGC        = self.MPRnb(0.0            ,"m/V"                   ,"" )
            self.LCIGC_given = 0

        if params_dict.get('NCIGC') != None:
            val =  params_dict['NCIGC']
            self.NCIGC        = self.MPRnb(val,"m/V"                   ,"" )
            self.NCIGC_given = 1
        else:
            self.NCIGC        = self.MPRnb(0.0            ,"m/V"                   ,"" )
            self.NCIGC_given = 0

        if params_dict.get('PCIGC') != None:
            val =  params_dict['PCIGC']
            self.PCIGC        = self.MPRnb(val,"(m^2)/V"               ,"" )
            self.PCIGC_given = 1
        else:
            self.PCIGC        = self.MPRnb(0.0            ,"(m^2)/V"               ,"" )
            self.PCIGC_given = 0

        if params_dict.get('LPIGCD') != None:
            val =  params_dict['LPIGCD']
            self.LPIGCD       = self.MPRnb(val,""                      ,"" )
            self.LPIGCD_given = 1
        else:
            self.LPIGCD       = self.MPRnb(0.0            ,""                      ,"" )
            self.LPIGCD_given = 0

        if params_dict.get('NPIGCD') != None:
            val =  params_dict['NPIGCD']
            self.NPIGCD       = self.MPRnb(val,""                      ,"" )
            self.NPIGCD_given = 1
        else:
            self.NPIGCD       = self.MPRnb(0.0            ,""                      ,"" )
            self.NPIGCD_given = 0

        if params_dict.get('PPIGCD') != None:
            val =  params_dict['PPIGCD']
            self.PPIGCD       = self.MPRnb(val,""                      ,"" )
            self.PPIGCD_given = 1
        else:
            self.PPIGCD       = self.MPRnb(0.0            ,""                      ,"" )
            self.PPIGCD_given = 0

        if params_dict.get('LAIGS') != None:
            val =  params_dict['LAIGS']
            self.LAIGS        = self.MPRnb(val,"(F*s^2/g)^0.5"         ,"" )
            self.LAIGS_given = 1
        else:
            self.LAIGS        = self.MPRnb(0.0            ,"(F*s^2/g)^0.5"         ,"" )
            self.LAIGS_given = 0

        if params_dict.get('NAIGS') != None:
            val =  params_dict['NAIGS']
            self.NAIGS        = self.MPRnb(val,"(F*s^2/g)^0.5"         ,"" )
            self.NAIGS_given = 1
        else:
            self.NAIGS        = self.MPRnb(0.0            ,"(F*s^2/g)^0.5"         ,"" )
            self.NAIGS_given = 0

        if params_dict.get('PAIGS') != None:
            val =  params_dict['PAIGS']
            self.PAIGS        = self.MPRnb(val,"((F*s^2/g)^0.5)*m"     ,"" )
            self.PAIGS_given = 1
        else:
            self.PAIGS        = self.MPRnb(0.0            ,"((F*s^2/g)^0.5)*m"     ,"" )
            self.PAIGS_given = 0

        if params_dict.get('LAIGS1') != None:
            val =  params_dict['LAIGS1']
            self.LAIGS1       = self.MPRnb(val,"((F*s^2/g)^0.5)/K"     ,"" )
            self.LAIGS1_given = 1
        else:
            self.LAIGS1       = self.MPRnb(0.0            ,"((F*s^2/g)^0.5)/K"     ,"" )
            self.LAIGS1_given = 0

        if params_dict.get('NAIGS1') != None:
            val =  params_dict['NAIGS1']
            self.NAIGS1       = self.MPRnb(val,"((F*s^2/g)^0.5)/K"     ,"" )
            self.NAIGS1_given = 1
        else:
            self.NAIGS1       = self.MPRnb(0.0            ,"((F*s^2/g)^0.5)/K"     ,"" )
            self.NAIGS1_given = 0

        if params_dict.get('PAIGS1') != None:
            val =  params_dict['PAIGS1']
            self.PAIGS1       = self.MPRnb(val,"((F*s^2/g)^0.5)*(m/K)" ,"" )
            self.PAIGS1_given = 1
        else:
            self.PAIGS1       = self.MPRnb(0.0            ,"((F*s^2/g)^0.5)*(m/K)" ,"" )
            self.PAIGS1_given = 0

        if params_dict.get('LBIGS') != None:
            val =  params_dict['LBIGS']
            self.LBIGS        = self.MPRnb(val,"((F*s^2/g)^0.5)/V"     ,"" )
            self.LBIGS_given = 1
        else:
            self.LBIGS        = self.MPRnb(0.0            ,"((F*s^2/g)^0.5)/V"     ,"" )
            self.LBIGS_given = 0

        if params_dict.get('NBIGS') != None:
            val =  params_dict['NBIGS']
            self.NBIGS        = self.MPRnb(val,"((F*s^2/g)^0.5)/V"     ,"" )
            self.NBIGS_given = 1
        else:
            self.NBIGS        = self.MPRnb(0.0            ,"((F*s^2/g)^0.5)/V"     ,"" )
            self.NBIGS_given = 0

        if params_dict.get('PBIGS') != None:
            val =  params_dict['PBIGS']
            self.PBIGS        = self.MPRnb(val,"((F*s^2/g)^0.5)*(m/V)" ,"" )
            self.PBIGS_given = 1
        else:
            self.PBIGS        = self.MPRnb(0.0            ,"((F*s^2/g)^0.5)*(m/V)" ,"" )
            self.PBIGS_given = 0

        if params_dict.get('LCIGS') != None:
            val =  params_dict['LCIGS']
            self.LCIGS        = self.MPRnb(val,"m/V"                   ,"" )
            self.LCIGS_given = 1
        else:
            self.LCIGS        = self.MPRnb(0.0            ,"m/V"                   ,"" )
            self.LCIGS_given = 0

        if params_dict.get('NCIGS') != None:
            val =  params_dict['NCIGS']
            self.NCIGS        = self.MPRnb(val,"m/V"                   ,"" )
            self.NCIGS_given = 1
        else:
            self.NCIGS        = self.MPRnb(0.0            ,"m/V"                   ,"" )
            self.NCIGS_given = 0

        if params_dict.get('PCIGS') != None:
            val =  params_dict['PCIGS']
            self.PCIGS        = self.MPRnb(val,"(m^2)/V"               ,"" )
            self.PCIGS_given = 1
        else:
            self.PCIGS        = self.MPRnb(0.0            ,"(m^2)/V"               ,"" )
            self.PCIGS_given = 0

        if params_dict.get('LAIGD') != None:
            val =  params_dict['LAIGD']
            self.LAIGD        = self.MPRnb(val,"(F*s^2/g)^0.5"         ,"" )
            self.LAIGD_given = 1
        else:
            self.LAIGD        = self.MPRnb(self.LAIGS          ,"(F*s^2/g)^0.5"         ,"" )
            self.LAIGD_given = 0

        if params_dict.get('NAIGD') != None:
            val =  params_dict['NAIGD']
            self.NAIGD        = self.MPRnb(val,"(F*s^2/g)^0.5"         ,"" )
            self.NAIGD_given = 1
        else:
            self.NAIGD        = self.MPRnb(self.NAIGS          ,"(F*s^2/g)^0.5"         ,"" )
            self.NAIGD_given = 0

        if params_dict.get('PAIGD') != None:
            val =  params_dict['PAIGD']
            self.PAIGD        = self.MPRnb(val,"((F*s^2/g)^0.5)*m"     ,"" )
            self.PAIGD_given = 1
        else:
            self.PAIGD        = self.MPRnb(self.PAIGS          ,"((F*s^2/g)^0.5)*m"     ,"" )
            self.PAIGD_given = 0

        if params_dict.get('LAIGD1') != None:
            val =  params_dict['LAIGD1']
            self.LAIGD1       = self.MPRnb(val,"((F*s^2/g)^0.5)/K"     ,"" )
            self.LAIGD1_given = 1
        else:
            self.LAIGD1       = self.MPRnb(self.LAIGS1         ,"((F*s^2/g)^0.5)/K"     ,"" )
            self.LAIGD1_given = 0

        if params_dict.get('NAIGD1') != None:
            val =  params_dict['NAIGD1']
            self.NAIGD1       = self.MPRnb(val,"((F*s^2/g)^0.5)/K"     ,"" )
            self.NAIGD1_given = 1
        else:
            self.NAIGD1       = self.MPRnb(self.NAIGS1         ,"((F*s^2/g)^0.5)/K"     ,"" )
            self.NAIGD1_given = 0

        if params_dict.get('PAIGD1') != None:
            val =  params_dict['PAIGD1']
            self.PAIGD1       = self.MPRnb(val,"((F*s^2/g)^0.5)*(m/K)" ,"" )
            self.PAIGD1_given = 1
        else:
            self.PAIGD1       = self.MPRnb(self.PAIGS1         ,"((F*s^2/g)^0.5)*(m/K)" ,"" )
            self.PAIGD1_given = 0

        if params_dict.get('LBIGD') != None:
            val =  params_dict['LBIGD']
            self.LBIGD        = self.MPRnb(val,"((F*s^2/g)^0.5)/V"     ,"" )
            self.LBIGD_given = 1
        else:
            self.LBIGD        = self.MPRnb(self.LBIGS          ,"((F*s^2/g)^0.5)/V"     ,"" )
            self.LBIGD_given = 0

        if params_dict.get('NBIGD') != None:
            val =  params_dict['NBIGD']
            self.NBIGD        = self.MPRnb(val,"((F*s^2/g)^0.5)/V"     ,"" )
            self.NBIGD_given = 1
        else:
            self.NBIGD        = self.MPRnb(self.NBIGS          ,"((F*s^2/g)^0.5)/V"     ,"" )
            self.NBIGD_given = 0

        if params_dict.get('PBIGD') != None:
            val =  params_dict['PBIGD']
            self.PBIGD        = self.MPRnb(val,"((F*s^2/g)^0.5)*(m/V)" ,"" )
            self.PBIGD_given = 1
        else:
            self.PBIGD        = self.MPRnb(self.PBIGS          ,"((F*s^2/g)^0.5)*(m/V)" ,"" )
            self.PBIGD_given = 0

        if params_dict.get('LCIGD') != None:
            val =  params_dict['LCIGD']
            self.LCIGD        = self.MPRnb(val,"m/V"                   ,"" )
            self.LCIGD_given = 1
        else:
            self.LCIGD        = self.MPRnb(self.LCIGS          ,"m/V"                   ,"" )
            self.LCIGD_given = 0

        if params_dict.get('NCIGD') != None:
            val =  params_dict['NCIGD']
            self.NCIGD        = self.MPRnb(val,"m/V"                   ,"" )
            self.NCIGD_given = 1
        else:
            self.NCIGD        = self.MPRnb(self.NCIGS          ,"m/V"                   ,"" )
            self.NCIGD_given = 0

        if params_dict.get('PCIGD') != None:
            val =  params_dict['PCIGD']
            self.PCIGD        = self.MPRnb(val,"(m^2)/V"               ,"" )
            self.PCIGD_given = 1
        else:
            self.PCIGD        = self.MPRnb(self.PCIGS          ,"(m^2)/V"               ,"" )
            self.PCIGD_given = 0

        if params_dict.get('LNTOX') != None:
            val =  params_dict['LNTOX']
            self.LNTOX        = self.MPRnb(val,""                      ,"" )
            self.LNTOX_given = 1
        else:
            self.LNTOX        = self.MPRnb(0.0            ,""                      ,"" )
            self.LNTOX_given = 0

        if params_dict.get('NNTOX') != None:
            val =  params_dict['NNTOX']
            self.NNTOX        = self.MPRnb(val,""                      ,"" )
            self.NNTOX_given = 1
        else:
            self.NNTOX        = self.MPRnb(0.0            ,""                      ,"" )
            self.NNTOX_given = 0

        if params_dict.get('PNTOX') != None:
            val =  params_dict['PNTOX']
            self.PNTOX        = self.MPRnb(val,""                      ,"" )
            self.PNTOX_given = 1
        else:
            self.PNTOX        = self.MPRnb(0.0            ,""                      ,"" )
            self.PNTOX_given = 0

        if params_dict.get('LPOXEDGE') != None:
            val =  params_dict['LPOXEDGE']
            self.LPOXEDGE     = self.MPRnb(val,""                      ,"" )
            self.LPOXEDGE_given = 1
        else:
            self.LPOXEDGE     = self.MPRnb(0.0            ,""                      ,"" )
            self.LPOXEDGE_given = 0

        if params_dict.get('NPOXEDGE') != None:
            val =  params_dict['NPOXEDGE']
            self.NPOXEDGE     = self.MPRnb(val,""                      ,"" )
            self.NPOXEDGE_given = 1
        else:
            self.NPOXEDGE     = self.MPRnb(0.0            ,""                      ,"" )
            self.NPOXEDGE_given = 0

        if params_dict.get('PPOXEDGE') != None:
            val =  params_dict['PPOXEDGE']
            self.PPOXEDGE     = self.MPRnb(val,""                      ,"" )
            self.PPOXEDGE_given = 1
        else:
            self.PPOXEDGE     = self.MPRnb(0.0            ,""                      ,"" )
            self.PPOXEDGE_given = 0

        if params_dict.get('LAGISL') != None:
            val =  params_dict['LAGISL']
            self.LAGISL       = self.MPRnb(val,"m/ohm"                 ,"" )
            self.LAGISL_given = 1
        else:
            self.LAGISL       = self.MPRnb(0.0            ,"m/ohm"                 ,"" )
            self.LAGISL_given = 0

        if params_dict.get('NAGISL') != None:
            val =  params_dict['NAGISL']
            self.NAGISL       = self.MPRnb(val,"m/ohm"                 ,"" )
            self.NAGISL_given = 1
        else:
            self.NAGISL       = self.MPRnb(0.0            ,"m/ohm"                 ,"" )
            self.NAGISL_given = 0

        if params_dict.get('PAGISL') != None:
            val =  params_dict['PAGISL']
            self.PAGISL       = self.MPRnb(val,"(m^2)/ohm"             ,"" )
            self.PAGISL_given = 1
        else:
            self.PAGISL       = self.MPRnb(0.0            ,"(m^2)/ohm"             ,"" )
            self.PAGISL_given = 0

        if params_dict.get('LBGISL') != None:
            val =  params_dict['LBGISL']
            self.LBGISL       = self.MPRnb(val,"V"                     ,"" )
            self.LBGISL_given = 1
        else:
            self.LBGISL       = self.MPRnb(0.0            ,"V"                     ,"" )
            self.LBGISL_given = 0

        if params_dict.get('NBGISL') != None:
            val =  params_dict['NBGISL']
            self.NBGISL       = self.MPRnb(val,"V"                     ,"" )
            self.NBGISL_given = 1
        else:
            self.NBGISL       = self.MPRnb(0.0            ,"V"                     ,"" )
            self.NBGISL_given = 0

        if params_dict.get('PBGISL') != None:
            val =  params_dict['PBGISL']
            self.PBGISL       = self.MPRnb(val,"m*V"                   ,"" )
            self.PBGISL_given = 1
        else:
            self.PBGISL       = self.MPRnb(0.0            ,"m*V"                   ,"" )
            self.PBGISL_given = 0

        if params_dict.get('LCGISL') != None:
            val =  params_dict['LCGISL']
            self.LCGISL       = self.MPRnb(val,"m*(V^3)"               ,"" )
            self.LCGISL_given = 1
        else:
            self.LCGISL       = self.MPRnb(0.0            ,"m*(V^3)"               ,"" )
            self.LCGISL_given = 0

        if params_dict.get('NCGISL') != None:
            val =  params_dict['NCGISL']
            self.NCGISL       = self.MPRnb(val,"m*(V^3)"               ,"" )
            self.NCGISL_given = 1
        else:
            self.NCGISL       = self.MPRnb(0.0            ,"m*(V^3)"               ,"" )
            self.NCGISL_given = 0

        if params_dict.get('PCGISL') != None:
            val =  params_dict['PCGISL']
            self.PCGISL       = self.MPRnb(val,"(m^2)*(V^3)"           ,"" )
            self.PCGISL_given = 1
        else:
            self.PCGISL       = self.MPRnb(0.0            ,"(m^2)*(V^3)"           ,"" )
            self.PCGISL_given = 0

        if params_dict.get('LEGISL') != None:
            val =  params_dict['LEGISL']
            self.LEGISL       = self.MPRnb(val,"m*V"                   ,"" )
            self.LEGISL_given = 1
        else:
            self.LEGISL       = self.MPRnb(0.0            ,"m*V"                   ,"" )
            self.LEGISL_given = 0

        if params_dict.get('NEGISL') != None:
            val =  params_dict['NEGISL']
            self.NEGISL       = self.MPRnb(val,"m*V"                   ,"" )
            self.NEGISL_given = 1
        else:
            self.NEGISL       = self.MPRnb(0.0            ,"m*V"                   ,"" )
            self.NEGISL_given = 0

        if params_dict.get('PEGISL') != None:
            val =  params_dict['PEGISL']
            self.PEGISL       = self.MPRnb(val,"(m^2)*V"               ,"" )
            self.PEGISL_given = 1
        else:
            self.PEGISL       = self.MPRnb(0.0            ,"(m^2)*V"               ,"" )
            self.PEGISL_given = 0

        if params_dict.get('LPGISL') != None:
            val =  params_dict['LPGISL']
            self.LPGISL       = self.MPRnb(val,""                      ,"" )
            self.LPGISL_given = 1
        else:
            self.LPGISL       = self.MPRnb(0.0            ,""                      ,"" )
            self.LPGISL_given = 0

        if params_dict.get('NPGISL') != None:
            val =  params_dict['NPGISL']
            self.NPGISL       = self.MPRnb(val,""                      ,"" )
            self.NPGISL_given = 1
        else:
            self.NPGISL       = self.MPRnb(0.0            ,""                      ,"" )
            self.NPGISL_given = 0

        if params_dict.get('PPGISL') != None:
            val =  params_dict['PPGISL']
            self.PPGISL       = self.MPRnb(val,""                      ,"" )
            self.PPGISL_given = 1
        else:
            self.PPGISL       = self.MPRnb(0.0            ,""                      ,"" )
            self.PPGISL_given = 0

        if params_dict.get('LAGIDL') != None:
            val =  params_dict['LAGIDL']
            self.LAGIDL       = self.MPRnb(val,"m/ohm"                 ,"" )
            self.LAGIDL_given = 1
        else:
            self.LAGIDL       = self.MPRnb(self.LAGISL         ,"m/ohm"                 ,"" )
            self.LAGIDL_given = 0

        if params_dict.get('NAGIDL') != None:
            val =  params_dict['NAGIDL']
            self.NAGIDL       = self.MPRnb(val,"m/ohm"                 ,"" )
            self.NAGIDL_given = 1
        else:
            self.NAGIDL       = self.MPRnb(self.NAGISL         ,"m/ohm"                 ,"" )
            self.NAGIDL_given = 0

        if params_dict.get('PAGIDL') != None:
            val =  params_dict['PAGIDL']
            self.PAGIDL       = self.MPRnb(val,"(m^2)/ohm"             ,"" )
            self.PAGIDL_given = 1
        else:
            self.PAGIDL       = self.MPRnb(self.PAGISL         ,"(m^2)/ohm"             ,"" )
            self.PAGIDL_given = 0

        if params_dict.get('LBGIDL') != None:
            val =  params_dict['LBGIDL']
            self.LBGIDL       = self.MPRnb(val,"V"                     ,"" )
            self.LBGIDL_given = 1
        else:
            self.LBGIDL       = self.MPRnb(self.LBGISL         ,"V"                     ,"" )
            self.LBGIDL_given = 0

        if params_dict.get('NBGIDL') != None:
            val =  params_dict['NBGIDL']
            self.NBGIDL       = self.MPRnb(val,"V"                     ,"" )
            self.NBGIDL_given = 1
        else:
            self.NBGIDL       = self.MPRnb(self.NBGISL         ,"V"                     ,"" )
            self.NBGIDL_given = 0

        if params_dict.get('PBGIDL') != None:
            val =  params_dict['PBGIDL']
            self.PBGIDL       = self.MPRnb(val,"m*V"                   ,"" )
            self.PBGIDL_given = 1
        else:
            self.PBGIDL       = self.MPRnb(self.PBGISL         ,"m*V"                   ,"" )
            self.PBGIDL_given = 0

        if params_dict.get('LCGIDL') != None:
            val =  params_dict['LCGIDL']
            self.LCGIDL       = self.MPRnb(val,"m*(V^3)"               ,"" )
            self.LCGIDL_given = 1
        else:
            self.LCGIDL       = self.MPRnb(self.LCGISL         ,"m*(V^3)"               ,"" )
            self.LCGIDL_given = 0

        if params_dict.get('NCGIDL') != None:
            val =  params_dict['NCGIDL']
            self.NCGIDL       = self.MPRnb(val,"m*(V^3)"               ,"" )
            self.NCGIDL_given = 1
        else:
            self.NCGIDL       = self.MPRnb(self.NCGISL         ,"m*(V^3)"               ,"" )
            self.NCGIDL_given = 0

        if params_dict.get('PCGIDL') != None:
            val =  params_dict['PCGIDL']
            self.PCGIDL       = self.MPRnb(val,"(m^2)*(V^3)"           ,"" )
            self.PCGIDL_given = 1
        else:
            self.PCGIDL       = self.MPRnb(self.PCGISL         ,"(m^2)*(V^3)"           ,"" )
            self.PCGIDL_given = 0

        if params_dict.get('LEGIDL') != None:
            val =  params_dict['LEGIDL']
            self.LEGIDL       = self.MPRnb(val,"m*V"                   ,"" )
            self.LEGIDL_given = 1
        else:
            self.LEGIDL       = self.MPRnb(self.LEGISL         ,"m*V"                   ,"" )
            self.LEGIDL_given = 0

        if params_dict.get('NEGIDL') != None:
            val =  params_dict['NEGIDL']
            self.NEGIDL       = self.MPRnb(val,"m*V"                   ,"" )
            self.NEGIDL_given = 1
        else:
            self.NEGIDL       = self.MPRnb(self.NEGISL         ,"m*V"                   ,"" )
            self.NEGIDL_given = 0

        if params_dict.get('PEGIDL') != None:
            val =  params_dict['PEGIDL']
            self.PEGIDL       = self.MPRnb(val,"(m^2)*V"               ,"" )
            self.PEGIDL_given = 1
        else:
            self.PEGIDL       = self.MPRnb(self.PEGISL         ,"(m^2)*V"               ,"" )
            self.PEGIDL_given = 0

        if params_dict.get('LPGIDL') != None:
            val =  params_dict['LPGIDL']
            self.LPGIDL       = self.MPRnb(val,""                      ,"" )
            self.LPGIDL_given = 1
        else:
            self.LPGIDL       = self.MPRnb(self.LPGISL         ,""                      ,"" )
            self.LPGIDL_given = 0

        if params_dict.get('NPGIDL') != None:
            val =  params_dict['NPGIDL']
            self.NPGIDL       = self.MPRnb(val,""                      ,"" )
            self.NPGIDL_given = 1
        else:
            self.NPGIDL       = self.MPRnb(self.NPGISL         ,""                      ,"" )
            self.NPGIDL_given = 0

        if params_dict.get('PPGIDL') != None:
            val =  params_dict['PPGIDL']
            self.PPGIDL       = self.MPRnb(val,""                      ,"" )
            self.PPGIDL_given = 1
        else:
            self.PPGIDL       = self.MPRnb(self.PPGISL         ,""                      ,"" )
            self.PPGIDL_given = 0

        if params_dict.get('LALPHA0') != None:
            val =  params_dict['LALPHA0']
            self.LALPHA0      = self.MPRnb(val,"(m^2)/V"               ,"" )
            self.LALPHA0_given = 1
        else:
            self.LALPHA0      = self.MPRnb(0.0            ,"(m^2)/V"               ,"" )
            self.LALPHA0_given = 0

        if params_dict.get('NALPHA0') != None:
            val =  params_dict['NALPHA0']
            self.NALPHA0      = self.MPRnb(val,"(m^2)/V"               ,"" )
            self.NALPHA0_given = 1
        else:
            self.NALPHA0      = self.MPRnb(0.0            ,"(m^2)/V"               ,"" )
            self.NALPHA0_given = 0

        if params_dict.get('PALPHA0') != None:
            val =  params_dict['PALPHA0']
            self.PALPHA0      = self.MPRnb(val,"(m^3)/V"               ,"" )
            self.PALPHA0_given = 1
        else:
            self.PALPHA0      = self.MPRnb(0.0            ,"(m^3)/V"               ,"" )
            self.PALPHA0_given = 0

        if params_dict.get('LALPHA1') != None:
            val =  params_dict['LALPHA1']
            self.LALPHA1      = self.MPRnb(val,"m/V"                   ,"" )
            self.LALPHA1_given = 1
        else:
            self.LALPHA1      = self.MPRnb(0.0            ,"m/V"                   ,"" )
            self.LALPHA1_given = 0

        if params_dict.get('NALPHA1') != None:
            val =  params_dict['NALPHA1']
            self.NALPHA1      = self.MPRnb(val,"m/V"                   ,"" )
            self.NALPHA1_given = 1
        else:
            self.NALPHA1      = self.MPRnb(0.0            ,"m/V"                   ,"" )
            self.NALPHA1_given = 0

        if params_dict.get('PALPHA1') != None:
            val =  params_dict['PALPHA1']
            self.PALPHA1      = self.MPRnb(val,"(m^2)/V"               ,"" )
            self.PALPHA1_given = 1
        else:
            self.PALPHA1      = self.MPRnb(0.0            ,"(m^2)/V"               ,"" )
            self.PALPHA1_given = 0

        if params_dict.get('LALPHAII0') != None:
            val =  params_dict['LALPHAII0']
            self.LALPHAII0    = self.MPRnb(val,"(m^2)/V"               ,"" )
            self.LALPHAII0_given = 1
        else:
            self.LALPHAII0    = self.MPRnb(0.0            ,"(m^2)/V"               ,"" )
            self.LALPHAII0_given = 0

        if params_dict.get('NALPHAII0') != None:
            val =  params_dict['NALPHAII0']
            self.NALPHAII0    = self.MPRnb(val,"(m^2)/V"               ,"" )
            self.NALPHAII0_given = 1
        else:
            self.NALPHAII0    = self.MPRnb(0.0            ,"(m^2)/V"               ,"" )
            self.NALPHAII0_given = 0

        if params_dict.get('PALPHAII0') != None:
            val =  params_dict['PALPHAII0']
            self.PALPHAII0    = self.MPRnb(val,"(m^3)/V"               ,"" )
            self.PALPHAII0_given = 1
        else:
            self.PALPHAII0    = self.MPRnb(0.0            ,"(m^3)/V"               ,"" )
            self.PALPHAII0_given = 0

        if params_dict.get('LALPHAII1') != None:
            val =  params_dict['LALPHAII1']
            self.LALPHAII1    = self.MPRnb(val,"m/V"                   ,"" )
            self.LALPHAII1_given = 1
        else:
            self.LALPHAII1    = self.MPRnb(0.0            ,"m/V"                   ,"" )
            self.LALPHAII1_given = 0

        if params_dict.get('NALPHAII1') != None:
            val =  params_dict['NALPHAII1']
            self.NALPHAII1    = self.MPRnb(val,"m/V"                   ,"" )
            self.NALPHAII1_given = 1
        else:
            self.NALPHAII1    = self.MPRnb(0.0            ,"m/V"                   ,"" )
            self.NALPHAII1_given = 0

        if params_dict.get('PALPHAII1') != None:
            val =  params_dict['PALPHAII1']
            self.PALPHAII1    = self.MPRnb(val,"(m^2)/V"               ,"" )
            self.PALPHAII1_given = 1
        else:
            self.PALPHAII1    = self.MPRnb(0.0            ,"(m^2)/V"               ,"" )
            self.PALPHAII1_given = 0

        if params_dict.get('LBETA0') != None:
            val =  params_dict['LBETA0']
            self.LBETA0       = self.MPRnb(val,"m/V"                   ,"" )
            self.LBETA0_given = 1
        else:
            self.LBETA0       = self.MPRnb(0.0            ,"m/V"                   ,"" )
            self.LBETA0_given = 0

        if params_dict.get('NBETA0') != None:
            val =  params_dict['NBETA0']
            self.NBETA0       = self.MPRnb(val,"m/V"                   ,"" )
            self.NBETA0_given = 1
        else:
            self.NBETA0       = self.MPRnb(0.0            ,"m/V"                   ,"" )
            self.NBETA0_given = 0

        if params_dict.get('PBETA0') != None:
            val =  params_dict['PBETA0']
            self.PBETA0       = self.MPRnb(val,"(m^2)/V"               ,"" )
            self.PBETA0_given = 1
        else:
            self.PBETA0       = self.MPRnb(0.0            ,"(m^2)/V"               ,"" )
            self.PBETA0_given = 0

        if params_dict.get('LBETAII0') != None:
            val =  params_dict['LBETAII0']
            self.LBETAII0     = self.MPRnb(val,"m/V"                   ,"" )
            self.LBETAII0_given = 1
        else:
            self.LBETAII0     = self.MPRnb(0.0            ,"m/V"                   ,"" )
            self.LBETAII0_given = 0

        if params_dict.get('NBETAII0') != None:
            val =  params_dict['NBETAII0']
            self.NBETAII0     = self.MPRnb(val,"m/V"                   ,"" )
            self.NBETAII0_given = 1
        else:
            self.NBETAII0     = self.MPRnb(0.0            ,"m/V"                   ,"" )
            self.NBETAII0_given = 0

        if params_dict.get('PBETAII0') != None:
            val =  params_dict['PBETAII0']
            self.PBETAII0     = self.MPRnb(val,"(m^2)/V"               ,"" )
            self.PBETAII0_given = 1
        else:
            self.PBETAII0     = self.MPRnb(0.0            ,"(m^2)/V"               ,"" )
            self.PBETAII0_given = 0

        if params_dict.get('LBETAII1') != None:
            val =  params_dict['LBETAII1']
            self.LBETAII1     = self.MPRnb(val,""                      ,"" )
            self.LBETAII1_given = 1
        else:
            self.LBETAII1     = self.MPRnb(0.0            ,""                      ,"" )
            self.LBETAII1_given = 0

        if params_dict.get('NBETAII1') != None:
            val =  params_dict['NBETAII1']
            self.NBETAII1     = self.MPRnb(val,""                      ,"" )
            self.NBETAII1_given = 1
        else:
            self.NBETAII1     = self.MPRnb(0.0            ,""                      ,"" )
            self.NBETAII1_given = 0

        if params_dict.get('PBETAII1') != None:
            val =  params_dict['PBETAII1']
            self.PBETAII1     = self.MPRnb(val,""                      ,"" )
            self.PBETAII1_given = 1
        else:
            self.PBETAII1     = self.MPRnb(0.0            ,""                      ,"" )
            self.PBETAII1_given = 0

        if params_dict.get('LBETAII2') != None:
            val =  params_dict['LBETAII2']
            self.LBETAII2     = self.MPRnb(val,"m*V"                   ,"" )
            self.LBETAII2_given = 1
        else:
            self.LBETAII2     = self.MPRnb(0.0            ,"m*V"                   ,"" )
            self.LBETAII2_given = 0

        if params_dict.get('NBETAII2') != None:
            val =  params_dict['NBETAII2']
            self.NBETAII2     = self.MPRnb(val,"m*V"                   ,"" )
            self.NBETAII2_given = 1
        else:
            self.NBETAII2     = self.MPRnb(0.0            ,"m*V"                   ,"" )
            self.NBETAII2_given = 0

        if params_dict.get('PBETAII2') != None:
            val =  params_dict['PBETAII2']
            self.PBETAII2     = self.MPRnb(val,"(m^2)*V"               ,"" )
            self.PBETAII2_given = 1
        else:
            self.PBETAII2     = self.MPRnb(0.0            ,"(m^2)*V"               ,"" )
            self.PBETAII2_given = 0

        if params_dict.get('LESATII') != None:
            val =  params_dict['LESATII']
            self.LESATII      = self.MPRnb(val,"V"                     ,"" )
            self.LESATII_given = 1
        else:
            self.LESATII      = self.MPRnb(0.0            ,"V"                     ,"" )
            self.LESATII_given = 0

        if params_dict.get('NESATII') != None:
            val =  params_dict['NESATII']
            self.NESATII      = self.MPRnb(val,"V"                     ,"" )
            self.NESATII_given = 1
        else:
            self.NESATII      = self.MPRnb(0.0            ,"V"                     ,"" )
            self.NESATII_given = 0

        if params_dict.get('PESATII') != None:
            val =  params_dict['PESATII']
            self.PESATII      = self.MPRnb(val,"m*V"                   ,"" )
            self.PESATII_given = 1
        else:
            self.PESATII      = self.MPRnb(0.0            ,"m*V"                   ,"" )
            self.PESATII_given = 0

        if params_dict.get('LLII') != None:
            val =  params_dict['LLII']
            self.LLII         = self.MPRnb(val,"(m^2)*V"               ,"" )
            self.LLII_given = 1
        else:
            self.LLII         = self.MPRnb(0.0            ,"(m^2)*V"               ,"" )
            self.LLII_given = 0

        if params_dict.get('NLII') != None:
            val =  params_dict['NLII']
            self.NLII         = self.MPRnb(val,"(m^2)*V"               ,"" )
            self.NLII_given = 1
        else:
            self.NLII         = self.MPRnb(0.0            ,"(m^2)*V"               ,"" )
            self.NLII_given = 0

        if params_dict.get('PLII') != None:
            val =  params_dict['PLII']
            self.PLII         = self.MPRnb(val,"(m^3)*V"               ,"" )
            self.PLII_given = 1
        else:
            self.PLII         = self.MPRnb(0.0            ,"(m^3)*V"               ,"" )
            self.PLII_given = 0

        if params_dict.get('LSII0') != None:
            val =  params_dict['LSII0']
            self.LSII0        = self.MPRnb(val,"m/V"                   ,"" )
            self.LSII0_given = 1
        else:
            self.LSII0        = self.MPRnb(0.0            ,"m/V"                   ,"" )
            self.LSII0_given = 0

        if params_dict.get('NSII0') != None:
            val =  params_dict['NSII0']
            self.NSII0        = self.MPRnb(val,"m/V"                   ,"" )
            self.NSII0_given = 1
        else:
            self.NSII0        = self.MPRnb(0.0            ,"m/V"                   ,"" )
            self.NSII0_given = 0

        if params_dict.get('PSII0') != None:
            val =  params_dict['PSII0']
            self.PSII0        = self.MPRnb(val,"(m^2)/V"               ,"" )
            self.PSII0_given = 1
        else:
            self.PSII0        = self.MPRnb(0.0            ,"(m^2)/V"               ,"" )
            self.PSII0_given = 0

        if params_dict.get('LSII1') != None:
            val =  params_dict['LSII1']
            self.LSII1        = self.MPRnb(val,""                      ,"" )
            self.LSII1_given = 1
        else:
            self.LSII1        = self.MPRnb(0.0            ,""                      ,"" )
            self.LSII1_given = 0

        if params_dict.get('NSII1') != None:
            val =  params_dict['NSII1']
            self.NSII1        = self.MPRnb(val,""                      ,"" )
            self.NSII1_given = 1
        else:
            self.NSII1        = self.MPRnb(0.0            ,""                      ,"" )
            self.NSII1_given = 0

        if params_dict.get('PSII1') != None:
            val =  params_dict['PSII1']
            self.PSII1        = self.MPRnb(val,""                      ,"" )
            self.PSII1_given = 1
        else:
            self.PSII1        = self.MPRnb(0.0            ,""                      ,"" )
            self.PSII1_given = 0

        if params_dict.get('LSII2') != None:
            val =  params_dict['LSII2']
            self.LSII2        = self.MPRnb(val,"m*V"                   ,"" )
            self.LSII2_given = 1
        else:
            self.LSII2        = self.MPRnb(0.0            ,"m*V"                   ,"" )
            self.LSII2_given = 0

        if params_dict.get('NSII2') != None:
            val =  params_dict['NSII2']
            self.NSII2        = self.MPRnb(val,"m*V"                   ,"" )
            self.NSII2_given = 1
        else:
            self.NSII2        = self.MPRnb(0.0            ,"m*V"                   ,"" )
            self.NSII2_given = 0

        if params_dict.get('PSII2') != None:
            val =  params_dict['PSII2']
            self.PSII2        = self.MPRnb(val,"(m^2)*V"               ,"" )
            self.PSII2_given = 1
        else:
            self.PSII2        = self.MPRnb(0.0            ,"(m^2)*V"               ,"" )
            self.PSII2_given = 0

        if params_dict.get('LSIID') != None:
            val =  params_dict['LSIID']
            self.LSIID        = self.MPRnb(val,"m*V"                   ,"" )
            self.LSIID_given = 1
        else:
            self.LSIID        = self.MPRnb(0.0            ,"m*V"                   ,"" )
            self.LSIID_given = 0

        if params_dict.get('NSIID') != None:
            val =  params_dict['NSIID']
            self.NSIID        = self.MPRnb(val,"m*V"                   ,"" )
            self.NSIID_given = 1
        else:
            self.NSIID        = self.MPRnb(0.0            ,"m*V"                   ,"" )
            self.NSIID_given = 0

        if params_dict.get('PSIID') != None:
            val =  params_dict['PSIID']
            self.PSIID        = self.MPRnb(val,"(m^2)*V"               ,"" )
            self.PSIID_given = 1
        else:
            self.PSIID        = self.MPRnb(0.0            ,"(m^2)*V"               ,"" )
            self.PSIID_given = 0

        if params_dict.get('LCFS') != None:
            val =  params_dict['LCFS']
            self.LCFS         = self.MPRnb(val,"F"                     ,"" )
            self.LCFS_given = 1
        else:
            self.LCFS         = self.MPRnb(0.0            ,"F"                     ,"" )
            self.LCFS_given = 0

        if params_dict.get('NCFS') != None:
            val =  params_dict['NCFS']
            self.NCFS         = self.MPRnb(val,"F"                     ,"" )
            self.NCFS_given = 1
        else:
            self.NCFS         = self.MPRnb(0.0            ,"F"                     ,"" )
            self.NCFS_given = 0

        if params_dict.get('PCFS') != None:
            val =  params_dict['PCFS']
            self.PCFS         = self.MPRnb(val,"F*m"                   ,"" )
            self.PCFS_given = 1
        else:
            self.PCFS         = self.MPRnb(0.0            ,"F*m"                   ,"" )
            self.PCFS_given = 0

        if params_dict.get('LCFD') != None:
            val =  params_dict['LCFD']
            self.LCFD         = self.MPRnb(val,"F"                     ,"" )
            self.LCFD_given = 1
        else:
            self.LCFD         = self.MPRnb(self.LCFS           ,"F"                     ,"" )
            self.LCFD_given = 0

        if params_dict.get('NCFD') != None:
            val =  params_dict['NCFD']
            self.NCFD         = self.MPRnb(val,"F"                     ,"" )
            self.NCFD_given = 1
        else:
            self.NCFD         = self.MPRnb(self.NCFS           ,"F"                     ,"" )
            self.NCFD_given = 0

        if params_dict.get('PCFD') != None:
            val =  params_dict['PCFD']
            self.PCFD         = self.MPRnb(val,"F*m"                   ,"" )
            self.PCFD_given = 1
        else:
            self.PCFD         = self.MPRnb(self.PCFS           ,"F*m"                   ,"" )
            self.PCFD_given = 0

        if params_dict.get('LCOVS') != None:
            val =  params_dict['LCOVS']
            self.LCOVS        = self.MPRnb(val,"F"                     ,"" )
            self.LCOVS_given = 1
        else:
            self.LCOVS        = self.MPRnb(0.0            ,"F"                     ,"" )
            self.LCOVS_given = 0

        if params_dict.get('NCOVS') != None:
            val =  params_dict['NCOVS']
            self.NCOVS        = self.MPRnb(val,"F"                     ,"" )
            self.NCOVS_given = 1
        else:
            self.NCOVS        = self.MPRnb(0.0            ,"F"                     ,"" )
            self.NCOVS_given = 0

        if params_dict.get('PCOVS') != None:
            val =  params_dict['PCOVS']
            self.PCOVS        = self.MPRnb(val,"F*m"                   ,"" )
            self.PCOVS_given = 1
        else:
            self.PCOVS        = self.MPRnb(0.0            ,"F*m"                   ,"" )
            self.PCOVS_given = 0

        if params_dict.get('LCOVD') != None:
            val =  params_dict['LCOVD']
            self.LCOVD        = self.MPRnb(val,"F"                     ,"" )
            self.LCOVD_given = 1
        else:
            self.LCOVD        = self.MPRnb(self.LCOVS          ,"F"                     ,"" )
            self.LCOVD_given = 0

        if params_dict.get('NCOVD') != None:
            val =  params_dict['NCOVD']
            self.NCOVD        = self.MPRnb(val,"F"                     ,"" )
            self.NCOVD_given = 1
        else:
            self.NCOVD        = self.MPRnb(self.NCOVS          ,"F"                     ,"" )
            self.NCOVD_given = 0

        if params_dict.get('PCOVD') != None:
            val =  params_dict['PCOVD']
            self.PCOVD        = self.MPRnb(val,"F*m"                   ,"" )
            self.PCOVD_given = 1
        else:
            self.PCOVD        = self.MPRnb(self.PCOVS          ,"F*m"                   ,"" )
            self.PCOVD_given = 0

        if params_dict.get('LCGSL') != None:
            val =  params_dict['LCGSL']
            self.LCGSL        = self.MPRnb(val,"F"                     ,"" )
            self.LCGSL_given = 1
        else:
            self.LCGSL        = self.MPRnb(0.0            ,"F"                     ,"" )
            self.LCGSL_given = 0

        if params_dict.get('NCGSL') != None:
            val =  params_dict['NCGSL']
            self.NCGSL        = self.MPRnb(val,"F"                     ,"" )
            self.NCGSL_given = 1
        else:
            self.NCGSL        = self.MPRnb(0.0            ,"F"                     ,"" )
            self.NCGSL_given = 0

        if params_dict.get('PCGSL') != None:
            val =  params_dict['PCGSL']
            self.PCGSL        = self.MPRnb(val,"F*m"                   ,"" )
            self.PCGSL_given = 1
        else:
            self.PCGSL        = self.MPRnb(0.0            ,"F*m"                   ,"" )
            self.PCGSL_given = 0

        if params_dict.get('LCGDL') != None:
            val =  params_dict['LCGDL']
            self.LCGDL        = self.MPRnb(val,"F"                     ,"" )
            self.LCGDL_given = 1
        else:
            self.LCGDL        = self.MPRnb(self.LCGSL          ,"F"                     ,"" )
            self.LCGDL_given = 0

        if params_dict.get('NCGDL') != None:
            val =  params_dict['NCGDL']
            self.NCGDL        = self.MPRnb(val,"F"                     ,"" )
            self.NCGDL_given = 1
        else:
            self.NCGDL        = self.MPRnb(self.NCGSL          ,"F"                     ,"" )
            self.NCGDL_given = 0

        if params_dict.get('PCGDL') != None:
            val =  params_dict['PCGDL']
            self.PCGDL        = self.MPRnb(val,"F*m"                   ,"" )
            self.PCGDL_given = 1
        else:
            self.PCGDL        = self.MPRnb(self.PCGSL          ,"F*m"                   ,"" )
            self.PCGDL_given = 0

        if params_dict.get('LCKAPPAS') != None:
            val =  params_dict['LCKAPPAS']
            self.LCKAPPAS     = self.MPRnb(val,"m*V"                   ,"" )
            self.LCKAPPAS_given = 1
        else:
            self.LCKAPPAS     = self.MPRnb(0.0            ,"m*V"                   ,"" )
            self.LCKAPPAS_given = 0

        if params_dict.get('NCKAPPAS') != None:
            val =  params_dict['NCKAPPAS']
            self.NCKAPPAS     = self.MPRnb(val,"m*V"                   ,"" )
            self.NCKAPPAS_given = 1
        else:
            self.NCKAPPAS     = self.MPRnb(0.0            ,"m*V"                   ,"" )
            self.NCKAPPAS_given = 0

        if params_dict.get('PCKAPPAS') != None:
            val =  params_dict['PCKAPPAS']
            self.PCKAPPAS     = self.MPRnb(val,"(m^2)*V"               ,"" )
            self.PCKAPPAS_given = 1
        else:
            self.PCKAPPAS     = self.MPRnb(0.0            ,"(m^2)*V"               ,"" )
            self.PCKAPPAS_given = 0

        if params_dict.get('LCKAPPAD') != None:
            val =  params_dict['LCKAPPAD']
            self.LCKAPPAD     = self.MPRnb(val,"m*V"                   ,"" )
            self.LCKAPPAD_given = 1
        else:
            self.LCKAPPAD     = self.MPRnb(self.LCKAPPAS       ,"m*V"                   ,"" )
            self.LCKAPPAD_given = 0

        if params_dict.get('NCKAPPAD') != None:
            val =  params_dict['NCKAPPAD']
            self.NCKAPPAD     = self.MPRnb(val,"m*V"                   ,"" )
            self.NCKAPPAD_given = 1
        else:
            self.NCKAPPAD     = self.MPRnb(self.NCKAPPAS       ,"m*V"                   ,"" )
            self.NCKAPPAD_given = 0

        if params_dict.get('PCKAPPAD') != None:
            val =  params_dict['PCKAPPAD']
            self.PCKAPPAD     = self.MPRnb(val,"(m^2)*V"               ,"" )
            self.PCKAPPAD_given = 1
        else:
            self.PCKAPPAD     = self.MPRnb(self.PCKAPPAS       ,"(m^2)*V"               ,"" )
            self.PCKAPPAD_given = 0

        if params_dict.get('LCGBL') != None:
            val =  params_dict['LCGBL']
            self.LCGBL        = self.MPRnb(val,"F"                     ,"" )
            self.LCGBL_given = 1
        else:
            self.LCGBL        = self.MPRnb(0.0            ,"F"                     ,"" )
            self.LCGBL_given = 0

        if params_dict.get('NCGBL') != None:
            val =  params_dict['NCGBL']
            self.NCGBL        = self.MPRnb(val,"F"                     ,"" )
            self.NCGBL_given = 1
        else:
            self.NCGBL        = self.MPRnb(0.0            ,"F"                     ,"" )
            self.NCGBL_given = 0

        if params_dict.get('PCGBL') != None:
            val =  params_dict['PCGBL']
            self.PCGBL        = self.MPRnb(val,"F*m"                   ,"" )
            self.PCGBL_given = 1
        else:
            self.PCGBL        = self.MPRnb(0.0            ,"F*m"                   ,"" )
            self.PCGBL_given = 0

        if params_dict.get('LCKAPPAB') != None:
            val =  params_dict['LCKAPPAB']
            self.LCKAPPAB     = self.MPRnb(val,"m*V"                   ,"" )
            self.LCKAPPAB_given = 1
        else:
            self.LCKAPPAB     = self.MPRnb(0.0            ,"m*V"                   ,"" )
            self.LCKAPPAB_given = 0

        if params_dict.get('NCKAPPAB') != None:
            val =  params_dict['NCKAPPAB']
            self.NCKAPPAB     = self.MPRnb(val,"m*V"                   ,"" )
            self.NCKAPPAB_given = 1
        else:
            self.NCKAPPAB     = self.MPRnb(0.0            ,"m*V"                   ,"" )
            self.NCKAPPAB_given = 0

        if params_dict.get('PCKAPPAB') != None:
            val =  params_dict['PCKAPPAB']
            self.PCKAPPAB     = self.MPRnb(val,"(m^2)*V"               ,"" )
            self.PCKAPPAB_given = 1
        else:
            self.PCKAPPAB     = self.MPRnb(0.0            ,"(m^2)*V"               ,"" )
            self.PCKAPPAB_given = 0

        if params_dict.get('LNTGEN') != None:
            val =  params_dict['LNTGEN']
            self.LNTGEN       = self.MPRnb(val,""                      ,"" )
            self.LNTGEN_given = 1
        else:
            self.LNTGEN       = self.MPRnb(0.0            ,""                      ,"" )
            self.LNTGEN_given = 0

        if params_dict.get('NNTGEN') != None:
            val =  params_dict['NNTGEN']
            self.NNTGEN       = self.MPRnb(val,""                      ,"" )
            self.NNTGEN_given = 1
        else:
            self.NNTGEN       = self.MPRnb(0.0            ,""                      ,"" )
            self.NNTGEN_given = 0

        if params_dict.get('PNTGEN') != None:
            val =  params_dict['PNTGEN']
            self.PNTGEN       = self.MPRnb(val,""                      ,"" )
            self.PNTGEN_given = 1
        else:
            self.PNTGEN       = self.MPRnb(0.0            ,""                      ,"" )
            self.PNTGEN_given = 0

        if params_dict.get('LAIGEN') != None:
            val =  params_dict['LAIGEN']
            self.LAIGEN       = self.MPRnb(val,"(m^-2)*(V^-1)"         ,"" )
            self.LAIGEN_given = 1
        else:
            self.LAIGEN       = self.MPRnb(0.0            ,"(m^-2)*(V^-1)"         ,"" )
            self.LAIGEN_given = 0

        if params_dict.get('NAIGEN') != None:
            val =  params_dict['NAIGEN']
            self.NAIGEN       = self.MPRnb(val,"(m^-2)*(V^-1)"         ,"" )
            self.NAIGEN_given = 1
        else:
            self.NAIGEN       = self.MPRnb(0.0            ,"(m^-2)*(V^-1)"         ,"" )
            self.NAIGEN_given = 0

        if params_dict.get('PAIGEN') != None:
            val =  params_dict['PAIGEN']
            self.PAIGEN       = self.MPRnb(val,"(m^-1)*(V^-1)"         ,"" )
            self.PAIGEN_given = 1
        else:
            self.PAIGEN       = self.MPRnb(0.0            ,"(m^-1)*(V^-1)"         ,"" )
            self.PAIGEN_given = 0

        if params_dict.get('LBIGEN') != None:
            val =  params_dict['LBIGEN']
            self.LBIGEN       = self.MPRnb(val,"(m^-2)*(V^-3)"         ,"" )
            self.LBIGEN_given = 1
        else:
            self.LBIGEN       = self.MPRnb(0.0            ,"(m^-2)*(V^-3)"         ,"" )
            self.LBIGEN_given = 0

        if params_dict.get('NBIGEN') != None:
            val =  params_dict['NBIGEN']
            self.NBIGEN       = self.MPRnb(val,"(m^-2)*(V^-3)"         ,"" )
            self.NBIGEN_given = 1
        else:
            self.NBIGEN       = self.MPRnb(0.0            ,"(m^-2)*(V^-3)"         ,"" )
            self.NBIGEN_given = 0

        if params_dict.get('PBIGEN') != None:
            val =  params_dict['PBIGEN']
            self.PBIGEN       = self.MPRnb(val,"(m^-1)*(V^-3)"         ,"" )
            self.PBIGEN_given = 1
        else:
            self.PBIGEN       = self.MPRnb(0.0            ,"(m^-1)*(V^-3)"         ,"" )
            self.PBIGEN_given = 0

        if params_dict.get('LXRCRG1') != None:
            val =  params_dict['LXRCRG1']
            self.LXRCRG1      = self.MPRnb(val,""                      ,"" )
            self.LXRCRG1_given = 1
        else:
            self.LXRCRG1      = self.MPRnb(0.0            ,""                      ,"" )
            self.LXRCRG1_given = 0

        if params_dict.get('NXRCRG1') != None:
            val =  params_dict['NXRCRG1']
            self.NXRCRG1      = self.MPRnb(val,""                      ,"" )
            self.NXRCRG1_given = 1
        else:
            self.NXRCRG1      = self.MPRnb(0.0            ,""                      ,"" )
            self.NXRCRG1_given = 0

        if params_dict.get('PXRCRG1') != None:
            val =  params_dict['PXRCRG1']
            self.PXRCRG1      = self.MPRnb(val,""                      ,"" )
            self.PXRCRG1_given = 1
        else:
            self.PXRCRG1      = self.MPRnb(0.0            ,""                      ,"" )
            self.PXRCRG1_given = 0

        if params_dict.get('LXRCRG2') != None:
            val =  params_dict['LXRCRG2']
            self.LXRCRG2      = self.MPRnb(val,""                      ,"" )
            self.LXRCRG2_given = 1
        else:
            self.LXRCRG2      = self.MPRnb(0.0            ,""                      ,"" )
            self.LXRCRG2_given = 0

        if params_dict.get('NXRCRG2') != None:
            val =  params_dict['NXRCRG2']
            self.NXRCRG2      = self.MPRnb(val,""                      ,"" )
            self.NXRCRG2_given = 1
        else:
            self.NXRCRG2      = self.MPRnb(0.0            ,""                      ,"" )
            self.NXRCRG2_given = 0

        if params_dict.get('PXRCRG2') != None:
            val =  params_dict['PXRCRG2']
            self.PXRCRG2      = self.MPRnb(val,""                      ,"" )
            self.PXRCRG2_given = 1
        else:
            self.PXRCRG2      = self.MPRnb(0.0            ,""                      ,"" )
            self.PXRCRG2_given = 0

        if params_dict.get('LUTE') != None:
            val =  params_dict['LUTE']
            self.LUTE         = self.MPRnb(val,""                      ,"" )
            self.LUTE_given = 1
        else:
            self.LUTE         = self.MPRnb(0.0            ,""                      ,"" )
            self.LUTE_given = 0

        if params_dict.get('NUTE') != None:
            val =  params_dict['NUTE']
            self.NUTE         = self.MPRnb(val,""                      ,"" )
            self.NUTE_given = 1
        else:
            self.NUTE         = self.MPRnb(0.0            ,""                      ,"" )
            self.NUTE_given = 0

        if params_dict.get('PUTE') != None:
            val =  params_dict['PUTE']
            self.PUTE         = self.MPRnb(val,""                      ,"" )
            self.PUTE_given = 1
        else:
            self.PUTE         = self.MPRnb(0.0            ,""                      ,"" )
            self.PUTE_given = 0

        if params_dict.get('LUTER') != None:
            val =  params_dict['LUTER']
            self.LUTER        = self.MPRnb(val,""                      ,"" )
            self.LUTER_given = 1
        else:
            self.LUTER        = self.MPRnb(self.LUTE           ,""                      ,"" )
            self.LUTER_given = 0

        if params_dict.get('NUTER') != None:
            val =  params_dict['NUTER']
            self.NUTER        = self.MPRnb(val,""                      ,"" )
            self.NUTER_given = 1
        else:
            self.NUTER        = self.MPRnb(self.NUTE           ,""                      ,"" )
            self.NUTER_given = 0

        if params_dict.get('PUTER') != None:
            val =  params_dict['PUTER']
            self.PUTER        = self.MPRnb(val,""                      ,"" )
            self.PUTER_given = 1
        else:
            self.PUTER        = self.MPRnb(self.PUTE           ,""                      ,"" )
            self.PUTER_given = 0

        if params_dict.get('LUTL') != None:
            val =  params_dict['LUTL']
            self.LUTL         = self.MPRnb(val,""                      ,"" )
            self.LUTL_given = 1
        else:
            self.LUTL         = self.MPRnb(0.0            ,""                      ,"" )
            self.LUTL_given = 0

        if params_dict.get('NUTL') != None:
            val =  params_dict['NUTL']
            self.NUTL         = self.MPRnb(val,""                      ,"" )
            self.NUTL_given = 1
        else:
            self.NUTL         = self.MPRnb(0.0            ,""                      ,"" )
            self.NUTL_given = 0

        if params_dict.get('PUTL') != None:
            val =  params_dict['PUTL']
            self.PUTL         = self.MPRnb(val,""                      ,"" )
            self.PUTL_given = 1
        else:
            self.PUTL         = self.MPRnb(0.0            ,""                      ,"" )
            self.PUTL_given = 0

        if params_dict.get('LUTLR') != None:
            val =  params_dict['LUTLR']
            self.LUTLR        = self.MPRnb(val,""                      ,"" )
            self.LUTLR_given = 1
        else:
            self.LUTLR        = self.MPRnb(self.LUTL           ,""                      ,"" )
            self.LUTLR_given = 0

        if params_dict.get('NUTLR') != None:
            val =  params_dict['NUTLR']
            self.NUTLR        = self.MPRnb(val,""                      ,"" )
            self.NUTLR_given = 1
        else:
            self.NUTLR        = self.MPRnb(self.NUTL           ,""                      ,"" )
            self.NUTLR_given = 0

        if params_dict.get('PUTLR') != None:
            val =  params_dict['PUTLR']
            self.PUTLR        = self.MPRnb(val,""                      ,"" )
            self.PUTLR_given = 1
        else:
            self.PUTLR        = self.MPRnb(self.PUTL           ,""                      ,"" )
            self.PUTLR_given = 0

        if params_dict.get('LEMOBT') != None:
            val =  params_dict['LEMOBT']
            self.LEMOBT       = self.MPRnb(val,""                      ,"" )
            self.LEMOBT_given = 1
        else:
            self.LEMOBT       = self.MPRnb(0.0            ,""                      ,"" )
            self.LEMOBT_given = 0

        if params_dict.get('NEMOBT') != None:
            val =  params_dict['NEMOBT']
            self.NEMOBT       = self.MPRnb(val,""                      ,"" )
            self.NEMOBT_given = 1
        else:
            self.NEMOBT       = self.MPRnb(0.0            ,""                      ,"" )
            self.NEMOBT_given = 0

        if params_dict.get('PEMOBT') != None:
            val =  params_dict['PEMOBT']
            self.PEMOBT       = self.MPRnb(val,""                      ,"" )
            self.PEMOBT_given = 1
        else:
            self.PEMOBT       = self.MPRnb(0.0            ,""                      ,"" )
            self.PEMOBT_given = 0

        if params_dict.get('LUA1') != None:
            val =  params_dict['LUA1']
            self.LUA1         = self.MPRnb(val,""                      ,"" )
            self.LUA1_given = 1
        else:
            self.LUA1         = self.MPRnb(0.0            ,""                      ,"" )
            self.LUA1_given = 0

        if params_dict.get('NUA1') != None:
            val =  params_dict['NUA1']
            self.NUA1         = self.MPRnb(val,""                      ,"" )
            self.NUA1_given = 1
        else:
            self.NUA1         = self.MPRnb(0.0            ,""                      ,"" )
            self.NUA1_given = 0

        if params_dict.get('PUA1') != None:
            val =  params_dict['PUA1']
            self.PUA1         = self.MPRnb(val,""                      ,"" )
            self.PUA1_given = 1
        else:
            self.PUA1         = self.MPRnb(0.0            ,""                      ,"" )
            self.PUA1_given = 0

        if params_dict.get('LUA1R') != None:
            val =  params_dict['LUA1R']
            self.LUA1R        = self.MPRnb(val,""                      ,"" )
            self.LUA1R_given = 1
        else:
            self.LUA1R        = self.MPRnb(self.LUA1           ,""                      ,"" )
            self.LUA1R_given = 0

        if params_dict.get('NUA1R') != None:
            val =  params_dict['NUA1R']
            self.NUA1R        = self.MPRnb(val,""                      ,"" )
            self.NUA1R_given = 1
        else:
            self.NUA1R        = self.MPRnb(self.NUA1           ,""                      ,"" )
            self.NUA1R_given = 0

        if params_dict.get('PUA1R') != None:
            val =  params_dict['PUA1R']
            self.PUA1R        = self.MPRnb(val,""                      ,"" )
            self.PUA1R_given = 1
        else:
            self.PUA1R        = self.MPRnb(self.PUA1           ,""                      ,"" )
            self.PUA1R_given = 0

        if params_dict.get('LUC1') != None:
            val =  params_dict['LUC1']
            self.LUC1         = self.MPRnb(val,""                      ,"" )
            self.LUC1_given = 1
        else:
            self.LUC1         = self.MPRnb(0.0            ,""                      ,"" )
            self.LUC1_given = 0

        if params_dict.get('NUC1') != None:
            val =  params_dict['NUC1']
            self.NUC1         = self.MPRnb(val,""                      ,"" )
            self.NUC1_given = 1
        else:
            self.NUC1         = self.MPRnb(0.0            ,""                      ,"" )
            self.NUC1_given = 0

        if params_dict.get('PUC1') != None:
            val =  params_dict['PUC1']
            self.PUC1         = self.MPRnb(val,""                      ,"" )
            self.PUC1_given = 1
        else:
            self.PUC1         = self.MPRnb(0.0            ,""                      ,"" )
            self.PUC1_given = 0

        if params_dict.get('LUC1R') != None:
            val =  params_dict['LUC1R']
            self.LUC1R        = self.MPRnb(val,""                      ,"" )
            self.LUC1R_given = 1
        else:
            self.LUC1R        = self.MPRnb(self.LUC1           ,""                      ,"" )
            self.LUC1R_given = 0

        if params_dict.get('NUC1R') != None:
            val =  params_dict['NUC1R']
            self.NUC1R        = self.MPRnb(val,""                      ,"" )
            self.NUC1R_given = 1
        else:
            self.NUC1R        = self.MPRnb(self.NUC1           ,""                      ,"" )
            self.NUC1R_given = 0

        if params_dict.get('PUC1R') != None:
            val =  params_dict['PUC1R']
            self.PUC1R        = self.MPRnb(val,""                      ,"" )
            self.PUC1R_given = 1
        else:
            self.PUC1R        = self.MPRnb(self.PUC1           ,""                      ,"" )
            self.PUC1R_given = 0

        if params_dict.get('LUD1') != None:
            val =  params_dict['LUD1']
            self.LUD1         = self.MPRnb(val,""                      ,"" )
            self.LUD1_given = 1
        else:
            self.LUD1         = self.MPRnb(0.0            ,""                      ,"" )
            self.LUD1_given = 0

        if params_dict.get('NUD1') != None:
            val =  params_dict['NUD1']
            self.NUD1         = self.MPRnb(val,""                      ,"" )
            self.NUD1_given = 1
        else:
            self.NUD1         = self.MPRnb(0.0            ,""                      ,"" )
            self.NUD1_given = 0

        if params_dict.get('PUD1') != None:
            val =  params_dict['PUD1']
            self.PUD1         = self.MPRnb(val,""                      ,"" )
            self.PUD1_given = 1
        else:
            self.PUD1         = self.MPRnb(0.0            ,""                      ,"" )
            self.PUD1_given = 0

        if params_dict.get('LUD1R') != None:
            val =  params_dict['LUD1R']
            self.LUD1R        = self.MPRnb(val,""                      ,"" )
            self.LUD1R_given = 1
        else:
            self.LUD1R        = self.MPRnb(self.LUD1           ,""                      ,"" )
            self.LUD1R_given = 0

        if params_dict.get('NUD1R') != None:
            val =  params_dict['NUD1R']
            self.NUD1R        = self.MPRnb(val,""                      ,"" )
            self.NUD1R_given = 1
        else:
            self.NUD1R        = self.MPRnb(self.NUD1           ,""                      ,"" )
            self.NUD1R_given = 0

        if params_dict.get('PUD1R') != None:
            val =  params_dict['PUD1R']
            self.PUD1R        = self.MPRnb(val,""                      ,"" )
            self.PUD1R_given = 1
        else:
            self.PUD1R        = self.MPRnb(self.PUD1           ,""                      ,"" )
            self.PUD1R_given = 0

        if params_dict.get('LUCSTE') != None:
            val =  params_dict['LUCSTE']
            self.LUCSTE       = self.MPRnb(val,""                      ,"" )
            self.LUCSTE_given = 1
        else:
            self.LUCSTE       = self.MPRnb(0.0            ,""                      ,"" )
            self.LUCSTE_given = 0

        if params_dict.get('NUCSTE') != None:
            val =  params_dict['NUCSTE']
            self.NUCSTE       = self.MPRnb(val,""                      ,"" )
            self.NUCSTE_given = 1
        else:
            self.NUCSTE       = self.MPRnb(0.0            ,""                      ,"" )
            self.NUCSTE_given = 0

        if params_dict.get('PUCSTE') != None:
            val =  params_dict['PUCSTE']
            self.PUCSTE       = self.MPRnb(val,""                      ,"" )
            self.PUCSTE_given = 1
        else:
            self.PUCSTE       = self.MPRnb(0.0            ,""                      ,"" )
            self.PUCSTE_given = 0

        if params_dict.get('LPTWGT') != None:
            val =  params_dict['LPTWGT']
            self.LPTWGT       = self.MPRnb(val,"m/K"                   ,"" )
            self.LPTWGT_given = 1
        else:
            self.LPTWGT       = self.MPRnb(0.0            ,"m/K"                   ,"" )
            self.LPTWGT_given = 0

        if params_dict.get('NPTWGT') != None:
            val =  params_dict['NPTWGT']
            self.NPTWGT       = self.MPRnb(val,"m/K"                   ,"" )
            self.NPTWGT_given = 1
        else:
            self.NPTWGT       = self.MPRnb(0.0            ,"m/K"                   ,"" )
            self.NPTWGT_given = 0

        if params_dict.get('PPTWGT') != None:
            val =  params_dict['PPTWGT']
            self.PPTWGT       = self.MPRnb(val,"(m^2)/K"               ,"" )
            self.PPTWGT_given = 1
        else:
            self.PPTWGT       = self.MPRnb(0.0            ,"(m^2)/K"               ,"" )
            self.PPTWGT_given = 0

        if params_dict.get('LAT') != None:
            val =  params_dict['LAT']
            self.LAT          = self.MPRnb(val,"m/K"                   ,"" )
            self.LAT_given = 1
        else:
            self.LAT          = self.MPRnb(0.0            ,"m/K"                   ,"" )
            self.LAT_given = 0

        if params_dict.get('NAT') != None:
            val =  params_dict['NAT']
            self.NAT          = self.MPRnb(val,"m/K"                   ,"" )
            self.NAT_given = 1
        else:
            self.NAT          = self.MPRnb(0.0            ,"m/K"                   ,"" )
            self.NAT_given = 0

        if params_dict.get('PAT') != None:
            val =  params_dict['PAT']
            self.PAT          = self.MPRnb(val,"(m^2)/K"               ,"" )
            self.PAT_given = 1
        else:
            self.PAT          = self.MPRnb(0.0            ,"(m^2)/K"               ,"" )
            self.PAT_given = 0

        if params_dict.get('LATR') != None:
            val =  params_dict['LATR']
            self.LATR         = self.MPRnb(val,""                      ,"" )
            self.LATR_given = 1
        else:
            self.LATR         = self.MPRnb(self.LAT            ,""                      ,"" )
            self.LATR_given = 0

        if params_dict.get('NATR') != None:
            val =  params_dict['NATR']
            self.NATR         = self.MPRnb(val,""                      ,"" )
            self.NATR_given = 1
        else:
            self.NATR         = self.MPRnb(self.NAT            ,""                      ,"" )
            self.NATR_given = 0

        if params_dict.get('PATR') != None:
            val =  params_dict['PATR']
            self.PATR         = self.MPRnb(val,""                      ,"" )
            self.PATR_given = 1
        else:
            self.PATR         = self.MPRnb(self.PAT            ,""                      ,"" )
            self.PATR_given = 0

        if params_dict.get('LATCV') != None:
            val =  params_dict['LATCV']
            self.LATCV        = self.MPRnb(val,"m/K"                   ,"" )
            self.LATCV_given = 1
        else:
            self.LATCV        = self.MPRnb(0.0            ,"m/K"                   ,"" )
            self.LATCV_given = 0

        if params_dict.get('NATCV') != None:
            val =  params_dict['NATCV']
            self.NATCV        = self.MPRnb(val,"m/K"                   ,"" )
            self.NATCV_given = 1
        else:
            self.NATCV        = self.MPRnb(0.0            ,"m/K"                   ,"" )
            self.NATCV_given = 0

        if params_dict.get('PATCV') != None:
            val =  params_dict['PATCV']
            self.PATCV        = self.MPRnb(val,"(m^2)/K"               ,"" )
            self.PATCV_given = 1
        else:
            self.PATCV        = self.MPRnb(0.0            ,"(m^2)/K"               ,"" )
            self.PATCV_given = 0

        if params_dict.get('LSTTHETASAT') != None:
            val =  params_dict['LSTTHETASAT']
            self.LSTTHETASAT  = self.MPRnb(val,""                      ,"" )
            self.LSTTHETASAT_given = 1
        else:
            self.LSTTHETASAT  = self.MPRnb(0.0            ,""                      ,"" )
            self.LSTTHETASAT_given = 0

        if params_dict.get('NSTTHETASAT') != None:
            val =  params_dict['NSTTHETASAT']
            self.NSTTHETASAT  = self.MPRnb(val,""                      ,"" )
            self.NSTTHETASAT_given = 1
        else:
            self.NSTTHETASAT  = self.MPRnb(0.0            ,""                      ,"" )
            self.NSTTHETASAT_given = 0

        if params_dict.get('PSTTHETASAT') != None:
            val =  params_dict['PSTTHETASAT']
            self.PSTTHETASAT  = self.MPRnb(val,""                      ,"" )
            self.PSTTHETASAT_given = 1
        else:
            self.PSTTHETASAT  = self.MPRnb(0.0            ,""                      ,"" )
            self.PSTTHETASAT_given = 0

        if params_dict.get('LPRT') != None:
            val =  params_dict['LPRT']
            self.LPRT         = self.MPRnb(val,"m/K"                   ,"" )
            self.LPRT_given = 1
        else:
            self.LPRT         = self.MPRnb(0.0            ,"m/K"                   ,"" )
            self.LPRT_given = 0

        if params_dict.get('NPRT') != None:
            val =  params_dict['NPRT']
            self.NPRT         = self.MPRnb(val,"m/K"                   ,"" )
            self.NPRT_given = 1
        else:
            self.NPRT         = self.MPRnb(0.0            ,"m/K"                   ,"" )
            self.NPRT_given = 0

        if params_dict.get('PPRT') != None:
            val =  params_dict['PPRT']
            self.PPRT         = self.MPRnb(val,"(m^2)/K"               ,"" )
            self.PPRT_given = 1
        else:
            self.PPRT         = self.MPRnb(0.0            ,"(m^2)/K"               ,"" )
            self.PPRT_given = 0

        if params_dict.get('LKT1') != None:
            val =  params_dict['LKT1']
            self.LKT1         = self.MPRnb(val,"m*V"                   ,"" )
            self.LKT1_given = 1
        else:
            self.LKT1         = self.MPRnb(0.0            ,"m*V"                   ,"" )
            self.LKT1_given = 0

        if params_dict.get('NKT1') != None:
            val =  params_dict['NKT1']
            self.NKT1         = self.MPRnb(val,"m*V"                   ,"" )
            self.NKT1_given = 1
        else:
            self.NKT1         = self.MPRnb(0.0            ,"m*V"                   ,"" )
            self.NKT1_given = 0

        if params_dict.get('PKT1') != None:
            val =  params_dict['PKT1']
            self.PKT1         = self.MPRnb(val,"(m^2)*V"               ,"" )
            self.PKT1_given = 1
        else:
            self.PKT1         = self.MPRnb(0.0            ,"(m^2)*V"               ,"" )
            self.PKT1_given = 0

        if params_dict.get('LTSS') != None:
            val =  params_dict['LTSS']
            self.LTSS         = self.MPRnb(val,""                      ,"" )
            self.LTSS_given = 1
        else:
            self.LTSS         = self.MPRnb(0.0            ,""                      ,"" )
            self.LTSS_given = 0

        if params_dict.get('NTSS') != None:
            val =  params_dict['NTSS']
            self.NTSS         = self.MPRnb(val,""                      ,"" )
            self.NTSS_given = 1
        else:
            self.NTSS         = self.MPRnb(0.0            ,""                      ,"" )
            self.NTSS_given = 0

        if params_dict.get('PTSS') != None:
            val =  params_dict['PTSS']
            self.PTSS         = self.MPRnb(val,""                      ,"" )
            self.PTSS_given = 1
        else:
            self.PTSS         = self.MPRnb(0.0            ,""                      ,"" )
            self.PTSS_given = 0

        if params_dict.get('LIIT') != None:
            val =  params_dict['LIIT']
            self.LIIT         = self.MPRnb(val,""                      ,"" )
            self.LIIT_given = 1
        else:
            self.LIIT         = self.MPRnb(0.0            ,""                      ,"" )
            self.LIIT_given = 0

        if params_dict.get('NIIT') != None:
            val =  params_dict['NIIT']
            self.NIIT         = self.MPRnb(val,""                      ,"" )
            self.NIIT_given = 1
        else:
            self.NIIT         = self.MPRnb(0.0            ,""                      ,"" )
            self.NIIT_given = 0

        if params_dict.get('PIIT') != None:
            val =  params_dict['PIIT']
            self.PIIT         = self.MPRnb(val,""                      ,"" )
            self.PIIT_given = 1
        else:
            self.PIIT         = self.MPRnb(0.0            ,""                      ,"" )
            self.PIIT_given = 0

        if params_dict.get('LTII') != None:
            val =  params_dict['LTII']
            self.LTII         = self.MPRnb(val,""                      ,"" )
            self.LTII_given = 1
        else:
            self.LTII         = self.MPRnb(0.0            ,""                      ,"" )
            self.LTII_given = 0

        if params_dict.get('NTII') != None:
            val =  params_dict['NTII']
            self.NTII         = self.MPRnb(val,""                      ,"" )
            self.NTII_given = 1
        else:
            self.NTII         = self.MPRnb(0.0            ,""                      ,"" )
            self.NTII_given = 0

        if params_dict.get('PTII') != None:
            val =  params_dict['PTII']
            self.PTII         = self.MPRnb(val,""                      ,"" )
            self.PTII_given = 1
        else:
            self.PTII         = self.MPRnb(0.0            ,""                      ,"" )
            self.PTII_given = 0

        if params_dict.get('LTGIDL') != None:
            val =  params_dict['LTGIDL']
            self.LTGIDL       = self.MPRnb(val,"m/K"                   ,"" )
            self.LTGIDL_given = 1
        else:
            self.LTGIDL       = self.MPRnb(0.0            ,"m/K"                   ,"" )
            self.LTGIDL_given = 0

        if params_dict.get('NTGIDL') != None:
            val =  params_dict['NTGIDL']
            self.NTGIDL       = self.MPRnb(val,"m/K"                   ,"" )
            self.NTGIDL_given = 1
        else:
            self.NTGIDL       = self.MPRnb(0.0            ,"m/K"                   ,"" )
            self.NTGIDL_given = 0

        if params_dict.get('PTGIDL') != None:
            val =  params_dict['PTGIDL']
            self.PTGIDL       = self.MPRnb(val,"(m^2)/K"               ,"" )
            self.PTGIDL_given = 1
        else:
            self.PTGIDL       = self.MPRnb(0.0            ,"(m^2)/K"               ,"" )
            self.PTGIDL_given = 0

        if params_dict.get('LIGT') != None:
            val =  params_dict['LIGT']
            self.LIGT         = self.MPRnb(val,""                      ,"" )
            self.LIGT_given = 1
        else:
            self.LIGT         = self.MPRnb(0.0            ,""                      ,"" )
            self.LIGT_given = 0

        if params_dict.get('NIGT') != None:
            val =  params_dict['NIGT']
            self.NIGT         = self.MPRnb(val,""                      ,"" )
            self.NIGT_given = 1
        else:
            self.NIGT         = self.MPRnb(0.0            ,""                      ,"" )
            self.NIGT_given = 0

        if params_dict.get('PIGT') != None:
            val =  params_dict['PIGT']
            self.PIGT         = self.MPRnb(val,""                      ,"" )
            self.PIGT_given = 1
        else:
            self.PIGT         = self.MPRnb(0.0            ,""                      ,"" )
            self.PIGT_given = 0

        # Output Variables
        #`ifdef __OPINFO__
        #   (* desc=  "WEFF" *)        real WEFF;
        #   (* desc=  "LEFF" *)        real LEFF;
        #   (* desc=  "WEFFCV" *)      real WEFFCV;
        #   (* desc=  "LEFFCV" *)      real LEFFCV;
        #   (* desc=  "IDS" *)         real IDS;
        #   (* desc=  "IDEFF" *)       real IDEFF;
        #   (* desc=  "ISEFF" *)       real ISEFF;
        #   (* desc=  "IGTOT" *)       real IGTOT;
        #   (* desc=  "IDSGEN" *)      real IDSGEN;
        #   (* desc=  "III" *)         real III;
        #   (* desc=  "IGS" *)         real IGS;
        #   (* desc=  "IGD" *)         real IGD;
        #   (* desc=  "IGCS" *)        real IGCS;
        #   (* desc=  "IGCD" *)        real IGCD;
        #   (* desc=  "IGBS" *)        real IGBS;
        #   (* desc=  "IGBD" *)        real IGBD;
        #   (* desc=  "IGIDL" *)       real IGIDL;
        #   (* desc=  "IGISL" *)       real IGISL;
        #   (* desc=  "IJSB" *)        real IJSB;
        #   (* desc=  "IJDB" *)        real IJDB;
        #   (* desc=  "ISUB" *)        real ISUB;
        #   (* desc=  "BETA" *)        real BETA;
        #   (* desc=  "VTH" *)         real VTH;
        #   (* desc=  "VDSSAT" *)      real VDSSAT;
        #   (* desc=  "VFB" *)         real VFB;
        #   (* desc=  "GM" *)          real GM;
        #   (* desc=  "GDS" *)         real GDS;
        #   (* desc=  "GMBS" *)        real GMBS;
        #   (* desc=  "QGI" *)         real QGI;
        #   (* desc=  "QDI" *)         real QDI;
        #   (* desc=  "QSI" *)         real QSI;
        #   (* desc=  "QBI" *)         real QBI;
        #   (* desc=  "QG" *)          real QG;
        #   (* desc=  "QD" *)          real QD;
        #   (* desc=  "QS" *)          real QS;
        #   (* desc=  "QB" *)          real QB;
        #   (* desc=  "CGGI" *)        real CGGI;
        #   (* desc=  "CGSI" *)        real CGSI;
        #   (* desc=  "CGDI" *)        real CGDI;
        #   (* desc=  "CGEI" *)        real CGEI;
        #   (* desc=  "CDGI" *)        real CDGI;
        #   (* desc=  "CDDI" *)        real CDDI;
        #   (* desc=  "CDSI" *)        real CDSI;
        #   (* desc=  "CDEI" *)        real CDEI;
        #   (* desc=  "CSGI" *)        real CSGI;
        #   (* desc=  "CSDI" *)        real CSDI;
        #   (* desc=  "CSSI" *)        real CSSI;
        #   (* desc=  "CSEI" *)        real CSEI;
        #   (* desc=  "CEGI" *)        real CEGI;
        #   (* desc=  "CEDI" *)        real CEDI;
        #   (* desc=  "CESI" *)        real CESI;
        #   (* desc=  "CEEI" *)        real CEEI;
        #   (* desc=  "CGG" *)         real CGG;
        #   (* desc=  "CGS" *)         real CGS;
        #   (* desc=  "CGD" *)         real CGD;
        #   (* desc=  "CGE" *)         real CGE;
        #   (* desc=  "CDG" *)         real CDG;
        #   (* desc=  "CDD" *)         real CDD;
        #   (* desc=  "CDS" *)         real CDS;
        #   (* desc=  "CDE" *)         real CDE;
        #   (* desc=  "CSG" *)         real CSG;
        #   (* desc=  "CSD" *)         real CSD;
        #   (* desc=  "CSS" *)         real CSS;
        #   (* desc=  "CSE" *)         real CSE;
        #   (* desc=  "CEG" *)         real CEG;
        #   (* desc=  "CED" *)         real CED;
        #   (* desc=  "CES" *)         real CES;
        #   (* desc=  "CEE" *)         real CEE;
        #   (* desc=  "CGSEXT" *)      real CGSEXT;
        #   (* desc=  "CGDEXT" *)      real CGDEXT;
        #   (* desc=  "CGBOV" *)       real CGBOV;
        #   (* desc=  "CJST" *)        real CJST;
        #   (* desc=  "CJDT" *)        real CJDT;
        #   (* desc=  "RSGEO" *)       real RSGEO;
        #   (* desc=  "RDGEO" *)       real RDGEO;
        #   (* desc=  "CFGEO" *)       real CFGEO;
        #   (* desc=  "T_TOTAL_K" *)   real T_TOTAL_K;
        #   (* desc=  "T_TOTAL_C" *)   real T_TOTAL_C;
        #   (* desc=  "T_DELTA_SH" *)  real T_DELTA_SH;
        #
        #   `ifdef __DEBUG__
        #      (* desc=  "IGBACC" *)    real IGBACC;
        #      (* desc=  "IGBINV" *)    real IGBINV;
        #      (* desc=  "DIDSDVG" *)   real DIDSDVG;
        #      (* desc=  "DIDSDVS" *)   real DIDSDVS;
        #      (* desc=  "DIDSDVD" *)   real DIDSDVD;
        #      (* desc=  "DIGSDVG" *)   real DIGSDVG;
        #      (* desc=  "DIGSDVS" *)   real DIGSDVS;
        #      (* desc=  "DIGSDVD" *)   real DIGSDVD;
        #      (* desc=  "DIGDDVG" *)   real DIGDDVG;
        #      (* desc=  "DIGDDVS" *)   real DIGDDVS;
        #      (* desc=  "DIGDDVD" *)   real DIGDDVD;
        #      (* desc=  "DIIIDVG" *)   real DIIIDVG;
        #      (* desc=  "DIIIDVS" *)   real DIIIDVS;
        #      (* desc=  "DIIIDVD" *)   real DIIIDVD;
        #      (* desc=  "DIGIDLDVG" *) real DIGIDLDVG;
        #      (* desc=  "DIGIDLDVS" *) real DIGIDLDVS;
        #      (* desc=  "DIGIDLDVD" *) real DIGIDLDVD;
        #      (* desc=  "DIGISLDVG" *) real DIGISLDVG;
        #      (* desc=  "DIGISLDVS" *) real DIGISLDVS;
        #      (* desc=  "DIGISLDVD" *) real DIGISLDVD;
        #
        #      `ifdef __SHMOD__
        #         (* desc=  "CGT" *)        real CGT;
        #         (* desc=  "CST" *)        real CST;
        #         (* desc=  "CDT" *)        real CDT;
        #         (* desc=  "DIDSDVTH" *)   real DIDSDVTH;
        #         (* desc=  "DIGSDVTH" *)   real DIGSDVTH;
        #         (* desc=  "DIGDDVTH" *)   real DIGDDVTH;
        #         (* desc=  "DIIIDVTH" *)   real DIIIDVTH;
        #         (* desc=  "DIGIDLDVTH" *) real DIGIDLDVTH;
        #         (* desc=  "DIGISLDVTH" *) real DIGISLDVTH;
        #         (* desc=  "DITHDVTH" *)   real DITHDVTH;
        #      `endif
        #
        #      (* desc=  "ITH" *)      real ITH;
        #      (* desc=  "DITHDVG" *)  real DITHDVG;
        #      (* desc=  "DITHDVS" *)  real DITHDVS;
        #      (* desc=  "DITHDVD" *)  real DITHDVD;
        #   `endif
        #`endif
        #
        ## Variables Inside the Model
        #integer devsign;
        #
        #real self.NFINtotal;
        #real DevTemp;
        #real ids0, ids0_ov_dqi, ids, vgs, vds, vdsx, sigvds, vch, etaiv;
        #real vgs_noswap, vds_noswap, vgd_noswap;
        #real qd, qg, qs, qb;
        #real ni, self.epssub, epssp, epsratio, self.Eg, self.Eg0, self.Nc;
        #real Lg, deltaL, deltaL1, deltaLCV, self.Leff, self.Leff1, self.LeffCV, self.LeffCV_acc, self.Weff0, self.WeffCV0;
        #real self.cox, cdsc, cbox;
        #real self.nbody, self.phib, self.deltaPhi;
        #real T0, T1, T2, T3, T4, T4a, T5, T6, T7, T8, T9;
        #real Vtm, Vtm0, nVtm;
        #real beta, beta0 ;
        #real wf, wr;
        #
        ## Temperature Effects
        #real Tnom, TRatio, self.dvth_temp, delTemp, self.ThetaSS;
        #real self.K0_t, K0SI_t, K2SI_t, self.K1_t, K2SAT_t, self.A1_t, self.A2_t;
        #real self.AIGBINV_t, self.AIGBACC_t, self.AIGC_t, self.AIGS_t, self.AIGD_t;
        #real BETA0_t, SII0_t, BGISL_t, BGIDL_t, self.igtemp, self.PTWG_t, PTWGR_t;
        #real ALPHA0_t, self.ALPHself.A1_t, ALPHAII0_t, ALPHAII1_t;
        #real CJS_t, CJSWS_t, CJSWGD_t, CJD_t, CJSWD_t, CJSWGS_t;
        #real self.PBS_t, self.PBSWS_t, self.PBSWGS_t, self.PBD_t, self.PBSWD_t, self.PBSWGD_t;
        #real JSS_t, JSWS_t, JSWGS_t, JSD_t, JSWD_t, JSWGD_t;
        #real self.JTSS_t, self.JTSD_t, self.JTSSWS_t, self.JTSSWD_t, self.JTSSWGS_t, JTSSWGD_t;
        #real NJTS_t, self.NJTSD_t, NJTSSW_t, Nself.JTSSWD_t, NJTSSWG_t, NJTSSWGD_t;
        #real K2_t;
        #real K0SISAT_t, K2SISAT_t;
        #
        ## Variables for analytical surface potential
        #real q0;
        #real T10, T11, T12;
        #real e0, e1, e2;
        #
        ## Accumulation Model
        #real vgsfb, vgsfbeff;
        #
        ## Short Channel Effect
        #real ETA0_t, ETA0R_t;
        #real self.scl, self.vbi, phist, dvth_vtroll, dvth_dibl, dvth_rsce, dvth_all;
        #real tmp, self.Theta_SCE, self.Theta_SW, self.Theta_DIBL, self.Theta_RSCE, self.Theta_DITS;
        #
        ## Lateral Non-uniform Doping Effect
        #real Mnud;
        #
        ## Body Effect for BULKMOD=1
        #real ves, vesx, vesmax, veseff;
        #real Mob;
        #
        ## Quantum mechanical correction [units are MKS]
        #real self.coxeff, Tcen0, Tcen, self.dvch_qm, MTcen;
        #real E0, E0prime, E1, E1prime, self.mx, self.mxprime, md, mdprime;
        #real gprime, gfactor, gam0, gam1, kT;
        #
        ## Drain Saturation Voltage
        #real Vdseff, qis, qid, self.qbs, Dmobs;
        #
        ## Midpoint Potential and Charge
        #real qia, qia2, qba, dqi;
        #real qb0;
        #real self.eta_mu, self.eta_mu_cv, Eeffm, Eeffm_cv, Dmob, Dmob_cv, self.u0, ueff, self.u0_a, self.u0r;
        #real self.UA_t, UAR_t, self.UC_t, UCR_t, self.UCS_t, self.UD_t, UDR_t, U0_t, U0R_t, ETAMOB_t, Eeffs, self.EeffFactor;
        #
        #real Dr, self.WeffWRFactor;
        #real self.RSourceGeo, self.RDrainGeo;
        #real RDSWMIN_i, self.RDWMIN_i, self.RSWMIN_i;
        #real Rdrain, Rsource;
        #
        #real self.rdstemp, Rdsi, Rdss;
        #real self.RSDR_t, RSDRR_t, self.RDDR_t, RDDRR_t;
        #
        #real DIBLfactor, PVAGfactor, diffVds, VaDIBL, Vgst2Vtm, Moc, Mclm;
        #real MclmCV, inv_MclmCV;
        #
        #real Dvsat, Vdsat, inv_MEXP, DvsatCV, Nsat;
        #real self.VSAT_t, self.VSAT1_t, VSAT1R_t, self.VSATCV_t, self.MEXP_t, MEXPR_t, Esat, EsatL, Esat1, Esat1L, EsatCV, EsatCVL;
        #real WVCox, Ta, Tb, Tc;
        #
        ## Asymmetry Model
        #real VSAT1_a, MEXP_a, PTWG_a, RSDR_a, RDDR_a, PDIBL1_a, VSAT_a;
        #
        ## Geometry dependent Source/Drain Resistance
        #real mu_max, mu_rsd, rhorsd, afin, thetarsp;
        #real Rsp, lt, arsd_total, prsd_total, alpha;
        #real eta, RrsdTML, Rrsdside, Rrsd;
        #real Rdsgeo, Arsd, Prsd;
        #
        ## Geometry dependent fringing capacitance
        #real Hg, Wg, Trsd, Hrsd, Cgg_top, Cgg_side, self.Cfr_geo, Acorner, Ccorner;
        #
        ## Gate Electrode Resistance
        #`ifdef __RGATEMOD__
        #   real self.ggeltd, Rgeltd;
        #`endif
        #
        ## Gate Current
        #real Vaux_Igbinv, igbinv, self.igsd_mult, self.igsd_mult0, igbs, igbd;
        #real Voxacc, Vaux_Igbacc, vfbzb, igbacc;
        #real igcs, igcd, igc0, Vdseffx, T1_exp;
        #real igisl, igidl, self.vfbsd, igs, igd, vgs_eff, vgd_eff;
        #real self.Aechvb, self.Bechvb, self.Toxratio, self.Toxratioedge;
        #
        ## Impact Ionization current
        #real Iii, Vdiff, Vdsatii, VgsStep, Ratio, ALPHAII;
        #
        ## Accumulation Capacitance
        #real self.cox_acc;
        #real qg_acc, qb_acc;
        #real vge;
        #
        ## Parasitic Capacitance
        #real qgs_ov, qgd_ov, qgs_fr, qgd_fr, qds_fr;
        #real qgs_parasitic, qgd_parasitic, Qes, Qed, Qeg;
        #real vgs_overlap, vgd_overlap, vge_overlap;
        #real cgsp, cgdp, self.csbox, self.cdbox, self.cgbox, self.vfbsdcv;
        #
        ## Junction Current and Capacitance
        #real Ies, Ied, ves_jct, ved_jct, vec;
        #real self.Czbs, self.Czbssw, self.Czbsswg, self.Czbd, self.Czbdsw, self.Czbdswg;
        #real pb2, arg, sarg, Qec;
        #real Qesj, Qesj1, Qesj2, Qesj3, Qedj, Qedj1, Qedj2, Qedj3;
        #real self.Isbs, self.Isbd, Nvtms, Nvtmd;
        #real SslpRev, IVjsmRev, VjsmRev, SslpFwd, IVjsmFwd, VjsmFwd, XExpBVS;
        #real DslpRev, IVjdmRev, VjdmRev, DslpFwd, IVjdmFwd, VjdmFwd, XExpBVD;
        #real self.igentemp, idsgen, self.LINTIGEN_i;
        #
        ## NQS Gate Resistance
        #`ifdef __NQSMOD1__
        #   real gcrg, XRCRG1_i, XRCRG2_i;
        #   real IdovVds;
        #`endif
        #
        ## NQS Charge Deficit Model
        #`ifdef __NQSMOD2__
        #   real xdpart, gtau, gcrg, XRCRG1_i, XRCRG2_i;
        #   real IdovVds;
        #`endif
        #
        ## Flicker Noise
        #real self.LIself.NTNOI_i;
        #real self.litl, Esatnoi, self.Leffnoi, self.Leffnoisq, DelClm;
        #real N0, Nl, Nstar, Ssi, Swi, FNPowerAt1Hz;
        #
        ## Thermal Noise
        #real self.NTNOI_i, qinv;
        #real Gtnoi, sid;
        #real gspr, gdpr;
        #
        ## Variables Controlled by Correlated Thermal Noise Switch
        #`ifdef  TNOIMOD1
        #   real Abulk, etaa, gamma, delta, epsilon, gche;
        #   real npart_beta, npart_theta, ctnoi, npart_c;
        #   real noiGd0, GammaGd0, C0, sf;
        #`endif
        #
        ## Self Heating
        #`ifdef __SHMOD__
        #   real gth, cth;
        #`endif
        #
        ## Binning
        #real Inv_L, Inv_NFIN, Inv_LNFIN;
        #real NBODY_i, PHIG_i, CFD_i, CFS_i, COVS_i, COVD_i, self.CGSO_i, self.CGDO_i;
        #real self.CGSL_i, self.CGDL_i, self.CGBL_i, self.CKAPPAS_i, self.CKAPPAD_i, self.CKAPPAB_i;
        #real QMFACTOR_i, self.QMTCENCV_i, self.QMTCENCVA_i, self.KSATIV_i, KSATIVR_i, KSATIV_a;
        #real self.CDSC_i, CDSCD_i, CDSCD_a, CDSCDR_i, self.CIT_i, self.DVT0_i, CITR_i, CIT_a;
        #real DVT1_i, DVT1SS_i, self.PHIN_i, ETA0_i, ETA0_a, ETA0R_i, DSUB_i, VSAT_i, VSATR_i, VSATR_t;
        #real self.DVTP0_i, self.DVTP1_i ;
        #real K0_i, K01_i, K0SI_i, K0SI1_i, K2SI_i, K2SI1_i, PHIBE_i, K1_i, K11_i, K2SAT_i, K2SAT1_i;
        #real self.DELTAVSAT_i, self.PSAT_i, self.DELTAVSATCV_i, self.PSATCV_i, VSAT1_i, VSAT1R_i, PTWG_i, PTWGR_i, VSATCV_i;
        #real UP_i, U0_i, U0R_i, ETAMOB_i, NGATE_i, RDSW_i, UPR_i;
        #real self.PRWGS_i, self.PRWGD_i, WR_i, PDIBL1_i, PDIBL1R_i, self.PDIBL2_i,PDIBL2R_i, PDIBL2_a ;
        #real self.DROUT_i, self.PVAG_i;
        #real AIGBINV_i, AIGBINV1_i, self.BIGBINV_i, self.CIGBINV_i, self.EIGBINV_i, self.NIGBINV_i;
        #real AIGBACC_i, AIGBACC1_i, self.BIGBACC_i, self.CIGBACC_i, self.NIGBACC_i;
        #real AIGC_i, AIGC1_i, self.BIGC_i, self.CIGC_i, self.PIGCD_i;
        #real AIGS_i, AIGS1_i, self.BIGS_i, self.CIGS_i, NTOX_i, self.POXEDGE_i;
        #real AIGD_i, AIGD1_i, self.BIGD_i, self.CIGD_i;
        #real AGIDL_i, BGIDL_i, CGIDL_i, EGIDL_i, PGIDL_i;
        #real AGISL_i, BGISL_i, CGISL_i, EGISL_i, PGISL_i;
        #real ALPHA0_i, ALPHA1_i, ALPHAII0_i, ALPHAII1_i, BETA0_i;
        #real BETAII0_i, BETAII1_i, BETAII2_i, ESATII_i;
        #real LII_i, SII0_i, SII1_i, SII2_i, SIID_i, TII_i;
        #real MEXP_i, MEXPR_i;
        #real self.PCLM_i, self.PCLMG_i, self.PCLMCV_i, PCLM_a, PCLMR_i;
        #real A1_i, A2_i, A11_i, A21_i;
        #real self.K1RSCE_i, LPE0_i, self.DVTSHIFT_i, DVTSHIFT_a, DVTSHIFTR_i ;
        #real UA_i, UC_i, self.EU_i, UD_i, UCS_i, UAR_i, EUR_i, UCR_i, UDR_i, UA_a, UD_a, UC_a, EU_a;
        #real UA1_i, UA1R_i, UC1_i, UD1_i, UCSTE_i, UTE_i, UTL_i, EMOBT_i, UC1R_i, UD1R_i, UTER_i, UTLR_i;
        #real PTWGT_i;
        #real AT_i, ATCV_i, ATR_i;
        #real self.RDW_i, self.RSW_i;
        #real PRT_i, KT1_i, TSS_i, IIT_i, IGT_i, TGIDL_i;
        #real NTGEN_i, self.AIGEN_i, self.BIGEN_i;
        #real K0SISAT_i, K0SISAT1_i;
        #real K2SISAT_i, K2SISAT1_i;
        #real K2_i, K21_i;
        #
        ## Variables of Unified Finfet Compact Model
        #real self.Cins, self.Ach, self.Weff_UFCM, qdep,rc, vth_fixed_factor_Sub, vth_fixed_factor_SI, qm, self.Qdep_ov_self, qi_acc_for_QM;
        #real fieldnormalizationfactor, auxQMfact, QMFACTORCVfinal;
        #real psipclamp, sqrtpsip, nq, F0;
        #
        #===================================================
        #               analog block begins
        #===================================================

    # ************************************************
    # *      Geometry dependent calculations         *
    # ************************************************
    def CMGBiasIndepCalc(self):
        # Variable Initialization to Prevent Hidden States
        qid         =  0.0
        qis         =  0.0
        self.qba         =  0.0
        T11         =  0.0
        T12         =  0.0
        ids         =  0.0
        sigvds      =  0.0
        Iii         =  0.0
        qd          =  0.0
        qg          =  0.0
        qs          =  0.0
        qb          =  0.0
        self.Weff0       =  0.0
        self.WeffCV0     =  0.0
        CJS_t       =  0.0
        CJSWS_t     =  0.0
        CJSWGS_t    =  0.0
        CJD_t       =  0.0
        CJSWD_t     =  0.0
        CJSWGD_t    =  0.0
        self.PBS_t       =  0.0
        self.PBSWS_t     =  0.0
        self.PBSWGS_t    =  0.0
        self.PBD_t       =  0.0
        self.PBSWD_t     =  0.0
        self.PBSWGD_t    =  0.0
        JSS_t       =  0.0
        JSWS_t      =  0.0
        JSWGS_t     =  0.0
        JSD_t       =  0.0
        JSWD_t      =  0.0
        JSWGD_t     =  0.0
        self.JTSS_t      =  0.0
        self.JTSSWS_t    =  0.0
        self.JTSSWGS_t   =  0.0
        self.JTSD_t      =  0.0
        self.JTSSWD_t    =  0.0
        self.JTSSWGD_t   =  0.0
        NJTS_t      =  0.0
        NJTSSW_t    =  0.0
        NJTSSWG_t   =  0.0
        self.NJTSD_t     =  0.0
        self.NJTSSWD_t   =  0.0
        NJTSSWGD_t  =  0.0
        Ies         =  0.0
        Ied         =  0.0
        self.Czbs        =  0.0
        self.Czbssw      =  0.0
        self.Czbsswg     =  0.0
        self.Czbd        =  0.0
        self.Czbdsw      =  0.0
        self.Czbdswg     =  0.0
        Qes         =  0.0
        Qed         =  0.0
        Qeg         =  0.0
        self.Isbs        =  0.0
        self.Isbd        =  0.0
        Nvtms       =  0.0
        Nvtmd       =  0.0
        SslpRev     =  0.0
        IVjsmRev    =  0.0
        VjsmRev     =  0.0
        SslpFwd     =  0.0
        IVjsmFwd    =  0.0
        VjsmFwd     =  0.0
        DslpRev     =  0.0
        IVjdmRev    =  0.0
        VjdmRev     =  0.0
        DslpFwd     =  0.0
        IVjdmFwd    =  0.0
        VjdmFwd     =  0.0
        XExpBVS     =  0.0
        XExpBVD     =  0.0
        idsgen      =  0.0
        q0          =  0.0
        Tcen        =  0.0
        MTcen       =  0.0
        Rdrain      =  0.0
        Rsource     =  0.0
        self.Cfr_geo     =  0.0
        igbinv      =  0.0
        igbs        =  0.0
        igbd        =  0.0
        igbacc      =  0.0
        igcs        =  0.0
        igcd        =  0.0
        igidl       =  0.0
        igisl       =  0.0
        igs         =  0.0
        igd         =  0.0
        self.cox_acc     =  0.0
        self.CGSO_i      =  0.0
        self.CGDO_i      =  0.0
        self.qb_acc      =  0.0
        self.qg_acc      =  0.0
        qgs_fr      =  0.0
        qgd_fr      =  0.0
        qds_fr      =  0.0
        qgs_parasitic  =  0.0
        qgd_parasitic  =  0.0
        FNPowerAt1Hz   =  0.0
        Gtnoi       =  0.0
        gspr        =  0.0
        gdpr        =  0.0
        Dr          =  1.0
        CDSCDR_i    =  0.0
        ETA0R_i     =  0.0
        VSAT1R_i    =  0.0
        VSAT1R_t    =  0.0
        MEXPR_i     =  0.0
        MEXPR_t     =  0.0
        PTWGR_i     =  0.0
        PTWGR_t     =  0.0
        PDIBL1R_i   =  0.0
        PDIBL2R_i   =  0.0
        self.PHIBE_i     =  0.0
        K1_i        =  0.0
        K11_i       =  0.0
        K2SAT_i     =  0.0
        K2SAT1_i    =  0.0
        KSATIVR_i   =  0.0
        K2_i        =  0.0
        K21_i       =  0.0
        UC_i        =  0.0
        UC1_i       =  0.0
        self.UC_t        =  0.0
        U0R_i       =  0.0
        UPR_i       =  0.0
        EUR_i       =  0.0
        ATR_i       =  0.0
        CITR_i      =  0.0
        ETA0R_i     =  0.0
        self.DVTP0_i     =  0.0
        self.DVTP1_i     =  0.0
        PDIBL2R_i   =  0.0
        PCLMR_i     =  0.0
        self.LeffCV_acc  =  0.0
        RDDRR_t     =  0.0
        RSDRR_t     =  0.0
        Rdsi        =  0.0
        T3          =  0.0
        Tcen0       =  0.0
        veseff      =  0.0
        U0R_t       =  0.0
        UAR_t       =  0.0
        UCR_t       =  0.0
        UDR_t       =  0.0
        VSAT_a      =  0.0
        DVTSHIFTR_i =  0.0
        UA1R_i      =  0.0
        UAR_i       =  0.0
        UC1R_i      =  0.0
        UCR_i       =  0.0
        UD1R_i      =  0.0
        UDR_i       =  0.0
        UTER_i      =  0.0
        UTLR_i      =  0.0
        VSATR_i     =  0.0
        VSATR_t     =  0.0
        self.u0r         =  0.0

        ## Voltages
        ## V(`IntrinsicGate, si)
        #V_IntrinsicGate_si = edge_voltages[self.V_IntrinsicGate_si_ref]
        #
        ## V(di, si)
        #V_di_si = edge_voltages[self.V_di_si_ref]
        #
        ## V(d, s)
        #V_d_s = edge_voltages[self.V_d_s_ref]
        #
        ## v(d, di)
        #V_d_di = edge_voltages[self.V_d_di_ref]
        #
        ## v(s, si)
        #V_s_si = edge_voltages[self.V_s_si_ref]
        #
        ## V(`IntrinsicGate, di)
        #V_IntrinsicGate_di = edge_voltages[self.V_IntrinsicGate_di_ref]
        #
        ## V(e, si)
        #V_e_si = edge_voltages[self.V_e_si_ref]
        #
        ## V(e, di)
        #V_e_di = edge_voltages[self.V_e_di_ref]
        #
        ## V(`IntrinsicGate, e)
        #V_IntrinsicGate_e = edge_voltages[self.V_IntrinsicGate_e_ref]
        #
        ## V(`GateEdgeNode, si)
        #V_GateEdgeNode_si = edge_voltages[self.V_GateEdgeNode_si_ref]
        #
        ## V(`GateEdgeNode, di)
        #V_GateEdgeNode_di = edge_voltages[self.V_GateEdgeNode_di_ref]
        #
        #if (self.CGEOMOD == 1):
        #    # V(`GateEdgeNode, s)
        #    V_GateEdgeNode_s = edge_voltages[self.V_GateEdgeNode_s_ref]
        #
        #    # V(`GateEdgeNode, d)
        #    V_GateEdgeNode_d = edge_voltages[self.V_GateEdgeNode_d_ref]
        #else:
        #   V_GateEdgeNode_s = 0
        #   V_GateEdgeNode_d = 0
        #
        #
        ## V(`GateEdgeNode, gi) # REVISIT
        #V_GateEdgeNode_gi = edge_voltages[0]
        #
        ## V(e, `GateEdgeNode)
        #V_e_GateEdgeNode = edge_voltages[self.V_e_GateEdgeNode_ref]
        #
        ## V(`GateEdgeNode, e)
        #V_GateEdgeNode_e = -edge_voltages[self.V_e_GateEdgeNode_ref]
        #
        #if self.__RGATEMOD__:
        #    # V(g, ge)
        #    V_g_ge = edge_voltages[self.V_g_ge_ref]
        #else:
        #    V_g_ge = 0
        #
        #V_si_s = -1 * edge_voltages[self.V_s_si_ref]
        #
        #V_di_d = -1 * edge_voltages[self.V_d_di_ref]

        # Initialise currents
        I_di_si            = 0
        I_si_di            = 0
        I_d_s              = 0
        I_d_di             = 0
        I_si_di            = 0
        I_s_si             = 0
        I_e_si             = 0
        I_e_di             = 0
        I_e_GateEdgeNode   = 0
        I_GateEdgeNode_si  = 0
        I_GateEdgeNode_di  = 0
        I_IntrinsicGate_si = 0
        I_IntrinsicGate_di = 0
        I_IntrinsicGate_e  = 0 #REVISIT
        I_g_ge             = 0 #REVISIT
        I_di_e             = 0
        I_si_e             = 0

        # Thermal Noise
        sid         =  0.0

        if self.__TNOIMOD1__:
            ctnoi  =  0.0
            sf     =  0.0
            C0     =  0.0
            gamma  =  0.0
            delta  =  0.0

        if self.__RGATEMOD__:
            self.ggeltd  =  0.0

        # Unified FinFET Model
        qm    =  1.0
        self.Cins  =  1.0
        self.Ach   =  1.0
        self.Weff_UFCM  =  1.0
        qdep  =  -1.0
        self.rc    =  1.0
        vth_fixed_factor_Sub  =  1.0
        vth_fixed_factor_SI   =  1.0
        qi_acc_for_QM         =  0.0
        fieldnormalizationfactor  =  0.0
        auxQMfact  =  0.0
        QMFACTORCVfinal  =  0.0
        psipclamp  =  1.0
        sqrtpsip   =  1.0
        nq  =  1.0
        F0  =  0.0
        e0  =  0.0
        e1  =  0.0
        e2  =  0.0
        self.Qdep_ov_self  =  0.0

        # Constants
        if ( self.TYPE == self.ntype ):
            self.devsign  =  1
        else:
            self.devsign  =  -1

        self.epssub    =  self.EPSRSUB * self.EPS0
        epssp     =  self.EPSRSP * self.EPS0
        cbox      =  self.EPSROX * self.EPS0 / self.EOTBOX
        epsratio  =  self.EPSRSUB / self.EPSROX

        #if ($port_connected(t) == 1):
        #    `ifdef __SHMOD__
        #        if (self.SHMOD == 0):
        #            if (self.SH_WARN == 1):
        #               print("The optional 5th terminal is present but not active because SHMOD=0.")
        #
        #
        #    `else
        #          Temp(t) <+ 0.0
        #          if (self.SH_WARN == 1):
        #             print("The optional 5th terminal is present but not active because the model was not compiled with self-heating enabled (__SHMOD__ was not activated).")
        #
        #    `endif


        # Constants for Quantum Mechanical Effects
        self.mx       =  0.916 * self.MEL
        self.mxprime  =  0.190 * self.MEL
        md       =  0.190 * self.MEL
        mdprime  =  0.417 * self.MEL
        gprime   =  4.0
        gfactor  =  2.0

        # Effective Channel Length for I-V / C-V
        Lg       =  self.L + self.XL
        deltaL   =  self.LINT + self.LL * pow(Lg, -self.LLN)
        deltaL1  =  self.LINT + self.LL * pow(Lg + self.DLBIN, -self.LLN)
        deltaLCV =  self.DLC + self.LLC * pow(Lg, -self.LLN)
        self.Leff     =  Lg - 2.0 * deltaL
        self.Leff1    =  Lg + self.DLBIN - 2.0 * deltaL1        #Used in the binning equations only
        self.LeffCV   =  Lg - 2.0 * deltaLCV
        if (self.BULKMOD != 0):
            self.LeffCV_acc  =  self.LeffCV - self.DLCACC

        # Total Fins
        self.NFINtotal  =  self.NFIN * self.NF

        # Range Checking on self.Leff and self.Leff1
        if (self.Leff <= 0.0):
            print("Fatal: self.Leff = %e is not positive.", self.Leff)
            sys.exit(0)
        elif (self.Leff <= 1.0e-9):
            print("Warning: self.Leff = %e <= 1.0e-9.", self.Leff)


        if (self.Leff1 <= 0.0):
           print("Fatal: self.Leff1 = %e is not positive.", self.Leff1)
           sys.exit(0)
        elif (self.Leff1 <= 1.0e-9):
           print("Warning: self.Leff1 = %e <= 1.0e-9.", self.Leff1)


        # Binning
        Inv_L      =  1.0e-6 / (self.Leff1)
        Inv_NFIN   =  1.0 / self.NFIN
        Inv_LNFIN  =  1.0e-6 / (self.Leff1 * self.NFIN)

        # Nbody Binning Equation for UFCM Parameters
        NBODY_i  =  self.NBODY + Inv_L * self.LNBODY + Inv_NFIN * self.NNBODY + Inv_LNFIN * self.PNBODY

        if (self.NBODYN1 != 0.0):
           NBODY_i  =  NBODY_i * (1.0 + NBODYN1/NFIN * self.lln(1.0 + NFIN/NBODYN2))

        # Model Parameters for Unified FinFET Compact Model by Juan Duarte 10/2013
        if self.GEOMOD == 0: # Double Gate
            if (not self.param_given("TFIN_TOP") or not self.param_given("TFIN_BASE")):
                self.Weff_UFCM  =  2.0 * self.HFIN
                self.Cins  =  self.Weff_UFCM * self.EPSROX * self.EPS0 / self.EOT
                self.Ach   =  self.HFIN * self.TFIN
                self.rc    =  (2.0 * self.Cins / (self.Weff_UFCM * self.Weff_UFCM * self.epssub / self.Ach))
                self.Qdep_ov_self  =  -self.q * NBODY_i * self.Ach / self.Cins
            else:
                self.Weff_UFCM  =  2.0 * math.sqrt( self.HFIN * self.HFIN + (self.TFIN_TOP - self.TFIN_BASE) * (self.TFIN_TOP - self.TFIN_BASE) / 4.0)
                self.Cins  =  self.Weff_UFCM * self.EPSROX * self.EPS0 / self.EOT
                self.Ach   =  self.HFIN * (self.TFIN_TOP + self.TFIN_BASE) / 2.0
                self.rc    =  (2.0 * self.Cins / (self.Weff_UFCM * self.Weff_UFCM * self.epssub / self.Ach))
                self.Qdep_ov_self  =  -self.q * NBODY_i * self.Ach / self.Cins

        elif self.GEOMOD == 1: # Triple Gate
            if (not self.param_given("TFIN_TOP") or not self.param_given("TFIN_BASE")):
                self.Weff_UFCM  =  2.0 * self.HFIN + self.TFIN
                self.Cins  =  self.Weff_UFCM * self.EPSROX * self.EPS0 / self.EOT
                self.Ach   =  self.HFIN * self.TFIN
                self.rc    =  (2.0 * self.Cins / (self.Weff_UFCM * self.Weff_UFCM * self.epssub / self.Ach))
                self.Qdep_ov_self  =  -self.q * NBODY_i * self.Ach / self.Cins
            else:
                self.Weff_UFCM  =  2.0 * math.sqrt(self.HFIN * self.HFIN + (self.TFIN_TOP - self.TFIN_BASE) * (self.TFIN_TOP - self.TFIN_BASE) / 4.0) + self.TFIN_TOP
                self.Cins  =  self.Weff_UFCM * self.EPSROX * self.EPS0 / self.EOT
                self.Ach   =  self.HFIN * (self.TFIN_TOP + self.TFIN_BASE) / 2.0
                self.rc    =  (2.0 * self.Cins /(self.Weff_UFCM * self.Weff_UFCM * self.epssub / self.Ach))
                self.Qdep_ov_self  =  -self.q * NBODY_i * self.Ach / self.Cins


        elif self.GEOMOD == 2: # Quadruple Gate
            if (not self.param_given("TFIN_TOP") or not self.param_given("TFIN_BASE")):
                self.Weff_UFCM  =  2.0 * self.HFIN + 2.0 * self.TFIN
                self.Cins  =  self.Weff_UFCM * self.EPSROX * self.EPS0 / self.EOT
                self.Ach   =  self.HFIN * self.TFIN
                self.rc    =  (2.0 * self.Cins / (self.Weff_UFCM * self.Weff_UFCM * self.epssub / self.Ach))
                self.Qdep_ov_self  =  -self.q * NBODY_i * self.Ach / self.Cins
            else:
                self.Weff_UFCM  =  2.0 * math.sqrt(self.HFIN * self.HFIN + (self.TFIN_TOP - self.TFIN_BASE) * (self.TFIN_TOP - self.TFIN_BASE) / 4.0) + self.TFIN_TOP + self.TFIN_BASE
                self.Cins  =  self.Weff_UFCM * self.EPSROX * self.EPS0 / self.EOT
                self.Ach   =  self.HFIN * (self.TFIN_TOP + self.TFIN_BASE) / 2.0
                self.rc    =  (2.0 * self.Cins / (self.Weff_UFCM * self.Weff_UFCM * self.epssub / self.Ach))
                self.Qdep_ov_self  =  -self.q * NBODY_i * self.Ach / self.Cins


        elif self.GEOMOD == 3: # Cylindrical Gate
            self.Weff_UFCM  =  math.pi * self.D
            self.Cins  =  2.0 * math.pi * self.EPSROX * self.EPS0 / math.log(1.0 + 2.0 * self.EOT / self.D)
            self.Ach   =  math.pi * self.D * self.D / 4.0
            self.rc    =  (2.0 * self.Cins / (self.Weff_UFCM * self.Weff_UFCM * self.epssub / self.Ach))
            self.Qdep_ov_self  =  -self.q * NBODY_i * self.Ach / self.Cins

        elif GEMOD == 4: # Unified Model
            self.Weff_UFCM  =  self.W_UFCM
            self.Cins  =  self.CINS_UFCM
            self.Ach   =  self.ACH_UFCM
            self.rc    =  (2.0 * self.Cins / (self.Weff_UFCM * self.Weff_UFCM * self.epssub / self.Ach))
            self.Qdep_ov_self  =  -self.q * NBODY_i * self.Ach / self.Cins

        # Cox Definition
        self.cox  =  self.Cins / self.Weff_UFCM
        if (self.BULKMOD != 0):
           self.cox_acc =  self.cox * self.EOT / self.EOTACC


        # Effective Width Calculation
        self.Weff0    =  self.Weff_UFCM - self.DELTAW
        self.WeffCV0  =  self.Weff_UFCM - self.DELTAWCV

        # SCE Scaling Length
        self.scl  =  math.sqrt((self.epssub * self.Ach / self.Cins) * (1.0 + self.Ach * self.Cins / (2.0 * self.epssub * self.Weff_UFCM * self.Weff_UFCM)))

        # Binning Equations (Process Parameters)
        PHIG_i   =  self.PHIG + Inv_L * self.LPHIG + Inv_NFIN * self.NPHIG + Inv_LNFIN * self.PPHIG
        NGATE_i  =  self.NGATE + Inv_L * self.LNGATE + Inv_NFIN * self.NNGATE + Inv_LNFIN * self.PNGATE

        # Binning Equations (Model Parameters)
        self.CIT_i       =  self.CIT + Inv_L * self.LCIT + Inv_NFIN * self.NCIT + Inv_LNFIN * self.PCIT
        self.CDSC_i      =  self.CDSC + Inv_L * self.LCDSC + Inv_NFIN * self.NCDSC + Inv_LNFIN * self.PCDSC
        self.CDSCD_i     =  self.CDSCD + Inv_L * self.LCDSCD + Inv_NFIN * self.NCDSCD + Inv_LNFIN * self.PCDSCD
        self.DVT0_i      =  self.DVT0 + Inv_L * self.LDVT0 + Inv_NFIN * self.NDVT0 + Inv_LNFIN * self.PDVT0
        DVT1_i      =  self.DVT1 + Inv_L * self.LDVT1 + Inv_NFIN * self.NDVT1 + Inv_LNFIN * self.PDVT1
        DVT1SS_i    =  self.DVT1SS + Inv_L * self.LDVT1SS + Inv_NFIN * self.NDVT1SS + Inv_LNFIN * self.PDVT1SS
        self.PHIN_i      =  self.PHIN + Inv_L * self.LPHIN + Inv_NFIN * self.NPHIN + Inv_LNFIN * self.PPHIN
        ETA0_i      =  self.ETA0 + Inv_L * self.LETA0 + Inv_NFIN * self.NETA0 + Inv_LNFIN * self.PETA0
        DSUB_i      =  self.DSUB + Inv_L * self.LDSUB + Inv_NFIN * self.NDSUB + Inv_LNFIN * self.PDSUB
        self.K1RSCE_i    =  self.K1RSCE + Inv_L * self.LK1RSCE + Inv_NFIN * self.NK1RSCE + Inv_LNFIN * self.PK1RSCE
        LPE0_i      =  self.LPE0 + Inv_L * self.LLPE0 + Inv_NFIN * self.NLPE0 + Inv_LNFIN * self.PLPE0
        self.DVTSHIFT_i  =  self.DVTSHIFT + Inv_L * self.LDVTSHIFT + Inv_NFIN * self.NDVTSHIFT + Inv_LNFIN * self.PDVTSHIFT
        K0_i        =  self.K0 + Inv_L * self.LK0 + Inv_NFIN * self.NK0 + Inv_LNFIN * self.PK0
        K01_i       =  self.K01 + Inv_L * self.LK01 + Inv_NFIN * self.NK01 + Inv_LNFIN * self.PK01
        K0SI_i      =  self.K0SI + Inv_L * self.LK0SI + Inv_NFIN * self.NK0SI + Inv_LNFIN * self.PK0SI
        K0SI1_i     =  self.K0SI1 + Inv_L * self.LK0SI1 + Inv_NFIN * self.NK0SI1 + Inv_LNFIN * self.PK0SI1
        K2SI_i      =  self.K2SI + Inv_L * self.LK2SI + Inv_NFIN * self.NK2SI + Inv_LNFIN * self.PK2SI
        K2SI1_i     =  self.K2SI1 + Inv_L * self.LK2SI1 + Inv_NFIN * self.NK2SI1 + Inv_LNFIN * self.PK2SI1
        K0SISAT_i   =  self.K0SISAT + Inv_L * self.LK0SISAT + Inv_NFIN * self.NK0SISAT + Inv_LNFIN * self.PK0SISAT
        K0SISAT1_i  =  self.K0SISAT1 + Inv_L * self.LK0SISAT1 + Inv_NFIN * self.NK0SISAT1 + Inv_LNFIN * self.PK0SISAT1
        K2SISAT_i   =  self.K2SISAT + Inv_L * self.LK2SISAT + Inv_NFIN * self.NK2SISAT + Inv_LNFIN * self.PK2SISAT
        K2SISAT1_i  =  self.K2SISAT1 + Inv_L * self.LK2SISAT1 + Inv_NFIN * self.NK2SISAT1 + Inv_LNFIN * self.PK2SISAT1

        if (self.BULKMOD != 0):
           if (self.BULKMOD == 2):
              K2_i      =  self.K2 + Inv_L * self.LK2 + Inv_NFIN * self.NK2 + Inv_LNFIN * self.PK2
              K21_i     =  self.K21 + Inv_L * self.LK21 + Inv_NFIN * self.NK21 + Inv_LNFIN * self.PK21
              K2SAT_i   =  self.K2SAT + Inv_L * self.LK2SAT + Inv_NFIN * self.NK2SAT + Inv_LNFIN * self.PK2SAT
              K2SAT1_i  =  self.K2SAT1 + Inv_L * self.LK2SAT1 + Inv_NFIN * self.NK2SAT1 + Inv_LNFIN * self.PK2SAT1

           self.PHIBE_i    =  self.PHIBE + Inv_L * self.LPHIBE + Inv_NFIN * self.NPHIBE + Inv_LNFIN * self.PPHIBE
           K1_i       =  self.K1 + Inv_L * self.LK1 + Inv_NFIN * self.NK1 + Inv_LNFIN * self.PK1
           K11_i      =  self.K11 + Inv_L * self.LK11 + Inv_NFIN * self.NK11 + Inv_LNFIN * self.PK11

        QMFACTOR_i    =  self.QMFACTOR + Inv_L * self.LQMFACTOR + Inv_NFIN * self.NQMFACTOR + Inv_LNFIN * self.PQMFACTOR
        self.QMTCENCV_i    =  self.QMTCENCV + Inv_L * self.LQMTCENCV + Inv_NFIN * self.NQMTCENCV + Inv_LNFIN * self.PQMTCENCV
        self.QMTCENCVA_i   =  self.QMTCENCVA + Inv_L * self.LQMTCENCVA + Inv_NFIN * self.NQMTCENCVA + Inv_LNFIN * self.PQMTCENCVA
        VSAT_i        =  self.VSAT + Inv_L * self.LVSAT + Inv_NFIN * self.NVSAT + Inv_LNFIN * self.PVSAT
        VSAT1_i       =  self.VSAT1 + Inv_L * self.LVSAT1 + Inv_NFIN * self.NVSAT1 + Inv_LNFIN * self.PVSAT1
        VSATCV_i      =  self.VSATCV + Inv_L * self.LVSATCV + Inv_NFIN * self.NVSATCV + Inv_LNFIN * self.PVSATCV
        self.DELTAVSAT_i   =  self.DELTAVSAT + Inv_L * self.LDELTAVSAT + Inv_NFIN * self.NDELTAVSAT + Inv_LNFIN * self.PDELTAVSAT
        self.PSAT_i        =  self.PSAT + Inv_L * self.LPSAT + Inv_NFIN * self.NPSAT + Inv_LNFIN * self.PPSAT
        self.DELTAVSATCV_i =  self.DELTAVSATCV + Inv_L * self.LDELTAVSATCV + Inv_NFIN * self.NDELTAVSATCV + Inv_LNFIN * self.PDELTAVSATCV
        self.PSATCV_i      =  self.PSATCV + Inv_L * self.LPSATCV + Inv_NFIN * self.NPSATCV + Inv_LNFIN * self.PPSATCV
        self.KSATIV_i      =  self.KSATIV + Inv_L * self.LKSATIV + Inv_NFIN * self.NKSATIV + Inv_LNFIN * self.PKSATIV
        MEXP_i        =  self.MEXP + Inv_L * self.LMEXP + Inv_NFIN * self.NMEXP + Inv_LNFIN * self.PMEXP
        PTWG_i        =  self.PTWG + Inv_L * self.LPTWG + Inv_NFIN * self.NPTWG + Inv_LNFIN * self.PPTWG
        U0_i          =  self.U0 + Inv_L * self.LU0 + Inv_NFIN * self.NU0 + Inv_LNFIN * self.PU0
        ETAMOB_i      =  self.ETAMOB + Inv_L * self.LETAMOB + Inv_NFIN * self.NETAMOB + Inv_LNFIN * self.PETAMOB
        UP_i          =  self.UP + Inv_L * self.LUP + Inv_NFIN * self.NUP + Inv_LNFIN * self.PUP
        UA_i          =  self.UA + Inv_L * self.LUA + Inv_NFIN * self.NUA + Inv_LNFIN * self.PUA
        if (self.BULKMOD != 0):
           UC_i       =  self.UC + Inv_L * self.LUC + Inv_NFIN * self.NUC + Inv_LNFIN * self.PUC

        self.EU_i          =  self.EU + Inv_L * self.LEU + Inv_NFIN * self.NEU + Inv_LNFIN * self.PEU
        UD_i          =  self.UD + Inv_L * self.LUD + Inv_NFIN * self.NUD + Inv_LNFIN * self.PUD
        UCS_i         =  self.UCS + Inv_L * self.LUCS + Inv_NFIN * self.NUCS + Inv_LNFIN * self.PUCS
        self.PCLM_i        =  self.PCLM + Inv_L * self.LPCLM + Inv_NFIN * self.NPCLM + Inv_LNFIN * self.PPCLM
        self.PCLMG_i       =  self.PCLMG + Inv_L * self.LPCLMG + Inv_NFIN * self.NPCLMG + Inv_LNFIN * self.PPCLMG
        self.PCLMCV_i      =  self.PCLMCV + Inv_L * self.LPCLMCV + Inv_NFIN * self.NPCLMCV + Inv_LNFIN * self.PPCLMCV
        A1_i          =  self.A1 + Inv_L * self.LA1 + Inv_NFIN * self.NA1 + Inv_LNFIN * self.PA1
        A11_i         =  self.A11 + Inv_L * self.LA11 + Inv_NFIN * self.NA11 + Inv_LNFIN * self.PA11
        A2_i          =  self.A2 + Inv_L * self.LA2 + Inv_NFIN * self.NA2 + Inv_LNFIN * self.PA2
        A21_i         =  self.A21 + Inv_L * self.LA21 + Inv_NFIN * self.NA21 + Inv_LNFIN * self.PA21
        self.RDSW_i        =  self.RDSW + Inv_L * self.LRDSW + Inv_NFIN * self.NRDSW + Inv_LNFIN * self.PRDSW
        self.RSW_i         =  self.RSW + Inv_L * self.LRSW + Inv_NFIN * self.NRSW + Inv_LNFIN * self.PRSW
        self.RDW_i         =  self.RDW + Inv_L * self.LRDW + Inv_NFIN * self.NRDW + Inv_LNFIN * self.PRDW
        self.PRWGD_i       =  self.PRWGD + Inv_L * self.LPRWGD + Inv_NFIN * self.NPRWGD + Inv_LNFIN * self.PPRWGD
        self.PRWGS_i       =  self.PRWGS + Inv_L * self.LPRWGS + Inv_NFIN * self.NPRWGS + Inv_LNFIN * self.PPRWGS
        WR_i          =  self.WR + Inv_L * self.LWR + Inv_NFIN * self.NWR + Inv_LNFIN * self.PWR
        self.PDIBL1      =  self.PDIBL1 + Inv_L * self.LPDIBL1 + Inv_NFIN * self.NPDIBL1 + Inv_LNFIN * self.PPDIBL1
        self.PDIBL2_i      =  self.PDIBL2 + Inv_L * self.LPDIBL2 + Inv_NFIN * self.NPDIBL2 + Inv_LNFIN * self.PPDIBL2
        self.DROUT_i       =  self.DROUT + Inv_L * self.LDROUT + Inv_NFIN * self.NDROUT + Inv_LNFIN * self.PDROUT
        self.PVAG_i        =  self.PVAG + Inv_L * self.LPVAG + Inv_NFIN * self.NPVAG + Inv_LNFIN * self.PPVAG
        AIGBINV_i     =  self.AIGBINV + Inv_L * self.LAIGBINV  + Inv_NFIN * self.NAIGBINV + Inv_LNFIN * self.PAIGBINV
        AIGBINV1_i    =  self.AIGBINV1 + Inv_L * self.LAIGBINV1 + Inv_NFIN * self.NAIGBINV1 + Inv_LNFIN * self.PAIGBINV1
        self.BIGBINV_i     =  self.BIGBINV + Inv_L * self.LBIGBINV + Inv_NFIN * self.NBIGBINV + Inv_LNFIN * self.PBIGBINV
        self.CIGBINV_i     =  self.CIGBINV + Inv_L * self.LCIGBINV + Inv_NFIN * self.NCIGBINV + Inv_LNFIN * self.PCIGBINV
        self.EIGBINV_i     =  self.EIGBINV + Inv_L * self.LEIGBINV + Inv_NFIN * self.NEIGBINV + Inv_LNFIN * self.PEIGBINV
        self.NIGBINV_i     =  self.NIGBINV + Inv_L * self.LNIGBINV + Inv_NFIN * self.NNIGBINV + Inv_LNFIN * self.PNIGBINV
        AIGBACC_i     =  self.AIGBACC + Inv_L * self.LAIGBACC + Inv_NFIN * self.NAIGBACC + Inv_LNFIN * self.PAIGBACC
        AIGBACC1_i    =  self.AIGBACC1 + Inv_L * self.LAIGBACC1 + Inv_NFIN * self.NAIGBACC1 + Inv_LNFIN * self.PAIGBACC1
        self.BIGBACC_i     =  self.BIGBACC + Inv_L * self.LBIGBACC + Inv_NFIN * self.NBIGBACC + Inv_LNFIN * self.PBIGBACC
        self.CIGBACC_i     =  self.CIGBACC + Inv_L * self.LCIGBACC + Inv_NFIN * self.NCIGBACC + Inv_LNFIN * self.PCIGBACC
        self.NIGBACC_i     =  self.NIGBACC + Inv_L * self.LNIGBACC + Inv_NFIN * self.NNIGBACC + Inv_LNFIN * self.PNIGBACC
        AIGC_i        =  self.AIGC + Inv_L * self.LAIGC + Inv_NFIN * self.NAIGC + Inv_LNFIN * self.PAIGC
        AIGC1_i       =  self.AIGC1 + Inv_L * self.LAIGC1 + Inv_NFIN * self.NAIGC1 + Inv_LNFIN * self.PAIGC1
        self.BIGC_i        =  self.BIGC + Inv_L * self.LBIGC + Inv_NFIN * self.NBIGC + Inv_LNFIN * self.PBIGC
        self.CIGC_i        =  self.CIGC + Inv_L * self.LCIGC + Inv_NFIN * self.NCIGC + Inv_LNFIN * self.PCIGC
        self.PIGCD_i       =  self.PIGCD + Inv_L * self.LPIGCD + Inv_NFIN * self.NPIGCD + Inv_LNFIN * self.PPIGCD
        AIGS_i        =  self.AIGS + Inv_L * self.LAIGS + Inv_NFIN * self.NAIGS + Inv_LNFIN * self.PAIGS
        AIGS1_i       =  self.AIGS1 + Inv_L * self.LAIGS1 + Inv_NFIN * self.NAIGS1 + Inv_LNFIN * self.PAIGS1
        self.BIGS_i        =  self.BIGS + Inv_L * self.LBIGS + Inv_NFIN * self.NBIGS + Inv_LNFIN * self.PBIGS
        self.CIGS_i        =  self.CIGS + Inv_L * self.LCIGS + Inv_NFIN * self.NCIGS + Inv_LNFIN * self.PCIGS
        AIGD_i        =  self.AIGD + Inv_L * self.LAIGD + Inv_NFIN * self.NAIGD + Inv_LNFIN * self.PAIGD
        AIGD1_i       =  self.AIGD1 + Inv_L * self.LAIGD1 + Inv_NFIN * self.NAIGD1 + Inv_LNFIN * self.PAIGD1
        self.BIGD_i        =  self.BIGD + Inv_L * self.LBIGD + Inv_NFIN * self.NBIGD + Inv_LNFIN * self.PBIGD
        self.CIGD_i        =  self.CIGD + Inv_L * self.LCIGD + Inv_NFIN * self.NCIGD + Inv_LNFIN * self.PCIGD
        NTOX_i        =  self.NTOX + Inv_L * self.LNTOX + Inv_NFIN * self.NNTOX + Inv_LNFIN * self.PNTOX
        self.POXEDGE_i     =  self.POXEDGE + Inv_L * self.LPOXEDGE + Inv_NFIN * self.NPOXEDGE + Inv_LNFIN * self.PPOXEDGE
        AGIDL_i       =  self.AGIDL + Inv_L * self.LAGIDL + Inv_NFIN * self.NAGIDL + Inv_LNFIN * self.PAGIDL
        BGIDL_i       =  self.BGIDL + Inv_L * self.LBGIDL + Inv_NFIN * self.NBGIDL + Inv_LNFIN * self.PBGIDL
        CGIDL_i       =  self.CGIDL + Inv_L * self.LCGIDL + Inv_NFIN * self.NCGIDL + Inv_LNFIN * self.PCGIDL
        EGIDL_i       =  self.EGIDL + Inv_L * self.LEGIDL + Inv_NFIN * self.NEGIDL + Inv_LNFIN * self.PEGIDL
        PGIDL_i       =  self.PGIDL + Inv_L * self.LPGIDL + Inv_NFIN * self.NPGIDL + Inv_LNFIN * self.PPGIDL
        AGISL_i       =  self.AGISL + Inv_L * self.LAGISL + Inv_NFIN * self.NAGISL + Inv_LNFIN * self.PAGISL
        BGISL_i       =  self.BGISL + Inv_L * self.LBGISL + Inv_NFIN * self.NBGISL + Inv_LNFIN * self.PBGISL
        CGISL_i       =  self.CGISL + Inv_L * self.LCGISL + Inv_NFIN * self.NCGISL + Inv_LNFIN * self.PCGISL
        EGISL_i       =  self.EGISL + Inv_L * self.LEGISL + Inv_NFIN * self.NEGISL + Inv_LNFIN * self.PEGISL
        PGISL_i       =  self.PGISL + Inv_L * self.LPGISL + Inv_NFIN * self.NPGISL + Inv_LNFIN * self.PPGISL
        ALPHA0_i      =  self.ALPHA0 + Inv_L * self.LALPHA0 + Inv_NFIN * self.NALPHA0 + Inv_LNFIN * self.PALPHA0
        ALPHA1_i      =  self.ALPHA1 + Inv_L * self.LALPHA1 + Inv_NFIN * self.NALPHA1 + Inv_LNFIN * self.PALPHA1
        ALPHAII0_i    =  self.ALPHAII0 + Inv_L * self.LALPHAII0 + Inv_NFIN * self.NALPHAII0 + Inv_LNFIN * self.PALPHAII0
        ALPHAII1_i    =  self.ALPHAII1 + Inv_L * self.LALPHAII1 + Inv_NFIN * self.NALPHAII1 + Inv_LNFIN * self.PALPHAII1
        BETA0_i       =  self.BETA0 + Inv_L * self.LBETA0 + Inv_NFIN * self.NBETA0 + Inv_LNFIN * self.PBETA0
        BETAII0_i     =  self.BETAII0 + Inv_L * self.LBETAII0 + Inv_NFIN * self.NBETAII0 + Inv_LNFIN * self.PBETAII0
        BETAII1_i     =  self.BETAII1 + Inv_L * self.LBETAII1 + Inv_NFIN * self.NBETAII1 + Inv_LNFIN * self.PBETAII1
        BETAII2_i     =  self.BETAII2 + Inv_L * self.LBETAII2 + Inv_NFIN * self.NBETAII2 + Inv_LNFIN * self.PBETAII2
        ESATII_i      =  self.ESATII + Inv_L * self.LESATII + Inv_NFIN * self.NESATII + Inv_LNFIN * self.PESATII
        LII_i         =  self.LII + Inv_L * self.LLII + Inv_NFIN * self.NLII + Inv_LNFIN * self.PLII
        SII0_i        =  self.SII0 + Inv_L * self.LSII0 + Inv_NFIN * self.NSII0 + Inv_LNFIN * self.PSII0
        SII1_i        =  self.SII1 + Inv_L * self.LSII1 + Inv_NFIN * self.NSII1 + Inv_LNFIN * self.PSII1
        SII2_i        =  self.SII2 + Inv_L * self.LSII2 + Inv_NFIN * self.NSII2 + Inv_LNFIN * self.PSII2
        SIID_i        =  self.SIID + Inv_L * self.LSIID + Inv_NFIN * self.NSIID + Inv_LNFIN * self.PSIID
        TII_i         =  self.TII + Inv_L * self.LTII + Inv_NFIN * self.NTII + Inv_LNFIN * self.PTII
        self.CFS_i    =  self.CFS + Inv_L * self.LCFS + Inv_NFIN * self.NCFS + Inv_LNFIN * self.PCFS
        self.CFD_i         =  self.CFD + Inv_L * self.LCFD + Inv_NFIN * self.NCFD + Inv_LNFIN * self.PCFD
        COVS_i        =  self.COVS + Inv_L * self.LCOVS + Inv_NFIN * self.NCOVS + Inv_LNFIN * self.PCOVS
        COVD_i        =  self.COVD + Inv_L * self.LCOVD + Inv_NFIN * self.NCOVD + Inv_LNFIN * self.PCOVD
        self.CGSL_i        =  self.CGSL + Inv_L * self.LCGSL + Inv_NFIN * self.NCGSL + Inv_LNFIN * self.PCGSL
        self.CGDL_i        =  self.CGDL + Inv_L * self.LCGDL + Inv_NFIN * self.NCGDL + Inv_LNFIN * self.PCGDL
        self.CGBL_i        =  self.CGBL + Inv_L * self.LCGBL + Inv_NFIN * self.NCGBL + Inv_LNFIN * self.PCGBL
        self.CKAPPAS_i     =  self.CKAPPAS + Inv_L * self.LCKAPPAS + Inv_NFIN * self.NCKAPPAS + Inv_LNFIN * self.PCKAPPAS
        self.CKAPPAD_i     =  self.CKAPPAD + Inv_L * self.LCKAPPAD + Inv_NFIN * self.NCKAPPAD + Inv_LNFIN * self.PCKAPPAD
        self.CKAPPAB_i     =  self.CKAPPAB + Inv_L * self.LCKAPPAB + Inv_NFIN * self.NCKAPPAB + Inv_LNFIN * self.PCKAPPAB
        NTGEN_i       =  self.NTGEN + Inv_L * self.LNTGEN + Inv_NFIN * self.NNTGEN + Inv_LNFIN * self.PNTGEN
        self.AIGEN_i       =  self.AIGEN + Inv_L * self.LAIGEN + Inv_NFIN * self.NAIGEN + Inv_LNFIN * self.PAIGEN
        self.BIGEN_i       =  self.BIGEN + Inv_L * self.LBIGEN + Inv_NFIN * self.NBIGEN + Inv_LNFIN * self.PBIGEN

        if (self.ASYMMOD != 0):
           CDSCDR_i     =  self.CDSCDR + Inv_L * self.LCDSCDR + Inv_NFIN * self.NCDSCDR + Inv_LNFIN * self.PCDSCDR
           CITR_i       =  self.CITR + Inv_L * self.LCITR + Inv_NFIN * self.NCITR + Inv_LNFIN * self.PCITR
           ETA0R_i      =  self.ETA0R + Inv_L * self.LETA0R + Inv_NFIN * self.NETA0R + Inv_LNFIN * self.PETA0R
           VSAT1R_i     =  self.VSAT1R + Inv_L * self.LVSAT1R + Inv_NFIN * self.NVSAT1R + Inv_LNFIN * self.PVSAT1R
           MEXPR_i      =  self.MEXPR + Inv_L * self.LMEXPR + Inv_NFIN * self.NMEXPR + Inv_LNFIN * self.PMEXPR
           PTWGR_i      =  self.PTWGR + Inv_L * self.LPTWGR + Inv_NFIN * self.NPTWGR + Inv_LNFIN * self.PPTWGR
           PDIBL1R_i    =  self.PDIBL1R + Inv_L * self.LPDIBL1R + Inv_NFIN * self.NPDIBL1R + Inv_LNFIN * self.PPDIBL1R
           PDIBL2R_i    =  self.PDIBL2R + Inv_L * self.LPDIBL2R + Inv_NFIN * self.NPDIBL2R + Inv_LNFIN * self.PPDIBL2R
           PCLMR_i      =  self.PCLMR + Inv_L * self.LPCLMR + Inv_NFIN * self.NPCLMR + Inv_LNFIN * self.PPCLMR
           DVTSHIFTR_i  =  self.DVTSHIFTR + Inv_L * self.LDVTSHIFTR + Inv_NFIN * self.NDVTSHIFTR + Inv_LNFIN * self.PDVTSHIFTR
           VSATR_i      =  self.VSATR + Inv_L * self.LVSATR + Inv_NFIN * self.NVSATR + Inv_LNFIN * self.PVSATR
           KSATIVR_i    =  self.KSATIVR + Inv_L * self.LKSATIVR + Inv_NFIN * self.NKSATIVR + Inv_LNFIN * self.PKSATIVR
           U0R_i        =  self.U0R + Inv_L * self.LU0R + Inv_NFIN * self.NU0R + Inv_LNFIN * self.PU0R
           UAR_i        =  self.UAR + Inv_L * self.LUAR + Inv_NFIN * self.NUAR + Inv_LNFIN * self.PUAR
           UPR_i        =  self.UPR + Inv_L * self.LUPR + Inv_NFIN * self.NUPR + Inv_LNFIN * self.PUPR
           if (self.BULKMOD != 0):
               UCR_i    =  self.UCR + Inv_L * self.LUCR + Inv_NFIN * self.NUCR + Inv_LNFIN * self.PUCR

           EUR_i        =  self.EUR + Inv_L * self.LEUR + Inv_NFIN * self.NEUR + Inv_LNFIN * self.PEUR
           UDR_i        =  self.UDR + Inv_L * self.LUDR + Inv_NFIN * self.NUDR + Inv_LNFIN * self.PUDR


        if self.__NQSMOD1__:
           if (self.NQSMOD == 1 and self.XRCRG1 != 0.0):
              XRCRG1_i  =  self.XRCRG1 + Inv_L * self.LXRCRG1 + Inv_NFIN * self.NXRCRG1 + Inv_LNFIN * self.PXRCRG1
              XRCRG2_i  =  self.XRCRG2 + Inv_L * self.LXRCRG2 + Inv_NFIN * self.NXRCRG2 + Inv_LNFIN * self.PXRCRG2
           else:
              XRCRG1_i  =  0.0
              XRCRG2_i  =  0.0

        else:
           if (self.NQSMOD == 1):
              print(" Although the model selector self.NQSMOD is set to 1, the NQS gate resistance model is not activated in the Verilog-A code. \
              Please uncomment \"`define __NQSMOD1__\" in bsimcmg.va to activate it.")

        if self.__NQSMOD2__:
           if (self.NQSMOD == 2 and self.XRCRG1 != 0.0):
              XRCRG1_i  =  self.XRCRG1 + Inv_L * self.LXRCRG1 + Inv_NFIN * self.NXRCRG1 + Inv_LNFIN * self.PXRCRG1
              XRCRG2_i  =  self.XRCRG2 + Inv_L * self.LXRCRG2 + Inv_NFIN * self.NXRCRG2 + Inv_LNFIN * self.PXRCRG2
           else:
              XRCRG1_i  =  0.0
              XRCRG2_i  =  0.0

        else:
           if (self.NQSMOD == 2):
              print(" Although the model selector self.NQSMOD is set to 2, the NQS gate resistance model is not activated in the Verilog-A code. \
              Please uncomment \"`define __NQSMOD2__\" in bsimcmg.va to activate it.")

        UTE_i    =  self.UTE + Inv_L * self.LUTE + Inv_NFIN * self.NUTE + Inv_LNFIN * self.PUTE
        UTL_i    =  self.UTL + Inv_L * self.LUTL + Inv_NFIN * self.NUTL + Inv_LNFIN * self.PUTL
        EMOBT_i  =  self.EMOBT + Inv_L * self.LEMOBT + Inv_NFIN * self.NEMOBT + Inv_LNFIN * self.PEMOBT
        UA1_i    =  self.UA1 + Inv_L * self.LUA1 + Inv_NFIN * self.NUA1 + Inv_LNFIN * self.PUA1

        if (self.BULKMOD != 0):
           UC1_i  =  self.UC1 + Inv_L * self.LUC1 + Inv_NFIN * self.NUC1 + Inv_LNFIN * self.PUC1

        UD1_i    =  self.UD1 + Inv_L * self.LUD1 + Inv_NFIN * self.NUD1 + Inv_LNFIN * self.PUD1
        UCSTE_i  =  self.UCSTE + Inv_L * self.LUCSTE + Inv_NFIN * self.NUCSTE + Inv_LNFIN * self.PUCSTE
        PTWGT_i  =  self.PTWGT + Inv_L * self.LPTWGT + Inv_NFIN * self.NPTWGT + Inv_LNFIN * self.PPTWGT
        AT_i     =  self.AT + Inv_L * self.LAT + Inv_NFIN * self.NAT + Inv_LNFIN * self.PAT
        ATCV_i   =  self.ATCV + Inv_L * self.LATCV  + Inv_NFIN * self.NATCV  + Inv_LNFIN * self.PATCV
        PRT_i    =  self.PRT + Inv_L * self.LPRT + Inv_NFIN * self.NPRT + Inv_LNFIN * self.PPRT
        KT1_i    =  self.KT1 + Inv_L * self.LKT1 + Inv_NFIN * self.NKT1 + Inv_LNFIN * self.PKT1
        TSS_i    =  self.TSS + Inv_L * self.LTSS + Inv_NFIN * self.NTSS + Inv_LNFIN * self.PTSS
        IIT_i    =  self.IIT + Inv_L * self.LIIT + Inv_NFIN * self.NIIT + Inv_LNFIN * self.PIIT
        TGIDL_i  =  self.TGIDL + Inv_L * self.LTGIDL + Inv_NFIN * self.NTGIDL + Inv_LNFIN * self.PTGIDL
        IGT_i    =  self.IGT + Inv_L * self.LIGT + Inv_NFIN * self.NIGT + Inv_LNFIN * self.PIGT

        if (self.ASYMMOD != 0):
            UTER_i  =  self.UTER + Inv_L * self.LUTER + Inv_NFIN * self.NUTER + Inv_LNFIN * self.PUTER
            UTLR_i  =  self.UTLR + Inv_L * self.LUTLR + Inv_NFIN * self.NUTLR + Inv_LNFIN * self.PUTLR
            UA1R_i  =  self.UA1R + Inv_L * self.LUA1R + Inv_NFIN * self.NUA1R + Inv_LNFIN * self.PUA1R
            UD1R_i  =  self.UD1R + Inv_L * self.LUD1R + Inv_NFIN * self.NUD1R + Inv_LNFIN * self.PUD1R
            ATR_i   =  self.ATR + Inv_L * self.LATR + Inv_NFIN * self.NATR + Inv_LNFIN * self.PATR
            if (self.BULKMOD != 0):
                UC1R_i  =  self.UC1R + Inv_L * self.LUC1R + Inv_NFIN * self.NUC1R + Inv_LNFIN * self.PUC1R

        # Geometrical Scaling
        # self.NFIN Scaling
        if (self.PHIGN1 != 0.0):
           PHIG_i  =  PHIG_i * (1.0 + self.PHIGN1 / self.NFIN * self.lln(1.0 + self.NFIN / self.PHIGN2))


        if (self.ETA0N1 != 0.0):
           ETA0_i  =  ETA0_i * (1.0 + self.ETA0N1 / self.NFIN * self.lln(1.0 + self.NFIN / self.ETA0N2))


        if (self.CDSCN1 != 0.0):
           self.CDSC_i  =  self.CDSC_i * (1.0 + self.CDSCN1 / self.NFIN * self.lln(1.0 + self.NFIN / self.CDSCN2))


        if (self.CDSCDN1 != 0.0):
           self.CDSCD_i  =  self.CDSCD_i * (1.0 + self.CDSCDN1 / self.NFIN * self.lln(1.0 + self.NFIN / self.CDSCDN2))


        if (self.CDSCDRN1 != 0.0):
           CDSCDR_i  =  CDSCDR_i * (1.0 + self.CDSCDRN1 / self.NFIN * self.lln(1.0 + self.NFIN / self.CDSCDRN2))


        if (self.VSATN1 != 0.0):
           VSAT_i  =  VSAT_i * (1.0 + self.VSATN1 / self.NFIN * self.lln(1.0 + self.NFIN / self.VSATN2))


        if (self.VSAT1N1 != 0.0):
           VSAT1_i  =  VSAT1_i * (1.0 + self.VSAT1N1 / self.NFIN * self.lln(1.0 + self.NFIN / self.VSAT1N2))


        if (self.VSAT1RN1 != 0.0):
           VSAT1R_i  =  VSAT1R_i * (1.0 + self.VSAT1RN1 / self.NFIN * self.lln(1.0 + self.NFIN / self.VSAT1RN2))


        if (self.U0N1 != 0.0):
           U0_i  =  U0_i * (1.0 + self.U0N1 / self.NFIN * self.lln(1.0 + self.NFIN / self.U0N2))


        if (self.param_given("NFINNOM")):
           PHIG_i  =  PHIG_i * (1.0 + (self.NFIN - self.NFINNOM) * self.PHIGLT * self.Leff)
           ETA0_i  =  ETA0_i * (1.0 + (self.NFIN - self.NFINNOM) * self.ETA0LT * self.Leff)
           U0_i    =  U0_i * (1.0 + (self.NFIN - self.NFINNOM) * self.U0LT * self.Leff)


        if (self.U0N1R != 0.0):
           U0R_i  =  U0R_i * (1.0 + self.U0N1R / self.NFIN * self.lln(1.0 + self.NFIN / self.U0N2R))


        # Length Scaling
        PHIG_i  =  PHIG_i + self.PHIGL * self.Leff
        if (self.LPA > 0.0):
           U0_i  =  U0_i * (1.0 - UP_i * pow(self.Leff, -self.LPA))
        else:
           U0_i  =  U0_i * (1.0 - UP_i)

        UA_i  =  UA_i + self.AUA * self.lexp(-self.Leff / self.BUA)
        UD_i  =  UD_i + self.AUD * self.lexp(-self.Leff / self.BUD)
        self.EU_i  =  self.EU_i + self.AEU * self.lexp(-self.Leff / self.BEU)

        if (self.ASYMMOD != 0):
           if (self.LPAR > 0.0):
              U0R_i  =  U0R_i * (1.0 - UPR_i * pow(self.Leff, -LPAR))
           else:
              U0R_i  =  U0R_i * (1.0 - UPR_i)

           UAR_i  =  UAR_i + self.AUAR * self.lexp(-self.Leff / self.BUAR)
           UDR_i  =  UDR_i + self.AUDR * self.lexp(-self.Leff / self.BUDR)
           EUR_i  =  EUR_i + self.AEUR * self.lexp(-self.Leff / self.BEUR)


        if (self.RDSMOD == 1):
           self.RSW_i  =  self.RSW_i + self.ARSW * self.lexp(-self.Leff / self.BRSW)
           self.RDW_i  =  self.RDW_i + self.ARDW * self.lexp(-self.Leff / self.BRDW)
        else:
           self.RDSW_i  =  self.RDSW_i + self.ARDSW * self.lexp(-self.Leff / self.BRDSW)


        self.PCLM_i  =  self.PCLM_i + self.APCLM * self.lexp(-self.Leff / self.BPCLM)
        if (self.ASYMMOD != 0):
           PCLMR_i  =  PCLMR_i + self.APCLMR * pow(self.Leff, -BPCLMR)


        MEXP_i  =  MEXP_i  + self.AMEXP  * pow(self.Leff, -self.BMEXP)
        if (self.ASYMMOD != 0):
           MEXPR_i  =  MEXPR_i + self.AMEXPR * pow(self.Leff, -BMEXPR)


        PTWG_i  =  PTWG_i + self.APTWG * self.lexp(-self.Leff / self.BPTWG)
        if (self.ASYMMOD != 0):
           PTWGR_i  =  PTWGR_i + self.APTWG * self.lexp(-self.Leff / self.BPTWG)


        VSAT_i    =  VSAT_i + self.AVSAT * self.lexp(-self.Leff / self.BVSAT)
        VSAT1_i   =  VSAT1_i + self.AVSAT1 * self.lexp(-self.Leff / self.BVSAT1)
        if (self.ASYMMOD != 0):
           VSAT1R_i =  VSAT1R_i + self.AVSAT1 * self.lexp(-self.Leff / self.BVSAT1)


        self.PSAT_i    =  self.PSAT_i + self.APSAT * self.lexp(-self.Leff / self.BPSAT)
        self.PSATCV_i  =  self.PSATCV_i + self.APSATCV * self.lexp(-self.Leff / self.BPSATCV)
        VSATCV_i  =  VSATCV_i + self.AVSATCV * self.lexp(-self.LeffCV / self.BVSATCV)

        # Scaling for DITS Parameters
        self.DVTP0_i   =  self.DVTP0 + self.ADVTP0 * self.lexp(-self.Leff / self.BDVTP0)
        self.DVTP1_i   =  self.DVTP1 + self.ADVTP1 * self.lexp(-self.Leff / self.BDVTP1)

        # Geometrical Scaling for Toxeff / Charge Centroid Tcen
        if (self.QMTCENCV_i > 0.0 or self.QMTCENCVA_i > 0.0):
           MTcen  =  1.0 + self.AQMTCEN * self.lexp(- (2.0 * self.Ach / self.Weff_UFCM) / self.BQMTCEN)
           Tcen0  =  (2.0 * self.Ach / self.Weff_UFCM ) * MTcen


        # **************************************
        # *        Parameter Checking          *
        # **************************************

        if (self.LeffCV <= 1.0e-9):
           print("Warning: self.LeffCV = %e <= 1.0e-9.", self.LeffCV)

        if (self.BULKMOD != 0):
           if (self.LeffCV_acc <= 1.0e-9):
              print("Warning: self.LeffCV_acc = %e <= 1.0e-9.", self.LeffCV_acc)

        if (self.Weff0 <= 1.0e-9):
           print("Warning: self.Weff0 = %e <= 1.0e-9.", self.Weff0)

        if (self.WeffCV0 <= 1.0e-9):
           print("Warning: self.WeffCV0 = %e <= 1.0e-9.", self.WeffCV0)

        if (NBODY_i <= 0.0):
           print("Fatal: NBODY_i = %e is not positive.", NBODY_i)
           sys.exit(0)
        elif (NBODY_i <= 1.0e18):
           print("Warning: NBODY_i = %e m^-3 may be too small.", NBODY_i)

        if (NGATE_i < 0.0):
           print("Fatal: NGATE_i = %e is negative.", NGATE_i)
           sys.exit(0)
        elif (NGATE_i != 0.0 and NGATE_i <= 1.0e24):
           print("Warning: NGATE_i = %e may be too small.", NGATE_i)
        elif (NGATE_i > 1.0e31):
           print("Fatal: NGATE_i = %e is too high.", NGATE_i)
           sys.exit(0)

        if (self.DVT0_i < 0.0):
           print("Warning: self.DVT0_i = %e is negative.", self.DVT0_i)

        if (PHIG_i <= 0.0):
           print("Fatal: PHIG_i = %e is not positive.", PHIG_i)
           sys.exit(0)

        if (VSAT_i <= 0.0):
           print("Fatal: VSAT_i = %e is not positive.", VSAT_i)
           sys.exit(0)

        if (VSAT1_i <= 0.0):
           print("Fatal: VSAT1_i = %e is not positive.", VSAT1_i)
           sys.exit(0)

        if (self.ASYMMOD != 0 and VSAT1R_i <= 0.0):
           print("Fatal: VSAT1R_i = %e is not positive.", VSAT1R_i)
           sys.exit(0)

        if (DVT1_i <= 0.0):
           print("Fatal: DVT1_i = %e is not positive.", DVT1_i)
           sys.exit(0)

        if (DVT1SS_i <= 0.0):
           print("Fatal: DVT1SS_i = %e is not positive.", DVT1SS_i)
           sys.exit(0)

        if (self.CDSC_i < 0.0):
           print("Warning: self.CDSC_i = %e is negative.", self.CDSC_i)

        if (self.CDSCD_i < 0.0):
           print("Warning: self.CDSCD_i = %e is negative.", self.CDSCD_i)

        if (self.ASYMMOD != 0 and CDSCDR_i < 0.0):
           print("Warning: CDSCDR_i = %e is negative.", CDSCDR_i)

        if (DSUB_i <= 0.0):
           print("Fatal: DSUB_i = %e is not positive.", DSUB_i)
           sys.exit(0)

        if (ETA0_i < 0.0):
           print("Warning: ETA0_i = %e is negative, setting it to 0", ETA0_i)
           ETA0_i  =  0.0


        if (ETA0R_i < 0.0):
           print("Warning: ETA0R_i = %e is negative, setting it to 0", ETA0R_i)
           ETA0R_i  =  0.0


        if (LPE0_i < -self.Leff):
           print("Warning: LPE0_i = %e is less than -self.Leff. Clipping LPE0_i to 0", LPE0_i)
           LPE0_i  =  0.0


        if (K0SI_i <= 0.0):
           print("Warning: K0SI_i = %e is not positive, setting it to 0.", K0SI_i)
           K0SI_i  =  0.0


        if (K2SI_i <= 0.0):
           print("Warning: K2SI_i = %e is not positive, setting it to 0.", K2SI_i)
           K2SI_i  =  0.0


        if (self.PHIBE_i < 0.2 and self.BULKMOD != 0):
           print("Warning: self.PHIBE_i = %e is less than 0.2, setting it to 0.2.", self.PHIBE_i)
           self.PHIBE_i  =  0.2


        if (self.PHIBE_i > 1.2 and self.BULKMOD != 0):
           print("Warning: self.PHIBE_i = %e is larger than 1.2, setting it to 1.2.", self.PHIBE_i)
           self.PHIBE_i  =  1.2


        if (self.PSAT_i < 2.0):
           print("Warning: self.PSAT_i = %e is less than 2.0, setting it to 2.0.", self.PSAT_i)
           self.PSAT_i  =  2.0


        if (self.PSATCV_i < 2.0):
           print("Warning: self.PSATCV_i = %e is less than 2.0, setting it to 2.0.", self.PSATCV_i)
           self.PSATCV_i  =  2.0


        if (U0_i < 0.0):
           print("Warning: U0_i = %e is negative, setting it to the default value.", U0_i)
           U0_i  =  0.03


        if (UA_i < 0.0):
           print("Warning: UA_i = %e is negative, setting it to 0.", UA_i)
           UA_i  =  0.0


        if (self.EU_i < 0.0):
           print("Warning: self.EU_i = %e is negative, setting it to 0.", self.EU_i)
           self.EU_i  =  0.0


        if (UD_i < 0.0):
           print("Warning: UD_i = %e is negative, setting it to 0.", UD_i)
           UD_i  =  0.0


        if (UCS_i < 0.0):
           print("Warning: UCS_i = %e is negative, setting it to 0.", UCS_i)
           UCS_i  =  0.0


        if (ETAMOB_i < 0.0):
           print("Warning: ETAMOB_i = %e is negative, setting it to 0", ETAMOB_i)
           ETAMOB_i  = 0.0


        self.RDSWMIN_i  =  self.RDSWMIN
        if (self.RDSWMIN_i < 0.0):
           print("Warning: self.RDSWMIN = %e is negative.  Set to zero", RDSWMIN_i)
           self.RDSWMIN_i  =  0.0


        if (self.RDSW_i < 0.0):
           print("Warning: RDSW_i = %e is negative.  Set to zero", self.RDSW_i)
           self.RDSW_i  =  0.0


        self.RSWMIN_i  =  self.RSWMIN
        if (self.RSWMIN_i < 0.0):
           print("Warning: self.RSWMIN = %e is negative.  Set to zero", self.RSWMIN_i)
           self.RSWMIN_i  =  0.0


        if (self.RSW_i < 0.0):
           print("Warning: self.RSW_i = %e is negative.  Set to zero", self.RSW_i)
           self.RSW_i  =  0.0


        self.RDWMIN_i  =  self.RDWMIN
        if (self.RDWMIN_i < 0.0):
           print("Warning: self.RDWMIN = %e is negative.  Set to zero", self.RDWMIN_i)
           self.RDWMIN_i  =  0.0


        if (self.RDW_i < 0):
           print("Warning: self.RDW_i = %e is negative.  Set to zero", self.RDW_i)
           self.RDW_i  =  0.0


        if (self.PRWGD_i < 0.0):
           print("Warning: self.PRWGD_i = %e is negative.  Set to zero", self.PRWGD_i)
           self.PRWGD_i  =  0.0


        if (self.PRWGS_i < 0.0):
           print("Warning: self.PRWGS_i = %e is negative.  Set to zero", self.PRWGS_i)
           self.PRWGS_i  =  0.0


        if (self.PCLM_i < 0):
           print("Warning: self.PCLM_i = %e is negative.", self.PCLM_i)


        if (self.PDIBL1 < 0.0):
           print("Warning: self.PDIBL1 = %e is negative.", self.PDIBL1)


        if (self.ASYMMOD != 0):
           if (PDIBL1R_i < 0.0):
              print("Warning: PDIBL1R_i = %e is negative.", PDIBL1R_i)

           if (PDIBL2R_i < 0.0):
              print("Warning: PDIBL2R_i = %e is negative.", PDIBL2R_i)

           if (U0R_i < 0):
               print("Warning: U0R_i = %e is negative, setting it to 0.", U0R_i)
               U0R_i  =  0.0

           if (UAR_i < 0.0):
               print("Warning: UAR_i = %e is negative, setting it to 0.", UAR_i)
               UAR_i  =  0.0

           if (EUR_i < 0.0):
               print("Warning: EUR_i = %e is negative, setting it to 0.", EUR_i)
               EUR_i  =  0.0

           if (UDR_i < 0.0):
               print("Warning: UDR_i = %e is negative, setting it to 0.", UDR_i)
               UDR_i  =  0.0



        if (self.PDIBL2_i < 0.0):
           print("Warning: self.PDIBL2_i = %e is negative.", self.PDIBL2_i)


        if (self.DROUT_i <= 0.0):
           print("Fatal: self.DROUT_i = %e is non-positive.", self.DROUT_i)
           sys.exit(0)


        if (MEXP_i < 2.0):
           print("Warning: MEXP_i = %e < 2. Setting MEXP_i = 2.", MEXP_i)
           MEXP_i  =  2.0


        if (self.ASYMMOD != 0):
           if (MEXPR_i < 2.0):
              print("Warning: MEXPR_i = %e < 2.  Setting MEXPR_i = 2.", MEXPR_i)
              MEXPR_i  =  2.0



        if (PTWG_i < 0):
           print("Warning: PTWG_i = %e is negative, setting it to 0.", PTWG_i )
           PTWG_i  =  0.0


        if (self.QMTCENCV_i > 0.0):
           if (self.QM0 <= 0.0):
              print("Fatal: self.QM0 = %e is non-positive.", self.QM0)
              sys.exit(0)



        if (self.BULKMOD != 0 and self.QMTCENCVA_i > 0.0):
           if (self.QM0ACC <= 0.0):
              print("Fatal: self.QM0ACC = %e is non-positive.", self.QM0ACC)
              sys.exit(0)



        if (CGIDL_i < 0.0):
           print("Warning: CGIDL_i = %e < 0.  Setting CGIDL_i = 0.", CGIDL_i)
           CGIDL_i  =  0.0


        if (CGISL_i < 0.0):
           print("Warning: CGISL_i = %e < 0.  Setting CGISL_i = 0.", CGISL_i)
           CGISL_i  =  0.0


        if (self.IGBMOD != 0):
           if (self.NIGBINV_i <= 0.0):
              print("Fatal: self.NIGBINV_i = %e is non-positive.", self.NIGBINV_i)
              sys.exit(0)

           if (self.NIGBACC_i <= 0.0):
              print("Fatal: self.NIGBACC_i = %e is non-positive.", self.NIGBACC_i)
              sys.exit(0)



        if (self.IGCMOD != 0):
           if (self.POXEDGE_i <= 0.0):
              print("Fatal: self.POXEDGE_i = %e is non-positive.", self.POXEDGE_i)
              sys.exit(0)

           if (self.PIGCD_i <= 0.0):
              print("Fatal: self.PIGCD_i = %e is non-positive.", self.PIGCD_i)
              sys.exit(0)



        if (self.IGCMOD != 0 or self.IGBMOD != 0):
           if (self.TOXREF <= 0):
              print("Fatal: self.TOXREF = %e is non-positive.", self.TOXREF)
              sys.exit(0)



        if (self.LINTIGEN >= (self.Leff / 2.0)):
           print("Warning: self.LINTIGEN = %e is too large - self.Leff for r/g current is negative.  Re-setting self.LINTIGEN = 0.", self.LINTIGEN)
           self.LINTIGEN_i  =  0.0
        else:
           self.LINTIGEN_i  =  self.LINTIGEN


        if (NTGEN_i <= 0.0):
           print("Fatal: NTGEN_i = %e is non-positive.", NTGEN_i)
           sys.exit(0)


        if self.__NQSMOD1__:
            if (self.NQSMOD == 1 and XRCRG1_i != 0.0 and XRCRG1_i < 1.0e-3):
                print("Warning: XRCRG1_i = %e.  Gate resistance may be too large. Disabling NQS Gate Resistance.", XRCRG1_i)
                XRCRG1_i  =  0.0

        if (self.IIMOD == 2):
           if (BETAII0_i < 0.0):
              print("Warning:  BETAII0_i = %e is negative.",BETAII0_i)

           if (BETAII1_i < 0.0):
              print("Warning:  BETAII1_i = %e is negative.", BETAII1_i)

           if (BETAII2_i < 0.0):
              print("Warning:  BETAII2_i = %e is negative.", BETAII2_i)

           if (ESATII_i < 0.0):
              print("Warning:  ESATII_i = %e is negative.", ESATII_i)

           if (LII_i < 0.0):
              print("Warning:  LII_i = %e is negative.", LII_i)

           if (SII1_i < 0.0):
              print("Warning:  SII1_i = %e is negative.", self.SII1)

           if (SII2_i < 0.0):
              print("Warning:  SII2_i = %e is negative.", SII2_i)

           if (SIID_i < 0.0):
              print("Warning:  SIID_i = %e is negative.", SIID_i)

        if (self.EF <= 0.0):
           print("Fatal: self.EF = %e is non-positive.", self.EF)
           sys.exit(0)
        elif (self.EF > 2.0):
           print("Fatal: self.EF = %e > 2.0.", self.EF)
           sys.exit(0)

        if (self.LINTNOI >= (self.Leff / 2.0)):
           print("Warning: self.LINTNOI = %e is too large - self.Leff for noise is negative.  Re-setting self.LINTNOI = 0.", self.LINTNOI)
           self.LINTNOI_i  =  0.0
        else:
           self.LINTNOI_i  =  self.LINTNOI

        if (self.NTNOI < 0):
           print("Warning: self.NTNOI = %e is negative. Set to zero.", self.NTNOI)
           self.NTNOI_i  =  0.0
        else:
           self.NTNOI_i  =  self.NTNOI


        # Self-Heating
        if self.__SHMOD__:
           if (self.SHMOD != 0 and self.RTH0 > 0.0):
              T1  =  self.WTH0
              if (self.BSHEXP != 0.0):
                  T1  =  self.WTH0 * pow(NF, self.BSHEXP)

              T2  =  self.FPITCH
              if (self.ASHEXP != 0.0):
                  T2 = self.FPITCH * pow(self.NFINtotal, self.ASHEXP)

              gth  =  (T1 + T2)/ self.RTH0
              cth  =  self.CTH0 * (T1 + T2)
           else:
              gth  =  1.0
              cth  =  0.0

        else:
           if (self.SHMOD != 0):
               print("Although the model selector self.SHMOD is set to 1, the self heating model is not activated in the Verilog-A code.  \
               Please uncomment \"`define __SHMOD__\" in bsimcmg.va to activate it.")

        # Gate Electrode Resistance
        if self.__RGATEMOD__:
            if self.RGATEMOD != 0:
                if self.NGCON == 2:
                    Rgeltd_p = 12.0
                else:
                    Rgeltd_p = 3.0

                Rgeltd  =  (self.RGEXT / self.NGCON + (self.RGFIN * self.NFIN) / Rgeltd_p) / self.NF
                self.ggeltd  =  1.0 / max(1.0e-3, Rgeltd)

        else:
           if (self.RGATEMOD != 0):
              print("Although the model selector self.RGATEMOD is set to 1, the gate electrode resistance model is not activated in the Verilog-A code.  \
              Please uncomment \"`define __RGATEMOD__\" in bsimcmg.va to activate it.")

        # Geometry-Dependent Source/Drain Resistance
        if (self.RGEOMOD == 0):
           self.RSourceGeo  =  self.RSHS * self.NRS
           self.RDrainGeo   =  self.RSHD * self.NRD
        else:
           # Area and perimeter calculation
           if (self.HEPI > 0.0):
              Arsd  =  self.FPITCH * self.HFIN + (self.TFIN + (self.FPITCH - self.TFIN) * self.CRATIO) * self.HEPI
           else:
              Arsd  =  self.FPITCH * max(1.0e-9, self.HFIN + self.HEPI)

           Prsd  =  self.FPITCH + self.DELTAPRSD

           # Resistivity Calculation
           if (self.param_given("RHORSD")):
              rhorsd  =  self.RHORSD
           else:
              if self.TYPE == self.ntype:
                  mu_max = 1417.0
              else:
                  mu_max = 470.5

              if (self.TYPE == self.ntype):
                  mu_rsd  =  (52.2 + (mu_max - 52.2) / (1.0 + pow(self.NSD / 9.68e22, 0.680)) - 43.4 / (1.0 + pow(3.43e26 / self.NSD, 2.0))) * 1.0e-4
              else:
                  mu_rsd  =  (44.9 + (mu_max - 44.9) / (1.0 + pow(self.NSD / 2.23e22, 0.719)) - 29.0 / (1.0 + pow(6.10e26 / self.NSD, 2.0))) * 1.0e-4

              rhorsd  =  1.0 / (self.q * self.NSD * mu_rsd)


           # Component: Spreading Resistance (extension -> hdd)
           thetarsp  =  55.0 * math.pi / 180.0
           afin      =  min(Arsd, max(1.0e-18, self.TFIN * (self.HFIN + min(0.0, self.HEPI))))
           T1        =  self.cot(thetarsp)
           Rsp       =  rhorsd * T1 / (math.sqrt(math.pi) * self.NFIN) * (1.0 / math.sqrt(afin) - 2.0 / math.sqrt(Arsd) + math.sqrt(afin / (Arsd*Arsd)))

           # Component: Contact Resistance
           arsd_total  =  Arsd * self.NFIN + self.ARSDEND
           prsd_total  =  Prsd * self.NFIN + self.PRSDEND
           lt          =  math.sqrt(self.RHOC * arsd_total / (rhorsd * prsd_total))
           alpha       =  self.LRSD / lt
           T0          =  self.lexp(alpha + alpha)

           if (self.SDTERM == 1.0):
              eta  =  rhorsd * lt / self.RHOC
              T1   =  T0 * (1.0 + eta)
              T2   =  T1 + 1.0 - eta
              T3   =  T1 - 1.0 + eta
           else:
              T2   =  T0 + 1.0
              T3   =  T0 - 1.0

           RrsdTML  =  rhorsd * lt * T2 / (arsd_total * T3)

           if (self.HEPI < -1.0e-10):
              Rrsdside  =  self.RHOC / (-self.HEPI * self.TFIN * self.NFIN)
              Rrsd      =  (RrsdTML + Rsp) * Rrsdside / ((RrsdTML + Rsp) + Rrsdside)
           else:
              Rrsd      =  RrsdTML + Rsp


           Rdsgeo      =  Rrsd / self.NF * max(0.0, self.RGEOA + self.RGEOB * self.TFIN + self.RGEOC * self.FPITCH + self.RGEOD * self.LRSD + self.RGEOE * self.HEPI)
           self.RSourceGeo  =  Rdsgeo
           self.RDrainGeo   =  Rdsgeo


        # Clamping of Source/Drain Resistances
        if (self.RSourceGeo <= 1.0e-3):
           self.RSourceGeo  =  1.0e-3


        if (self.RDrainGeo <= 1.0e-3):
           self.RDrainGeo  =  1.0e-3


        if (self.RDSMOD == 1):
           if (self.RSWMIN_i <= 0.0):
              self.RSWMIN_i  =  0.0

           if (self.RDWMIN_i <= 0.0):
              self.RDWMIN_i  =  0.0

           if (self.RSW_i <= 0.0):
              self.RSW_i  =  0.0

           if (self.RDW_i <= 0.0):
              self.RDW_i  =  0.0

        else:
           if (self.RDSWMIN_i <= 0.0):
              self.RDSWMIN_i  =  0.0

           if (self.RDSW_i <= 0.0):
              self.RDSW_i  =  0.0

          # End of Clamping of Source/Drain Resistances

        if (self.CGEOMOD != 1):
           if (self.param_given("CGSO")):
              self.CGSO_i  =  self.CGSO
           else:
              if (self.param_given("DLC") and self.DLC > 0.0):
                 self.CGSO_i  =  max(0.0, self.DLC * self.cox - self.CGSL_i)
              else:
                 self.CGSO_i  =  0.3 * self.TFIN * self.cox


           if (self.param_given("CGDO")):
              self.CGDO_i  =  self.CGDO
           else:
              if (self.param_given("DLC") and self.DLC > 0.0):
                 self.CGDO_i  =  max(0.0, self.DLC * self.cox - self.CGDL_i)
              else:
                 self.CGDO_i  =  0.3 * self.TFIN * self.cox

        # Parasitic Source/Drain to Gate Fringe Capacitance Model
        if (self.CGEOMOD == 2):
           Hg    =  self.TGATE + self.TMASK
           Trsd  =  0.5 * (self.FPITCH - self.TFIN)
           Wg    =  max(0.0, Trsd - self.TOXP)
           Hrsd  =  max(0.0, self.HEPI + self.TSILI)

           # Top Component
           if (self.TMASK > 0.0):
              # Capacitance Model by Chung-Hsun Lin (IBM)
              T0       =  3.467e-11 * self.lln(1.0e-7 * self.EPSRSP / (3.9 * self.LSP))
              T1       =  0.942 * Hrsd * epssp / self.LSP
              Cgg_top  =  (T0 + T1) * (self.TFIN + (self.FPITCH - self.TFIN) * self.CRATIO)
           else:
              Cgg_top = self.Cfringe_2d('cfr_top_trigate', Hg, Hrsd, self.LSP, self.TFIN, self.LRSD, Lg, self.TOXP, 0.85, epssp)


           # Side Component
           if (self.TMASK > 0):
              Cgg_side = self.Cfringe_2d('cfr_side_dblgate', Wg, Trsd, self.LSP, self.HFIN, self.LRSD, Lg, self.TOXP, 0.70, epssp)
           else:
              Cgg_side = self.Cfringe_2d('cfr_side_trigate', Wg, Trsd, self.LSP, self.HFIN, self.LRSD, Lg, self.TOXP, 0.85, epssp)


           # Corner Component
           if (self.TMASK > 0.0):
              Acorner  =  0.0
           else:
              if (self.HEPI > 0.0):
                  Acorner  =  (self.FPITCH - self.TFIN) * (self.HEPI * self.CRATIO + self.TSILI)
              else:
                  Acorner  =  (self.FPITCH - self.TFIN) * Hrsd


           Ccorner  =  (self.NFIN * Acorner + self.ARSDEND + self.ASILIEND) * epssp / self.LSP
           self.Cfr_geo  =  (Ccorner + Cgg_top * self.NFIN + self.CGEOE * Cgg_side * self.NFIN * 2.0) * self.NF
           self.Cfr_geo  =  self.Cfr_geo * max(0.0, self.CGEOA + self.CGEOB * self.TFIN + self.CGEOC * self.FPITCH + self.CGEOD * self.LRSD)


        # Source/Gate/Drain-to-Substrate Parasitic Capacitances
        T0     =  self.CSDESW * self.lln(1.0 + self.HFIN / self.EOTBOX)
        self.csbox  =  cbox * self.ASEO + T0 * max(0.0, self.PSEO - self.FPITCH * self.NFINtotal)
        self.cdbox  =  cbox * self.ADEO + T0 * max(0.0, self.PDEO - self.FPITCH * self.NFINtotal)
        self.cgbox  =  (self.CGBO * self.NF * self.NGCON + self.CGBN * self.NFINtotal) * Lg

        # Mobility Degradation
        self.EeffFactor    =  1.0e-8 / (epsratio * (self.EOT))
        self.WeffWRFactor  =  1.0 / (pow((self.Weff0) * 1.0e6, WR_i) * self.NFINtotal)
        self.litl          =  math.sqrt(epsratio * self.EOT * 0.5 * self.TFIN)

        if (not self.param_given("THETASCE")):
           tmp  =  DVT1_i * self.Leff / self.scl + 1.0e-6
           if (tmp < 40.0):
              self.Theta_SCE  =  0.5 / (self.cosh(tmp) - 1.0)
           else:
              self.Theta_SCE  =  math.exp(-tmp)

        else:
           self.Theta_SCE  =  self.THETASCE


        if (not self.param_given("THETASW")):
           tmp  =  DVT1SS_i * self.Leff / self.scl + 1.0e-6
           if (tmp < 40.0):
              self.Theta_SW  =  0.5 / (self.cosh(tmp) - 1.0)
           else:
              self.Theta_SW  =  math.exp(-tmp)

        else:
           self.Theta_SW  =  self.THETASW


        if (not self.param_given("THETADIBL")):
           tmp  =  DSUB_i * self.Leff / self.scl + 1.0e-6
           if (tmp < 40.0):
              self.Theta_DIBL  =  0.5 / (self.cosh(tmp) - 1.0)
           else:
              self.Theta_DIBL  =  math.exp(-tmp)

        else:
           self.Theta_DIBL  =  self.THETADIBL


        self.Theta_RSCE  =  math.sqrt(1.0 + LPE0_i / self.Leff) - 1.0

        tmp  =  DSUB_i * self.Leff / self.scl + 1.0e-6
        if (tmp < 40.0):
           T0  =  1.0 / max((1.0 + self.DVTP2 * (self.cosh(tmp) - 2.0)), 1.0e-6)
        else:
           T0  =  math.exp(-tmp) / max((math.exp(-tmp) + self.DVTP2), 1.0e-6)

        self.Theta_DITS  =  T0
        self.nbody       =  NBODY_i
        self.qbs         =  self.q * self.nbody * self.Ach / self.Cins

        # Gate Current
        if (self.TYPE == self.ntype):
            self.Aechvb  =  4.97232e-7  # NMOS
            self.Bechvb  =  7.45669e11  # NMOS
        else:
            self.Aechvb  =  3.42537e-7  # PMOS
            self.Bechvb  =  1.16645e12  # PMOS

        T0  =  self.TOXG * self.TOXG
        T1  =  self.TOXG * self.POXEDGE_i
        T2  =  T1 * T1
        self.Toxratio      =  self.lexp(NTOX_i * self.lln(self.TOXREF / self.TOXG)) / T0
        self.Toxratioedge  =  self.lexp(NTOX_i * self.lln(self.TOXREF / T1)) / T2
        self.igsd_mult0    =  self.Weff0 * self.Aechvb * self.Toxratioedge

        if (self.TNOM < -self.P_CELSIUS0):
            print("Warning: (TNOM=%e) < -P_CELSIUS0. Set to 27 C.", self.TNOM)
            Tnom  =  self.REFTEMP
        else:
            Tnom  =  self.TNOM + self.CONSTCtoK

        # initial_step

        # ************************************************
        # *     Temperature Dependence Calculations      *
        # ************************************************
        if self.__SHMOD__:
            if (self.SHMOD != 0 and self.RTH0 > 0.0):
                DevTemp  =  self.temperature + Temp(rth_branch) + self.DTEMP
        else:
            DevTemp  =  self.temperature + self.DTEMP

        # CMGTempDepCalc
        TRatio   =  DevTemp / Tnom
        delTemp  =  DevTemp - Tnom
        self.Vtm      =  self.KboQ * DevTemp
        Vtm0     =  self.KboQ * Tnom
        self.Eg       =  self.BG0SUB - self.TBGASUB * DevTemp * DevTemp / (DevTemp + self.TBGBSUB)
        self.Eg0      =  self.BG0SUB - self.TBGASUB * Tnom * Tnom / (Tnom + self.TBGBSUB)
        T1       =  (DevTemp / 300.15) * math.sqrt(DevTemp / 300.15)
        ni       =  self.NI0SUB * T1 * self.lexp(self.BG0SUB / (2.0 * self.KboQ * 300.15) - self.Eg / (2.0 * self.Vtm))
        self.Nc       =  self.NC0SUB * T1
        self.ThetaSS  =  self.hypsmooth(1.0 + TSS_i * delTemp - 1.0e-6, 1.0e-3)

        # Quantum Mechanical Vth Correction (Ref: Trivedi et al., EDL 2005)
        kT       =  self.Vtm * self.q
        T0       =  self.HBAR * math.pi / (2*self.Ach/self.Weff_UFCM )
        E0       =  T0 * T0 / (2.0 * self.mx)
        E0prime  =  T0 * T0 / (2.0 * self.mxprime)
        E1       =  4.0 * E0
        E1prime  =  4.0 * E0prime
        T1       =  gprime * mdprime / (gfactor * md)
        gam0     =  1.0 + T1 * self.lexp((E0 - E0prime) / kT)
        gam1     =  gam0 + self.lexp((E0 - E1) / kT) + T1 * self.lexp((E0 - E1prime) / kT)
        T2       =  -self.Vtm * self.lln(gfactor * md / (math.pi * self.HBAR * self.HBAR * self.Nc) * kT / (2.0 * self.Ach / self.Weff_UFCM) * gam1)
        self.dvch_qm  =  QMFACTOR_i * (E0 / self.q + T2)

        # Temperature Dependence
        self.ETA0_t   =  self.Tempdep(ETA0_i, self.TETA0, delTemp, self.TEMPMOD)
        ETA0R_t  =  self.Tempdep(ETA0R_i, self.TETA0R, delTemp, self.TEMPMOD)
        T1       =  U0_i * pow(TRatio, UTE_i)
        U0_t     =  T1 + self.hypmax(UTL_i * delTemp, -0.9 * T1, 1.0e-4)
        self.u0       =  U0_t
        if (self.ASYMMOD == 1):
            T1     =  U0R_i * pow(TRatio, UTER_i)
            U0R_t  =  T1 + self.hypmax(UTLR_i * delTemp, -0.9 * T1, 1.0e-4)
            self.u0r    =  U0R_t

        ETAMOB_t  =  self.Tempdep(ETAMOB_i, EMOBT_i, delTemp, self.TEMPMOD)
        self.UA_t      =  UA_i + self.hypmax(UA1_i*delTemp, -UA_i, 1.0e-6)
        if (self.ASYMMOD != 0):
            UAR_t  =  UAR_i + self.hypmax(UA1R_i * delTemp, -UAR_i, 1.0e-6)


        if (self.BULKMOD != 0):
            if (self.TEMPMOD == 0):
                self.UC_t  =  self.Tempdep(UC_i, UC1_i, delTemp, 0)
                if (self.ASYMMOD != 0):
                    UCR_t  =  self.Tempdep(UCR_i, UC1R_i, delTemp, 0)

            else:
                self.UC_t  =  UC_i + UC1_i * delTemp
                if (self.ASYMMOD != 0):
                    UCR_t  =  UCR_i + UC1R_i * delTemp

        self.UD_t  =  UD_i * pow(TRatio, UD1_i)
        if (self.ASYMMOD != 0):
           UDR_t  =  UDR_i * pow(TRatio, UD1R_i)


        self.UCS_t  =  UCS_i * pow(TRatio, UCSTE_i)

        self.rdstemp  =  self.hypsmooth(1.0 + PRT_i * delTemp - 1.0e-6, 1.0e-3)
        self.RSDR_t   =  self.Tempdep(self.RSDR, self.TRSDR, delTemp, self.TEMPMOD)
        if (self.ASYMMOD != 0):
           RSDRR_t  =  self.Tempdep(self.RSDRR, self.TRSDR, delTemp, self.TEMPMOD)


        self.RDDR_t  = self.Tempdep(self.RDDR, self.TRDDR, delTemp, self.TEMPMOD)
        if (self.ASYMMOD != 0):
           RDDRR_t  =  self.Tempdep(self.RDDRR, self.TRDDR, delTemp, self.TEMPMOD)


        self.VSAT_t  =  self.Tempdep(VSAT_i, -AT_i, delTemp, self.TEMPMOD)
        if (self.VSAT_t < 1000):
           print("Warning: VSAT(%f) = %e is less than 1K, setting it to 1K.", DevTemp, self.VSAT_t)
           self.VSAT_t =  1000


        if (self.ASYMMOD != 0):
            VSATR_t   = self.Tempdep(VSATR_i, -ATR_i, delTemp, self.TEMPMOD)
            if (VSATR_t < 1000):
               print("Warning: VSATR(%f) = %e is less than 1K, setting it to 1K.", DevTemp, VSATR_t)
               VSATR_t =  1000



        self.VSAT1_t  =  self.Tempdep(VSAT1_i, -AT_i, delTemp, self.TEMPMOD)
        if (self.VSAT1_t < 1000):
           print("Warning: VSAT1(%f) = %e is less than 1K, setting it to 1K.", DevTemp, self.VSAT1_t)
           self.VSAT1_t  =  1000


        if (self.ASYMMOD != 0):
           VSAT1R_t   = self.Tempdep(VSAT1R_i, -AT_i, delTemp, self.TEMPMOD)
           if (VSAT1R_t < 1000):
              print("Warning: VSAT1R(%f) = %e is less than 1K, setting it to 1K.", DevTemp, VSAT1R_t)
              VSAT1R_t  =  1000



        self.VSATCV_t  =  self.Tempdep(VSATCV_i, -ATCV_i, delTemp, self.TEMPMOD)
        if (self.VSATCV_t < 1000):
           print("Warning: VSATCV(%f) = %e is less than 1K, setting it to 1K.", DevTemp, self.VSATCV_t)
           self.VSATCV_t  =  1000


        self.MEXP_t  =  self.hypsmooth(MEXP_i * (1.0 + self.TMEXP * delTemp) - 2.0, 1.0e-3) + 2.0
        if (self.ASYMMOD != 0):
           MEXPR_t  =  self.hypsmooth(MEXPR_i * (1.0 + self.TMEXPR * delTemp) - 2.0, 1.0e-3) + 2.0


        self.PTWG_t  =  self.Tempdep(PTWG_i, -PTWGT_i, delTemp, self.TEMPMOD)
        if (self.ASYMMOD != 0):
           PTWGR_t  =  self.Tempdep(PTWGR_i, -PTWGT_i, delTemp, self.TEMPMOD)


        self.dvth_temp   =  (KT1_i + self.KT1L / self.Leff) * (TRatio - 1.0)
        BETA0_t     =  BETA0_i * pow(TRatio, IIT_i)
        SII0_t      =  SII0_i * (self.hypsmooth(1.0 + TII_i * (TRatio - 1.0) - 0.01, 1.0e-3) + 0.01)

        self.K0_t        =  K0_i + K01_i * delTemp
        K0SI_t      =  K0SI_i + self.hypmax(K0SI1_i * delTemp, -K0SI_i, 1.0e-6)
        K2SI_t      =  K2SI_i + self.hypmax(K2SI1_i * delTemp, -K2SI_i, 1.0e-6)
        self.K1_t        =  K1_i + self.hypmax(K11_i * delTemp, -K1_i, 1.0e-6)
        K2SAT_t     =  K2SAT_i + K2SAT1_i * delTemp
        self.A1_t        =  A1_i + A11_i * delTemp
        self.A2_t        =  A2_i + A21_i * delTemp
        K2_t        =  K2_i + self.hypmax(K21_i * delTemp, -K2_i, 1.0e-6)
        K0SISAT_t   =  K0SISAT_i + K0SISAT1_i * delTemp
        K2SISAT_t   =  K2SISAT_i + K2SISAT1_i * delTemp
        self.AIGBINV_t   =  AIGBINV_i + self.hypmax(AIGBINV1_i * delTemp, -AIGBINV_i, 1.0e-6)
        self.AIGBACC_t   =  AIGBACC_i + self.hypmax(AIGBACC1_i * delTemp, -AIGBACC_i, 1.0e-6)
        self.AIGC_t      =  AIGC_i + self.hypmax(AIGC1_i * delTemp, -AIGC_i, 1.0e-6)
        self.AIGS_t      =  AIGS_i + self.hypmax(AIGS1_i * delTemp, -AIGS_i, 1.0e-6)
        self.AIGD_t      =  AIGD_i + self.hypmax(AIGD1_i * delTemp, -AIGD_i, 1.0e-6)
        BGIDL_t     =  BGIDL_i * self.hypsmooth(1.0 + TGIDL_i * delTemp - 1.0e-6, 1.0e-3)
        BGISL_t     =  BGISL_i * self.hypsmooth(1.0 + TGIDL_i * delTemp - 1.0e-6, 1.0e-3)
        ALPHA0_t    =  ALPHA0_i + self.hypmax(self.ALPHA01 * delTemp, -ALPHA0_i, 1.0e-6)
        self.ALPHA1_t    =  ALPHA1_i + self.hypmax(self.ALPHA11 * delTemp, -ALPHA1_i, 1.0e-6)
        ALPHAII0_t  =  ALPHAII0_i + self.hypmax(self.ALPHAII01 * delTemp, -ALPHAII0_i, 1.0e-25)
        ALPHAII1_t  =  ALPHAII1_i + self.hypmax(self.ALPHAII11 * delTemp, -ALPHAII1_i, 1.0e-20)
        self.igtemp      =  self.lexp(IGT_i * self.lln(TRatio))
        self.igsd_mult   =  self.igsd_mult0 * self.igtemp

        if (self.BULKMOD != 0):
           CJS_t     =  self.Tempdep(self.CJS, self.TCJ, delTemp, self.TEMPMOD)
           CJD_t     =  self.Tempdep(self.CJD, self.TCJ, delTemp, self.TEMPMOD)
           CJSWS_t   =  self.Tempdep(self.CJSWS, self.TCJSW, delTemp, self.TEMPMOD)
           CJSWD_t   =  self.Tempdep(self.CJSWD, self.TCJSW, delTemp, self.TEMPMOD)
           CJSWGS_t  =  self.Tempdep(self.CJSWGS, self.TCJSWG, delTemp, self.TEMPMOD)
           CJSWGD_t  =  self.Tempdep(self.CJSWGD, self.TCJSWG, delTemp, self.TEMPMOD)

           self.PBS_t     =  self.hypsmooth(self.PBS - self.TPB * delTemp - 0.01, 1.0e-3) + 0.01
           self.PBD_t     =  self.hypsmooth(self.PBD - self.TPB * delTemp - 0.01, 1.0e-3) + 0.01
           self.PBSWS_t   =  self.hypsmooth(self.PBSWS - self.TPBSW * delTemp - 0.01, 1.0e-3) + 0.01
           self.PBSWD_t   =  self.hypsmooth(self.PBSWD - self.TPBSW * delTemp - 0.01, 1.0e-3) + 0.01
           self.PBSWGS_t  =  self.hypsmooth(self.PBSWGS - self.TPBSWG * delTemp - 0.01, 1.0e-3) + 0.01
           self.PBSWGD_t  =  self.hypsmooth(self.PBSWGD - self.TPBSWG * delTemp - 0.01, 1.0e-3) + 0.01

           T0        =  self.Eg0 / Vtm0 - self.Eg / self.Vtm
           T1        =  self.lln(TRatio)
           T3        =  self.lexp((T0 + self.XTIS * T1) / self.NJS)
           JSS_t     =  self.JSS * T3
           JSWS_t    =  self.JSWS * T3
           JSWGS_t   =  self.JSWGS * T3

           T3        =  self.lexp((T0 + self.XTID * T1) / self.NJD)
           JSD_t     =  self.JSD * T3
           JSWD_t    =  self.JSWD * T3
           JSWGD_t   =  self.JSWGD * T3

           self.JTSS_t     =  self.JTSS * self.lexp(self.Eg0 * self.XTSS * (TRatio - 1.0) / self.Vtm)
           self.JTSD_t     =  self.JTSD * self.lexp(self.Eg0 * self.XTSD * (TRatio - 1.0) / self.Vtm)
           self.JTSSWS_t   =  self.JTSSWS * self.lexp(self.Eg0 * self.XTSSWS * (TRatio - 1.0) / self.Vtm)
           self.JTSSWD_t   =  self.JTSSWD * self.lexp(self.Eg0 * self.XTSSWD * (TRatio - 1.0) / self.Vtm)
           self.JTSSWGS_t  =  self.JTSSWGS * (math.sqrt(self.JTWEFF / self.Weff0) + 1.0) * self.lexp(self.Eg0 * self.XTSSWGS * (TRatio - 1.0) / self.Vtm)
           self.JTSSWGD_t  =  self.JTSSWGD * (math.sqrt(self.JTWEFF / self.Weff0) + 1.0) * self.lexp(self.Eg0 * self.XTSSWGD * (TRatio - 1.0) / self.Vtm)

           # All NJT's Smoothed to 0.01 to Prevent Divide-by-zero / Negative Values
           NJTS_t      =  self.hypsmooth(self.NJTS * (1.0 + self.TNJTS * (TRatio-1.0)) - 0.01, 1.0e-3) + 0.01
           self.NJTSD_t     =  self.hypsmooth(self.NJTSD * (1.0 + self.TNJTSD * (TRatio-1.0)) - 0.01, 1.0e-3) + 0.01
           NJTSSW_t    =  self.hypsmooth(self.NJTSSW * (1.0 + self.TNJTSSW * (TRatio-1.0)) - 0.01, 1.0e-3) + 0.01
           self.NJTSSWD_t   =  self.hypsmooth(self.NJTSSWD * (1.0 + self.TNJTSSWD * (TRatio-1.0)) - 0.01, 1.0e-3) + 0.01
           NJTSSWG_t   =  self.hypsmooth(self.NJTSSWG * (1.0 + self.TNJTSSWG * (TRatio-1.0)) - 0.01, 1.0e-3) + 0.01
           NJTSSWGD_t  =  self.hypsmooth(self.NJTSSWGD * (1.0 + self.TNJTSSWGD * (TRatio-1.0)) - 0.01, 1.0e-3) + 0.01


        if (not self.param_given("VFBSD")):
            if (self.NGATE > 0.0):
               self.vfbsd  =  self.devsign * (self.hypsmooth(0.5 * self.Eg - self.Vtm * self.lln(self.NGATE / ni), 1.0e-4) - (0.5 * self.Eg - self.devsign * (0.5 * self.Eg - self.hypsmooth(0.5 * self.Eg - self.Vtm * self.lln(self.NSD / ni), 1.0e-4))))
            else:
               self.vfbsd  =  self.devsign * (PHIG_i - (self.EASUB + 0.5 * self.Eg - self.devsign * (0.5 * self.Eg - self.hypsmooth(0.5 * self.Eg - self.Vtm * self.lln(self.NSD / ni), 1.0e-4))))

        else:
           self.vfbsd  =  self.VFBSD

        if (not self.param_given("VFBSDCV")):
           self.vfbsdcv  =  self.vfbsd
        else:
           self.vfbsdcv  =  self.VFBSDCV

        if self.__SHMOD__:
           if (self.SHMOD != 0 and self.RTH0 > 0.0):
              T0    =  self.Vtm * self.lln(self.nbody / ni)
              self.phib  =  math.sqrt(T0 * T0 + 1.0e-6)
           else:
              self.phib  =  self.Vtm * self.lln(self.nbody / ni)

        else:
           self.phib  =  self.Vtm * self.lln(self.nbody/ni)

        if self.__SHMOD__:
           if (self.SHMOD != 0 and self.RTH0 > 0.0):
              T0   =  self.Vtm * self.lln(self.nbody * self.NSD / (ni * ni))
              self.vbi  =  math.sqrt(T0 * T0 + 1.0e-6)
           else:
              self.vbi  =  self.Vtm * self.lln(self.nbody * self.NSD / (ni * ni))

        else:
           self.vbi  =  self.Vtm * self.lln(self.nbody * self.NSD / (ni * ni))

        # self.deltaPhi definition and Polysilicon Depletion
        # self.deltaPhi: workfunction difference between the gate and the n+ source.
        if self.TYPE == self.ntype:
            deltaPhi_p = 0
        else:
            deltaPhi_p = self.Eg

        self.deltaPhi  =  self.devsign*(PHIG_i - (self.EASUB + deltaPhi_p))

        # Mobility Degradation
        self.eta_mu     =  0.5 * ETAMOB_t
        self.eta_mu_cv  =  0.5
        if ( self.TYPE != self.ntype ):
           self.eta_mu     =  1.0 / 3.0 * ETAMOB_t
           self.eta_mu_cv  =  1.0 / 3.0


        # Junction Current and Capacitance
        if (self.BULKMOD != 0):
           # Source-Side Junction Current
           self.Isbs  =  self.ASEJ * JSS_t + self.PSEJ * JSWS_t + self.TFIN * self.NFINtotal * JSWGS_t
           if (self.Isbs > 0.0):
              Nvtms     =  self.Vtm * self.NJS
              XExpBVS   =  self.lexp(-self.BVS / Nvtms) * self.XJBVS
              T2        =  max(self.IJTHSFWD / self.Isbs, 10.0)
              Tb        =  1.0 + T2 - XExpBVS
              VjsmFwd   =  Nvtms * self.lln(0.5 * (Tb + math.sqrt(Tb * Tb + 4.0 * XExpBVS)))
              T0        =  self.lexp(VjsmFwd / Nvtms)
              IVjsmFwd  =  self.Isbs * (T0 - XExpBVS / T0 + XExpBVS - 1.0)
              SslpFwd   =  self.Isbs * (T0 + XExpBVS / T0) / Nvtms
              T2        =  self.hypsmooth(self.IJTHSREV / self.Isbs - 10.0, 1.0e-3) + 10.0
              VjsmRev   =  -self.BVS - Nvtms * self.lln((T2 - 1.0) / self.XJBVS)
              T1        =  self.XJBVS * self.lexp(-(self.BVS + VjsmRev) / Nvtms)
              IVjsmRev  =  self.Isbs * (1.0 + T1)
              SslpRev   =  -self.Isbs * T1 / Nvtms


           # Drain-Side Junction Current
           self.Isbd  =  self.ADEJ * JSD_t + self.PDEJ * JSWD_t + self.TFIN * self.NFINtotal * JSWGD_t
           if (self.Isbd > 0.0):
              Nvtmd     =  self.Vtm * self.NJD
              XExpBVD   =  self.lexp(-self.BVD / Nvtmd) * self.XJBVD
              T2        =  max(self.IJTHDFWD / self.Isbd, 10.0)
              Tb        =  1.0 + T2 - XExpBVD
              VjdmFwd   =  Nvtmd * self.lln(0.5 * (Tb + math.sqrt(Tb * Tb + 4.0 * XExpBVD)))
              T0        =  self.lexp(VjdmFwd / Nvtmd)
              IVjdmFwd  =  self.Isbd * (T0 - XExpBVD / T0 + XExpBVD - 1.0)
              DslpFwd   =  self.Isbd * (T0 + XExpBVD / T0) / Nvtmd
              T2        =  self.hypsmooth(self.IJTHDREV / self.Isbd - 10.0, 1.0e-3) + 10.0
              VjdmRev   =  -self.BVD - Nvtmd * self.lln((T2 - 1.0) / self.XJBVD)
              T1        =  self.XJBVD * self.lexp(-(self.BVD + VjdmRev) / Nvtmd)
              IVjdmRev  =  self.Isbd * (1.0 + T1)
              DslpRev   =  -self.Isbd * T1 / Nvtmd


           # Junction Capacitance
           self.Czbs     =  CJS_t * self.ASEJ
           self.Czbssw   =  CJSWS_t * self.PSEJ
           self.Czbsswg  =  CJSWGS_t * self.Weff0 * self.NFINtotal
           self.Czbd     =  CJD_t * self.ADEJ
           self.Czbdsw   =  CJSWD_t * self.PDEJ
           self.Czbdswg  =  CJSWGD_t * self.Weff0 * self.NFINtotal


        # Generation-Recombination Current
        T0        =  self.Eg / self.Vtm * (TRatio - 1.0)
        T1        =  T0 / NTGEN_i
        self.igentemp  =  self.lexp(T1)

        #  End of temperature dependent calculations

        # ************************************************
        # *     Bias dependent calculations follow       *
        # ************************************************
    def CMGBiasDepCalc(self, edge_currents, edge_voltages, delta_t):

        # Voltages
        # V(`IntrinsicGate, si)
        V_IntrinsicGate_si = edge_voltages[self.V_IntrinsicGate_si_ref]
        #print (V_IntrinsicGate_si)

        # V(di, si)
        V_di_si = edge_voltages[self.V_di_si_ref]

        # V(d, s)
        V_d_s = edge_voltages[self.V_d_s_ref]

        # v(d, di)
        V_d_di = edge_voltages[self.V_d_di_ref]

        # v(s, si)
        V_s_si = edge_voltages[self.V_s_si_ref]

        # V(`IntrinsicGate, di)
        V_IntrinsicGate_di = edge_voltages[self.V_IntrinsicGate_di_ref]

        # V(e, si)
        V_e_si = edge_voltages[self.V_e_si_ref]

        # V(e, di)
        V_e_di = edge_voltages[self.V_e_di_ref]

        # V(`IntrinsicGate, e)
        V_IntrinsicGate_e = edge_voltages[self.V_IntrinsicGate_e_ref]

        # V(`GateEdgeNode, si)
        V_GateEdgeNode_si = edge_voltages[self.V_GateEdgeNode_si_ref]

        # V(`GateEdgeNode, di)
        V_GateEdgeNode_di = edge_voltages[self.V_GateEdgeNode_di_ref]

        if (self.CGEOMOD == 1):
            # V(`GateEdgeNode, s)
            V_GateEdgeNode_s = edge_voltages[self.V_GateEdgeNode_s_ref]

            # V(`GateEdgeNode, d)
            V_GateEdgeNode_d = edge_voltages[self.V_GateEdgeNode_d_ref]
        else:
           V_GateEdgeNode_s = 0
           V_GateEdgeNode_d = 0


        # V(`GateEdgeNode, gi) # REVISIT
        V_GateEdgeNode_gi = edge_voltages[0]

        # V(e, `GateEdgeNode)
        V_e_GateEdgeNode = edge_voltages[self.V_e_GateEdgeNode_ref]

        # V(`GateEdgeNode, e)
        V_GateEdgeNode_e = -edge_voltages[self.V_e_GateEdgeNode_ref]

        if self.__RGATEMOD__:
            # V(g, ge)
            V_g_ge = edge_voltages[self.V_g_ge_ref]
        else:
            V_g_ge = 0

        V_si_s = -1 * edge_voltages[self.V_s_si_ref]

        V_di_d = -1 * edge_voltages[self.V_d_di_ref]

        # Initialise currents
        I_di_si            = 0
        I_si_di            = 0
        I_d_s              = 0
        I_d_di             = 0
        I_si_di            = 0
        I_s_si             = 0
        I_e_si             = 0
        I_e_di             = 0
        I_e_GateEdgeNode   = 0
        I_GateEdgeNode_si  = 0
        I_GateEdgeNode_di  = 0
        I_IntrinsicGate_si = 0
        I_IntrinsicGate_di = 0
        I_IntrinsicGate_e  = 0 #REVISIT
        I_g_ge             = 0 #REVISIT
        I_di_e             = 0
        I_si_e             = 0

        # Load Terminal Voltages
        vgs_noswap  =  self.devsign * V_IntrinsicGate_si
        vds_noswap  =  self.devsign * V_di_si
        vgd_noswap  =  self.devsign * V_IntrinsicGate_di
        ves_jct     =  self.devsign * V_e_si
        ved_jct     =  self.devsign * V_e_di
        vge         =  self.devsign * V_IntrinsicGate_e

        # Source-Drain Interchange
        sigvds  =  1.0
        if (vds_noswap < 0.0):
           sigvds  =  -1.0
           vgs     =  vgs_noswap - vds_noswap
           vds     =  -1.0 * vds_noswap
           ves     =  ved_jct
        else:
           vgs     =  vgs_noswap
           vds     =  vds_noswap
           ves     =  ves_jct

        vgsfb  =  vgs - self.deltaPhi

        # Initialize Certain Variables to Zero to Prevent Unnecessary Update
        etaiv  =  0.0
        Qes    =  0.0
        Qesj   =  0.0
        Qeg    =  0.0
        Qed    =  0.0
        Qedj   =  0.0

        # Vds Smoothing
        vdsx  =  math.sqrt (vds * vds + 0.01) - 0.1

        # Ves Smoothing
        if (self.BULKMOD != 0):
           vesx    =  ves - 0.5 * (vds - vdsx)
           vesmax  =  0.95 * self.PHIBE_i
           T2      =  vesmax - vesx - 1.0e-3
           veseff  =  vesmax - 0.5 * (T2 + math.sqrt(T2 * T2 + 0.004 * vesmax))


        # Asymmetry Model
        T0  =  math.tanh(0.6 * vds_noswap / self.Vtm)
        wf  =  0.5 + 0.5 * T0
        wr  =  1.0 - wf
        if (self.ASYMMOD != 0):
           CDSCD_a     =  CDSCDR_i * wr + self.CDSCD_i * wf
           ETA0_a      =  ETA0R_t * wr + self.ETA0_t * wf
           PDIBL1_a    =  PDIBL1R_i * wr + self.PDIBL1 * wf
           PDIBL2_a    =  PDIBL2R_i * wr + self.PDIBL2_i * wf
           MEXP_a      =  MEXPR_t * wr + self.MEXP_t * wf
           PTWG_a      =  PTWGR_t * wr + self.PTWG_t * wf
           VSAT1_a     =  VSAT1R_t * wr + self.VSAT1_t * wf
           RSDR_a      =  RSDRR_t * wr + self.RSDR_t * wf
           RDDR_a      =  RDDRR_t * wr + self.RDDR_t * wf
           PCLM_a      =  PCLMR_i * wr + self.PCLM_i * wf
           VSAT_a      =  VSATR_t * wr + self.VSAT_t * wf
           KSATIV_a    =  KSATIVR_i * wr + self.KSATIV_i * wf
           DVTSHIFT_a  =  DVTSHIFTR_i * wr + self.DVTSHIFT_i * wf
           CIT_a       =  CITR_i * wr + self.CIT_i * wf
           self.u0_a        =  self.u0r * wr + self.u0 * wf
           UA_a        =  UAR_t*wr + self.UA_t * wf
           UD_a        =  UDR_t * wr + self.UD_t * wf
           UC_a        =  UCR_t * wr + self.UC_t * wf
           EU_a        =  EUR_i * wr + self.EU_i * wf
        else:
           CDSCD_a     =  self.CDSCD_i
           ETA0_a      =  self.ETA0_t
           PDIBL1_a    =  self.PDIBL1
           PDIBL2_a    =  self.PDIBL2_i
           MEXP_a      =  self.MEXP_t
           PTWG_a      =  self.PTWG_t
           VSAT1_a     =  self.VSAT1_t
           RSDR_a      =  self.RSDR_t
           RDDR_a      =  self.RDDR_t
           PCLM_a      =  self.PCLM_i
           VSAT_a      =  self.VSAT_t
           KSATIV_a    =  self.KSATIV_i
           DVTSHIFT_a  =  self.DVTSHIFT_i
           CIT_a       =  self.CIT_i
           self.u0_a        =  self.u0
           UA_a        =  self.UA_t
           UD_a        =  self.UD_t
           UC_a        =  self.UC_t
           EU_a        =  self.EU_i


        # Drain Saturation Voltage
        inv_MEXP  =  1.0 / MEXP_a

        # SCE, DIBL, SS Degradation Effects (Ref: BSIM4 Model)
        phist  =  0.4 + self.phib + self.PHIN_i
        T1     =  2.0 * (self.Cins / self.Weff_UFCM) / (self.rc + 2.0)
        cdsc   =  self.Theta_SW * (self.CDSC_i + CDSCD_a * vdsx)

        if (not self.param_given("NVTM")):
            nVtm  =  self.Vtm * self.ThetaSS * (1.0 + (CIT_a + cdsc) / T1)
        else:
            nVtm  =  self.NVTM

        # temp deped UFCM
        qdep                  =  self.Qdep_ov_self / nVtm
        vth_fixed_factor_SI   =  math.log(self.Cins * nVtm/(self.q * self.Nc * 2.0 * self.Ach)) # Natural log
        vth_fixed_factor_Sub  =  math.log((qdep * self.rc) * (qdep * self.rc) / ((math.exp(qdep * self.rc) - qdep * self.rc - 1.0))) + vth_fixed_factor_SI # Natural log
        q0                    =  10.0 * nVtm / self.rc + 2.0 * self.qbs

        # New QM parameter calculation: fieldnormalizationfactor, auxQMfact, QMFACTORCVfinal
        fieldnormalizationfactor  =  self.Vtm * self.Cins / (self.Weff_UFCM * self.epssub)
        auxQMfact                 =  pow(((3.0 / 4.0) * 3.0 * self.HBAR * 2.0 * math.pi * self.q / (4.0 * math.sqrt(2.0 * self.mx))), 2.0 / 3.0)
        QMFACTORCVfinal           =  self.QMFACTORCV * auxQMfact * pow(fieldnormalizationfactor, 2.0 / 3.0) * (1/(self.q * self.Vtm))

        dvth_vtroll  =  -self.DVT0_i * self.Theta_SCE * (self.vbi - phist)
        dvth_dibl    =  -ETA0_a * self.Theta_DIBL * vdsx + (self.DVTP0_i * self.Theta_DITS * pow(vdsx, self.DVTP1_i))
        dvth_rsce    =  self.K1RSCE_i * self.Theta_RSCE * math.sqrt(phist)
        dvth_all     =  dvth_vtroll + dvth_dibl + dvth_rsce + self.dvth_temp + DVTSHIFT_a
        vgsfb        =  vgsfb - dvth_all

        # Vgs Clamping for Inversion Region Calculation in Accumulation
        beta0     =  self.u0_a * self.cox * self.Weff0 / self.Leff
        T0        =  -(self.dvch_qm + nVtm * self.lln(2.0 * self.cox * self.Imin / (beta0 * nVtm * self.q * self.Nc * self.TFIN)))
        T1        =  vgsfb + T0 + self.DELVTRAND
        vgsfbeff  =  self.hypsmooth(T1 , 1.0e-4) - T0

        # Core Model Calculation at Source Side
        vch  =  0.0 + self.dvch_qm

        if (self.BULKMOD != 0):
            T1  =  self.hypsmooth(2.0 * self.phib + vch - ves, 0.1)
            T3  =  (-self.K1_t / (2.0 * nVtm)) * (math.sqrt(T1) - math.sqrt(2.0 * self.phib))
            T0  =  -qdep - T3 + vth_fixed_factor_Sub + QMFACTORCVfinal * pow(-qdep, 2.0/3.0)
            T1  =  -qdep - T3 + vth_fixed_factor_SI
        else:
            T0  =  -qdep + vth_fixed_factor_Sub + QMFACTORCVfinal * pow(-qdep, 2.0/3.0)
            T1  =  -qdep + vth_fixed_factor_SI

        T2  =  (vgsfbeff - vch) / nVtm
        F0  =  -T2 + T1
        T3  =  0.5 * (T2 - T0)
        qm  =  math.exp(T3)
        if (qm > 1.0e-7):
            T7  =  math.log(1.0 + qm)
            qm  =  2.0 * (1.0 - math.sqrt(1.0 + T7 * T7))
            T8  =  (qm * self.ALPHA_UFCM + qdep) * self.rc
            T4  =  T8 / (math.exp(T8) - T8 - 1.0)
            T5  =  T8 * T4
            e0  =  F0 - qm + math.log(-qm) + math.log(T5) + QMFACTORCVfinal * pow(-(qm + qdep), 2.0 / 3.0)
            e1  =  -1.0 + 1.0 / qm + (2.0 / T8 - T4 - 1.0) * self.rc - (2.0 / 3.0) * QMFACTORCVfinal * pow(-(qm + qdep), -1.0 / 3.0)
            e2  =  -1.0 / (qm * qm) - (2.0 / 9.0) * QMFACTORCVfinal * pow(-(qm + qdep), -4.0/3.0)
            qm  =  qm - (e0 / e1) * (1.0 + (e0 * e2) / (2.0 * e1 * e1))
            T8  =  (qm * self.ALPHA_UFCM + qdep) * self.rc
            T4  =  T8 / (math.exp(T8) - T8 - 1.0)
            T5  =  T8 * T4
            e0  =  F0 - qm + math.log(-qm) + math.log(T5) + QMFACTORCVfinal * pow(-(qm + qdep), 2.0 / 3.0)
            e1  =  -1.0 + 1.0 / qm + (2.0 / T8 - T4 - 1.0) * self.rc - (2.0 / 3.0) * QMFACTORCVfinal * pow(-(qm + qdep), -1.0/3.0)
            e2  =  -1.0 / (qm * qm) - (2.0 / 9.0) * QMFACTORCVfinal * pow(-(qm + qdep), -4.0/3.0)
            qm  =  qm - (e0 / e1) * (1.0 + (e0 * e2) / (2.0 * e1 * e1))
        else:
            qm  =  -qm * qm

        qis  =  -qm * nVtm

        # Drain Saturation Voltage
        Eeffs  =  self.EeffFactor * (self.qbs + self.eta_mu * qis)
        qb0    =  1.0e-2 / self.cox
        T2     =  pow(0.5 * (1.0 + abs((qis) / qb0)), self.UCS_t)
        if (self.BULKMOD != 0):
           T3  =  (UA_a + UC_a * veseff) * pow(abs(Eeffs), EU_a) + UD_a / T2
        else:
           T3  =  UA_a * pow(abs(Eeffs), EU_a) + UD_a / T2

        Dmobs  =  1.0 + T3
        Dmobs  =  Dmobs / self.U0MULT

        if (self.RDSMOD == 1):
            Rdss  =  0.0
        elif (self.RDSMOD == 0):
            T4    =  1.0 + self.PRWGS_i * qis
            T1    =  1.0 / T4
            T0    =  0.5 * (T1 + math.sqrt(T1 * T1 + 0.01))
            Rdss  =  (self.RDSWMIN_i + self.RDSW_i * T0) * self.WeffWRFactor * self.NFINtotal * self.rdstemp
        else:
            T4    =  1.0 + self.PRWGS_i * qis
            T1    =  1.0 / T4
            T0    =  0.5 * (T1 + math.sqrt(T1 * T1 + 0.01))
            Rdss  =  (self.RSourceGeo + self.RDrainGeo + self.RDSWMIN_i + self.RDSW_i * T0) * self.WeffWRFactor * self.NFINtotal * self.rdstemp

        Esat   =  2.0 * VSAT_a / self.u0_a * Dmobs
        EsatL  =  Esat * self.Leff
        T6     =  KSATIV_a * (qis +  2 * self.Vtm)

        if (Rdss == 0.0):
           Vdsat  =  EsatL * T6 / (EsatL + T6)
        else:
           WVCox  =  self.Weff0 * VSAT_a * self.cox
           T0     =  WVCox * Rdss
           Ta     =  2.0 * T0
           Tb     =  T6 + EsatL + 3.0 * T6 * T0
           Tc     =  T6 * (EsatL + 2.0 * T6 * T0)
           Vdsat  =  (Tb - math.sqrt(Tb * Tb - 2.0 * Ta * Tc)) / Ta

        Vdsat   =  self.hypsmooth(Vdsat - 1.0e-3, 1.0e-5) + 1.0e-3
        T7      =  pow(vds / Vdsat , MEXP_a)
        T8      =  pow(1.0 + T7, inv_MEXP)
        Vdseff  =  vds / T8

        if (Vdseff > vds):
           Vdseff  =  vds


        # Core Model Calculation at Drain Side
        vch  =  Vdseff + self.dvch_qm

        if (self.BULKMOD != 0):
           T1  = self.hypsmooth(2.0 * self.phib + vch - ves, 0.1)
           T3  = (-self.K1_t / (2.0 * nVtm)) * (math.sqrt(T1) - math.sqrt(2.0 * self.phib))
           T0  = -qdep - T3 + vth_fixed_factor_Sub + QMFACTORCVfinal * pow(-qdep, 2.0 / 3.0)
           T1  = -qdep - T3 + vth_fixed_factor_SI
        else:
           T0  = -qdep + vth_fixed_factor_Sub + QMFACTORCVfinal * pow(-qdep, 2.0 / 3.0)
           T1  = -qdep + vth_fixed_factor_SI

        T2  =  (vgsfbeff - vch) / nVtm
        F0  =  -T2 + T1
        T3  =  (T2 - T0) * 0.5
        qm  =  math.exp(T3)
        if (qm > 1.0e-7):
           T7  =  math.log(1.0 + qm)
           qm  =  2.0 * (1.0 - math.sqrt(1.0 + T7 * T7))
           T8  =  (qm * self.ALPHA_UFCM + qdep) * self.rc
           T4  =  T8 / (math.exp(T8) - T8 - 1.0)
           T5  =  T8 * T4
           e0  =  F0 - qm + math.log(-qm) + math.log(T5) + QMFACTORCVfinal * pow(-(qm + qdep), 2.0 / 3.0)
           e1  =  -1.0 + (1.0 / qm) + (2.0 / T8 - T4 - 1.0) * self.rc - (2.0 / 3.0) * QMFACTORCVfinal * pow(-(qm + qdep), -1.0 / 3.0)
           e2  =  -1.0 / (qm * qm) - (2.0 / 9.0) * QMFACTORCVfinal * pow(-(qm + qdep), -4.0 / 3.0)
           qm  =  qm - (e0 / e1)*(1.0 + (e0 * e2) / (2.0 * e1 * e1))
           T8  =  (qm * self.ALPHA_UFCM + qdep) * self.rc
           T4  =  T8 / (math.exp(T8) - T8 - 1.0)
           T5  =  T8 * T4
           e0  =  F0 - qm + math.log(-qm) + math.log(T5) + QMFACTORCVfinal * pow(-(qm + qdep), 2.0/3.0)
           e1  =  -1.0 + (1.0 / qm) + (2.0 / T8 - T4 - 1.0) * self.rc - (2.0 / 3.0) * QMFACTORCVfinal * pow(-(qm + qdep), -1.0 / 3.0)
           e2  =  -1.0 / (qm * qm) - (2.0 / 9.0) * QMFACTORCVfinal * pow(-(qm + qdep), -4.0 / 3.0)
           qm  =  qm - (e0 / e1) * (1.0 + (e0 * e2) / (2.0 * e1 * e1))
        else:
           qm  =  -qm * qm

        qid  =  -qm * nVtm

        if (self.BULKMOD != 0):
           T9  =  (self.K1_t / (2.0 * nVtm)) * math.sqrt(self.Vtm)
           T0  =  T9 / 2.0
           T2  =  (vge - (self.deltaPhi - self.Eg - self.Vtm * math.log(self.NBODY / self.Nc) + self.DELVFBACC)) / self.Vtm
           if ((T2 * self.Vtm) > self.phib + T9 * math.sqrt(self.phib * self.Vtm)):
               T1   =  math.sqrt(T2 - 1.0 + T0 * T0) - T0
               T10  =  1.0 + T1 * T1
           else:
               T3   =  T2 * 0.5 - 3.0 * (1.0 + T9 / math.sqrt(2.0))
               T10  =  T3 + math.sqrt(T3 * T3 + 6.0 * T2)
               if (T2 < 0.0):
                   T4   =  (T2 - T10) / T9
                   T10  = -math.log(1.0 - T10 + T4 * T4 )
               else:
                   T11  = math.exp(-T10)
                   T4   = math.sqrt(T2 - 1.0 + T11 + T0 * T0) - T0
                   T10  = 1.0 - T11 + T4 * T4


           T6  =  math.exp(-T10) - 1.0
           T7  =  math.sqrt(T6 + T10)
           if (T10 > 1.0e-15):
              e0   =  -(T2 - T10) + T9 * T7
              e1   =  1.0 - T9 * 0.5 * T6 / T7
              T8   =  T10 - (e0 / e1)
              T11  =  math.exp(-T8) - 1.0
              T12  =  math.sqrt(T11 + T8)
              self.qba  =  -T9 * T12 * self.Vtm
           else:
              if (T10 < -1.0e-15):
                  e0   =  -(T2 - T10) - T9 * T7
                  e1   =  1.0 + T9 * 0.5 * T6 / T7
                  T8   =  T10 - e0 / e1
                  T12  =  T9 * math.sqrt(math.exp(-T8) + T8 - 1.0)
              else:
                  T12  =  0.0
                  T8   =  0.0

              self.qba  =  T12 * self.Vtm

           qi_acc_for_QM  =  T9 * math.exp(-T8 / 2.0) * self.Vtm

           psipclamp  =  0.5 * (T8 + 1.0 + math.sqrt((T8 - 1.0) * (T8 - 1.0) + 0.25 * 2.0 * 2.0))
           sqrtpsip   =  math.sqrt(psipclamp)
           nq         =  1.0 + T9 / sqrtpsip


        # Drain Side and Average Potential / Charge
        qia  =  0.5 * (qis + qid)
        dqi  =  qis - qid

        T0   = pow(Vdseff, 2.0) / 6.25e-4   # pow(Vdseff,2.0) / pow(25e-3, 2.0)
        if (self.CHARGEWF != 0.0):
            qia2 = 0.5 * (qis + qid) + self.CHARGEWF * (1.0 - self.lexp(-T0)) * 0.5 * dqi
        else:
            qia2 = 0.5 * (qis + qid)

        #if self.__DEBUG__: REVISIT
        #   if (qis < 0.0) print("Warning: negative source-side inversion carrier density.  Vgs=%f  Vds=%f  Vbs=%f qis=%e", V(g, s), V_d_s, V(e, s), qis)
        #   if (qid < 0.0) print("Warning: negative drain-side inversion carrier density.  Vgs=%f  Vds=%f  Vbs=%f qid=%e", V(g, s), V_d_s, V(e, s), qid)

        # Toxeff model for quantum mechanical effects
        # Normal operation (Vgs > Vfb)
        if (self.QMTCENCV_i > 0.0):
           T4      =  qia / self.QM0
           T5      =  1.0 + pow(T4, self.PQM)
           Tcen    =  Tcen0 / T5
           self.coxeff  =  1.0 / (1.0 / (self.cox * self.EOT / self.TOXP) + Tcen * self.QMTCENCV_i / self.epssub)
        else:
           self.coxeff  =  self.cox


        # Quantum Mechanical Effect Correction for Accumulation Side Cap (Vgs < Vfb)
        if (self.BULKMOD != 0 and self.QMTCENCVA_i != 0.0):
           T6       =  1.0 + pow(qi_acc_for_QM / self.QM0ACC, self.PQMACC)
           Tcen     =  Tcen0 / T6
           self.cox_acc  =  1.0 / (1.0 / self.cox_acc + Tcen * self.QMTCENCVA_i / self.epssub)


        # Multiplication Factor for I-V
        beta  =  self.u0_a * self.cox * self.Weff0 / self.Leff

        # Mobility Degradation
        Eeffm  =  self.EeffFactor * (self.qba + self.eta_mu * qia2)
        T2     =  pow(0.5 * (1.0 + abs((qia2) / qb0)), self.UCS_t)
        if (self.BULKMOD != 0):
            T3  =  (UA_a + UC_a * veseff) * pow(abs(Eeffm), EU_a) + UD_a / T2
        else:
            T3  =  UA_a * pow(abs(Eeffm), EU_a) + UD_a / T2

        Dmob  =  1.0 + T3
        Dmob  =  Dmob / self.U0MULT
        ueff  =  self.u0_a / Dmob

        # Mobility Degradation for C-V
        Eeffm_cv  =  self.EeffFactor * (self.qba + self.eta_mu_cv * qia2)
        T3        =  UA_a * pow(abs(Eeffm_cv), EU_a) + UD_a / T2
        Dmob_cv   =  1.0 + T3
        Dmob_cv   =  Dmob_cv / self.U0MULT

        # Calculate current and capacitance enhancement factors due to CLM and DIBL
        tmp  =  self.DROUT_i * self.Leff / self.scl + 1.0e-6

        if (tmp < 40.0):
           DIBLfactor  =  0.5 * PDIBL1_a / (self.cosh(tmp) - 1.0) + PDIBL2_a
        else:
           DIBLfactor  =  PDIBL1_a * math.exp(-tmp) + PDIBL2_a


        if (self.PVAG_i > 0.0):
           PVAGfactor  =  1.0 + self.PVAG_i * qia / EsatL
        else:
           PVAGfactor  =  1.0 / (1.0 - self.PVAG_i * qia / EsatL)


        if (Vdseff > vds):
           Vdseff  =  vds

        diffVds   =  vds - Vdseff
        Vgst2Vtm  =  qia + 2.0 * self.Vtm
        if (DIBLfactor > 0):
           T1      =  Vgst2Vtm
           T3      =  T1 / (Vdsat + T1)
           VaDIBL  =  T1 / DIBLfactor * T3 * PVAGfactor
           Moc     =  1.0 + diffVds / VaDIBL
        else:
           Moc     =  1.0


        if (PCLM_a > 0.0):
           if (self.PCLMG_i < 0.0):
              T1  =  1.0 / (1.0 / PCLM_a - self.PCLMG_i * qia)
           else:
              T1  =  PCLM_a + self.PCLMG_i * qia

           Mclm  =  1.0 + T1 * self.lln(1.0 + (vds - Vdseff) / T1 / (Vdsat + EsatL))
        else:
           Mclm  =  1.0


        Moc  =  Moc * Mclm

        # Current Degradation Factor Due to Velocity Saturation
        Esat1   =  2.0 * VSAT1_a / ueff
        Esat1L  =  Esat1 * self.Leff
        T0      =  self.lexp(self.PSAT_i * self.lln(dqi / Esat1L))
        Ta      =  (1.0 + self.lexp(1.0 / self.PSAT_i * self.lln(self.DELTAVSAT_i)))
        Dvsat   =  (1.0 + self.lexp(1.0 / self.PSAT_i * self.lln(self.DELTAVSAT_i + T0))) / Ta
        Dvsat   =  Dvsat + 0.5 * PTWG_a * qia * dqi * dqi

        # Non-Saturation Effect
        T0     =  self.A1_t + self.A2_t / (qia + 2.0 * nVtm)
        T1     =  T0 * dqi * dqi
        T2     =  T1 + 1.0 - 0.001
        T3     =  -1.0 + 0.5 * (T2 + math.sqrt(T2 * T2 + 0.004))   # max(T1, -1.0)
        Nsat   =  0.5 * (1.0 + math.sqrt(1.0 + T3))
        Dvsat  =  Dvsat * Nsat

        # Lateral Non-uniform doping effect (IV-CV Vth shift) factor
        if (self.K0_t != 0):
           T1    =  self.K0_t / (max(0, K0SI_t + K0SISAT_t * dqi * dqi) * qia + 2.0 * nVtm)
           Mnud  =  self.lexp(-T1)
        else:
           Mnud  =  1.0


        # Body-Effect Factor for self.BULKMOD = 2
        if (self.BULKMOD == 2):
           T0      =  self.hypsmooth((K2_t + K2SAT_t * vdsx), 1.0e-6)
           T1      =  T0 / (max(0, K2SI_t + K2SISAT_t * dqi * dqi) * qia + 2.0 * nVtm)
           T3      =  math.sqrt(self.PHIBE_i - veseff) - math.sqrt(self.PHIBE_i)
           Mob     =  self.lexp(- T1 * T3)
        else:
           Mob  =  1.0

        # Velocity Saturation Factor for C-V
        EsatCV   =  2.0 * self.VSATCV_t * Dmob_cv / self.u0_a
        EsatCVL  =  EsatCV * self.LeffCV
        T0       =  self.lexp(self.PSATCV_i * self.lln(dqi / EsatCVL))
        Ta       =  (1.0 + self.lexp(1.0 / self.PSATCV_i * self.lln(self.DELTAVSATCV_i)))
        DvsatCV  =  (1.0 + self.lexp(1.0 / self.PSATCV_i * self.lln(self.DELTAVSATCV_i + T0))) / Ta

        # Channel Length Modulation factor for C-V
        if (self.PCLMCV_i != 0):
           MclmCV  =  1.0 + self.PCLMCV_i * self.lln(1.0 + (vds - Vdseff) / self.PCLMCV_i / (Vdsat + EsatCVL))
        else:
           MclmCV  =  1.0


        # Calculating fixed body charge qb with sign (Here to avoid multiple calculation in NQSMOD=3 case)
        qb  =  -self.q * self.nbody * self.Ach * self.LeffCV

        # ************************************************
        # *     Current and Charges Calculations         *
        # ************************************************
        # Quasi Static I-V Model
        T1     =  qia
        etaiv  =  q0 / (q0 + qia)
        T2     =  (2.0 - etaiv) * nVtm
        ids0_ov_dqi  =  T1 + T2
        ids0   =  ids0_ov_dqi * dqi

        # S/D Series Resistance
        #`include "bsimcmg_rdsmod.include"
        if self.RDSMOD == 1:
            Rdsi     =  0.0
            Dr       =  1.0

            T2       =  vgs_noswap - self.vfbsd
            T3       =  math.sqrt(T2 * T2 + 1.0e-1)
            vgs_eff  =  0.5 * (T2 + T3)
            T4       =  1.0 + self.PRWGS_i * vgs_eff
            T1       =  1.0 / T4
            T0       =  0.5 * (T1 + math.sqrt(T1 * T1 + 0.01))
            T5       =  self.RSW_i * (1.0 + RSDR_a * self.lexp(0.5 * self.PRSDR * self.lln(V_si_s * V_si_s + 1.0E-6)))
            Rsource  =  self.rdstemp * (self.RSourceGeo + (self.RSWMIN_i + T5 * T0) * self.WeffWRFactor)

            T2       =  vgd_noswap - self.vfbsd
            T3       =  math.sqrt(T2 * T2 + 1.0e-1)
            vgd_eff  =  0.5 * (T2 + T3)
            T4       =  1.0 + self.PRWGD_i * vgd_eff
            T1       =  1.0 / T4
            T0       =  0.5 * (T1 + math.sqrt(T1 * T1 + 0.01))
            T5       =  self.RDW_i * (1.0 + RDDR_a * self.lexp(0.5 * self.PRDDR * self.lln(V_di_d * V_di_d + 1.0E-6)))
            Rdrain   =  self.rdstemp * (self.RDrainGeo + (self.RDWMIN_i + T5 * T0) * self.WeffWRFactor)

        elif self.RDSMOD == 0:
            Rsource  =  self.RSourceGeo
            Rdrain   =  self.RDrainGeo
            T4       =  1.0 + self.PRWGS_i * qia
            T1       =  1.0 / T4
            T0       =  0.5 * (T1 + math.sqrt(T1 * T1 + 0.01))
            Rdsi     =  self.rdstemp * (self.RDSWMIN_i + self.RDSW_i * T0) * self.WeffWRFactor
            Dr       =  1.0 + (self.NFINtotal) * beta * ids0_ov_dqi / (Dmob * Dvsat) * Rdsi

        elif self.RDSMOD == 2:
            T4       =  1.0 + self.PRWGS_i * qia
            T1       =  1.0 / T4
            T0       =  0.5 * (T1 + math.sqrt(T1 * T1 + 0.01))
            Rdsi     =  self.rdstemp * (self.RSourceGeo + self.RDrainGeo + self.RDSWMIN_i + self.RDSW_i * T0) * self.WeffWRFactor
            Dr       =  1.0 + (self.NFINtotal) * beta * ids0_ov_dqi / (Dmob * Dvsat) * Rdsi
            Rsource  =  0.0
            Rdrain   =  0.0

        ids  =  self.NFINtotal * beta * ids0 * Moc * Mnud * Mob / (Dmob * Dvsat * Dr)
        ids  =  ids * self.IDS0MULT

        # Quasi Static C-V Model
        # `include "bsimcmg_quasi_static_cv.include"

        T11 = (2.0*qia+nVtm)/DvsatCV # G
        qg  = qia+dqi*dqi/(6.0*T11) # qc
        qd  = -0.5*(qia-(dqi/(6.0))*(1.0-(dqi/T11)*(1+dqi/(5.0*T11)))) # qd
        # Including CLM in qg and qd
        inv_MclmCV = 1.0 / MclmCV
        qg         = inv_MclmCV * qg + (MclmCV - 1.0) * qid
        qd         = inv_MclmCV * inv_MclmCV * qd + 0.5 * (MclmCV - inv_MclmCV) * qid

        # Calculating partition for NQSMOD2
        if self.__NQSMOD2__:
            if(NQSMOD == 2):
                xdpart  =  qd / qg
            else:
                xdpart  =  0

        qs     = -qg-qd # from charge conservation qs = -qg-qd
        T6     = self.NFINtotal*self.WeffCV0 * self.LeffCV * self.coxeff

        qg     = T6*qg
        qd     = T6*qd
        qs     = T6*qs
        qinv    = qg

        if(self.BULKMOD != 0):
            T1  =  self.NFINtotal * self.WeffCV0 * self.LeffCV_acc * self.cox_acc
            T7  =  qi_acc_for_QM # qbulk
            T10 =  T7 * T1
            self.qg_acc =  - T10
            self.qb_acc =  T10
            T1 =  self.NFINtotal * self.WeffCV0 * self.LeffCV * self.cox
            T2 = self.qba - qi_acc_for_QM
            T10 = T1*T2
            self.qg_acc =  self.qg_acc - T10
            self.qb_acc =  self.qb_acc + T10
            T1 =  self.NFINtotal * self.WeffCV0 * self.LeffCV * self.cox
            T2 = (nq-1.0)*0.5*(qia+(dqi*dqi/(6.0*T11)))
            T10 = T1*T2
            self.qg_acc =  self.qg_acc - T10
            self.qb_acc =  self.qb_acc + T10

        # if vds is negative, physical charge on qd is qs
        if (sigvds < 0):
            T1  =  qd
            qd  =  qs
            qs  =  T1

        # Parasitic Capacitances
        # Bias-dependent overlap capacitances (self.CGEOMOD = 0 and 2)
        qgs_ov  =  0.0
        qgd_ov  =  0.0
        if (self.CGEOMOD != 1):
           T1           =  self.NFINtotal * self.WeffCV0 * self.devsign
           T2           =  self.devsign * V_GateEdgeNode_si
           T0           =  T2 - self.vfbsdcv + self.DELTA_1
           vgs_overlap  =  0.5 * (T0 - math.sqrt(T0 * T0 + 4.0 * self.DELTA_1))
           qgs_ov       =  T1 * (self.CGSL_i * (T2 - self.vfbsdcv - vgs_overlap - 0.5 * self.CKAPPAS_i * (math.sqrt(1.0 - 4.0 * vgs_overlap / self.CKAPPAS_i) - 1.0)) + self.CGSO_i * T2)
           T2           =  self.devsign * V_GateEdgeNode_di
           T0           =  T2 - self.vfbsdcv + self.DELTA_1
           vgd_overlap  =  0.5 * (T0 - math.sqrt(T0 * T0 + 4.0 * self.DELTA_1))
           qgd_ov       =  T1 * (self.CGDL_i * (T2 - self.vfbsdcv - vgd_overlap - 0.5 * self.CKAPPAD_i * (math.sqrt(1.0 - 4.0 * vgd_overlap / self.CKAPPAD_i) - 1.0)) + self.CGDO_i * T2)


        if (self.CGEOMOD == 0):
           T1      =  self.NFINtotal * self.WeffCV0      # Fringe caps dont see QM effects
           qgs_fr  =  T1 * self.CFS_i * V_GateEdgeNode_si
           qgd_fr  =  T1 * self.CFD_i * V_GateEdgeNode_di
           qgs_parasitic  =  qgs_ov + qgs_fr
           qgd_parasitic  =  qgd_ov + qgd_fr
        elif (self.CGEOMOD == 1):        # CGEO1SW=1 enables parameters to be in F per fin, per gate-finger, per unit channel width
           if (self.CGEO1SW == 1):
                T0      =  self.NFINtotal * self.WeffCV0
                COVS_i  =  T0 * COVS_i
                COVD_i  =  T0 * COVD_i
                cgsp    =  T0 * self.CGSP
                cgdp    =  T0 * self.CGDP
           else:
                cgsp    =  self.CGSP
                cgdp    =  self.CGDP

           qgs_ov         =  COVS_i * V_GateEdgeNode_si
           qgd_ov         =  COVD_i * V_GateEdgeNode_di
           qgs_parasitic  =  qgs_ov
           qgd_parasitic  =  qgd_ov
           qgs_fr         =  cgsp * V_GateEdgeNode_s
           qgd_fr         =  cgdp * V_GateEdgeNode_d
        else:
           qgs_fr         =  self.Cfr_geo * V_GateEdgeNode_si
           qgd_fr         =  self.Cfr_geo * V_GateEdgeNode_di
           qgs_parasitic  =  qgs_ov + qgs_fr
           qgd_parasitic  =  qgd_ov + qgd_fr


        # Drain-to-Source Fringe Capacitance Available for all CGEOMOD
        qds_fr  =  self.CDSP * V_d_s

        # Impact Ionization Current (Ref: self.IIMOD = 1 from BSIM4 Model, self.IIMOD = 2 from BSIMSOI Model)
        Iii  =  0.0
        if (self.IIMOD == 1):
            T0 =  (ALPHA0_t + self.ALPHA1_t * self.Leff) / self.Leff
            if ((T0 <= 0.0) or (BETA0_t <= 0.0)):
                Iii  =  0.0
            else:
                T1   =  -BETA0_t / (diffVds + 1.0e-30)
                Iii  =  T0 * diffVds * ids * self.lexp(T1)

        elif (self.IIMOD == 2):    #End of IIMOD=1
            ALPHAII  =  (ALPHAII0_t + ALPHAII1_t * self.Leff) / self.Leff
            if (ALPHAII <= 0.0):
                Iii  =  0.0
            else:
                T0       =  ESATII_i * self.Leff
                T1       =  SII0_t * T0 / (1.0 + T0)
                T0       =  1.0 / (1.0 + self.hypsmooth(SII1_i * vgsfbeff, self.IIMOD2CLAMP1)) # T0 = 1 / (1 + SII1_i * vgsfbeff)
                T3       =  T0 + SII2_i
                T2       =  self.hypsmooth(vgsfbeff * T3, self.IIMOD2CLAMP2)                   # T2 = vgsfbeff * T3
                T3       =  1.0 / (1.0 + SIID_i * vds)
                VgsStep  =  T1 * T2 * T3
                Vdsatii  =  VgsStep * (1.0 - LII_i / self.Leff)
                Vdiff    =  vds - Vdsatii
                T0       =  BETAII2_i + BETAII1_i * Vdiff + BETAII0_i * Vdiff * Vdiff
                T1       =  math.sqrt(T0 * T0 + 1.0e-10)
                Ratio    =  -self.hypmax( -ALPHAII * self.lexp(Vdiff / T1), -10.0, self.IIMOD2CLAMP3)
                Iii      =  Ratio * ids
        # End of IIMOD=2

        # Gate Current (Ref: BSIM4 Model)
        igbinv  =  0.0
        igbacc  =  0.0
        igcs    =  0.0
        igcd    =  0.0
        igs     =  0.0
        igd     =  0.0

        # Igb
        if (self.IGBMOD != 0):
           # Igbinv
           T1      =  (qia - self.EIGBINV_i) / self.NIGBINV_i / self.Vtm
           Vaux_Igbinv  =  self.NIGBINV_i * self.Vtm * self.lln(1.0 + self.lexp(T1))
           T2      =  self.AIGBINV_t - self.BIGBINV_i * qia
           T3      =  1.0 + self.CIGBINV_i * qia
           T4      =  -9.82222e11 * self.TOXG * T2 * T3
           T5      =  self.lexp(T4)
           T6      =  3.75956e-7
           igbinv  =  self.Weff0 * self.Leff * T6 * self.Toxratio * vge * Vaux_Igbinv * T5
           igbinv  =  igbinv * self.igtemp

           # Igbacc
           vfbzb  =  self.deltaPhi - (self.Eg / 2.0) - self.phib
           T0     =  vfbzb - vge
           T1     =  T0 / self.NIGBACC_i / self.Vtm
           Vaux_Igbacc  =  self.NIGBACC_i * self.Vtm * self.lln(1.0 + self.lexp(T1))
           if (self.BULKMOD != 0):
              Voxacc  =  qi_acc_for_QM
           else:
              if (vfbzb <= 0):
                  Voxacc  =  0.5 * (T0 - 0.02 + math.sqrt((T0 - 0.02) * (T0 - 0.02) - 0.08 * vfbzb))
              else:
                  Voxacc  =  0.5 * (T0 - 0.02 + math.sqrt((T0 - 0.02) * (T0 - 0.02) + 0.08 * vfbzb))

           T2      =  self.AIGBACC_t - self.BIGBACC_i * Voxacc
           T3      =  1.0 + self.CIGBACC_i * Voxacc
           T4      =  -7.45669e11 * self.TOXG * T2 * T3
           T5      =  self.lexp(T4)
           T6      =  4.97232e-7
           igbacc  =  self.Weff0 * self.Leff * T6 * self.Toxratio * vge * Vaux_Igbacc * T5
           igbacc  =  igbacc * self.igtemp

        if (self.IGCMOD != 0):
           # Igcinv
           T1    =  self.AIGC_t - self.BIGC_i * qia
           T2    =  1.0 + self.CIGC_i * qia
           T3    =  -self.Bechvb * self.TOXG * T1 * T2
           T4    =  qia * self.lexp(T3)
           T5    =  (vge + 0.5 * vdsx + 0.5 * (ves_jct + ved_jct))
           igc0  =  self.Weff0 * self.Leff * self.Aechvb * self.Toxratio * T4 * T5 * self.igtemp

           # Gate-Current Partitioning
           Vdseffx  =  math.sqrt(Vdseff * Vdseff + 0.01) - 0.1
           T1       =  self.PIGCD_i * Vdseffx
           T1_exp   =  self.lexp(-T1)
           T3       =  T1 + T1_exp - 1.0 + 1.0e-4
           T4       =  1.0 - (T1 + 1.0) * T1_exp + 1.0e-4
           T5       =  T1 * T1 + 2.0e-4
           igcd     =  igc0 * T4 / T5
           igcs     =  igc0 * T3 / T5

           # Igs
           T0       =  vgs_noswap - self.vfbsd
           vgs_eff  =  math.sqrt(T0 * T0 + 1.0e-4)
           if (self.IGCLAMP == 1):
               T1  =  self.hypsmooth((self.AIGS_t - self.BIGS_i * vgs_eff), 1.0e-6)
               if (self.CIGS_i < 0.01):
                   self.CIGS_i = 0.01

           else:
               T1  =  self.AIGS_t - self.BIGS_i * vgs_eff

           T2  =  1.0 + self.CIGS_i * vgs_eff
           T3  =  -self.Bechvb * self.TOXG * self.POXEDGE_i * T1 * T2
           T4  =  self.lexp(T3)
           if (sigvds > 0.0):
              igs  =  self.igsd_mult * self.DLCIGS * vgs_noswap * vgs_eff * T4
           else:
              igd  =  self.igsd_mult * self.DLCIGS * vgs_noswap * vgs_eff * T4


           # Igd
           T0       =  vgd_noswap - self.vfbsd
           vgd_eff  =  math.sqrt(T0 * T0 + 1.0e-4)
           if (self.IGCLAMP == 1):
               T1  =  self.hypsmooth((self.AIGD_t - self.BIGD_i * vgd_eff), 1.0e-6)
               if (self.CIGD_i < 0.01):
                   self.CIGD_i = 0.01

           else:
               T1  =  self.AIGD_t - self.BIGD_i * vgd_eff

           T2  =  1.0 + self.CIGD_i * vgd_eff
           T3  =  -self.Bechvb * self.TOXG * self.POXEDGE_i * T1 * T2
           T4  =  self.lexp(T3)

           if (sigvds > 0.0):
              igd  =  self.igsd_mult * self.DLCIGD * vgd_noswap * vgd_eff * T4
           else:
              igs  =  self.igsd_mult * self.DLCIGD * vgd_noswap * vgd_eff * T4

        # GIDL/GISL Current (Ref: BSIM4 Model)
        igisl  =  0.0
        igidl  =  0.0

        if (self.GIDLMOD != 0):
           T0  =  epsratio * self.EOT
           # GIDL
           if ((AGIDL_i <= 0.0) or (BGIDL_t <= 0.0)):
              T6  =  0.0
           else:
              T1  =  (-vgd_noswap - EGIDL_i + self.vfbsd) / T0
              T1  =  self.hypsmooth(T1, 1.0e-2)
              T2  =  BGIDL_t / (T1 + 1.0e-3)
              T3  =  self.lexp(PGIDL_i * self.lln(T1))
              if (self.BULKMOD != 0):
                  T4   =  -ved_jct*ved_jct*ved_jct
                  T4a  =  CGIDL_i + abs(T4) + 1.0e-5
                  T5   =  self.hypsmooth(T4/T4a, 1.0e-6) - 1.0e-6
                  T6   =  AGIDL_i * self.Weff0 * T3 * self.lexp(-T2) * T5
              else:
                  T6   =  AGIDL_i * self.Weff0 * T3 * self.lexp(-T2) * vds_noswap

           if (sigvds > 0.0):
              igidl  =  T6
           else:
              igisl  =  T6

           # GISL
           if ((AGISL_i <= 0.0) or (BGISL_t <= 0.0)):
              T6  =  0.0
           else:
              T1  =  (-vgs_noswap - EGISL_i + self.vfbsd) / T0
              T1  =  self.hypsmooth(T1, 1.0e-2)
              T2  =  BGISL_t / (T1 + 1.0e-3)
              T3  =  self.lexp(PGISL_i * self.lln(T1))
              if (self.BULKMOD != 0):
                  T4   =  -ves_jct * ves_jct * ves_jct
                  T4a  =  CGISL_i + abs(T4) + 1.0e-5
                  T5   =  self.hypsmooth(T4/T4a, 1.0e-6) - 1.0e-6
                  T6   =  AGISL_i * self.Weff0 * T3 * self.lexp(-T2) * T5
              else:
                  T6   =  AGISL_i * self.Weff0 * T3 * self.lexp(-T2) * (-vds_noswap)

           if (sigvds > 0.0):
              igisl  =  T6
           else:
              igidl  =  T6
          # End of GIDLMOD

        # Junction Current
        if (self.BULKMOD != 0):
           # Source-Side Junction Current
           if (self.Isbs > 0.0):
              if (ves_jct < VjsmRev):
                  T0   =  ves_jct / Nvtms
                  T1   =  self.lexp(T0) - 1.0
                  T2   =  IVjsmRev + SslpRev * (ves_jct - VjsmRev)
                  Ies  =  T1 * T2
              elif (ves_jct <= VjsmFwd):
                  T0   =  ves_jct / Nvtms
                  T1   =  (self.BVS + ves_jct) / Nvtms
                  T2   =  self.lexp(-T1)
                  Ies  =  self.Isbs * (self.lexp(T0) + XExpBVS - 1.0 - self.XJBVS * T2)
              else:
                  Ies  =  IVjsmFwd + SslpFwd * (ves_jct - VjsmFwd)

           else:
              Ies  =  0.0


           # Source-Side Junction Tunneling Current
           if (self.JTSS_t > 0.0):
              if ((self.VTSS - ves_jct) < (self.VTSS * 1.0e-3)):
                  T0   =  -ves_jct / Vtm0 / NJTS_t
                  T1   =  self.lexp(T0 * 1.0e3) - 1.0
                  Ies  =  Ies - self.ASEJ * self.JTSS_t * T1
              else:
                  T0   =  -ves_jct / Vtm0 / NJTS_t
                  T1   =  self.lexp(T0 * self.VTSS / (self.VTSS - ves_jct)) - 1.0
                  Ies  =  Ies - self.ASEJ * self.JTSS_t * T1



           if (self.JTSSWS_t > 0.0):
              if ((self.VTSSWS - ves_jct) < (self.VTSSWS * 1.0e-3)):
                  T0   =  -ves_jct / Vtm0 / NJTSSW_t
                  T1   =  self.lexp(T0 * 1.0e3) - 1.0
                  Ies  =  Ies - self.PSEJ * self.JTSSWS_t * T1
              else:
                  T0   =  -ves_jct / Vtm0 / NJTSSW_t
                  T1   =  self.lexp(T0 * self.VTSSWS / (self.VTSSWS - ves_jct)) - 1.0
                  Ies  =  Ies - self.PSEJ * self.JTSSWS_t * T1



           if (self.JTSSWGS_t > 0.0):
              if ((self.VTSSWGS - ves_jct) < (self.VTSSWGS * 1.0e-3)):
                  T0   =  -ves_jct / Vtm0 / NJTSSWG_t
                  T1   =  self.lexp(T0 * 1.0e3) - 1.0
                  Ies  =  Ies - self.Weff0 * self.NFINtotal * self.JTSSWGS_t * T1
              else:
                  T0   =  -ves_jct / Vtm0 / NJTSSWG_t
                  T1   =  self.lexp(T0 * self.VTSSWGS / (self.VTSSWGS - ves_jct)) - 1.0
                  Ies  =  Ies - self.Weff0 * self.NFINtotal * self.JTSSWGS_t * T1

           # Drain-Side Junction Current
           if (self.Isbd > 0.0):
               if (ved_jct < VjdmRev):
                   T0   =  ved_jct / Nvtmd
                   T1   =  self.lexp(T0) - 1.0
                   T2   =  IVjdmRev + DslpRev * (ved_jct - VjdmRev)
                   Ied  =  T1 * T2
               elif (ved_jct <= VjdmFwd):
                   T0   =  ved_jct / Nvtmd
                   T1   =  (self.BVD + ved_jct) / Nvtmd
                   T2   =  self.lexp(-T1)
                   Ied  =  self.Isbd * (self.lexp(T0) + XExpBVD - 1.0 - self.XJBVD * T2)
               else:
                   Ied  =  IVjdmFwd + DslpFwd * (ved_jct - VjdmFwd)
           else:
               Ied  =  0.0

           # Drain-Side Junction Tunneling Current
           if (self.JTSD_t > 0.0):
              if ((self.VTSD - ved_jct) < (self.VTSD * 1.0e-3)):
                  T0 =  -ved_jct / Vtm0 / self.NJTSD_t
                  T1 =  self.lexp(T0 * 1.0e3) - 1.0
                  Ied  =  Ied - self.ADEJ * self.JTSD_t * T1
              else:
                  T0   =  -ved_jct / Vtm0 / self.NJTSD_t
                  T1   =  self.lexp(T0 * VTSD/ (self.VTSD - ved_jct)) - 1.0
                  Ied  =  Ied - self.ADEJ * self.JTSD_t * T1


           if (self.JTSSWD_t > 0.0):
              if ((self.VTSSWD - ved_jct) < (self.VTSSWD * 1.0e-3)):
                  T0   =  -ved_jct / Vtm0 / Nself.JTSSWD_t
                  T1   =  self.lexp(T0 * 1.0e3) - 1.0
                  Ied  =  Ied - self.PDEJ * self.JTSSWD_t * T1
              else:
                  T0   =  -ved_jct / Vtm0 / Nself.JTSSWD_t
                  T1   =  self.lexp(T0 * self.VTSSWD / (self.VTSSWD - ved_jct)) - 1.0
                  Ied  =  Ied - self.PDEJ * self.JTSSWD_t * T1


           if (self.JTSSWGD_t > 0.0):
              if ((self.VTSSWGD - ved_jct) < (self.VTSSWGD * 1.0e-3)):
                  T0   =  -ved_jct / Vtm0 / NJTSSWGD_t
                  T1   =  self.lexp(T0 * 1.0e3) - 1.0
                  Ied  =  Ied - self.Weff0 * self.NFINtotal * self.JTSSWGD_t * T1
              else:
                  T0   =  -ved_jct / Vtm0 / NJTSSWGD_t
                  T1   =  self.lexp(T0 * self.VTSSWGD / (self.VTSSWGD - ved_jct)) - 1.0
                  Ied  =  Ied - self.Weff0 * self.NFINtotal * self.JTSSWGD_t * T1

           # Junction Capacitance (No Swapping)
           # Source-Substrate Junction
           Qesj1 = self.BSIM6JunctnCap(ves_jct, self.Czbs, self.PBS_t, self.SJS, self.MJS, self.MJS2)
           Qesj2 = self.BSIM6JunctnCap(ves_jct, self.Czbssw, self.PBSWS_t, self.SJSWS, self.MJSWS, self.MJSWS2)
           Qesj3 = self.BSIM6JunctnCap(ves_jct, self.Czbsswg, self.PBSWGS_t, self.SJSWGS, self.MJSWGS, self.MJSWGS2)
           Qesj  =  Qesj1 + Qesj2 + Qesj3

           # Drain-Substrate Junction
           Qedj1 = self.BSIM6JunctnCap(ved_jct, self.Czbd, self.PBD_t, self.SJD, self.MJD, self.MJD2)
           Qedj2 = self.BSIM6JunctnCap(ved_jct, self.Czbdsw, self.PBSWD_t, self.SJSWD, self.MJSWD, self.MJSWD2)
           Qedj3 = self.BSIM6JunctnCap(ved_jct, self.Czbdswg, self.PBSWGD_t, self.SJSWGD, self.MJSWGD, self.MJSWGD2)
           Qedj  =  Qedj1 + Qedj2 + Qedj3

        # BULKMOD=0

        Qes  =  Qesj + self.csbox * ves_jct
        Qed  =  Qedj + self.cdbox * ved_jct

        # Gate-to-Substrate Parasitic Capacitance
        # Bias Independent Component
        Qeg  =  self.cgbox * self.devsign * V_e_GateEdgeNode
        if (self.BULKMOD != 0):
           # Bias Dependent Component
           T2           =  self.devsign * V_GateEdgeNode_e
           T3           =  T2 - self.deltaPhi + self.Eg / 2.0 + self.phib - self.DELVFBACC
           T0           =  T3 + self.DELTA_1
           vge_overlap  =  0.5 * (T0 + math.sqrt(T0 * T0 + 4.0 * self.DELTA_1))
           Qeg          =  Qeg - self.NFINtotal * self.LeffCV * (self.CGBL_i * (T3 - vge_overlap + 0.5 * self.CKAPPAB_i * ( math.sqrt(1.0 + 4.0 * vge_overlap / self.CKAPPAB_i) - 1.0 )))


        # Generation-Recombination Component
        T0      =  vds
        T1      =  T0 * (self.AIGEN_i + self.BIGEN_i * T0 * T0)
        idsgen  =  self.HFIN * self.TFIN * (self.Leff - 2.0 * self.LINTIGEN_i) * self.igentemp * T1

        # NQS Gate Resistance (Ref: BSIM4 Model)
        T0      =  ueff * self.coxeff * self.Weff0 / self.Leff

        if self.__NQSMOD1__:
           if (self.NQSMOD == 1 and XRCRG1_i != 0):
              IdovVds  =  beta * ids0_ov_dqi * Moc / (Dmob * Dvsat * Dr)
              gcrg     =  self.NFINtotal * XRCRG1_i * (IdovVds + XRCRG2_i * self.Vtm * T0)

        if self.__NQSMOD2__:
           if (self.NQSMOD == 2):
              IdovVds  =  beta * ids0_ov_dqi * Moc / (Dmob * Dvsat * Dr)
              gcrg     =  self.NFINtotal * XRCRG1_i * (IdovVds + XRCRG2_i * self.Vtm * T0)
              gtau     =  gcrg / (self.cox * self.Weff0 * self.Leff)

        # *** Multiply all current and charge components by self.NFINtotal ***
        # Note: Do not multiply ids, qg, qs, qd, qb, Ies, Ied, Qbs, Qbd with self.NFINtotal
        #       since it is already considered.

        igidl   =  self.NFINtotal * igidl
        igisl   =  self.NFINtotal * igisl
        igcd    =  self.NFINtotal * igcd
        igcs    =  self.NFINtotal * igcs
        igs     =  self.NFINtotal * igs
        igd     =  self.NFINtotal * igd
        igbinv  =  self.NFINtotal * igbinv
        igbacc  =  self.NFINtotal * igbacc
        idsgen  =  self.NFINtotal * idsgen

        # Gate to Body Tunneling Current Empirical Partition for self.BULKMOD = 0
        igbs  =  0.0
        igbd  =  0.0
        if (self.BULKMOD == 0):
           igbs  =  (igbinv + igbacc) * wf
           igbd  =  (igbinv + igbacc) * wr


        # Noise Models
        Esatnoi  =  2.0 * VSAT_a / ueff  # Thermal noise and flicker noise

        # Flicker Noise (Ref: BSIM4 Model from K. K. Hung et al. TED 1990)
        if (self.NOIA > 0.0 or self.NOIB > 0.0 or self.NOIC > 0.0):
           self.Leffnoi    =  self.Leff - 2.0 * self.LINTNOI_i
           self.Leffnoisq  =  self.Leffnoi * self.Leffnoi
           if (self.EM <= 0.0):
              DelClm  =  0.0
           else:
              T0      =  (diffVds / self.litl + self.EM) / Esatnoi
              DelClm  =  self.litl * self.lln(T0)
              if (DelClm < 0.0):
                 DelClm  =  0.0


           T1     =  self.q * self.q * self.q * self.Vtm * abs(ids) * ueff
           T2     =  1.0e10 * self.coxeff * self.Leffnoisq
           N0     =  self.coxeff * qis / self.q
           Nl     =  self.coxeff * qid / self.q
           Nstar  =  self.Vtm / self.q * (self.coxeff + CIT_a)
           T3     =  self.NOIA * self.lln((N0 + Nstar) / (Nl + Nstar))
           T4     =  self.NOIB * (N0 - Nl)
           T5     =  0.5 * self.NOIC * (N0 * N0 - Nl * Nl)
           T6     =  self.q * self.Vtm * ids * ids
           T7     =  1.0e10 * self.Leffnoisq * self.Weff0 * self.NFINtotal
           T8     =  self.NOIA + self.NOIB * Nl + self.NOIC * Nl * Nl
           T9     =  (Nl + Nstar) * (Nl + Nstar)
           Ssi    =  T1 / T2 * (T3 + T4 + T5) + T6 / T7 * DelClm * T8 / T9
           T10    =  self.NOIA * self.q * self.Vtm
           T11    =  self.Weff0 * self.NFINtotal * self.Leffnoi * 1.0e10 * Nstar * Nstar
           Swi    =  T10 / T11 * ids * ids
           T1     =  Swi + Ssi
           if (T1 > 0.0):
              FNPowerAt1Hz  =  (Ssi * Swi) / T1
           else:
              FNPowerAt1Hz  =  0.0

        else:
           FNPowerAt1Hz  =  0.0


        # Thermal Noise
        if self.TNOIMOD == 0: # Charge-based model (BSIM4 - TNOIMOD=0)
            T0     =  ueff * qinv
            T1     =  T0 * Rdsi + self.Leff * self.Leff
            Gtnoi  =  (T0 / T1) * self.NTNOI_i
            sid    =  4.0 * self.Vtm * self.q * Gtnoi

        elif self.TNOIMOD == 1: # Correlated Thermal Noise by Navid, November 2013, Reference BSIMSOI4.5.0
            if self.TNOIMOD1:
                Abulk    =  1.0
                Vgst2self.Vtm =  KSATIV_a * (qis + 2.0 * self.Vtm)
                etaa     =  1.0 - Vdseff * Abulk / Vgst2self.Vtm
                T0       =  1.0 - etaa
                T1       =  1.0 + etaa
                T2       =  T1 + 2.0 * Abulk * self.Vtm / (qia + 1.0e-10)
                T3       =  T2 * T2
                T4       =  T0 * T0
                T5       =  T3 * T3
                T6       =  1.0 / (1.0 + Vdseff / EsatL)
                gamma    =  T6 * (0.5 * T1 + T0 * T0 / (6.0 * T2))
                delta    =  ((T1 / T3) - (5.0 * T1 + T2) * T4 / (15.0 * T5) + T4 * T4 / (9.0 * T5 * T2)) / (6.0 * T6 * T6 * T6)
                T7       =  T0 / T2
                epsilon  =  (T7 + T7 * T7 * T7 / 3.0) / (6.0 * T6)
                T8       =  qia / EsatL
                T8       =  T8 * T8
                npart_c  =  self.RNOIC * (1.0 + T8 * self.TNOIC * self.Leff)
                T9       =  gamma * delta
                if (T9 > 0.0):
                    ctnoi  =  epsilon / math.sqrt( gamma * delta) * (2.5316 * npart_c)
                else:
                    ctnoi  =  1.0

                if (ctnoi > 1):
                    ctnoi = 1.0

                if (ctnoi < 0):
                  ctnoi = 0.0

                npart_beta  =  self.RNOIA * (1.0 + T8 * self.TNOIA * self.Leff)
                npart_theta =  self.RNOIB * (1.0 + T8 * self.TNOIB * self.Leff)
                gamma       =  gamma * (3.0 * npart_beta * npart_beta)
                delta       =  delta * (3.75 * npart_theta * npart_theta)
                T9          =  qia * 0.5 * T1
                gche        =  beta * T9 * T6
                noiGd0      =  self.NFINtotal * beta * qia / (1.0 + gche * Rdsi)
                GammaGd0    =  gamma * noiGd0
                sid         =  4.0 * self.Vtm * self.q * GammaGd0
                C0          = self.NFINtotal * self.coxeff * self.WeffCV0 * self.LeffCV
                if (gamma > 0.0 and delta > 0.0):
                    sf  =  (noiGd0 + 1.0e-15) / math.sqrt(delta / gamma)
                else:
                    sf  =  0.0
            else:
                print("[Warning!] Although the model selector self.TNOIMOD is set to 1, the new correlated thermal noise model is not activated. \
                Please uncomment \"`define TNOIMOD1\" in the bsimcmg.va.")

        # Source and Drain Conductance for Thermal Noise Contribution
        if (self.RDSMOD != 2):
            gspr  =  1.0 / Rsource    # Note: gspr considers all fins
            gdpr  =  1.0 / Rdrain     # Note: gdpr considers all fins


        # Loading Ids, Gate and Drain charges
        if self.__NQSMOD2__:
            if (sigvds > 0.0):
                I_di_si += self.devsign * ids
            else:
                I_si_di += self.devsign * ids

            if (self.NQSMOD == 2):
                I_IntrinsicGate_si += self.devsign * gtau * -V(q)
                I_di_si            += self.devsign * xdpart * gtau * V(q)
            else:  # Quasi-static Stamping (Normal case)
                I_di_si            += self.devsign * ddt(qd)
                I_IntrinsicGate_si += self.devsign * ddt(qg)

        else:
            if (sigvds > 0.0):
                I_di_si += self.devsign * ids
            else:
                I_si_di += self.devsign * ids

            I_di_si += self.devsign * self.ddt(qd, self.qd_step, delta_t)
            I_IntrinsicGate_si += self.devsign * self.ddt(qg, self.qg_step, delta_t)

        # Loading Other Currents
        if (sigvds > 0.0):
           I_di_si += self.devsign * idsgen
           I_IntrinsicGate_si += self.devsign * (igcs + igs)
           I_IntrinsicGate_di += self.devsign * (igcd + igd)
           if (self.BULKMOD != 0):
              I_di_e += self.devsign * (igidl + Iii)
              I_si_e += self.devsign * igisl
              I_IntrinsicGate_e += self.devsign * (igbinv + igbacc)
           else:
              I_di_si += self.devsign * (igidl + Iii)
              I_si_di += self.devsign * igisl

        else:
           I_si_di += self.devsign * idsgen
           I_IntrinsicGate_di += self.devsign * (igcs + igs)
           I_IntrinsicGate_si += self.devsign * (igcd + igd)
           if (self.BULKMOD != 0):
              I_si_e += self.devsign * (igidl + Iii)
              I_di_e += self.devsign * igisl
              I_IntrinsicGate_e += self.devsign * (igbinv + igbacc)
           else:
              I_si_di += self.devsign * (igidl + Iii)
              I_di_si += self.devsign * igisl


        if (self.BULKMOD == 0):
           I_IntrinsicGate_si += self.devsign * igbs
           I_IntrinsicGate_di += self.devsign * igbd


        if (self.BULKMOD != 0):
           I_e_si += self.devsign * Ies
           I_e_di += self.devsign * Ied

        I_e_si += self.devsign * self.ddt(Qes, self.Qes_step, delta_t)
        I_e_di += self.devsign * self.ddt(Qed, self.Qed_step, delta_t)
        I_e_GateEdgeNode += self.devsign * self.ddt(Qeg, self.Qeg_step, delta_t)

        # Loading other charges
        I_GateEdgeNode_si += self.ddt(qgs_parasitic, self.qgs_parasitic_step, delta_t)
        I_GateEdgeNode_di += self.ddt(qgd_parasitic, self.qgd_parasitic_step, delta_t)
        I_d_s += self.ddt(qds_fr, self.qds_fr_step, delta_t)
        if (self.CGEOMOD == 1):
           I_GateEdgeNode_s += ddt(qgs_fr)
           I_GateEdgeNode_d += ddt(qgd_fr)


        # Accumulation Charge for Bulk FET
        if (self.BULKMOD != 0):
            I_IntrinsicGate_si += self.devsign * self.ddt(self.qg_acc, self.qg_acc_step, delta_t)
            I_e_si             += self.devsign * self.ddt(self.qb_acc, self.qb_acc_step, delta_t)


        # External S/D Resistance
        if (self.RDSMOD == 2):
            #V(d, di) += 0.0
            #V(s, si) += 0.0
            print ("revisit")
        else:
            I_d_di += V_d_di / Rdrain
            I_s_si += V_s_si / Rsource


        # NQSMOD1 Gate Resistance Model
        if self.__NQSMOD1__:
            if (self.NQSMOD == 1 and XRCRG1_i != 0):
                I_GateEdgeNode_gi += V_GateEdgeNode_gi * gcrg
            else:
                V_GateEdgeNode_gi += 0.0

        # NQSMOD2 BSIM4 Charge Deficit Model
        if self.__NQSMOD2__:
            if (self.NQSMOD ==2):
                I_q_gnd += ddt(qg - qb)
                I_q_gnd += V(q) * gtau
                I_q_gnd += ddt(V(q))
            else:
                V_q_gnd += 0.0

        # Gate Electrode Resistance
        if self.__RGATEMOD__:
           if (self.RGATEMOD != 0):
               I_g_ge += V_g_ge * self.ggeltd
           else:
               V_g_ge += 0.0

        # Flicker Noise
        I_di_si += 0.0 # REVISIT flicker_noise(FNPowerAt1Hz, self.EF, "flicker")

        # Thermal noise for parasitics
        if (self.RDSMOD != 2):
           I_d_di += 0.0 # REVISIT white_noise(4.0 * self.Vtm * self.q * gdpr, "thermal")
           I_s_si += 0.0 # REVISIT white_noise(4.0 * self.Vtm * self.q * gspr, "thermal")


        if self.__RGATEMOD__:
            if (self.RGATEMOD != 0):
                I_g_ge +=  0.0 # REVISIT white_noise(4.0 * self.Vtm * self.q * self.ggeltd, "thermal")

        # Channel thermal noise and induced gate noise stamping
        # Implementation of correlated noise follows C. C. McAndrew, WCM 2005
        if (self.TNOIMOD == 0):
           I_di_si += 0.0 # REVISIT white_noise(sid, "thermal")
           if self.__TNOIMOD1__:
               V_N_gnd = 0.0
        else:
           if self.__TNOIMOD1__:
               I_di_si += white_noise(sid * abs(1.0 - ctnoi * ctnoi), "thermal")
               I_di_si += ctnoi * V_N_gnd * sf * self.SCALEN
               if (gamma > 0 and delta > 0):
                   I_N_gnd += V_N_gnd * sf * self.SCALEN
                   I_N_gnd += white_noise(sid/(sf*sf*SCALEN*SCALEN))
               else:
                   I_N_gnd += V_N_gnd

               I_IntrinsicGate_si += ddt(0.5 * C0 * self.SCALEN * V_N_gnd)
               I_IntrinsicGate_di += ddt(0.5 * C0 * self.SCALEN * V_N_gnd)
           else:
              print("[Warning!] Although the model selector self.TNOIMOD is set to 1, the new correlated thermal noise model is not activated. \
              Please uncomment \"`define TNOIMOD1\" in the bsimcmg.va.")

        # Gate Current Shot Noise
        if (self.IGCMOD != 0):
           if (sigvds > 0):
              I_IntrinsicGate_si += 0.0 #REVISIT white_noise(2.0 * self.q * abs(igcs + igs), "shot")
              I_IntrinsicGate_di += 0.0 #REVISIT white_noise(2.0 * self.q * abs(igcd + igd), "shot")
           else:
              I_IntrinsicGate_di += 0.0 #REVISIT white_noise(2.0 * self.q * abs(igcs + igs), "shot")
              I_IntrinsicGate_si += 0.0 #REVISIT white_noise(2.0 * self.q * abs(igcd + igd), "shot")

        if (self.IGBMOD != 0):
           if (self.BULKMOD != 0):
              I_IntrinsicGate_e  +=  0.0 #REVISIT white_noise(2.0 * self.q * abs(igbinv + igbacc), "shot")
           else:
              I_IntrinsicGate_si +=  0.0 #REVISIT white_noise(2.0 * self.q * abs(igbs), "shot")
              I_IntrinsicGate_di +=  0.0 #REVISIT white_noise(2.0 * self.q * abs(igbd), "shot")

        # Charge state
        self.qd_current            = qd
        self.qg_current            = qg
        self.Qes_current           = Qes
        self.Qed_current           = Qed
        self.Qeg_current           = Qeg
        self.qgs_parasitic_current = qgs_parasitic
        self.qgd_parasitic_current = qgd_parasitic
        self.qds_fr_current        = qds_fr
        self.qg_acc_current        = self.qg_acc
        self.qb_acc_current        = self.qb_acc

        # Current state
        self.I_IntrinsicGate_si = I_IntrinsicGate_si
        self.I_di_si            = I_di_si
        self.I_si_di            = I_si_di
        self.I_d_s              = I_d_s
        self.I_d_di             = I_d_di
        self.I_s_si             = I_s_si
        self.I_IntrinsicGate_di = I_IntrinsicGate_di
        self.I_e_si             = I_e_si
        self.I_e_di             = I_e_di
        self.I_IntrinsicGate_e  = I_IntrinsicGate_e
        self.I_GateEdgeNode_si  = I_GateEdgeNode_si
        self.I_GateEdgeNode_di  = I_GateEdgeNode_di

        if (self.CGEOMOD == 1):
            self.I_GateEdgeNode_s = I_GateEdgeNode_s
            self.I_GateEdgeNode_d = I_GateEdgeNode_d

        self.I_e_GateEdgeNode = I_e_GateEdgeNode

        if self.__RGATEMOD__:
            self.I_g_ge = I_g_ge

        # Update edge currents
        edge_currents[self.V_IntrinsicGate_si_ref] = I_IntrinsicGate_si
        edge_currents[self.V_di_si_ref]            = I_di_si
        edge_currents[self.V_si_di_ref]            = I_si_di
        edge_currents[self.V_d_s_ref]              = I_d_s
        edge_currents[self.V_d_di_ref]             = I_d_di
        edge_currents[self.V_s_si_ref]             = I_s_si
        edge_currents[self.V_IntrinsicGate_di_ref] = I_IntrinsicGate_di
        edge_currents[self.V_e_si_ref]             = I_e_si
        edge_currents[self.V_e_di_ref]             = I_e_di
        edge_currents[self.V_IntrinsicGate_e_ref]  = I_IntrinsicGate_e
        edge_currents[self.V_GateEdgeNode_si_ref]  = I_GateEdgeNode_si
        edge_currents[self.V_GateEdgeNode_di_ref]  = I_GateEdgeNode_di

        if (self.CGEOMOD == 1):
            edge_currents[self.V_GateEdgeNode_s_ref] = I_GateEdgeNode_s
            edge_currents[self.V_GateEdgeNode_d_ref] = I_GateEdgeNode_d

        edge_currents[self.V_e_GateEdgeNode_ref] = I_e_GateEdgeNode

        if self.__RGATEMOD__:
            edge_currents[self.V_g_ge_ref] = I_g_ge

        return edge_currents

        # Self Heating (REVISIT)
        #if self.__SHMOD__:
        #   if (self.SHMOD != 0 and self.RTH0 > 0):
        #         if (self.RDSMOD != 2):
        #             Pwr(ith_branch) += -(self.devsign * sigvds * V_di_si * ids + V_d_di * V_d_di / Rdrain + V_s_si * V_s_si / Rsource)
        #         else:
        #             Pwr(ith_branch) += -(self.devsign * sigvds * V_di_si * ids )
        #
        #
        #   Pwr(rth_branch) += Temp(rth_branch) * gth
        #   Pwr(rth_branch) += ddt(Temp(rth_branch) * cth)

        # Operating-Point information
#if self.__OPINFO__:
#   # W & L
#   WEFF    =  self.Weff0                            # Effective width for IV
#   LEFF    =  self.Leff                             # Effective length for IV
#   WEFFCV  =  self.WeffCV0                          # Effective width for CV
#   LEFFCV  =  self.LeffCV                           # Effective length for CV
#
#   # Currents
#   IDS     =  self.devsign * ids                    # Intrinsic Drain Current (Electrical)
#   if (sigvds > 0):                        # Total Source/Drain Currents (Physical)
#      if (self.BULKMOD != 0):
#         IDEFF  =  IDS + self.devsign * idsgen - self.devsign * (igd + igcd) + self.devsign * (Iii + igidl) - self.devsign * Ied
#         ISEFF  =  -IDS - self.devsign * idsgen - self.devsign * (igs + igcs) + self.devsign * (igisl) - self.devsign * Ies
#      else:
#         IDEFF  =  IDS + self.devsign * idsgen - self.devsign * (igd + igcd + igbd) + self.devsign * (Iii + igidl - igisl)
#         ISEFF  =  -IDS - self.devsign * idsgen - self.devsign * (igs + igcs + igbs) + self.devsign* (igisl - igidl)
#
#   else:
#      if (self.BULKMOD != 0):
#         IDEFF  =  -IDS - self.devsign * idsgen - self.devsign * (igs + igcs) + self.devsign * (igisl) - self.devsign * Ied
#         ISEFF  =  IDS + self.devsign * idsgen - self.devsign * (igd + igcd) + self.devsign * (Iii + igidl) -  self.devsign * Ies
#      else:
#         IDEFF  =  -IDS - self.devsign * idsgen - self.devsign * (igs + igcs + igbd) + self.devsign * (igisl - igidl)
#         ISEFF  =  IDS + self.devsign * idsgen - self.devsign * (igd + igcd + igbs) + self.devsign * (Iii + igidl - igisl)
#
#
#
#   if (self.BULKMOD == 0):                      # Total Gate Current
#      IGTOT  =  self.devsign * (igs + igd + igcs + igcd + igbs + igbd)
#   else:
#      IGTOT  =  self.devsign * (igs + igd + igcs + igcd + igbacc + igbinv)
#
#
#   IDSGEN =  sigvds * self.devsign * idsgen         # Generation-Recombination Current (Physical)
#   III    =  self.devsign * Iii                     # Impact Ionization Current
#   if (sigvds > 0):
#      IGIDL  =  self.devsign * igidl                # GIDL Current (Physical)
#      IGISL  =  self.devsign * igisl                # GISL Current (Physical)
#   else:
#      IGIDL  =  self.devsign * igisl                # GIDL Current (Physical)
#      IGISL  =  self.devsign * igidl                # GISL Current (Physical)
#
#
#   if (self.BULKMOD != 0):
#      IJSB  =  -self.devsign * Ies                 # Source-Body Junction Current (Physical)
#      IJDB  =  -self.devsign * Ied                 # Drain-Body Junction Current (Physical)
#   else:
#      IJSB  =  0.0
#      IJDB  =  0.0
#
#
#   if (self.BULKMOD != 0):
#      ISUB  =  -III - IGIDL - IGISL - IJSB - IJDB - self.devsign * (igbinv + igbacc)      # Substrate Current
#   else:
#      ISUB  =  0.0
#
#
#   # Misc Variables
#   BETA    =  beta                               # Drain Current prefactor per fin per finger
#   VDSSAT  =  Vdsat                              # Drain-Source saturation Voltage
#   if (NGATE_i > 0):                             # Flatband Voltage
#       VFB  =  -self.devsign * (self.phib + self.Vtm * self.lln(NGATE_i / ni))
#   else:
#       VFB  =  PHIG_i - (self.EASUB + 0.5 * self.Eg + self.devsign * self.phib)
#
#   # Threshold Voltage Calculation
#   q0   =  10.0 * self.Vtm / self.rc + 2.0 * self.qbs
#   T1   =  self.Vtm * (self.Vtm + q0)
#   T2   =  self.cox * self.cox * T1
#   T3   =  2.0 * self.q * ni * self.epssub * self.Vtm
#   VTH  =  VFB + self.devsign * (self.Vtm * self.lln(T2 / T3) + self.dvch_qm + self.phib + self.qbs + self.Vtm + dvth_all - self.DELVTRAND)
#
#   # Conductances
#   GM   =  ddx(IDS,V(`IntrinsicGate))      # Transconductance
#   GDS  =  ddx(IDS,V(di))                  # Output Conductance
#   if (self.BULKMOD != 0)
#      GMBS  =  ddx(IDS,V(e))               # Body Transconductance
#   else
#      GMBS  =  0.0
#
#   # Intrinsic Charges (Physical) (Sriram: Not accurate for NQSMOD= 2 and 3)
#   QGI  =  self.devsign * qg + self.devsign * qg_acc
#   QDI  =  self.devsign * qd
#   QSI  =  self.devsign * qs
#   QBI  =  self.devsign * (qb + qb_acc)
#
#   # Total Charges (Sriram: Not accurate for NQSMOD= 2 and 3)
#   QG  =  self.devsign * qg + qgs_parasitic + qgd_parasitic + (self.CGEOMOD == 1 ? qgs_fr + qgd_fr : 0) + self.devsign * qg_acc - self.devsign * Qeg
#   QD  =  self.devsign * qd - qgd_parasitic - (self.CGEOMOD == 1 ? qgd_fr : 0) - self.devsign * Qed
#   QS  =  self.devsign * qs - qgs_parasitic - (self.CGEOMOD == 1 ? qgs_fr : 0) - self.devsign * Qes
#   QB  =  self.devsign * (qb + qb_acc) + self.devsign * (Qeg + Qes + Qed)
#
#   # Intrinsic Capacitances (Physical)
#   CGGI  =  ddx(QGI, V(`IntrinsicGate))
#   CGSI  =  -ddx(QGI, V(si))
#   CGDI  =  -ddx(QGI, V(di))
#   CGEI  =  -ddx(QGI, V(e))
#
#   CSGI  =  -ddx(QSI, V(`IntrinsicGate))
#   CSDI  =  -ddx(QSI, V(di))
#   CSSI  =  ddx(QSI, V(si))
#   CSEI  =  -ddx(QSI, V(e))                    # Should be zero everywhere
#
#   CDGI  =  -ddx(QDI, V(`IntrinsicGate))
#   CDDI  =  ddx(QDI, V(di))
#   CDSI  =  -ddx(QDI, V(si))
#   CDEI  =  -ddx(QDI, V(e))
#
#   CEGI  =  -ddx(QBI, V(`IntrinsicGate))
#   CEDI  =  -ddx(QBI, V(di))                   # Should be zero everywhere
#   CESI  =  -ddx(QBI, V(si))                   # Should be zero everywhere
#   CEEI  =  ddx(QBI, V(e))
#
#   # Total Capacitances
#   CGG  =  ddx(QG, V(`IntrinsicGate))
#   CGS  =  -ddx(QG, V(si))
#   CGD  =  -ddx(QG, V(di))
#   CGE  =  -ddx(QG, V(e))
#
#   CSG  =  -ddx(QS, V(`IntrinsicGate))
#   CSD  =  -ddx(QS, V(di))
#   CSS  =  ddx(QS, V(si))
#   CSE  =  -ddx(QS, V(e))
#
#   CDG  =  -ddx(QD, V(`IntrinsicGate))
#   CDD  =  ddx(QD, V(di))
#   CDS  =  -ddx(QD, V(si))
#   CDE  =  -ddx(QD, V(e))
#
#   CEG  =  -ddx(QB, V(`IntrinsicGate))
#   CED  =  -ddx(QB, V(di))
#   CES  =  -ddx(QB, V(si))
#   CEE  =  ddx(QB, V(e))
#
#   # Total extrinsic capacitance
#   CGSEXT  =  -ddx(qgs_parasitic + (self.CGEOMOD == 1 ? qgs_fr : 0),V(si))  # Gate-Source Overlap + outer fringing
#   CGDEXT  =  -ddx(qgd_parasitic + (self.CGEOMOD == 1 ? qgd_fr : 0),V(di))  # Gate-Drain Overlap + outer fringing
#   CGBOV   =  -self.devsign * ddx(Qeg,V(e))                                 # Gate-Body Overlap
#
#   # Total of Junction Capacitance and Source/Drain-Body Overlap Capacitance
#   CJST  =  -self.devsign * ddx(Qes, V(si))
#   CJDT  =  -self.devsign * ddx(Qed, V(di))
#
#   RSGEO  =  self.RSourceGeo                                                # External bias independent Source Resistance
#   RDGEO  =  self.RDrainGeo                                                 # External bias independent Drain Resistance
#   CFGEO  =  self.Cfr_geo                                                   # Geometric Parasitic Cap for CGEOMOD=1
#
#   # Output for Self-Heating Temperature
#   T_TOTAL_K   =  DevTemp
#   T_TOTAL_C   =  DevTemp - P_CELSIUS0
#   T_DELTA_SH  =  Temp(t)
#
#   `ifdef __DEBUG__
#      # Individual Gate Current Components
#      IGS   =  self.devsign * igs
#      IGD   =  self.devsign * igd
#      IGCS  =  self.devsign * igcs
#      IGCD  =  self.devsign * igcd
#      if (self.BULKMOD == 0):
#         IGBS  =  self.devsign * igbs
#         IGBD  =  self.devsign * igbd
#      else:
#         IGBINV  =  self.devsign * igbinv
#         IGBACC  =  self.devsign * igbacc
#
#
#      DIDSDVG  =  self.devsign * sigvds * ddx(ids, V(`IntrinsicGate))
#      DIDSDVS  =  self.devsign * sigvds * ddx(ids, V(si))
#      DIDSDVD  =  self.devsign * sigvds * ddx(ids, V(di))
#      `ifdef __SHMOD__
#         DIDSDVTH  =  self.devsign * sigvds * ddx(ids, Temp(t))
#      `endif
#      DIGSDVG  =  self.devsign * ddx(igs + igcs, V(`IntrinsicGate))
#      DIGSDVS  =  self.devsign * ddx(igs + igcs, V(si))
#      DIGSDVD  =  self.devsign * ddx(igs + igcs, V(di))
#      `ifdef __SHMOD__
#         DIGSDVTH  =  self.devsign * ddx(igs + igcs, Temp(t))
#      `endif
#      DIGDDVG  =  self.devsign * ddx(igd + igcd, V(`IntrinsicGate))
#      DIGDDVS  =  self.devsign * ddx(igd + igcd, V(si))
#      DIGDDVD  =  self.devsign * ddx(igd + igcd, V(di))
#      `ifdef __SHMOD__
#         DIGDDVTH  =  self.devsign * ddx(igd + igcd, Temp(t))
#      `endif
#      DIIIDVG  =  self.devsign * ddx(Iii, V(`IntrinsicGate))
#      DIIIDVS  =  self.devsign * ddx(Iii, V(si))
#      DIIIDVD  =  self.devsign * ddx(Iii, V(di))
#      `ifdef __SHMOD__
#         DIIIDVTH  =  self.devsign * ddx(Iii, Temp(t))
#      `endif
#      DIGIDLDVG  =  self.devsign * ddx(igidl, V(`IntrinsicGate))
#      DIGIDLDVS  =  self.devsign * ddx(igidl, V(si))
#      DIGIDLDVD  =  self.devsign * ddx(igidl, V(di))
#      `ifdef __SHMOD__
#         DIGIDLDVTH  =  self.devsign * ddx(igidl, Temp(t))
#      `endif
#      DIGISLDVG  =  self.devsign * ddx(igisl, V(`IntrinsicGate))
#      DIGISLDVS  =  self.devsign * ddx(igisl, V(si))
#      DIGISLDVD  =  self.devsign * ddx(igisl, V(di))
#      `ifdef __SHMOD__
#         DIGISLDVTH  =  self.devsign * ddx(igisl, Temp(t))
#      `endif
#
#      `ifdef __SHMOD__
#         CGT  =  ddx(QG, Temp(t))
#         CST  =  ddx(QS, Temp(t))
#         CDT  =  ddx(QD, Temp(t))
#      `endif
#      ITH  =  ids * vds
#      `ifdef __SHMOD__
#         DITHDVTH  =  ddx(ITH, Temp(t))
#      `endif
#      DITHDVG  =  ddx(ITH, V(`IntrinsicGate))
#      DITHDVS  =  ddx(ITH, V(si))
#      DITHDVD  =  ddx(ITH, V(di))
#   `endif # __DEBUG__
#`endif # __OPINFO__
      #           analog block ends
  #================================================
